Search.additionalData = [
["history.html","Revision History ",
"Revision History\n",0]
,["history.html","Version 1.2 (2015-01-15)",
"Version 1.2 (2015-01-15)\n",0]
,["history.html","Changes",
"Changes\n \t4.1.16. Adding a Block-to-Linear Image Conversion and Transfer Command Request \tAdded a note about the image size to specify in a transfer request. \tAdded descriptions of the errors generated by the nngxAddB2LTransferCommand function.\n",0]
,["history.html","Version 1.1 (2014-11-10)",
"Version 1.1 (2014-11-10)\n",0]
,["history.html","Changes ",
"Changes\n \t3.5. Copying From the Color Buffer to the Display Buffer \tAdded a description of the minimum value for the copy region in calls to nngxTransferRenderImage and the error that occurs if a smaller value is specified.\n",0]
,["history.html","Version 1.0 (2014-09-04)",
"Version 1.0 (2014-09-04)\n",0]
,["history.html","Additions and Changes ",
"Additions and Changes\n \tInitial version.\nCONFIDENTIAL\n",0]
,["title.html","3DS Programming Manual",
"3DS Programming Manual: Basic Graphics\nVersion 1.2\n\tNintendo Confidential\n\tThe content of this document is highly confidential and should be handled accordingly.\nThe content of this document cannot be duplicated, copied, reprinted, transferred, distributed, or loaned in whole or in part without the prior approval of Nintendo.\nThis document contains confidential and proprietary information of Nintendo and is also protected under the copyright laws of the United States and foreign countries.No part of this document may be released, distributed, transmitted or reproduced in any form or by any electronic or mechanical means, including information storage and retrieval systems, without permission in writing from Nintendo.\n© 2014–2015 Nintendo. All rights reserved.\nCTR-06-0077-002-B\nAll company and product names in this document are the trademarks or registered trademarks of their respective companies.\n",0]
,["Pages/Page_55588594.html","1. Introduction ",
"1. Introduction\nThis document explains basic information and programming procedures necessary to use 3DS graphics features. 3D graphics features for 3DS are based on OpenGL ES 1.1. To read this document, you must have an understanding of OpenGL in addition to matrix and vector math.\n Warning:\n Even though the 3DS system supports the use of OpenGL, for performance reasons, it is often advisable to avoid calling GL functions when you frequently employ 3D graphics features. Instead, write directly to PICA registers using 3D commands and command caches, as explained in the 3DS Programming Manual: Advanced Graphics. Please review the CTR Programming Manual: Advanced Graphics, which explains many other features, after you read this document.\n Four libraries are provided for drawing graphics: GL, GD, GR, and the collection of graphics libraries included in NintendoWare for CTR (NW4C), distributed by Nintendo.\n GL is a basic library that provides functions conforming to OpenGL ES specifications and a rich variety of features, including error handling and state delta management. Accordingly, this library places a heavy processing load on the system. We do not recommend using the GL library when high performance is required.\n GD is a lighter-weight library than GL. Although it is not compatible with the GL library, it has equivalent features and provides an easy-to-use API. The GD library requires a certain amount of CPU cycles for internal processing.\n GR is a library intended to support the direct creation of 3D commands. Although this library executes the fastest, its use requires in-depth knowledge of how registers are set, meaning that error handling is the responsibility of the developer.\n NW4C provides libraries and source code files that use command cache-related features, so be sure to look into the possible use of NW4C. Although NW4C has been implemented to take fullest advantage of optimized performance when used without modification, developers who want to create customized processing can use the GR and GD libraries in combination with NW4C.\n",0]
,["Pages/Page_55588594.html#PageId_55588595","1.1. Document Structure ",
"\n1.1. Document Structure\n2. GPU gives an overview of the GPU built into the 3DS system, in addition to a brief summary of its features. This chapter also explains the order in which graphics are processed. Read this chapter to get a grasp of what you can render using 3DS graphics, in addition to an overall picture of graphics processing.\n3. LCD explains the relationship between the GX library and the LCD screens built into the 3DS system. Read this chapter to understand the close connection between the GX library and the images displayed on the LCDs.\n4. Command Lists explains command lists, which are required to execute 3D graphics commands.\n5. Shader Programs describes the types of shader programs and explains how to use them.\n6. Vertex Buffers explains how to create and use vertex buffers to input batches of vertex data to the vertex shader.\n7. Textures describes the types of texture images that can be handled by the 3DS system, and the CTR native format.\n8. Vertex Shaders describes input vertex data, and the vertex attributes that are output for use in later-stage processing.\n9. Geometry Shaders gives an overview of and explains how to use the shaders provided by the SDK for generating basic geometry, such as points and lines, and for other useful features, such as silhouettes and subdivision.\n10. Rasterization describes the processing that immediately precedes per-fragment operations. The 3DS system runs the scissor test at this stage.\n11. Texture Processing explains how texture units, combiners, and combiner buffers determine texel colors; how to combine lights with colors; and how to create procedural textures.\n12. Reserved Fragment Shaders describes per-fragment operations that can be controlled using reserved uniforms, such as fragment lighting, shadows, fog, and gas rendering.\n13. Per-Fragment Operations describes the operations performed on fragments, before they are written to the render buffer. Refer to this chapter for more information about blending, and the alpha test and other tests.\n14. Framebuffer Operations describes framebuffer processing and explains how to clear buffers.\n15. Miscellaneous summarizes the differences between the CTR graphics libraries and the OpenGL ES specifications.\n Note:\n 3DS Programming Manual: Advanced Graphics presents PICA register information, graphics command caching, and sample implementations of lighting models. It also introduces features that require special settings such as stereoscopic display, block mode, early depth tests, and so on.\n",0]
,["Pages/Page_55588594.html#PageId_55588596","1.2. Graphics Processing Errors ",
"\n1.2. Graphics Processing Errors\n3D graphics for 3DS are based on OpenGL ES. Behavior follows the same specifications as OpenGL ES if an error occurs during a call to a 3D graphics function.\n\tThe processing of the function in question is ignored, except when there is a GL_OUT_OF_MEMORY error. \tYou can get error codes with the glGetError function. \tOnly a single error code is recorded. The error code is not updated until it is read by the glGetError function, even if another error is generated in the meantime.\nThe following error codes are recorded.\nTable 1-1. Error Code List\n \t Error Code\n \t Conditions for Generating an Error\n \t GL_INVALID_ENUM\n \t An out-of-range value was specified as a GLenum parameter.\n \t GL_INVALID_VALUE\n \t An out-of-range value was specified as a numeric argument (and similar cases).\n \t GL_INVALID_OPERATION\n \t A function was called when it was prohibited to do so (and similar cases).\n \t GL_OUT_OF_MEMORY\n \t There is not enough memory to execute the function.\n ",0]
,["Pages/Page_55588594.html#PageId_55588597","1.3. Single-Threaded Model ",
"\n1.3. Single-Threaded Model\nThe 3D graphics library was designed under the assumption that it would be called from a single thread. Operation is not guaranteed when functions are called from multiple threads. Functions must also be called on only one rendering context.\nCONFIDENTIAL\n",0]
,["Pages/Page_55588598.html","2. GPU ",
"2. GPU\nA PICA graphics core (268 MHz) developed by Digital Media Professionals (DMP) is installed as the 3DS GPU. It employs a framebuffer architecture. There are no equivalents of the OBJ and BG concepts previously employed on the Nintendo DS. Unlike rendering with the 3D engine for the Nintendo DS (which was closely connected with the VSync), on CTR rendering is possible until buffers are swapped. In other words, you can control the trade-off between the frame rate and rendering quality.\n3D graphics features are based on OpenGL ES 1.1, but some of them correspond to OpenGL ES 2.0. Frequently used 3D graphics features are built into the hardware. Shader programs are supported. Programmable vertex shaders and non-programmable pixel shaders are also provided.\n Note:\n A simple comparison of the GPU reveals that its specifications do not match the Wii or Nintendo GameCube GPU in terms of vertex operations or fill rate. However, built-in hardware features allow richer imagery to be shown for some scenes than is possible with the Wii or Nintendo GameCube.\n",0]
,["Pages/Page_55588598.html#PageId_55588599","2.1. Built-In Hardware Features ",
"\n2.1. Built-In Hardware Features\nFrequently-used graphics features, including some that are not stipulated by the OpenGL ES standard, are built into the hardware. This makes it possible to express varied imagery in a small number of processing steps.\nThe following features have been provided.\n\tSilhouette generation \tParticle generation \tProcedural textures \tFragment lighting \tGas rendering \tSelf-shadowing and soft shadowing ",0]
,["Pages/Page_55588598.html#PageId_55588600","2.1.1. Silhouette Generation ",
" \tPolygon subdivision\nShader programs and reserved uniform settings can use these features for graphics processing.\n2.1.1. Silhouette Generation\nSilhouette generation is a feature that uses the GPU to detect object edges and render only those edges. You can use this feature to outline a selected object for emphasis or to render soft shadows.\n",0]
,["Pages/Page_55588598.html#PageId_55588601","2.1.2. Particle Generation ",
"\n2.1.2. Particle Generation\nParticle generation is a feature that renders a large number of random particles. You can use this feature to render explosions, snow, and other effects.\n",0]
,["Pages/Page_55588598.html#PageId_55588602","2.1.3. Procedural Textures ",
"\n2.1.3. Procedural Textures\nProcedural textures represent a feature that automatically generates textures using a combination of random noise and geometric patterns. This feature is a fast, low-cost way to generate textures with regular geometric patterns, in addition to textures that have some randomness in addition to a regular pattern, such as wood grain and marble.\n",0]
,["Pages/Page_55588598.html#PageId_55588603","2.1.4. Fragment Lighting ",
"\n2.1.4. Fragment Lighting\nFragment lighting is used on the 3DS system to light scenes on a per-fragment basis. This feature allows you to use bump mapping, apply environment maps, and run lighting calculations quickly.\n",0]
,["Pages/Page_55588598.html#PageId_55588604","2.1.5. Gas Rendering ",
"\n2.1.5. Gas Rendering\nGas rendering is a feature used to render gaseous objects, while accounting for intersections and foreground/background relationships with polygon objects. This feature allows you to render surface boundaries with other objects more naturally than, for example, gaseous objects rendered by particles.\n",0]
,["Pages/Page_55588598.html#PageId_55588605","2.1.6. Self-Shadowing and Soft Shadows ",
"\n2.1.6. Self-Shadowing and Soft Shadows\nSelf-shadowing and soft shadows are supported by the 3DS system. These features allow an object to cast shadows on itself, and soft shadows to be rendered more naturally around the contours of an object.\n",0]
,["Pages/Page_55588598.html#PageId_55588606","2.1.7. Polygon Subdivision ",
"\n2.1.7. Polygon Subdivision\nPolygon subdivision is a feature that takes a group of vertices that are input according to fixed rules and automatically splits them into smooth polygons, using the GPU. This feature allows you to render objects with curved surfaces, given a small number of input vertices.\n",0]
,["Pages/Page_55588598.html#PageId_55588607","2.2. Rendering Pipeline ",
"\n2.2. Rendering Pipeline\n3D graphics for 3DS are processed in a series of steps known as the rendering pipeline.\nFigure 2-1 shows a block view of the rendering pipeline and Table 2-1 shows what each process does. For more information, see the relevant pages for each process.\nFigure 2-1. Schematic View of the Rendering Pipeline\nTable 2-1. Actions Performed by Each Process\n \t Process\n \t Description\n \t Vertex input\n \t Processes vertex data input from the application.\n \t Vertex operations\n \t Runs the vertex shaders.\n \t Geometry generation\n \t Runs the geometry shaders.\n \t Triangle generation\n \t Converts all primitives into triangles.\n \t Clipping\n \t Uses a clipping volume to clip primitives.\n \t Rasterization\n \t Converts primitives into fragments.\n \t Scissor test\n \t Eliminates unnecessary fragments via scissoring.\n \t Early depth test\n \t Runs depth tests before per-fragment operations.\n \t Fragment lighting\n \t Processes per-fragment lighting.\n \t Texel generation\n \t Determines texel colors from texture coordinates.\n \t Texture combiners\n \t Combines fragment light colors, texture colors, and so on.\n \t Fog\n \t Renders fog and gas.\n \t Per-fragment operations\n \t Runs tests, blending, logical operations, and other processing on fragments.\n \t Frame buffer operations\n \t Copies data from the framebuffer, runs pixel-read operations, and so on.\nCONFIDENTIAL\n",0]
,["Pages/Page_55588608.html","3. LCD",
"3. LCD\nOn the 3DS system, the GX (graphics) library is closely involved with the images displayed on the LCD screens. This chapter first describes LCD screen resolution, placement, and orientation. It then continues with the following topics: the required initialization process, the steps taken to display the rendering results on the LCDs, the methods for allocating various buffer types, screen update synchronization, and the shutdown process.\n Note:\n For more information about stereoscopic display, see the 3DS Programming Manual: Advanced Graphics.\n",0]
,["Pages/Page_55588608.html#PageId_55588609","3.1. LCD Resolution, Placement, and Orientation",
"3.1. LCD Resolution, Placement, and Orientation\nThe LCDs on the 3DS system differ from the LCDs on the Nintendo DS (NITRO) and Nintendo DSi (TWL) systems, both in terms of resolution and arrangement. Table 3-1 shows the differences in resolution and Figure 3-1 shows the differences in placement and orientation.\nTable 3-1. LCD Resolution\n \t LCD\n \t NITRO/TWL\n \t3DS\n \t Upper screen (width × height)\n \t 256 × 192\n \t 240 × 400\n \t Lower screen (width × height)\n \t 256 × 192\n \t 240 × 320\n Figure 3-1. LCD Placement and Orientation\nThe standard LCD arrangement on the NITRO and TWL systems places the long edges of the LCDs (the screen width) horizontally, with the lower screen in front. The standard LCD arrangement on the 3DS system places the short edges of the LCDs horizontally, with the lower screen to the left of the upper screen (as when the front of the controller is rotated 90 degrees to the right).\n",0]
,["Pages/Page_55588608.html#PageId_55588610","3.2. Required Initialization Process",
"3.2. Required Initialization Process\nA framebuffer architecture has been adopted for 3DS. Because the images displayed on the LCDs are based on the framebuffer content, the GX library must be initialized first.\n",0]
,["Pages/Page_55588608.html#PageId_55588611","3.2.1. Initializing the GX Library",
"3.2.1. Initializing the GX Library\nCall the nngxInitialize function to initialize the GX library.\nCode 3-1. Initialization Function for the GX Library\n GLboolean nngxInitialize(\n GLvoid* (*allocator)(GLenum, GLenum, GLuint, GLsizei),\n void (deallocator)(GLenum, GLenum, GLuint, GLvoid));\nGLboolean nngxGetIsInitialized();\nAn allocator and a deallocator are specified to the nngxInitialize function, using the allocator and deallocator parameters, respectively. The allocator is called when a display buffer or other memory region is allocated, and the deallocator is called when the same memory region is freed. For more information about the allocator and deallocator, see 3.2.1.1. Allocator and 3.2.1.2. Deallocator.\nA value of GL_TRUE is returned when initialization succeeds, and GL_FALSE is returned when initialization fails. A value of GL_FALSE is also returned if this function is called after the library has already been initialized but before the nngxFinalize function has been called to shut down the library. Behavior is not guaranteed if any other gl or nngx functions are called before using this function to initialize the library.\nThe nngxGetIsInitialized function returns GL_TRUE if the nngxInitialize function has initialized the GX library.\nThe nngxInitialize function does not allocate buffers such as the default render buffer. These buffers must be allocated by the application after initialization.\n",0]
,["Pages/Page_55588608.html#PageId_55588612","3.2.1.1. Allocator ",
"3.2.1.1. Allocator\nThe first allocator argument indicates the memory from which to allocate a region. The following values can be passed.\n \t NN_GX_MEM_FCRAM:\nAllocate from main (device) memory. \t NN_GX_MEM_VRAMA:\nAllocate from VRAM-A. \t NN_GX_MEM_VRAMB:\nAllocate from VRAM-B.\n Given a value of NN_GX_MEM_FCRAM, the allocator allocates from main memory, but the region to allocate must also be located in device memory. Device memory is a memory region that guarantees address consistency when it is accessed by peripheral devices. Applications are responsible for memory management. You can use the nn::os::GetDeviceMemoryAddress and nn::os::GetDeviceMemorySize functions to get the starting address and size of the memory region, allocated as device memory.\n Note:\n For more information about device memory, see the 3DS Programming Manual: System.\nIf the allocator is given a value of NN_GX_MEM_VRAMA or NN_GX_MEM_VRAMB, use the nn::gx::GetVramStartAddr, nn::gx::GetVramEndAddr, and nn::gx::GetVramSize functions to get the starting address, ending address, and size of VRAM, respectively. As with main memory, the application is responsible for memory management.\nThe second allocator argument is passed a value indicating the usage (buffer type) of the memory to allocate. Because addresses are aligned differently for each type, implement your application's allocator to comply with the following rules.\nTable 3-2. Differences in Alignment Between Buffer Types\n \t Argument Value\n \t Buffer Type\n \t Alignment\n \t NN_GX_MEM_TEXTURE\n \t Texture\n(2D, environment map)\n \t 128 bytes for all formats.\n \t NN_GX_MEM_VERTEXBUFFER\n \t Vertex buffer\n \t Depends on the vertex attribute.\n 4 bytes (GLfloat type)\n2 bytes (GLshort type, GLushort type)\n1 byte (GLbyte type, GLubyte type)\n \t NN_GX_MEM_RENDERBUFFER\n \t Color buffer\n \t 64 bytes\n \t Depth buffer\n(stencil buffer)\n \t 32 bytes (16-bit depth)\n96 bytes (24-bit depth)\n64 bytes (24-bit depth + 8-bit stencil)\n \t NN_GX_MEM_DISPLAYBUFFER\n \t Display buffer\n \t 16 bytes\n \t NN_GX_MEM_COMMANDBUFFER\n \t 3D command buffer\n \t 16 bytes\n \t NN_GX_MEM_SYSTEM\n \t System buffer\n \t 4 bytes (when the memory size to allocate is a multiple of 4).\n2 bytes (when the memory size to allocate is a multiple of 2, but not a multiple of 4).\n1 byte (when none of the above).\n In addition to the rules given above, your implementation must comply with the following hardware specifications.\n \tAll six faces of a cube map texture must fit within the same 32 MB boundaries. \tAll six faces of a cube map texture must have addresses that share the same value for the most-significant 7 bits. \tThe GL_TEXTURE_CUBE_MAP_POSITIVE_X face of a cube map texture must have a smaller address, or the same address, as every other face. \tData must not be placed so that it spans VRAM-A and VRAM-B.\n Warning:\n Do not allocate a display buffer in the last 1.5 MB of VRAM-A or VRAM-B.\n If texture addresses are not correctly aligned, the GPU may hang, rendered results may be corrupted, or other problems may arise.\nIf NN_GX_MEM_VERTEXBUFFER, NN_GX_MEM_RENDERBUFFER, NN_GX_MEM_DISPLAYBUFFER, or NN_GX_MEM_COMMANDBUFFER was specified for the second argument, the third allocator argument is passed the name (ID) of that object.\nThe fourth allocator argument is passed the size of the region to allocate.\nThe application must account for address alignment and allocate the specified memory region based on these arguments, and then return the region's starting address. The application must return a value of 0 if it fails to allocate the region.\n",0]
,["Pages/Page_55588608.html#PageId_55588613","3.2.1.2. Deallocator ",
"3.2.1.2. Deallocator\nThe first, second, and third deallocator arguments are passed the same values that were used when the memory region was allocated. The fourth argument is passed the starting address of the memory region. The application must use these arguments to release the memory region allocated by the allocator.\n",0]
,["Pages/Page_55588608.html#PageId_55588614","3.2.1.3. Getting an Allocator ",
"3.2.1.3. Getting an Allocator\nYou can get the allocator that was configured when the GX library was initialized by calling the nngxGetAllocator function.\nCode 3-2. Getting an Allocator\n void nngxGetAllocator (\n GLvoid* (**allocator)(GLenum, GLenum, GLuint, GLsizei),\n void (*deallocator)(GLenum, GLenum, GLuint, GLvoid));\n The allocator and deallocator parameters specify pointers that will be assigned pointers to the allocator and deallocator. No allocator or deallocator is obtained when specifying NULL for each of the arguments.\n",0]
,["Pages/Page_55588608.html#PageId_55588615","3.2.1.4. Getting an Initialization Register Setting Command ",
"3.2.1.4. Getting an Initialization Register Setting Command\nYou can get an executable Initialization Register Setting Command when calling the nngxGetInitializationCommand and nngxInitialize functions.\n Note:\n This function was added as a measure related to rendering that takes place when returning to an application from the HOME Menu when generating a setting command directly in the register without using the graphics library supported by SDK. Consequently, it is not normally necessary to use it.\n Code 3-3. Getting an Initialization Register Setting Command\n GLsizei nngxGetInitializationCommand(GLsizei datasize, GLvoid* data);\nFor the data parameter, you can specify a pointer to the command that receives the register setting command. For the datasize parameter, you can specify the buffer byte size indicated by the data parameter.\nThis function returns the size in bytes of the Initialization Register Setting Command. Specifying 0 (NULL) for data gets the Register Setting Command, and returns the buffer size to allocate. After you specify 0 for data, prepare that size of the buffer and get the command by making a call-out again.\nIf you are calling the nngxInitialize function before initialization, 0 is returned without getting the command.\nTable 3-3. Error Generated by the nngxGetInitializationCommand Function\n \t Error Code\n \t Cause\n \t GL_ERROR_80B4_DMP\n \t The value specified for datasize is smaller than the acquired register command.\n",0]
,["Pages/Page_55588608.html#PageId_55588616","3.2.2. Command-List Object Creation ",
"3.2.2. Command-List Object Creation\nYou must create command list objects after initializing the GX library. A command list object is a CTR-specific mechanism for running command lists. These objects serve as the units in which command lists are run. This section only explains how to create them. For more information about command lists and command list objects, see 4. Command Lists.\nFirst use the nngxGenCmdlists function to create one or more command list objects, individually bind them to the GPU with the nngxBindCmdlist function, and then allocate individual memory regions with the nngxCmdlistStorage function.\nSet the bufsize and requestcount parameters of the nngxCmdlistStorage function to the size of the 3D command buffer and the number of command requests that can be queued, respectively. When you create multiple command list objects, you must call the nngxBindCmdlist and nngxCmdlistStorage function on each of them.\nThe following code sample creates a single command list object that has a 3D command buffer size of 256 KB and can queue 128 command requests.\nCode 3-4. How to Create a Command List Object\n GLuint commandList;\nnngxGenCmdlists(1, &commandList);\nnngxBindCmdlist(commandList);\nnngxCmdlistStorage(256 * 1024, 128);\n",0]
,["Pages/Page_55588608.html#PageId_55588617","3.3. Path of the Rendering Results Prior to Being Displayed on the LCDs ",
"3.3. Path of the Rendering Results Prior to Being Displayed on the LCDs\nFigure 3-2 shows the flow of rendering results from the GPU to the LCDs.\nFigure 3-2. Path of the Rendering Results From the GPU to the LCDs\nThis process is roughly split into the following steps.\n \tRender. \tCopy data from the render buffer to the display buffer and convert its format. \tSwap buffers to update the regions displayed on the LCDs.\nRendering is described in later chapters. This chapter describes buffer allocation, which is required for output to the LCDs, and then describes the other steps in the diagram above.\n",0]
,["Pages/Page_55588608.html#PageId_55588618","3.4. Allocating Buffers ",
"3.4. Allocating Buffers\nTwo of the buffers used to render graphics are involved in LCD output: the framebuffer, which is the GPU render target; and the display buffer, which is used to copy rendering results and display them on the LCDs.\n",0]
,["Pages/Page_55588608.html#PageId_55588619","3.4.1. Allocating Render Buffers ",
"3.4.1. Allocating Render Buffers\nFirst, you must use the glGenFramebuffers function to create a framebuffer object to specify as the render target. You can then bind the various render buffers (color, depth, or stencil) to the framebuffer object to specify those render buffers as the render target. The 3DS system has two screens (an upper and a lower screen), but if they have the same format and do not need to be rendered in parallel, we recommend that they share an object to conserve memory resources. In this case, specify the dimensions of the upper screen as the width and height of the buffer.\n Code 3-5. Creating a Framebuffer Object\n GLuint frameBufferObject;\nglGenFramebuffers(1, &frameBufferObject);\nNext, use the glGenRenderbuffers function to create the render buffer objects. If the render target includes a depth buffer, stencil buffer, or both, in addition to a color buffer, you need two render buffer objects for the framebuffer object.\nCode 3-6. Creating Render Buffer Objects\n GLuint renderBuffer[2];\nglGenRenderbuffers(2, renderBuffer);\nUse the glBindRenderbuffer function to specify the render buffer object to bind to the framebuffer object, and then use the glRenderbufferStorage function to allocate a render buffer. The glRenderbufferStorage function is defined as follows.\nCode 3-7. Definition of the glRenderbufferStorage Function\n void glRenderbufferStorage(GLenum target, GLenum internalformat,\n GLsizei width, GLsizei height);\nSet width and height to the width and height of the render buffer. Neither the width nor the height can be greater than 1024 pixels.\n Warning:\n Block-shaped noise may be rendered on some pixels if the render buffer has 262,128 or more pixels (the product of its width and height). For more information, see 15.8. Block-Shaped Noise Is Rendered on Certain Pixels.\nBy setting target to the bitwise OR of GL_RENDERBUFFER and the following bitmasks, you can specify the memory from which to allocate the buffer. If no bitmask is specified, the buffer is allocated as if NN_GX_MEM_VRAMA was specified.\nTable 3-4. Specifiable Bitmasks for the target Parameter of glRenderbufferStorage\n \t Bitmask\n \t Buffer Is Allocated In\n \t NN_GX_MEM_VRAMA\n \t VRAM-A.\n \t NN_GX_MEM_VRAMB\n \t VRAM-B.\n Specify the buffer type (format) in the internalformat parameter. You can choose from the following formats on the 3DS system.\nTable 3-5. Specifiable Formats for the internalformat Parameter of glRenderbufferStorage\n \t Format\n \t Bits\n \t Description of Format\n \t GL_DEPTH_COMPONENT16\n \t 16\n \t 16-bit depth.\n \t GL_DEPTH_COMPONENT24_OES\n \t 24\n \t 24-bit depth.\n \t GL_RGBA4\n \t 16\n \t The R, G, B, and alpha components are 4 bits each.\n \t GL_RGB5_A1\n \t 16\n \t The R, G, and B components are 5 bits each, and the alpha component is 1 bit.\n \t GL_RGB565\n \t 16\n \t 5-bit RB components and 6-bit G component. No alpha component.\n \t GL_RGBA8_OES\n \t 32\n \t The R, G, B, and alpha components are 8 bits each.\n \t GL_DEPTH24_STENCIL8_EXT\n \t 32\n \t 24-bit depth and 8-bit stencil value.\n \t GL_GAS_DMP\n \t 32\n \t Density values used during gas rendering (the rendered results cannot be copied to the display buffer).\n To specify an allocated render buffer as the render target, use the glFramebufferRenderbuffer function to bind it to the framebuffer object that was specified by the glBindFramebuffer function.\nCode 3-8. Definitions of glBindFramebuffer and glFramebufferRenderbuffer Functions\n void glBindFramebuffer(GLenum target, GLuint framebuffer);\nvoid glFramebufferRenderbuffer(GLenum target, GLenum attachment,\n GLenum renderbuffertarget, GLuint renderbuffer);\nSet target to GL_FRAMEBUFFER for both functions.\nSet framebuffer to the framebuffer object and renderbuffer to the render buffer object. Set renderbuffertarget to GL_RENDERBUFFER.\nThe value to specify for attachment depends on the render buffer format.\nTable 3-6. Render Buffer Formats and the attachment Values to Use With Them\n \t Format\n \t Value of attachment\n \t GL_DEPTH_COMPONENT16\n \t GL_DEPTH_ATTACHMENT\n \t GL_DEPTH_COMPONENT24_OES\n \t GL_RGBA4\n \t GL_COLOR_ATTACHMENT0\n \t GL_RGB5_A1\n \t GL_RGB565\n \t GL_RGBA8_OES\n \t GL_GAS_DMP\n \t GL_DEPTH24_STENCIL8_EXT\n \t GL_DEPTH_STENCIL_ATTACHMENT\n You can call the glCheckFramebufferStatus function to get the state of the render buffer object that is bound to the framebuffer object. Set the function’s target parameter to GL_FRAMEBUFFER. A GL_INVALID_ENUM error is generated if you specify any other value.\nIf GL_FRAMEBUFFER_COMPLETE is returned, a proper render buffer object has been bound.\nIf GL_FRAMEBUFFER_INCOMPLETE_MISSING_ATTACHMENT is returned, neither the color buffer nor the depth (stencil) buffer has been attached.\nIf GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT is returned, either memory has not been allocated for the buffer that is bound, or the same render buffer object has been bound to both the color buffer and the depth buffer.\nIf GL_FRAMEBUFFER_INCOMPLETE_DIMENSIONS is returned, buffers with different sizes have been bound.\nThe following sample code allocates render buffers. Note that settings differ between the color and depth (stencil) buffers. The render buffer is shared by the upper and lower screens, so its width and height are specified using the dimensions of the upper screen.\nCode 3-9. Allocating Render Buffers (Color, Depth, Stencil)\n // FrameBuffer\nglBindFramebuffer(GL_FRAMEBUFFER, frameBufferObject);\n// Color\nglBindRenderbuffer(GL_RENDERBUFFER, renderBuffer[0]);\nglRenderbufferStorage(GL_RENDERBUFFER | NN_GX_MEM_VRAMA, GL_RGBA8_OES,\n nn::gx::DISPLAY0_WIDTH, nn::gx::DISPLAY0_HEIGHT);\nglFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0,\n GL_RENDERBUFFER, renderBuffer[0]);\n// Depth / Stencil\nglBindRenderbuffer(GL_RENDERBUFFER, renderBuffer[1]);\nglRenderbufferStorage(GL_RENDERBUFFER | NN_GX_MEM_VRAMB,\n GL_DEPTH24_STENCIL8_EXT, nn::gx::DISPLAY0_WIDTH, nn::gx::DISPLAY0_HEIGHT);\nglFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_STENCIL_ATTACHMENT,\n GL_RENDERBUFFER, renderBuffer[1]);\n",0]
,["Pages/Page_55588608.html#PageId_55588620","3.4.2. Allocating Display Buffers ",
"\n3.4.2. Allocating Display Buffers\nBefore you allocate a display buffer, you must call the nngxActiveDisplay function to specify whether it will be used for the upper or lower screen.\nCode 3-10. Definition of the nngxActiveDisplay Function\n void nngxActiveDisplay(GLenum display);\nSpecify either the upper or lower screen with the value you pass to display. A GL_ERROR_801F_DMP error occurs when display is set to a value other than the ones listed in the table below.\nTable 3-7. Values to Set for display\n \t Value of display\n \t Specified Screen\n \t NN_GX_DISPLAY0\n \t Upper screen (during stereoscopic display, this is the image for the left eye).\n \t NN_GX_DISPLAY0_EXT\n \t Upper screen (specifiable only during stereoscopic display, when this is the image for the right eye).\n \t NN_GX_DISPLAY1\n \t Lower screen.\n Note:\n For more information about stereoscopic display, see the 3DS Programming Manual: Advanced Graphics.\nNext, use the nngxGenDisplaybuffers function to create the display buffer objects.\nCode 3-11. Definition of the nngxGenDisplaybuffers Function\n void nngxGenDisplaybuffers(GLsizei n, GLuint* buffers);\nSet n to the number of display buffer objects to be created. Set buffers to point to an array in which to store the display buffer objects. If you are using more than one display buffer for a single screen through multi-buffering, create only as many objects as you need.\nTable 3-8. Errors Generated by the nngxGetInitializationCommand Function\n \t Error Code\n \t Cause\n \t GL_ERROR_801C_DMP\n \t A negative value was specified for n.\n \t GL_ERROR_801D_DMP\n \t The internal buffer failed to be allocated.\n Call the nngxBindDisplaybuffer function on the generated display buffer object to set it as the target display buffer.\nCode 3-12. Definition of the nngxGenDisplaybuffers Function\n void nngxBindDisplaybuffer(GLuint buffer);\nAn object is created as long as an unused object name is specified for buffer. A GL_ERROR_8020_DMP error occurs when the internal buffer failed to be allocated.\nUse the nngxDisplaybufferStorage function to allocate a display buffer. The nngxDisplaybufferStorage function is defined as follows.\nCode 3-13. Definition of the nngxDisplaybufferStorage Function\n void nngxDisplaybufferStorage(GLenum format, GLsizei width, GLsizei height,\n GLenum area);\nFor format specify one of the following buffer formats. You cannot specify a format that uses more bits per pixel than the format set when the color buffer was allocated.\nTable 3-9. Specifiable Formats for format of nngxDisplaybufferStorage\n \t Format\n \t Bits\n \t Description of Format\n \t GL_RGBA4\n \t 16\n \t The R, G, B, and alpha components are 4 bits each.\n \t GL_RGB5_A1\n \t 16\n \t The R, G, and B components are 5 bits each, and the alpha component is 1 bit.\n \t GL_RGB565\n \t 16\n \t 5-bit RB components and 6-bit G component. No alpha component.\n \t GL_RGB8_OES\n \t 24\n \t 8-bit RGB components. No alpha component.\n Use width and height to specify the display buffer size. Both dimensions must be positive numbers that are multiples of the block size. The block size is either 8 or 32, depending on the render buffer's current block mode. For information about block mode settings, see the 3DS Programming Manual: Advanced Graphics.\nFor area, specify the memory in which to allocate the buffer.\nTable 3-10. Specifiable Flags for area of nngxDisplaybufferStorage Function\n \t Flag\n \t Buffer Is Allocated In\n \t NN_GX_MEM_FCRAM\n \t Main (device) memory.\n \t NN_GX_MEM_VRAMA\n \t VRAM-A.\n \t NN_GX_MEM_VRAMB\n \t VRAM-B.\n To capture and save a screen image, the display buffer must be allocated in main memory where it can be accessed by the CPU.\nIf a display buffer object already has a display buffer allocated and you allocate a new display buffer for it, the old memory region is freed and a new memory region is allocated.\nTable 3-11. Errors Generated by the nngxGetInitializationCommand Function\n \t Error Code\n \t Cause\n \t GL_ERROR_8021_DMP\n \t The target is a display buffer whose object name is 0.\n \t GL_ERROR_8022_DMP\n \t An illegal value is specified for width or height.\n \t GL_ERROR_8023_DMP\n \t An illegal value is specified for format.\n \t GL_ERROR_8024_DMP\n \t An illegal value is specified for area.\n \t GL_ERROR_8025_DMP\n \t The memory region failed to be allocated.\n You can allocate multiple display buffers. The following sample code uses double-buffering.\nCode 3-14. Allocating Display Buffers\n GLuint display0Buffers[2];\nGLuint display1Buffers[2];\n// Displaybuffer for UpperLCD\nnngxActiveDisplay(NN_GX_DISPLAY0);\nnngxGenDisplaybuffers(2, display0Buffers);\nnngxBindDisplaybuffer(display0Buffers[0]);\nnngxDisplaybufferStorage(GL_RGB8_OES, nn::gx::DISPLAY0_WIDTH,\n nn::gx::DISPLAY0_HEIGHT, NN_GX_MEM_FCRAM);\nnngxBindDisplaybuffer(display0Buffers[1]);\nnngxDisplaybufferStorage(GL_RGB8_OES, nn::gx::DISPLAY0_WIDTH,\n nn::gx::DISPLAY0_HEIGHT, NN_GX_MEM_FCRAM);\nnngxDisplayEnv(0, 0);\n// Displaybuffer for LowerLCD\nnngxActiveDisplay(NN_GX_DISPLAY1);\nnngxGenDisplaybuffers(2, display1Buffers);\nnngxBindDisplaybuffer(display1Buffers[0]);\nnngxDisplaybufferStorage(GL_RGB8_OES, nn::gx::DISPLAY1_WIDTH,\n nn::gx::DISPLAY1_HEIGHT, NN_GX_MEM_FCRAM);\nnngxBindDisplaybuffer(display1Buffers[1]);\nnngxDisplaybufferStorage(GL_RGB8_OES, nn::gx::DISPLAY1_WIDTH,\n nn::gx::DISPLAY1_HEIGHT, NN_GX_MEM_FCRAM);\nnngxDisplayEnv(0, 0);\n",0]
,["Pages/Page_55588608.html#PageId_55588621","3.4.2.1. Getting Parameters ",
"\n3.4.2.1. Getting Parameters\nYou can use the nngxGetDisplaybufferParameteri function to get information about the target display buffer.\nCode 3-15. Function for Obtaining Display Buffer Parameters\n void nngxGetDisplaybufferParameteri(GLenum pname, GLint* param);\nYou can set pname to one of the following values. Specifying any other value results in a GL_ERROR_8033_DMP error.\nTable 3-12. Display Buffer Parameters\n \t pname\n \t Parameter Information to Get\n \t NN_GX_DISPLAYBUFFER_ADDRESS\n \t Starting address of the display buffer.\n \t NN_GX_DISPLAYBUFFER_FORMAT\n \t The display buffer format.\n \t NN_GX_DISPLAYBUFFER_WIDTH\n \t The display buffer width.\n \t NN_GX_DISPLAYBUFFER_HEIGHT\n \t The display buffer height.\n",0]
,["Pages/Page_55588608.html#PageId_55588622","3.5. Copying From the Color Buffer to the Display Buffer ",
"\n3.5. Copying From the Color Buffer to the Display Buffer\nWhen rendering is done, the content of the color buffer is in block format. This cannot be output to the LCDs, which can only display data in a linear format. When rendering is done, the content of the color buffer is in block format. This cannot be output to the LCDs, which can only display data in a linear format. You must call the nngxTransferRenderImage function to convert the data into a format that can be output to the LCDs. This function also copies the data to the display buffer from the color buffer that is bound to the framebuffer object specified by glBindFramebuffer.\nCode 3-16. Function for Copying Data From the Color Buffer to the Display Buffer\n void nngxTransferRenderImage(GLuint buffer, GLenum mode, GLboolean yflip,\n GLint colorx, GLint colory);\nThe buffer parameter specifies the display buffer object into which to copy the data. If the 3D command buffer has accumulated unsplit commands, the function adds a split command and a transfer command request to the command requests. There is no guarantee that any added transfer commands have completed running after this function has executed. Wait until the command list has finished executing before doing other things, such as deleting color buffers or changing buffer contents.\nThe mode parameter specifies the degree of anti-aliasing to apply when the rendered results are copied to the display buffer.\nTable 3-13. Anti-Aliasing Specifications\n \t mode\n \t Anti-Aliasing Specification\n \t Width\n \t Height\n \t NN_GX_ANTIALIASE_NOT_USED\n \t Off (none).\n \t Equal\n \t Equal\n \t NN_GX_ANTIALIASE_2x1\n \t 2×1 anti-aliasing.\n \t 2 times\n \t Equal\n \t NN_GX_ANTIALIASE_2x2\n \t 2×2 anti-aliasing.\n \t 2 times\n \t 2 times\n The value of yflip specifies whether to apply a y-flip (a vertical flip) when the rendered results are copied to the display buffer. It is applied if the argument is set to GL_TRUE. Any value other than 0 is treated as GL_TRUE.\nThe values of colorx and colory specify the offsets to use when copying the data from the color buffer. (The function assumes that the origin is at the lower-left corner and that the positive axes are pointing up and right.) You must specify offsets that are positive multiples of the block size, which is 8 in block-8 mode and 32 in block-32 mode. (For more information about block mode settings, see the 3DS Programming Manual: Advanced Graphics.)\nStarting from the offset position in the color buffer, this function copies a region with the same width and height as the display buffer to the display buffer. Subtract the offset values from the color buffer's width and height to find the dimensions of the region that can be copied from the color buffer.\nThe height and width of the region to copy, as measured in pixels, must be at least as big as the minimum allowed. The minimum height and width when copying from a color buffer is 128. The minimum height and width when copying to a display buffer depends on the anti-alias setting. If anti-aliasing is disabled, the minimum for both height and width is 128. If 2x1 anti-aliasing is enabled, the height minimum is 128 and the width minimum is 64. If 2x2 anti-aliasing is enabled, the minimum for both height and width is 64.\nTable 3-14. Errors Generated by the nngxTransferRenderImage Function\n \t Error Code\n \t Cause\n \t GL_ERROR_8027_DMP\n \t Called when the bound command list’s object name is 0.\n \t GL_ERROR_8028_DMP\n \t The command request has already reached the maximum number of accumulated command requests allowable.\n \t GL_ERROR_8029_DMP\n \t A valid display buffer has not been bound.\n \t GL_ERROR_802A_DMP\n \t A valid color buffer has not been bound.\n \t GL_ERROR_802B_DMP\n \t An illegal value was specified for mode.\n \t GL_ERROR_802C_DMP\n \t The display buffer is larger than the region that can be copied. (When anti-aliasing, apply the width and height ratios from Table 3-13.)\n \t GL_ERROR_802D_DMP\n \t Illegal values were specified for colorx and colory.\n \t GL_ERROR_802E_DMP\n \t The display buffer uses more bits per pixel than the color buffer.\n \t GL_ERROR_802F_DMP\n \t The 3D command buffer is full because of commands added by this function.\n \t GL_ERROR_8059_DMP\n \t The width and height of the color buffer and display buffer are not multiples of 32 in block-32 mode.\n \t GL_ERROR_805A_DMP\n \t The width of the color buffer and display buffer are not multiples of 16 when the display buffer’s pixel size is 24 bits and the block mode is block 8.\n \tGL_ERROR_80B5_DMP \tThe specified height or width for copying from a color buffer was smaller than the minimum.\n \t GL_ERROR_80B6_DMP\n \tThe specified height or width for copying to a display buffer was smaller than the minimum.\n",0]
,["Pages/Page_55588608.html#PageId_55588623","3.6. Updating the LCD Render Regions by Swapping Buffers ",
"\n3.6. Updating the LCD Render Regions by Swapping Buffers\nAfter data has finished being copied to the display buffer, the buffer-swap function displays the rendered results to one or both LCDs.\nUse the nngxActiveDisplay function to specify a display and the nngxBindDisplaybuffer function to bind the display buffer from which to display. You can then call the nngxDisplayEnv function to specify the offsets to use when data is output from the display buffer to the LCDs. The function assumes that the origin is at the lower-left corner and the positive axes are pointing up and right, and that they are always positive values. Set (0, 0) for the offsets, if the display buffer is the same size as the LCD. Specifying a negative value for the offset results in a GL_ERROR_8026_DMP error.\nCode 3-17. Functions for Specifying the Display and the Display Buffer\n void nngxActiveDisplay(GLenum display);\nvoid nngxBindDisplaybuffer(GLuint buffer);\nvoid nngxDisplayEnv(GLint displayx, GLint displayy);\nNext, use the buffer-swap function to switch the buffer that is output to the LCD.\nCode 3-18. Buffer-Swap Function\n void nngxSwapBuffers(GLenum display);\nThe nngxSwapBuffers function swaps the buffers during the next VSync. This function can be called at any time, but if it is called more than once before a VSync, only the last call is valid.\nUse display to specify which display is affected when the buffers are swapped. Specify NN_GX_DISPLAY0 to target only the upper screen, NN_GX_DISPLAY1 to target only the lower screen, orNN_GX_DISPLAY_BOTH to target both screens.\nThis function configures the GPU with the address of the display buffer to display, switching the image that is displayed on the LCDs. The display buffer address that is ultimately set in the GPU is calculated from the starting address of the buffer allocated by the nngxDisplaybufferStorage function, taking into account a number of variables; including the display buffer resolution, the pixel size or number of bits per pixel (bpp), the LCD resolution, and the offsets configured by the nngxDisplayEnv function.\nThe following equation calculates the address to configure.\nBufferAddress + PixelSize × (DisplayBufferWidth × (DisplayBufferHeight - LcdHeight - DisplayY) + DisplayX)\nTable 3-15. Errors Generated by the nngxTransferRenderImage Function\n \t Error Code\n \t Cause\n \t GL_ERROR_8030_DMP\n \t An illegal value was specified for display.\n \t GL_ERROR_8031_DMP\n \t A valid display buffer has not been bound.\n \t GL_ERROR_8032_DMP\n \t The display region that considered the offset will fall outside of the display buffer.\n \t GL_ERROR_8053_DMP\n \t The display buffer address set in the GPU is not 16-byte aligned.\n \t GL_ERROR_9000_DMP\n \t The display buffer bound to the upper screen for the right eye during stereoscopic display (NN_GX_DISPLAY0_EXT) is 0 or that region has not been allocated.\n \t GL_ERROR_9001_DMP\n \t The nngxDisplayEnv function specifies a display region that is outside the display buffer.\n \t GL_ERROR_9002_DMP\n \t The display buffers bound to the two upper screens (NN_GX_DISPLAY0 and NN_GX_DISPLAY0_EXT) have different resolutions, formats, or memory regions.\n In the system’s initial state, a black screen is forced to be displayed on the LCDs. If a valid display buffer for LCD output has been prepared in the buffer-swap function, call the nngxStartLcdDisplay function at the same time as a VSync to start LCD output. You only need to call this function the first time.\nCode 3-19. Function for Starting LCD Output\n void nngxStartLcdDisplay( void );\n",0]
,["Pages/Page_55588608.html#PageId_55588624","3.6.1. Swapping Buffers for a Specified Address ",
"\n3.6.1. Swapping Buffers for a Specified Address\nBy calling the nngxSwapBuffersByAddress function, you can specify the address of a buffer to display (on the LCDs) without using a display buffer.\nCode 3-20. Swapping Buffers for a Specified Address\n void nngxSwapBuffersByAddress(GLenum display,\n const GLvoid* addr, const GLvoid* addrB,\n GLsizei width, GLenum format);\nBuffers are swapped during the first VSync after this function (like the nngxSwapBuffers function) is called. If this function is called more than once on the same display before a VSync occurs, only the last function call is valid.\nUse display to specify which display is affected when the buffers are swapped. Specify NN_GX_DISPLAY0 for the upper screen or NN_GX_DISPLAY1 for the lower screen.\nUse addr to specify the address of the buffer to display. If stereoscopic display is enabled and you have specified the upper screen with display, this argument gives the address of images to display for the left eye. You must specify an address that is 16-byte aligned.\nUse addrB to specify the address of images to display for the right eye when stereoscopic display is enabled. This argument is only valid when you have specified the upper screen with display. It is ignored if stereoscopic display is disabled or if you have specified NN_GX_DISPLAY1 in display. You must specify an address that is 16-byte aligned.\nThe display position specified by the nngxDisplayEnv function is ignored when the buffer specified by this function is displayed. Consequently, consider offsets and other factors in the addresses that you specify in addr and addrB. For more information about how addresses are calculated, see 3.6. Updating the LCD Render Regions by Swapping Buffers.\nUse width to specify the width (in pixels) of the buffer to display. Note that width specifies the width of the buffer rather than the width of the LCD. Both the upper and lower screens have a width of 240 pixels, but if you want to partially display a buffer that is wider than that, specify the width of the entire buffer (in pixels), including the parts that will not be displayed. width must be a multiple of 8 and cannot be less than 240.\nUse format to specify the buffer format. You can specify the same formats that can be specified when allocating a display buffer.\nTable 3-16. Errors Generated by the nngxSwapBuffersbyAddress Function\n \t Error Code\n \t Cause\n \t GL_ERROR_8087_DMP\n \t An illegal value was specified for display.\n \t GL_ERROR_8088_DMP\n \t The address specified in addr is not 16-byte aligned.\n \t GL_ERROR_8089_DMP\n \t The address specified in addrB is not 16-byte aligned.\n \t GL_ERROR_808A_DMP\n \t An illegal value was specified for width.\n \t GL_ERROR_808B_DMP\n \t A value other than a specifiable format was specified for format.\n ",0]
,["Pages/Page_55588608.html#PageId_55588625","3.6.2. Details of Swapping Buffers ",
"\n3.6.2. Details of Swapping Buffers\nThe nngxSwapBuffers and nngxSwapBuffersByAddress functions do not show the content of the display buffer after they swap buffers. These functions simply schedule the display buffer address (used when the screen is rendered after a VSync) to be changed during a VBlank.\nThe GPU displays the content of the display buffer to the LCD. Because the GPU reads a single line of data from the display buffer for each scan line, memory is accessed frequently outside of a VBlank. This causes tearing to occur if the content of the display buffer is overwritten outside of a VBlank.\nThe GPU accesses memory at the highest priority when it transfers (displays) the content of the display buffer to the LCD. By placing the display buffer in VRAM that can only be accessed by the GPU, you can avoid problems caused by memory access conflicts. If, however, you place the display buffer in main memory (device memory), there may be memory access conflicts with the CPU and other devices.\nYou can use the nngxSetMemAccessPrioMode function to adjust the priority at which the GPU, CPU, and other devices access main memory.\nCode 3-21. Function for Adjusting the Priority of Main Memory Access\n void nngxSetMemAccessPrioMode(nngxMemAccessPrioMode mode);\nTable 3-17. Differences in Main Memory Access Priorities\n \t mode\n \t GPU\n \t CPU\n \t Other Devices\n \t NN_GX_MEM_ACCESS_PRIO_MODE_0\n \t Uniform priority.\n \t NN_GX_MEM_ACCESS_PRIO_MODE_1\n \t\n \t High priority.\n \t\n \t NN_GX_MEM_ACCESS_PRIO_MODE_2\n \t\n \t Extremely high priority.\n \t\n \t NN_GX_MEM_ACCESS_PRIO_MODE_3\n \t High priority.\n \t High priority.\n \t\n \t NN_GX_MEM_ACCESS_PRIO_MODE_4\n \t High priority.\n \t\n \t\n NN_GX_MEM_ACCESS_PRIO_MODE_1 is the default setting.\nBy raising the priority of memory accesses from the CPU, you can reduce the effect of the GPU and other devices on the time taken by processes that involve accessing main memory from the CPU.\n Warning:\n If you place the display buffer in main memory and specify NN_GX_MEM_ACCESS_PRIO_MODE_2, a large number of memory accesses from the CPU may cause noise resembling vertical lines to appear on the screen because of insufficient bandwidth for transferring images to be displayed on the LCD. To avoid this, either place the display buffer in VRAM or specify another mode.\n Note:\n If there is insufficient bandwidth for displaying images on the LCD, the nngxGetCmdlistParameteri function returns a bit array with a value of 1 stored in both bit 17 and bit 18 when NN_GX_CMDLIST_HW_STATE is specified as the param parameter. For more information about the nngxGetCmdlistParameteri function, see 4.1.10. Getting Command List Parameters.\n",0]
,["Pages/Page_55588608.html#PageId_55588626","3.7. Screen Update Synchronization ",
"\n3.7. Screen Update Synchronization\nYou can use the following functions for processing that needs to line up with vertical LCD synchronization (VSync).\nCode 3-22. VSync Functions\n GLint nngxCheckVSync(GLenum display);\nvoid nngxWaitVSync(GLenum display);\nvoid nngxSetVSyncCallback(GLenum display, void (*func)(GLenum));\nThe nngxCheckVSync function returns the value of the VSync counter updated within the library. By checking for changes in this value you can asynchronously check for VSync updates. Because the return value is an update counter used internally by the library, it wraps around to 0 after it exceeds the implementation's maximum value. (This may change in future implementations.)\nThe nngxWaitVSync function waits for a VSync update on the specified LCD. Control does not return until the VSync update.\nThe nngxSetVSyncCallback function registers the callback function that is invoked when VSync is updated. If this function is called with func set to 0 (NULL), it unregisters the callback function. The registered callback function is called from a different thread than the main thread, so mutual exclusion is required when referencing any data shared with the main thread. However, mutual exclusion is not required for data shared with any interrupt handlers registered using the nngxSetCmdlistCallback function even if they are for the same graphics processing.\n Warning:\n You can call the nngx functions from within a VSync callback, but note that command request completion interrupts are forced to wait until the callback function completes. Consequently, minimize any calls within callback functions to functions that issue command requests.\nSetting display to NN_GX_DISPLAY0, NN_GX_DISPLAY1, or NN_GX_DISPLAY_BOTH in any of these functions causes it to operate on the upper screen, lower screen, or both, respectively. Passing any other value causes a GL_ERROR_8019_DMP error in calls to nngxCheckVSync, a GL_ERROR_801A_DMP error in calls to nngxWaitVSync, and a GL_ERROR_801B_DMP error in calls to nngxSetVSyncCallback.\nThe system tries to prevent extreme time delay between screens, but there are approximately 100 microseconds between VSync updates for the upper and lower screens. Avoid creating code that strongly depends on this delay because heavy processing in the VSync callback for the upper screen or the start of an extremely high-priority thread may force the VSync callback for the lower screen to wait.\nThe LCD screen VSync interval is 59.831 Hz for both the upper and lower screens. This does not change when stereoscopic display (the parallax barrier) is enabled or disabled.\n",0]
,["Pages/Page_55588608.html#PageId_55588627","3.8. Finalizing ",
"\n3.8. Finalizing\nCall the nngxFinalize function when you stop using the GX library (such as, when the application shuts down). This releases all remaining unreleased objects.\nCode 3-23. Definition of the nngxFinalize Function\n void nngxFinalize(void);\nTo destroy the framebuffer objects, render buffers, and display buffers allocated for displaying graphics on the LCDs, call glDeleteFramebuffers, glDeleteRenderbuffers, and nngxDeleteDisplaybuffers, respectively.\nCode 3-24. Functions for Destroying Framebuffer Objects, Render Buffers, and Display Buffers\n void glDeleteFramebuffers(GLsizei n, const GLuint* framebuffers);\nvoid glDeleteRenderbuffers(GLsizei n, const GLuint* renderbuffers);\nvoid nngxDeleteDisplaybuffers(GLsizei n, GLuint* buffers);\nIn each function, n specifies the number of objects in the array given by the second argument. A GL_ERROR_801E_DMP error occurs when n is set to a negative value in the function.\nIf any of the specified framebuffer objects or render buffers is in use, it is unaffected. All other objects are deleted. Display buffers are swapped for display buffers with an object name of 0 and the objects in use are destroyed.\nCONFIDENTIAL\n",0]
,["Pages/Page_55588628.html","4. Command Lists ",
"4. Command Lists\nAll gl and nngx functions, whether called by 3D graphics rendering or otherwise, are ultimately processed by the GPU through register write operations and similar operations. Multiple commands that write to registers (3D commands) are handled and run together as a single command set.\n",0]
,["Pages/Page_55588628.html#PageId_55588629","4.1. How to Use ",
"3D commands are accumulated in a command list's 3D command buffer when, for example, glDrawElements is run. Command requests that (among other things) control command set execution, are queued in the command list separately from the 3D command buffer. For information about the types of command requests, see 4.2. Command Request Types.\n4.1. How to Use\n",0]
,["Pages/Page_55588628.html#PageId_55588630","4.1.1. Creating Objects ",
"The 3DS GPU renders 3D graphics by running commands in units of command lists. Applications create command list objects into which gl functions and other functions accumulate 3D commands, which are then executed as one batch when the GPU runs the command list.\n4.1.1. Creating Objects\nFirst, use the nngxGenCmdlists function to create the command list objects.\nCode 4-1. Function for Creating Command Lists\n void nngxGenCmdlists(GLsizei n, GLuint* cmdlists);\nThis code creates n command list objects and stores their object names in cmdlists.\nCommand lists have their own namespace. The command list with an object name of 0 is reserved by the system.\nTable 4-1. Errors Generated by the nngxGenCmdlists Function\n \t Error Code\n \t Cause\n \t GL_ERROR_8000_DMP\n \t A negative value was specified for n.\n \t GL_ERROR_8001_DMP\n \t The internal buffer failed to be allocated.\n ",0]
,["Pages/Page_55588628.html#PageId_55588631","4.1.2. Binding Command Lists ",
"\n4.1.2. Binding Command Lists\nNext, use the nngxBindCmdlist function to bind a generated command list object to the GPU. 3D commands are accumulated in the bound command list's 3D command buffer.\nCode 4-2. Function for Binding Command Lists\n void nngxBindCmdlist(GLuint cmdlist);\nIf cmdlist is set to an unused object name, that object is created.\nTable 4-2. Errors Generated by the nngxBindCmdlist Function\n \t Error Code\n \t Cause\n \t GL_ERROR_8004_DMP\n \t The internal buffer failed to be allocated.\n \t GL_ERROR_8005_DMP\n \t Called while command caches and command lists were in a state of being saved (for more information, see the CTR Programming Manual: Advanced Graphics).\n ",0]
,["Pages/Page_55588628.html#PageId_55588632","4.1.3. Allocating Memory Regions ",
"\n4.1.3. Allocating Memory Regions\nUse the nngxCmdlistStorage function to allocate a memory region for the bound command list.\nCode 4-3. Allocating a Memory Region for a Command List\n void nngxCmdlistStorage(GLsizei bufsize, GLsizei requestcount);\nSet bufsize to the size of the 3D command buffer and requestcount to the number of command requests that can be queued.\nYou must call the nngxBindCmdlist and nngxCmdlistStorage functions on each command list object that you create. Calls to these functions are ignored when a command list with an object name of 0 is bound. If you call this function again on an object that already has an allocated region, that region is freed and reallocated.\nA GL_ERROR_COMMANDBUFFER_FULL_DMP error is generated by the relevant function if 3D commands have accumulated past this allocated 3D command buffer size or if the 3D command buffer has not been set. A GL_ERROR_COMMANDREQUEST_FULL_DMP error is generated by the relevant function if command requests have been queued past this maximum queue size or if the buffer has not been set.\nTable 4-3. Common Errors When Allocating a Memory Region for a Command List\n \t Error Code\n \t Cause\n \t GL_ERROR_8006_DMP\n \t Failed to allocate a memory region.\n \t GL_ERROR_8007_DMP\n \t Called on a command list that is being executed.\n \t GL_ERROR_8008_DMP\n \t A negative value was specified as an argument.\n ",0]
,["Pages/Page_55588628.html#PageId_55588633","4.1.4. Running Command List Objects ",
"\n4.1.4. Running Command List Objects\nCall the nngxRunCmdlist function to start executing command requests that have been queued in the bound command list.\nCode 4-4. Function for Executing Command Lists\n void nngxRunCmdlist(void);\nvoid nngxRunCmdlistByID(GLuint cmdlist);\nExecution is ignored if the bound command list has an object name of 0. Likewise, attempts to bind a different command list and run this function are ignored while command requests are executing.\nAfter command requests have started executing, you can accumulate more commands in that same list or you can bind another command list and accumulate commands there. However, commands must be executed in the same order in which they were accumulated.\nThe nngxRunCmdlistByID function runs the command list specified by cmdlist rather than the currently bound command list. Besides running the specified command list, it works the same as the nngxRunCmdlist function.\nTable 4-4. Common Errors When Running Command Lists\n \t Error Code\n \t Cause\n \t GL_ERROR_8009_DMP\n \t Called on a command list for which a memory region has notbeen allocated (nngxRunCmdlist).\n \t GL_ERROR_80B1_DMP\n \t Called on a command list for which a memory region has notbeen allocated (nngxRuncmdlistByID).\n ",0]
,["Pages/Page_55588628.html#PageId_55588634","4.1.4.1. Getting the State of an Executing Command List ",
"4.1.4.1. Getting the State of an Executing Command List\nDetermine whether a command list is running using the nngxGetIsRunning function.\nCode 4-5. Function for Determining Command List Execution State\n GLboolean nngxGetIsRunning(void);\nThis function returns GL_TRUE if a command list is currently running, regardless of whether the command list is currently bound.\nSimilarly, you can determine whether a command list is currently running by passing the NN_GX_CMDLIST_IS_RUNNING value to the pname parameter of the nngxGetCmdlistParameteri function. This method can be used to determine whether the currently bound command list is running. For information about the nngxGetCmdlistParameteri function, see 4.1.10. Getting Command List Parameters.\n",0]
,["Pages/Page_55588628.html#PageId_55588635","4.1.5. Destroying Command List Objects ",
"\n4.1.5. Destroying Command List Objects\nYou can call the nngxDeleteCmdlists function to destroy command list objects that are no longer necessary.\nCode 4-6. Function for Destroying Command Lists\n void nngxDeleteCmdlists(GLsizei n, const GLuint* cmdlists);\nThis destroys the command list objects specified by the n object names in cmdlists. A GL_ERROR_8003_DMP error occurs if any of the specified command list objects are currently being executed, but all other specified command list objects are still destroyed.\nTable 4-5. Errors Generated by the nngxDeleteCmdlist Function\n \t Error Code\n \t Cause\n \t GL_ERROR_8002_DMP\n \t A negative value was specified for n.\n \t GL_ERROR_8003_DMP\n \t A command list included in cmdlists is still being executed.\n ",0]
,["Pages/Page_55588628.html#PageId_55588636","4.1.6. Stopping a Command List ",
"\n4.1.6. Stopping a Command List\nCall either of the following functions to stop a command list that is being executed.\nCode 4-7. Functions for Stopping Command Lists\n void nngxStopCmdlist(void);\nvoid nngxReserveStopCmdlist(GLint id);\nWhen the nngxStopCmdlist function is called, it waits for any executing command request to complete and then stops the command list. You cannot stop a command request after it has started executing (or is waiting to commence execution).\nThe nngxReserveStopCmdlist function stops the command list immediately after the idth accumulated command request finishes executing.\nCall the nngxRunCmdlist function to resume a stopped command list. Note, however, that this function will be ignored if it is called after the instruction to stop the command list but before the executing command requests finish executing.\nTable 4-6. Errors Generated by the nngxReserveStopCmdlist Function\n \t Error Code\n \t Cause\n \t GL_ERROR_800A_DMP\n \t Called on a command list that is being executed.\n \t GL_ERROR_800B_DMP\n \t Zero, a negative number, or a value that exceeds the maximum number of command requests was specified for id.\n ",0]
,["Pages/Page_55588628.html#PageId_55588637","4.1.7. Splitting the 3D Command Buffer ",
"\n4.1.7. Splitting the 3D Command Buffer\nUse the nngxSplitDrawCmdlist function to add a buffer loading complete command to the 3D command buffer and start queuing render command requests. If a command list accumulates 3D commands while it is executing, its 3D commands are run up to the point at which they are split by this function.\nCode 4-8. Function for Splitting the 3D Command Buffer\n void nngxSplitDrawCmdlist(void);\nRender command requests are not queued until the buffer loading complete command is added. In addition to this function, some other functions also queue render command requests. Because functions such as glClear and glTexImage2D must stop 3D command execution, they each add a buffer loading complete command, and then queue the render command requests.\nTable 4-7. Common Errors Generated by the nngxSplitDrawCmdlist Function\n \t Error Code\n \t Cause\n \t GL_ERROR_800C_DMP\n \t Called when the bound command list’s object name is 0.\n \t GL_ERROR_800D_DMP\n \t The command request has already reached the maximum number of accumulated command requests allowable.\n \t GL_ERROR_800E_DMP\n \t The 3D command buffer is full because of commands added by this function.\n ",0]
,["Pages/Page_55588628.html#PageId_55588638","4.1.7.1. Flushing the Accumulated 3D Command Buffer ",
"These errors may also be generated by other functions that call this one internally.\n4.1.7.1. Flushing the Accumulated 3D Command Buffer\nWhen the nngxSplitDrawCmdlist function is called, the buffer loading complete command is added and queuing of render command requests is carried out, even if there are no 3D commands accumulated in the 3D command buffer. In other words, this function can potentially add unneeded commands. We recommend calling the nngxFlush3DCommand function (which adds a command to split the 3D command buffer) and the nngxFlush3DCommandNoCacheFlush function only when 3D commands have accumulated. If a cache flush occurs multiple times, call a later function that does not perform cache flush and perform cache flush all at once by using the nngxUpdateBuffer function. This can reduce CPU overhead.\nCode 4-9. 3D Function for Flushing the Command Buffer\n void nngxFlush3DCommand(void);\nvoid nngxFlush3DCommandNoCacheFlush(void);\nThese functions will not add a buffer loading complete command and render command request if there are no 3D commands accumulated in the 3D command buffer of the bound command list after the buffer has been split for the last time. This function only adds a buffer loading complete command and render command request when 3D commands have been accumulated in the buffer. If 3D commands are accumulating as they are executing, the 3D commands execute up to where the buffer was split by this function.\nTable 4-8. Errors Generated by the nngxFlush3DCommand and nngxFlush3DCommandNoCacheFlush Functions\n \t Error Code\n \t Cause\n \t GL_ERROR_8084_DMP\nGL_ERROR_80AE_DMP\n \t Called when the bound command list’s object name is 0.\n \t GL_ERROR_8085_DMP\nGL_ERROR_80AF_DMP\n \t The command request has already reached the maximum number of accumulated command requests allowable.\n \t GL_ERROR_8086_DMP\nGL_ERROR_80E0_DMP\n \t The 3D command buffer is full because of commands added by this function.\n ",0]
,["Pages/Page_55588628.html#PageId_55588639","4.1.7.2. Partially Flushing the Accumulated 3D Command Buffer ",
"\n4.1.7.2. Partially Flushing the Accumulated 3D Command Buffer\nThe nngxFlush3DCommandPartially function is provided for executing a 3D command of a specified size. This function is an extension of the features provided with the nngxFlush3DCommand function and can be called to correctly execute 3D commands, including the command buffer execution register kick command added to functions such as nngxAdd3DCommand.\nCode 4-10. Function for Partially Flushing the Command Buffer\n void nngxFlush3DCommandPartially(GLsizei buffersize);\nSpecify the size, in bytes, of the command buffer to be executed in buffersize. The number must be a multiple of 16.\nThe size must be correctly specified in buffersize from the address following the previous command flush to the first kick command (and including the kick command). If the wrong value is specified, the command is executed in an unintended order and the operation may not be able to complete properly.\nUsing the application, accurately perform cache flush on 3D commands accumulated from the previous command flush up to the point this function is called. Overall flushing of the cache must be performed after calling the function because commands that generate an interrupt are generated within this function. In addition, as a means of avoiding cache execution before the cache is flushed, this function cannot be called for command lists whose execution is in progress. Functions such as glClear, nngxTransferRenderImage, and glCopyTexImage2D execute flushes according to the same method as the nngxFlush3DCommand function. Always perform a flush using this function before calling those functions.\nWhen a kick command is added using the nngxAddJumpCommand or nngxAddSubroutineCommand function, the driver adjusts the size so that the command kicks the execution size up to the first kick command. It is unnecessary to call the nngxFlush3DCommandPartially function when using those functions.\nNote that when a partial flush is performed for a command buffer with a kick command added with the nngxAddSubroutineCommand function, the execution size used is specified in buffersize instead of an execution size calculated by the driver.\nTable 4-9. Errors Generated by the nngxFlush3DCommandPartially Function\n \t Error Code\n \t Cause\n \t GL_ERROR_80A9_DMP\n \t Called when the bound command list’s object name is 0.\n \t GL_ERROR_80AA_DMP\n \t The command request has already reached the maximum number of accumulated command requests allowable.\n \t GL_ERROR_80AB_DMP\n \t The 3D command buffer is full because of commands added by this function.\n \t GL_ERROR_80AC_DMP\n \t A value less than 0 or which is not a multiple of 16 was specified in buffersize.\n \t GL_ERROR_80AD_DMP\n \t Called on a command list that is being executed.\n",0]
,["Pages/Page_55588628.html#PageId_55588640","4.1.8. Clearing the Command List ",
"\n4.1.8. Clearing the Command List\nThe following function clears the command list and sets both the 3D command buffer and command request queue to the unused state (the state immediately after their memory regions are allocated).\nCode 4-11. Function for Clearing the Command List\n void nngxClearCmdlist(void);\nTable 4-10. Errors Generated by the nngxClearCmdlist Function\n \t Error Code\n \t Cause\n \t GL_ERROR_800F_DMP\n \t Called on a command list that is being executed.\n ",0]
,["Pages/Page_55588628.html#PageId_55588641","4.1.8.1. Clearing the Command List and Filling Its 3D Command Buffer ",
"4.1.8.1. Clearing the Command List and Filling Its 3D Command Buffer\nThe following function clears the command list and initializes the 3D command buffer with the specified data. Both the 3D command buffer and command request queue enter the unused state.\nCode 4-12. Function for Clearing the Command List and Filling Its 3D Command Buffer\n void nngxClearFillCmdlist(GLuint data);\nTable 4-11 Errors Generated by the nngxClearFillCmdlist Function\n \t Error Code\n \t Cause\n \t GL_ERROR_8065_DMP\n \t Called on a command list that is being executed.\n",0]
,["Pages/Page_55588628.html#PageId_55588642","4.1.9. Setting Command List Parameters ",
"\n4.1.9. Setting Command List Parameters\nYou can call the nngxSetCmdlistParameteri function to set command list parameters\nCode 4-13. Function for Setting Command List Parameters\n void nngxSetCmdlistParameteri(GLenum pname, GLint param);\nTable 4-12. Command List Parameters That Can Be Set\n \t pname\n \t Setting\n \t NN_GX_CMDLIST_GAS_UPDATE\n \t This parameter is set for individual command list objects and can have one of the following values.\n\tGL_TRUE: Update the additive blend results for rendering gas density information. \tGL_FALSE: Normal behavior (default)\nIf this parameter is GL_TRUE, when the nngxSplitDrawCmdlist or nngxFlush3DCommand function is called, additive blend results are updated for rendered gas density information after the accumulated render command requests have finished running.\n If this parameter is GL_FALSE, normal behavior is restored; that is, commands that update gas density information are only accumulated when necessary.\n This setting takes effect depending on whether it is GL_TRUE when nngxSplitDrawCmdlist or nngxFlush3DCommand is called. A setting of GL_TRUE does not have any effect when render command requests are executed. This setting also does not affect render command requests accumulated by any function call other than nngxSplitDrawCmdlist or nngxFlush3DCommand.\n For more information about how additive blend results are updated for rendered gas density information, see the Gas Control Setting Registers section in the 3DS Programming Manual: Advanced Graphics.\n Table 4-13. Errors Generated by the nngxSetCmdlistParameteri Function\n \t Error Code\n \t Cause\n \t GL_ERROR_8015_DMP\n \t Called on a command list that is being executed.\n \t GL_ERROR_8016_DMP\n \t An invalid value was specified for pname or param.\n ",0]
,["Pages/Page_55588628.html#PageId_55588643","4.1.10. Getting Command List Parameters ",
"\n4.1.10. Getting Command List Parameters\nYou can call the nngxGetCmdlistParameteri function to get command list parameters.\nCode 4-14. Function for Getting Command List Parameters\n void nngxGetCmdlistParameteri(GLenum pname, GLint* param);\nTable 4-14. Getting Command List Parameters\n \t pname\n \t Parameter Obtained\n \t NN_GX_CMDLIST_IS_RUNNING\n \t The command list execution state.\n GL_TRUE: The command list is currently being executed.\nGL_FALSE: The command list is not currently being executed.\n \t NN_GX_CMDLIST_USED_BUFSIZE\n \t The number of bytes accumulated in the 3D command buffer.\n \t NN_GX_CMDLIST_USED_REQCOUNT\n \t The number of accumulated command requests.\n \t NN_GX_CMDLIST_MAX_BUFSIZE\n \t The maximum 3D command buffer size.\n This value is specified as bufsize to the nngxCmdlistStorage function.\n \t NN_GX_CMDLIST_MAX_REQCOUNT\n \t The maximum number of command requests.\n This value was specified in the requestcount parameter to the nngxCmdlistStorage function.\n \t NN_GX_CMDLIST_TOP_BUFADDR\n \t The starting address of the 3D command buffer.\n \t NN_GX_CMDLIST_BINDING\n \t The object name of the command list that is currently bound.\n \t NN_GX_CMDLIST_RUN_BUFSIZE\n \t The number of executed bytes in the 3D command buffer.\n \t NN_GX_CMDLIST_RUN_REQCOUNT\n \t The number of executed command requests.\n \t NN_GX_CMDLIST_TOP_REQADDR\n \t The starting address of the data region used for the command request queue.\n \t NN_GX_CMDLIST_NEXT_REQTYPE\n \t The type of command request that is currently executing or will be executed next.\n The value returned in param depends on the state of the command list that is currently bound. If the command list is currently stopped, the return value is the type of command request that will be executed next. If the command list is executing, the return value is the type of command request that is currently executing. NULL is returned when all command requests have finished executing.\n Command request types are defined by the following macros.\n NN_GX_CMDLIST_REQTYPE_DMA: DMA transfer command request.\nNN_GX_CMDLIST_REQTYPE_RUN3D: Render command request.\nNN_GX_CMDLIST_REQTYPE_FILLMEM: Memory fill command request.\nNN_GX_CMDLIST_REQTYPE_POSTTRANS: Post transfer command request.\nNN_GX_CMDLIST_REQTYPE_COPYTEX: Copy texture command request.\n \t NN_GX_CMDLIST_NEXT_REQINFO\n \t The command buffer's address and byte size.\n The command buffer's address is stored in the first element of param and its size (in bytes) is stored in the second element. You must pass param a pointer to an array of at least two GLint values.\n If the bound command list is currently stopped, parameter information is returned for the command request that will be executed next. If the bound command list is executing, parameter information is returned for the command request that is currently executing. Nothing is returned if all command requests have finished executing.\n The function only returns information when a render command request is the command request that is currently executing or that will be executed next. Nothing is returned for any other type of command.\n \t NN_GX_CMDLIST_HW_STATE\n \t 32 bits of data indicating the hardware state.\n Each bit below is set to 1 to indicate that the command list is in the specified state.\n bit 20: A post transfer command request is executing.\nbit 19: A memory fill is in progress.\nbit 18: An underrun error has occurred in the FIFO for the lower screen.\nbit 17: An underrun error has occurred in the FIFO for the upper screen.\nbit 16: The post vertex cache is busy.\nbit 15: Bits [1:0] of register 0x0252 are 1.\nbit 14: Vertex processor 3 is busy.\nbit 13: Vertex processor 2 is busy.\nbit 12: Vertex processor 1 is busy.\nbit 11: Vertex processor 0 (which can also be used as the geometry shader processor) is busy.\nbit 10: Bits [1:0] of register 0x0229 are nonzero.\nbit 9: Input to the module that loads command buffers and vertex arrays is busy.\nbit 8: Output from the module that loads command buffers and vertex arrays is busy.\nbit 7: The early depth test module is busy.\nbit 6: The per-fragment operation module is busy processing data from the module in the previous stage.\nbit 5: The per-fragment operation module is busy accessing the framebuffer.\nbit 4: The texture combiners are busy.\nbit 3: Fragment lighting is busy.\nbit 2: The texture units are busy.\nbit 1: The rasterization module is busy.\nbit 0: Triangle setup is busy.\n \t NN_GX_CMDLIST_CURRENT_BUFADDR\n \t Buffer address of the next 3D command stored in the currently bound command list.\n Table 4-15. Common Errors When Setting Command List Parameters\n \t Error Code\n \t Cause\n \t GL_ERROR_8017_DMP\n \t An invalid value was specified for pname or param.\n \t GL_ERROR_8018_DMP\n \t NN_GX_CMDLIST_BINDING is not specified for pname when the bound command list’s object name is 0.\n ",0]
,["Pages/Page_55588628.html#PageId_55588644","4.1.11. Command Completion Interrupts ",
"\n4.1.11. Command Completion Interrupts\nYou can cause interrupts to occur and call interrupt handlers, when the command requests in a command list finish. You can register an interrupt handler with the nngxSetCmdlistCallback function.\nCode 4-15. Function for Registering an Interrupt Handler\n void nngxSetCmdlistCallback(void (*func)(GLint));\nAn interrupt handler is valid only for the bound command list. If this function is called with func set to 0 (NULL), the handler is unregistered. A GL_ERROR_8010_DMP error is returned when this function is called on a command list that is being executed. The interrupt handler is called from a different thread than the main thread, so mutual exclusion is needed when referencing any data shared with the main thread. However, mutual exclusion is not needed for data shared with any callback functions for the same graphics processing registered using the nngxSetVSyncCallback function. The command list is still executing when a handler other than the interrupt handler is called. (The interrupt handler call is the last command request accumulated in the command list.) Consequently, the interrupt handler cannot, itself, call any functions that cannot be called while a command list is executing.\nUse the nngxEnableCmdlistCallback function to specify a command request that normally triggers an interrupt when it ends. The nngxDisableCmdlistCallback function can disable interrupts.\nCode 4-16. Interrupt Control Functions\n void nngxEnableCmdlistCallback(GLint id);\nvoid nngxDisableCmdlistCallback(GLint id);\nAn interrupt occurs upon completion of the id'th accumulated command request. You can call this function on a single command list several times with separate id values to cause multiple interrupts to occur. Note that id indicates a command request in the order that it was accumulated, not in the order that it was executed. You can call nngxGetCmdlistParameteri with pname set to NN_GX_CMDLIST_USED_REQCOUNT to get a value to specify for id.\nIf id is -1, an interrupt occurs when all command requests accumulated in the command list have finished. Even without registering an interrupt handler, you can determine when a command request has finished executing by calling nngxGetCmdlistParameteri passing pname as NN_GX_CMDLIST_IS_RUNNING, and then waiting until you get a value of GL_FALSE.\nThe nngxEnableCmdlistCallback function returns a GL_ERROR_8012_DMP error, and the nngxDisableCmdlistCallback function returns a GL_ERROR_8014_DMP error, when id is 0, a negative value other than -1, or the maximum number of command requests.\n",0]
,["Pages/Page_55588628.html#PageId_55588645","4.1.12. Waiting for Command Execution to Complete ",
"\n4.1.12. Waiting for Command Execution to Complete\nYou can call nngxWaitCmdlistDone to wait for all of the command requests accumulated in the command list to complete.\nCode 4-17. Function That Waits for Commands to Complete\n void nngxWaitCmdlistDone(void);\nRender command requests are executed until the point at which they are split. To execute all of the accumulated render command requests, call nngxSplitDrawCmdlist before this function.\nThis function does not return until command execution is complete. However, you can use the nngxSetTimeout function to set a timeout period.\nCode 4-18. Function for Setting a Timeout When Waiting for Command Execution to Complete\n void nngxSetTimeout(GLint64EXT time, void (*callback)(void));\nSet time to the number of ticks to wait before the nngxWaitCmdlistDone function times out. Timeouts do not occur when a value of 0 is specified.\nSet callback to the callback function to invoke when a timeout occurs. If this is NULL, a callback function is not invoked when the timeout occurs.\nNo timeouts occur by default because the initial values for time and callback are 0 and NULL respectively.\n",0]
,["Pages/Page_55588628.html#PageId_55588646","4.1.13. Adding a DMA Transfer Command Request ",
"\n4.1.13. Adding a DMA Transfer Command Request\nWhen the nngxAddVramDmaCommand or nngxAddVramDmaCommandNoCacheFlush function is called, a command request that runs a DMA transfer to VRAM is accumulated in the command list. The former flushes the source cache, but the latter does not. This function can only use DMA transfers from main memory to VRAM.\nCode 4-19. Function for Adding a DMA Transfer Command Request\n void nngxAddVramDmaCommand(\n const GLvoid* srcaddr, GLvoid* dstaddr, GLsizei size);\nvoid nngxAddVramDmaCommandNoCacheFlush(\n const GLvoid* srcaddr, GLvoid* dstaddr, GLsizei size);\nAn amount of data specified by size is transferred from the address specified by srcaddr to the address specified by dstaddr.\nWhen calling the nngxAddVramDmaCommand function, a GL_ERROR_8062_DMP error indicates that this function was called when no valid command list was bound, while a GL_ERROR_8064_DMP error indicates that size is negative.\nWhen calling the nngxAddVramDmaCommandNoCacheFlush function, a GL_ERROR_8090_DMP error indicates that this function was called when no valid command list was bound, and a GL_ERROR_8091_DMP error indicates that size is negative.\n",0]
,["Pages/Page_55588628.html#PageId_55588647","4.1.14. Adding an Anti-Aliasing Filter Transfer Command Request ",
"\n4.1.14. Adding an Anti-Aliasing Filter Transfer Command Request\nWhen the nngxFilterBlockImage function is called, a command request that transfers an image with an anti-aliasing filter applied is accumulated in the command list. (This is one kind of post-filter command request.) The image is transferred in block format, unconverted. The only supported antialiasing specification is 2x2.\nCode 4-20. Function for Adding an Anti-Aliasing Filter Transfer Command Request\n void nngxFilterBlockImage(const GLvoid* srcaddr, GLvoid* dstaddr,\n GLsizei width, GLsizei height, GLenum format);\nAn image with a width, height, and format specified by width, height, and format respectively is transferred from the address specified by srcaddr to the address specified by dstaddr.\nwidth and height are restricted as follows by the value specified for format.\nTable 4-16. Format Restrictions on the Width and Height of Images to Be Transferred\n \t format\n \t width\n \t height\n \t GL_RGBA8_OES\n GL_RGB8_OES\n \t A multiple of 64, greater than or equal to 64.\n \t A multiple of 16, greater than or equal to 64.\n \t GL_RGBA4\n GL_RGB5_A1\n GL_RGB565\n \t A multiple of 128, greater than or equal to 128.\n \t A multiple of 16, greater than or equal to 128.\n If the transfer source and destination memory regions overlap, the function works properly when the scraddr and dstaddr values are the same, or when the scraddr value is bigger than the dstaddr value. The transfer results could be corrupted if the scraddr value is smaller than the dstaddr value.\nWhen the value for srcaddr specifies an address in device memory, the transfer results could be incorrect if the destination memory cache has not been flushed.\nTable 4-17. Errors Generated by the nngxFilterBlockImage Function\n \t Error Code\n \t Cause\n \t GL_ERROR_8068_DMP\n \t Called when a command list with an object name of 0 is bound or when there is no space in the command request queue.\n \t GL_ERROR_8069_DMP\n \t The address specified for srcaddr or dstaddr is not 8-byte aligned.\n \t GL_ERROR_806A_DMP\n \t A width or height value is specified that violates the restrictions.\n \t GL_ERROR_806B_DMP\n \t A format value is specified that is not listed in the restrictions.\n ",0]
,["Pages/Page_55588628.html#PageId_55588648","4.1.15. Adding an Image Transfer Command Request ",
"\n4.1.15. Adding an Image Transfer Command Request\nWhen the nngxTransferLinearImage function is called, a command request that transfers an image to a render buffer or texture is accumulated in the command list. (This is one kind of copy-texture command request.) If the current 3D command buffer has accumulated unsplit commands, a split command is added, and then the transfer command request is added.\nAlthough images are converted from linear format to block format while they are transferred, this conversion only affects addressing. If this function is called on a render buffer, the block mode setting automatically determines whether a conversion to 8 block addressing or 32 block addressing is applied during the transfer. If this function is called on a texture, a conversion to 8 block addressing is applied. In either case, you must flip an image in the V direction and convert its byte order before you transfer it.\n Note:\n For information about block mode, see Block Mode Settings in the 3DS Programming Manual: Advanced Graphics.\nCode 4-21. Function for Adding an Image Transfer Command Request\n void nngxTransferLinearImage(const GLvoid* srcaddr, GLuint dstid,\n GLenum target);\nFor srcaddr specify the starting address of the image to transfer. The image must have the same format, width, and height as the render buffer or texture to which it is transferred. However, the source pixel format must be 32-bit when the target pixel format is 24-bit because the hardware does not support transfers between 24-bit pixel formats. In this case, for each 4 bytes that are transferred, the first byte (the internal format's alpha component) is truncated.\nThe image is transferred to the render buffer or texture that has the object ID specified by dstid and the object type specified by target.\nTable 4-18. Values to Specify for target and dstid\n \t When target is:\n \t Set dstid to:\n \t GL_RENDERBUFFER\n \t The object ID of a render buffer.\n If a value of 0 is specified, data is transferred to the color buffer that is attached to the current framebuffer.\n \t GL_TEXTURE_2D\n \t The object ID of a 2D texture.\n \t GL_TEXTURE_CUBE_MAP_POSITIVE_X{,Y,Z}\n GL_TEXTURE_CUBE_MAP_NEGATIVE_X{,Y,Z}\n \t The object ID of a cube map texture.\n The width and height of the target render buffer must be multiples of 8, in block 8 mode or multiples of 32, in block 32 mode. Both the width and height must be at least 128.\nTable 4-19. Errors Generated by the nngxTransferRenderImage Function\n \t Error Code\n \t Cause\n \t GL_ERROR_805B_DMP\n \t Called when the bound command list’s object name is 0.\n \t GL_ERROR_805C_DMP\n \t The maximum number of command requests has already accumulated.\n \t GL_ERROR_805D_DMP\n \t The 3D command buffer is full because of commands added by this function.\n \t GL_ERROR_805E_DMP\n \t The render buffer or texture specified for dstid does not exist, or it does not have an allocated memory region.\n \t GL_ERROR_805F_DMP\n \t There is a violation of the width and height restrictions for the target render buffer.\n \t GL_ERROR_8060_DMP\n \t An illegal value was specified for target.\n \t GL_ERROR_8067_DMP\n \t The target render buffer or texture does not use 32-bit, 24-bit, or 16-bit pixel sizes.\n ",0]
,["Pages/Page_55588628.html#PageId_55588649","4.1.16. Adding a Block-to-Linear Image Conversion and Transfer Command Request ",
"\n4.1.16. Adding a Block-to-Linear Image Conversion and Transfer Command Request\nA command request for converting a block image to a linear image and transferring the result can be added to the command list by calling the nngxAddB2LTransferCommand function. (This is one kind of post-filter command request.) Although the nngxTransferRenderImage function provides the same functionality, the nngxAddB2LTransferCommand function is more versatile. They also differ in that the latter function only adds a transfer request command and does not add a split command.\nCode 4-22. Function for Adding a Command Request for Converting From a Block Image to a Linear Image and Transferring\n void nngxAddB2LTransferCommand(\n const GLvoid* srcaddr, GLsizei srcwidth, GLsizei srcheight, GLenum srcformat,\n GLvoid* dstaddr, GLsizei dstwidth, GLsizei dstheight, GLenum dstformat,\n GLenum aamode, GLboolean yflip, GLsizei blocksize);\nThe srcaddr parameter specifies the transfer source (block image) address. The dstaddr parameter specifies the transfer destination (linear image) address. Both srcaddr and dstaddr must be 16-byte aligned.\nThe srcwidth, srcheight, dstwidth, and dstheight parameters specify the transfer source image width and height and transfer destination width and height, in pixels. The height and width of the source image and destination image must be a multiple of the block size (8 or 32). Finally, if the pixel size of the destination image is 24 bits and the block size is 8, the width of the source image and width of the destination image must be a multiple of 16. If 0 is specified for srcwidth, srcheight, dstwidth, or dstheight, the command is not issued. The height and width of the destination image in pixels must be equal to, or less than, that of the source image.\nThe height and width of the source and destination images, as measured in pixels, must be at least as big as the minimum allowed. The minimum height and width for source images is 128. The minimum height and width for destination images depends on the anti-alias setting. If anti-aliasing is disabled, the minimum for both height and width is 128. If 2x1 anti-aliasing is enabled, the height minimum is 128 and the width minimum is 64. If 2x2 anti-aliasing is enabled, the minimum for both height and width is 64.\nThe srcformat and dstformat parameters specify the pixel format of the source and destination image. The five types of pixel formats that can be specified are listed in the table below.\nTable 4-20. Pixel Format Specifications\n \t Definition\n \t Bits\n \t Description of Format\n \t GL_RGBA4\n \t 16\n \t The R, G, B, and alpha components are 4 bits each.\n \t GL_RGB5_A1\n \t 16\n \t The R, G, and B components are 5 bits each, and the alpha component is 1 bit.\n \t GL_RGB565\n \t 16\n \t 5-bit RB components and 6-bit G component. No alpha component.\n \t GL_RGB8_OES\n \t 24\n \t 8-bit RGB components. No alpha component.\n \t GL_RGBA8_OES\n \t 32\n \t The R, G, B, and alpha components are 8 bits each.\n Conversion to a pixel format with a higher pixel depth is not supported. For example, you cannot convert from a 24-bitformat to a 32-bit format, or from a 16-bit format to the 24-bit or 32-bit format.\naamode specifies the anti-alias filter mode. The three modes that can be specified are listed in the table below. The widths and heights indicate the minimum dimensions of the source image relative to the destination image.\nTable 4-21. Anti-Aliasing Specifications\n \t Definition\n \t Anti-Aliasing\n \t Width\n \t Height\n \t NN_GX_ANTIALIASE_NOT_USED\n \t No anti-aliasing.\n \t Equal\n \t Equal\n \t NN_GX_ANTIALIASE_2x1\n \t Transferred using 2x1 anti-aliasing.\n \t 2 times\n \t Equal\n \t NN_GX_ANTIALIASE_2x2\n \t Transferred using 2×2 anti-aliasing.\n \t 2 times\n \t 2 times\n yflip specifies whether vertical flipping is enabled during image transfer. Flipping is performed if GL_TRUE (or a value other than 0) is specified. Flipping is not performed if GL_FALSE (or 0) is specified.\nFor blocksize, specify the block size used for the transfer source image (8 or 32).\nTable 4-22. Errors Generated by the nngxTransferRenderImage Function\n \t Error Code\n \t Cause\n \t GL_ERROR_807C_DMP\n \t A command list with object name 0 was bound or there is no space in the command request queue.\n \t GL_ERROR_807D_DMP\n \t Either srcaddr or dstaddr is not 16-byte aligned.\n \t GL_ERROR_807E_DMP\n \t A value other than 8 or 32 is specified in blocksize.\n \t GL_ERROR_807F_DMP\n \t An illegal value is specified in aamode.\n \t GL_ERROR_8080_DMP\n \t An illegal value is specified in either srcformat or dstformat.\n \t GL_ERROR_8081_DMP\n \t The pixel size of srcformat is greater than that of dstformat.\n \t GL_ERROR_8082_DMP\n \t An illegal value is specified for any of srcwidth, srcheight, dstwidth, or dstheight.\n \tGL_ERROR_8083_DMP \t The specified width or height of the destination image is greater than the width or height in pixels of the source image.\n \t GL_ERROR_80B7_DMP\n \t The specified height or width of the source image was smaller than the minimum.\n \t GL_ERROR_80B8_DMP\n \tThe specified height or width of the destination image was smaller than the minimum.\n ",0]
,["Pages/Page_55588628.html#PageId_55588650","4.1.17. Adding a Linear-to-Block Image Conversion and Transfer Command Request ",
"\n4.1.17. Adding a Linear-to-Block Image Conversion and Transfer Command Request\nA command for converting from a linear image to a block image and then transferring the result can be added to the command list by calling the nngxAddL2BTransferCommand function. (This is one kind of post-filter command request.) The nngxTransferLinearImage function also provides the same functionality, but the nngxAddL2BTransferCommand function is more versatile. They also differ in that the latter function adds only a transfer request command and does not add a split command.\nCode 4-23. Function for Adding a Command Request for Converting From a Linear Image to a Block Image and Transferring\n void nngxAddL2BTransferCommand(\n const GLvoid* srcaddr, GLvoid* dstaddr,\n GLsizei width, GLsizei height, GLenum format, GLsizei blocksize);\nsrcaddr specifies the transfer source (linear image) address. dstaddr specifies the transfer destination (block image) address. Both srcaddr and dstaddr must be 16-byte aligned.\nwidth and height specify the height and width, in pixels, of the transfer source and transfer destination images. The transfer source and transfer destination images must have the same width and height, and each dimension must be 128 or greater and a multiple of the block size (8 or 32). Finally, if the bit depth of the source image is 24 bits, the image width must be a multiple of 32, even if the block size is 8. The command is not added if 0 is specified for either width or height.\nformat specifies the pixel format of the image being transferred. The source and destination images must have the same pixel format. Note, however, that if format is set to 24-bit, the source image must be in 32-bit format because hardware does not support 24-bit to 24-bit transfer. In this case, the last byte of every 4 bytes of source data is thrown away. The pixel formats that can be specified are the same as those in Table 4-20.\nThe blocksize parameter specifies the block size of the source image as either 8 or 32.\nTable 4-23. Errors Generated by the nngxTransferRenderImage Function\n \t Error Code\n \t Cause\n \t GL_ERROR_806F_DMP\n \t A command list with object name 0 was bound or there is no space in the command request queue.\n \t GL_ERROR_8070_DMP\n \t Either srcaddr or dstaddr is not 16-byte aligned.\n \t GL_ERROR_8071_DMP\n \t A value other than 8 or 32 is specified in blocksize.\n \t GL_ERROR_8072_DMP\n \t An illegal value is specified in either width or height.\n \t GL_ERROR_8073_DMP\n \t An illegal value is specified in format.\n ",0]
,["Pages/Page_55588628.html#PageId_55588651","4.1.18. Adding a Block Image Transfer Command Request ",
"\n4.1.18. Adding a Block Image Transfer Command Request\nA command request for transferring a block image is added to the command list by calling the nngxAddBlockImageCopyCommand function. The added command request allows you to copy graphics between textures and render buffers that contain rendered images. Because transfer is performed by specifying a combination of transfer size and skip size, you can clip part of the source image region or paste to part of the destination image region. The main purpose of this function is to transfer block format images. It can be used for transfer of various types of data because it does not perform format conversion.\nCode 4-24. Function for Adding a Block Image Transfer Command Request\n void nngxAddBlockImageCopyCommand(\n const GLvoid* srcaddr, GLsizei srcunit, GLsizei srcinterval,\n GLvoid* dstaddr, GLsizei dstunit, GLsizei dstinterval,\n GLsizei totalsize);\nUse the srcaddr parameter to specify the transfer source start address. dstaddr specifies the transfer destination start address. Both srcaddr and dstaddr must be 16-byte aligned.\ntotalsize specifies the total amount of data to be transferred, in bytes. totalsize must be 16-byte aligned.\nsrcunit and srcinterval specify the unit size used for reading each transfer and the skip size, respectively. Transfer of srcunit bytes of data and a skip of srcinterval bytes in the address being read ia alternately repeated. Transfer ends when the amount of data transferred reaches totalsize. If 0 is specified for srcinterval, memory is read continuously from the start address until totalsize is reached. If a value other than 0 is specified for srcinterval, srcunit bytes of data are read and then srcinterval bytes are skipped, repeatedly, allowing part of the source image to be clipped.\ndstunit and dstinterval, respectively, specify the write unit size of the transfer destination and the skip size, in bytes. Writing of dstunit bytes of data and a skip of dstinterval bytes in the address being written will be alternately repeated. Transfer ends when the amount of data transferred reaches totalsize. If 0 is specified for dstinterval, memory is written continuously from the start address until totalsize is reached. If a value other than 0 is specified for dstinterval, writing and skipping are repeated, allowing the image to be inserted into a portion of the memory region for the transfer destination image.\nFigure 4-1. Sample Block Image Transfer\nThe srcunit, srcinterval, dstunit, and dstinterval parameters must be multiples of 16. Negative values and values greater than or equal to 0x100000 cannot be specified.\nWhen transferring rendering results, such as block images, note that the start address of the transfer image (at both the source and destination) is normally the upper-left corner of the image (or the lower-left corner in OpenGL ES), and that data is arranged in block units of 8×8 pixels when using a format with a block size of 8. For more information about the block format, see 7.10. Native PICA Format.\nTable 4-24. Errors Generated by the nngxAddBlockImageCopyCommand Function\n \t Error Code\n \t Cause\n \t GL_ERROR_8074_DMP\n \t A command list with object name 0 was bound, or there is no space in the command request queue.\n \t GL_ERROR_8075_DMP\n \t Either srcaddr or dstaddr is not 16-byte aligned.\n \t GL_ERROR_8076_DMP\n \t totalsize is not a multiple of 16.\n \t GL_ERROR_8077_DMP\n \t An illegal value was specified in srcunit, srcinterval, dstunit, or dstinterval.\n ",0]
,["Pages/Page_55588628.html#PageId_55588652","4.1.19. Adding a Memory Fill Command Request ",
"\n4.1.19. Adding a Memory Fill Command Request\nA command request for filling the specified region of memory with the specified data can be added to the command list by calling the nngxAddMemoryFillCommand function. The command request added by this function can be used for things like clearing the color buffer or depth buffer (stencil buffer). The glClear function provides the same functionality, but this function is more versatile. Two memory regions of different sizes can be cleared simultaneously by making settings for two channels with independently specifiable parameters.\nCode 4-25. Function for Adding a Memory Fill Command Request\n void nngxAddMemoryFillCommand(\n GLvoid* startaddr0, GLsizei size0, GLuint data0, GLsizei width0,\n GLvoid* startaddr1, GLsizei size1, GLuint data1, GLsizei width1);\nstartaddr0, size0, data0, and width0 represent settings for Channel 0. startaddr1, size1, data1, and width1 represent settings for Channel 1. Memory is filled simultaneously for both channel 0 and channel 1. If the memory regions specified for Channel 0 and Channel 1 overlap, it is undefined as to which fill data is ultimately reflected in the overlapping part.\nstartaddr0 and startaddr1 specify the start addresses of the memory regions. Addresses must be 16-byte aligned. If 0 is specified for an address, that channel is not used. If 0 is specified for startaddr0, no error checking will be performed for size0, data0, or width0. If 0 is specified for startaddr1, no error checking will be performed for size1, data1, or width1.\nsize0 and size1 specify the sizes of the memory regions, in bytes. Sizes must be multiples of 16.\ndata0 and data1 specify the fill pattern data. The specified fill pattern is repeatedly inserted into the memory region until it is full.\nwidth0 and width1 specify the bit width of the fill pattern. The values 16, 24, or 32 can be specified for the bit width. If 16 is specified, the memory region is filled in 16-bit units using bits [15:0] of the data. If 24 is specified, the given memory region will be filled in 24-bit units using bits [23:0] of the data. If 32 is specified, the given memory region will be filled in 32-bit units using bits [31:0] of the data.\nThe table below provides fill pattern specifications (bit width and various parameter values) according to the render buffer format being used.\nTable 4-25. Fill Pattern by Render Buffer Format\n \t Render Buffer Format\n \t Bit Width\n \t R / D\n \t G / S\n \t B\n \t A\n \t GL_RGBA8_OES\n \t 32\n \t [31:24]\n0 through 255\n \t [23:16]\n0 through 255\n \t [15:8]\n0 through 255\n \t [7:0]\n0 through 255\n \t GL_RGB8_OES\n \t 24\n \t [23:16]\n0 through 255\n \t [15:8]\n0 through 255\n \t [7:0]\n0 through 255\n \t -\n \t GL_RGBA4\n \t 16\n \t [15:12]\n0 through 15\n \t [11:8]\n0 through 15\n \t [7:4]\n0 through 15\n \t [3:0]\n0 through 15\n \t GL_RGB5_A1\n \t 16\n \t [15:11]\n0 through 31\n \t [10:6]\n0 through 31\n \t [5:1]\n0 through 31\n \t [1:0]\n0 through 1\n \t GL_RGB565\n \t 16\n \t [15:11]\n0 through 31\n \t [10:5]\n0 through 63\n \t [4:0]\n0 through 31\n \t -\n \t GL_DEPTH24_STENCIL8_EXT\n \t 32\n \t [23:0]\n \t [31:24]\n \t -\n \t -\n \t GL_DEPTH_COMPONENT24_OES\n \t 24\n \t [23:0]\n \t -\n \t -\n \t -\n \t GL_DEPTH_COMPONENT16\n \t 16\n \t [15:0]\n \t -\n \t -\n \t -\n Table 4-26. Errors Generated by the nngxAddBlockImageCopyCommand Function\n \t Error Code\n \t Cause\n \t GL_ERROR_8078_DMP\n \t A command list with object name 0 was bound, or there is no space in the command request queue.\n \t GL_ERROR_8079_DMP\n \t startaddr0 or startaddr1 is not 16-byte aligned.\n \t GL_ERROR_807A_DMP\n \t size0 or size1 is not a multiple of 16.\n \t GL_ERROR_807B_DMP\n \t An illegal value is specified in width0 or width1.\n ",0]
,["Pages/Page_55588628.html#PageId_55588653","4.1.20. Moving the 3D Command Buffer Pointer ",
"\n4.1.20. Moving the 3D Command Buffer Pointer\nCall the nngxMoveCommandbufferPointer function to move the pointer in the 3D command buffer of the currently bound command list. (This 3D command buffer pointer is the position in the 3D commands from which to start running the 3D commands.)\nCode 4-26. Function for Moving the 3D Command Buffer Pointer\n void nngxMoveCommandbufferPointer(GLint offset);\nSpecify the amount by which to move the pointer (in bytes) as the offset parameter.\nA GL_ERROR_8061_DMP error occurs when no command list is bound, or this operation would move the pointer outside of the 3D command buffer region.\n",0]
,["Pages/Page_55588628.html#PageId_55588654","4.1.21. Adding Jump Commands ",
"\n4.1.21. Adding Jump Commands\nCall the nngxAddJumpCommand function to add to the currently bound command list a jump command that executes a 3D command in the specified memory region. Use a jump command to move execution to a different command list without causing any interrupts.\nThis function uses the command buffer execution PICA register. This only uses channel 0, so the content of two registers (0x0238 and 0x023A) are both written when this function is run. For more information, see 8.8.23. Command Buffer Execution Registers (0x0238 – 0x023D) and 8.8.23.1. Consecutive Execution of Command Buffers in 3DS Programming Manual: Advanced Graphics.\nCode 4-27. Function for Adding Jump Commands\n void nngxAddJumpCommand(const GLvoid* bufferaddr, GLsizei buffersize);\nIn bufferaddr and buffersize, specify the address and size of the command buffer to move execution to. Both bufferaddr and buffersize must be multiples of 16.\nThe content of the destination command buffer (the command list specified by bufferaddr and buffersize) is not copied to the command buffer of the currently bound command list. A jump command changes the execution address of a command buffer and directly executes the destination command buffer. Consequently, the application must ensure that the jump destination memory cache has been flushed.\nThe last command executed at the jump destination must be a split command (a command to write to the split command setting register, added by the nngxSplitDrawCmdlist function). Alternatively, this command could be another jump command. When using multiple jump commands, the last command in the last command buffer in the chain must be a split command.\nThis function adds a command request for a 3D execution command. A GL_ERROR_809A_DMP error occurs when this function is called immediately after the command buffer has been flushed (for example, by a call to the nngxFlush3DCommand function) because doing so is meaningless. To add a 3D command to the command buffer immediately after a flush, call the nngxAdd3DCommand function.\nTable 4-27. Errors Generated by the nngxAddJumpCommand Function\n \t Error Code\n \t Cause\n \t GL_ERROR_8096_DMP\n \t The bound command list’s object name is 0.\n \t GL_ERROR_8097_DMP\n \t buffersize is 0 or less.\n \t GL_ERROR_8098_DMP\n \t buffersize is not a multiple of 16.\n \t GL_ERROR_8099_DMP\n \t bufferaddr is not a multiple of 16.\n \t GL_ERROR_809A_DMP\n \t This function was called immediately after the command buffer was flushed.\n \t GL_ERROR_809B_DMP\n \t The command request added by this function makes the queue overflow.\n \t GL_ERROR_809C_DMP\n \t The command added by this function makes the command buffer overflow.\n ",0]
,["Pages/Page_55588628.html#PageId_55588655","4.1.22. Adding Subroutine Commands ",
"\n4.1.22. Adding Subroutine Commands\nCall the nngxAddSubroutineCommand function to add both a jump command to execute a 3D command in the specified memory region and a command to set the address for returning to the command buffer jumped from, to the currently bound command list. Use a subroutine command to execute another command list without causing any interrupts, as if it were a subroutine.\nThis function uses the command buffer execution PICA register. This uses all channels, so the content of four registers (0x0238 through 0x023B) are written when this function is run. For more information, see 8.8.23. Command Buffer Execution Registers (0x0238 – 0x023D) and 8.8.23.1. Consecutive Execution of Command Buffers in 3DS Programming Manual: Advanced Graphics.\nCode 4-28. Function for Adding Subroutine Commands\n void nngxAddSubroutineCommand(const GLvoid* bufferaddr, GLsizei buffersize);\nIn bufferaddr and buffersize, specify the address and size of the command buffer to move execution to. Both bufferaddr and buffersize must be multiples of 16.\nThe content of the destination command buffer (the command list specified by bufferaddr and buffersize) is not copied to the command buffer of the currently bound command list. A jump command changes the execution address of a command buffer and directly executes the destination command buffer. Consequently, the application must ensure that the jump destination memory cache has been flushed.\nThe jump command is executed on channel 0, and the command to return to the command buffer jumped from is executed on channel 1. Consequently, the last command executed at the jump destination must be a kick command for channel 1 (a command to write to the command buffer execution register 0x023D). Alternatively, this command could be a jump command to another command buffer, but the channel used by the jump must not be channel 0, and the last command in the last command buffer in the chain must be a kick command for channel 1. In addition, you must not write to the channel 1 address setting registers (0x0239 and 0x023B). This function adds a jump command (channel 0) and an address setting (channel 1). The application must place the channel 1 kick command and the jump commands within the subroutine.\nThis function does not add a command request for a 3D execution command. After calling this function, continue accumulating commands, and then execute them after flushing the command buffer, such as by using the nngxFlush3DCommand function. Values written to the channel 1 size setting register (0x023B) added by this function are undefined until the command buffer is flushed. Operation is similarly undefined if you reuse the copied content of this register until the command buffer is flushed.\nTable 4-28. Errors Generated by the nngxAddJumpCommand Function\n \t Error Code\n \t Cause\n \t GL_ERROR_809D_DMP\n \t The bound command list’s object name is 0.\n \t GL_ERROR_809E_DMP\n \t buffersize is 0 or less.\n \t GL_ERROR_809F_DMP\n \t buffersize is not a multiple of 16.\n \t GL_ERROR_80A0_DMP\n \t bufferaddr is not a multiple of 16.\n \t GL_ERROR_80A1_DMP\n \t The command added by this function makes the command buffer overflow.\n",0]
,["Pages/Page_55588628.html#PageId_55588656","4.2. Command Request Types ",
"\n4.2. Command Request Types\nThe following command requests are queued in a command list.\n",0]
,["Pages/Page_55588628.html#Anchor_55588656_DMA転送コマンド","DMA Transfer Command Requests ",
"DMA Transfer Command Requests\nThese command requests use DMA transfers to send texture images and vertex buffers from main memory into VRAM.\nThese command requests are queued by glTexImage2D and other functions that allocate texture regions, and by glBufferData and other functions that allocate vertex buffer regions.\n",0]
,["Pages/Page_55588628.html#Anchor_55588656_3D実行コマンド","Render Command Requests ",
"Render Command Requests\nThese command requests execute a single command set of 3D commands accumulated in the 3D command buffer.\nWhen glClear, glTexImage2D, and other functions are called, they write a buffer loading complete 3D command and then queue the accumulated 3D command buffer as a single render command request.\nThe nngxSplitDrawCmdlist function allows you to queue render command requests at any time.\n",0]
,["Pages/Page_55588628.html#Anchor_55588656_メモリフィルコマンド","Memory-Fill Command Requests ",
"Memory-Fill Command Requests\nThese command requests use the GPU memory-fill feature to clear a region allocated in VRAM using a specified data pattern.\nThese command requests specify a render buffer and are queued when the glClear function is called. The glClear function also requires a 3D command other than a memory-fill command request to be executed. In other words, when the glClear function is called, it first writes 3D commands for the glClear function and a buffer loading complete 3D command, and then it queues a render command request and a memory-fill command request.\n",0]
,["Pages/Page_55588628.html#Anchor_55588656_ポスト転送コマンド","Post-Transfer Command Requests ",
"Post-Transfer Command Requests\nThese command requests use the GPU post-filter feature to convert images rendered in PICA block format into a linear format that can be read by the LCDs.\nThese command requests are queued when the nngxTransferRenderImage function is called. If the nngxSplitDrawCmdlist function has not been called in advance to stop reading from the 3D command buffer, these command requests are queued after a buffer loading complete command is written and a render command request is queued.\n",0]
,["Pages/Page_55588628.html#Anchor_55588656_レンダーテクスチャ転送コマンド","Copy Texture Command Requests ",
"Copy Texture Command Requests\nThese command requests copy GPU rendering results into memory as texture images.\nThese command requests are queued when glCopyTexImage2D or glCopyTexSubImage2D is called.\nIf the nngxSplitDrawCmdlist function has not been called in advance to stop reading from the 3D command buffer, these command requests are queued after a buffer loading complete command is written and a render command request is queued.\n",0]
,["Pages/Page_55588628.html#PageId_55588662","4.3. Methods for Optimizing 3D Command Buffer Performance ",
"\n4.3. Methods for Optimizing 3D Command Buffer Performance\n",0]
,["Pages/Page_55588628.html#PageId_55588663","4.3.1. Changes in Load Speed due to Address and Size ",
"Methods for optimizing performance during 3D command buffer execution are described below.\n4.3.1. Changes in Load Speed due to Address and Size\nThe address and size of a 3D command buffer can have an effect on load speed at run time.\nThere are two types of command buffer execution: executing 3D execution commands queued in a command request, and executing the command buffer execution register.\nWhen executing 3D execution commands, execution is affected by the size from the address immediately after a split command added by nngxFlush3DCommand or nngxSplitDrawCmdlist, up to the next split command added. You can get the address of 3D commands being accumulated in the 3D command buffer by calling nngxGetCmdlistParameteri and passing NN_GX_CMDLIST_CURRENT_BUFADDR as the pname parameter.\nWhen executing using the command buffer execution register, execution is affected by the address and size of the following command buffers: added by nngxAddJumpCommand, added as subroutines by nngxAddSubroutineCommand, or executed to return from a subroutine to the calling location.\nIf the 3D command buffer address is 128-byte aligned, and the size is a multiple of 256 bytes (256, 512, 768, and so on), transfer speed may be faster.\nIf the 3D command buffer address is not 128-byte aligned and the size starting from the previous 128-byte aligned address to the end of the 3D command buffer is a multiple of 256, speed may be increased. For example, if the 3D command buffer address and size are 0x20000010 and 0x1F0 respectively, the preceding 128-byte aligned address is only 0x10 earlier, at 0x20000000. The distance from there to the end is 0x1F0 + 0x10, which is 0x200, and thus is a multiple of 256.\nThe address and size of 3D command buffers have these characteristics due to implementation details of the GPU, but they may not have significant effect in some cases due to factors such as: where the buffer is stored, details of the 3D commands, or memory access conflicts with other modules.\n",0]
,["Pages/Page_55588628.html#PageId_55588664","4.3.2. Using Subroutine Execution ",
"\n4.3.2. Using Subroutine Execution\n",0]
,["Pages/Page_55588628.html#PageId_55588665","4.3.2.1. Overview ",
"It may be possible to improve performance by using 3D command buffer subroutine execution.\n4.3.2.1. Overview\n3D command buffer subroutine execution uses the command buffer execution register for execution. In contrast to the ordinary method of storing 3D commands in a sequence of 3D command buffers and executing it, a command buffer stored in a different location is executed successively using a command buffer address jump feature. This method is called command buffer subroutine execution because of performing the following controls: first performing an address jump specifying the address of a 3D command buffer, executing the 3D command buffer at that location, and then returning to the calling location.\nFor more information about using command buffer subroutine execution, see 4.1.22. Adding Subroutine Commands and Command Buffer Execution Registers in the 3DS Programming Manual: Advanced Graphics.\n",0]
,["Pages/Page_55588628.html#PageId_55588666","4.3.2.2. Effect on Behavior ",
"\n4.3.2.2. Effect on Behavior\nCommand buffer subroutine execution has the following advantages.\n\tOnly a jump command to the subroutine command buffer needs to be stored, eliminating the CPU processing needed to copy the 3D commands. The technique is effective for tasks that are quite large and configured frequently, such as loading reference table data or shader programs. \tThe subroutine command buffer is not copied to the 3D command buffer, but is referenced directly by the GPU, allowing the total size of the command buffer to be reduced. \tIf the subroutine command buffer is stored in VRAM, GPU access to the command buffer is faster than if it is in main memory (device memory). If memory access to the command buffer is a performance bottleneck, this technique could improve overall system processing speed.\nOn the other hand, it has the following disadvantage.\n\tSwitching the address due to a jump command incurs memory access overhead. If the granularity of subroutines in the implementation is small and they are called frequently, a decrease in GPU processing speed could result.\nThe effect of converting to subroutines on processing performance is heavily influenced by issues such as memory access conflicts, so it is strongly dependent on the actual implementation of the application.\n",0]
,["Pages/Page_55588628.html#PageId_55588667","4.3.2.3. Storage Location ",
"\n4.3.2.3. Storage Location\nCommand buffer access speed is faster in VRAM than in main memory (device memory), so we recommend storing subroutine command buffers in VRAM.\nThere is some memory access overhead when executing a subroutine command buffer using a jump command, but if the executed command buffer is stored in VRAM, this overhead is decreased.\nTo store a command buffer in VRAM, it must first be generated in device memory and then transferred to VRAM by DMA using nngxAddVramDmaCommand. For information about DMA transfers to VRAM, see 4.1.13. Adding a DMA Transfer Command Request.\n",0]
,["Pages/Page_55588628.html#PageId_55588668","4.3.2.4. Balance Between Execution and Access Processes ",
"\n4.3.2.4. Balance Between Execution and Access Processes\nDepending on the content of subroutine command buffers, the processing bottleneck could move between accessing and executing 3D commands.\nIf the 3D command is the register write command of the rasterization module or a later module (including the rasterization module), each 3D command requires 2 cycles to process, so it is relatively processor-intensive. When 3D commands are composed of burst commands, execution is even more processor-intensive relative to access processing. In this case, the bottleneck is in command execution, and the processing cost of memory access due to conversion to subroutines is hidden.\nIf the 3D command is the register write command of a module before the rasterization module (not including the rasterization module), each 3D command requires only one cycle to process, so processing emphasis is light relative to commands discussed in the previous paragraph. In this case, the bottleneck is more likely to be access processing, and the memory access processing cost incurred by conversion to subroutines is more likely to affect the overall performance.\nFor information about the relative positioning of each module, see 2.2. Rendering Pipeline.\nCONFIDENTIAL\n",0]
,["Pages/Page_55588669.html","5. Shader Programs ",
"5. Shader Programs\nShader programs can customize the 3D graphics pipeline and control various graphics effects on the 3DS system.\nThere are three types of 3DS shader programs: one processes vertices, one creates geometry, and one processes fragments.\nThe shader program that processes vertices (the vertex shader) can be a unique shader programmed by developers.\nThe shader program that creates geometry (the geometry shader) must be one of the shader programs provided by the SDK.\n",0]
,["Pages/Page_55588669.html#PageId_55588670","5.1. Creating Shaders ",
"The shader program that processes fragments (the fragment shader) is not programmable. Fragment processing is implemented as a fixed pipeline, but it can be controlled through reserved uniforms. This document uses the terms reserved fragment processing and reserved fragment shader to refer to the fragment pipeline and shader program, respectively.\n5.1. Creating Shaders\nVertex shaders are the only shader programs that can be created by developers. The series of procedures related to vertex processing follow the OpenGL ES 2.0 specifications, but some features are not supported.\nShader programs are written in an assembly language that is unique to the PICA core. For an application to use a shader program, it must load and then attach a binary generated by a special-purpose assembler and linker. The glShaderSource and glCompileShader functions in the OpenGL ES 2.0 specifications are not implemented.\nFor more information about how to create a vertex shader, see 8. Vertex Shader and the Vertex Shader Reference.\n",0]
,["Pages/Page_55588669.html#PageId_55588671","5.2. Loading Shaders ",
"\n5.2. Loading Shaders\nAs explained in 5.1. Creating Shaders, you must load and then attach a shader program's binary data. Use the glCreateShader function first to create a shader object.\nCode 5-1. Definition of the glCreateShader Function\n GLuint glCreateShader(GLenum type);\nThe argument to pass as type depends on the shader program being loaded. The reserved fragment shader uses a fixed implementation and does not need to be loaded.\nTable 5-1. Shader Object Types\n \t Type\n \t Generated Object\n \t GL_VERTEX_SHADER\n \t A developer-created vertex shader object.\n \t GL_GEOMETRY_SHADER_DMP\n \t A geometry shader object provided by the SDK.\n Load the shader program binary into memory and then bind it to the GPU with the glShaderBinary function.\nCode 5-2. Definition of the glShaderBinary Function\n void glShaderBinary(GLint n, const GLuint* shaders, GLenum binaryformat,\n const void* binary, GLint length);\nFor shaders, specify an array of shader objects and for n specify the number of array elements. Because you can only load a binary that was created by the assembler and linker, set binaryformat to GL_PLATFORM_BINARY_DMP. Set binary to the address at which the shader program binary was loaded and length to the binary's size (in bytes).\nThe loaded shader programs are bound to the array in the order that they were passed to the linker. You can use the map file output by the linker to check the number of elements in the array and the type of shader objects to set. For more information, see the Vertex Shader Reference.\n",0]
,["Pages/Page_55588669.html#PageId_55588672","5.3. Attaching Shaders ",
"\n5.3. Attaching Shaders\nTo use a loaded shader program, an application must attach the shader program to a program object created by glCreateProgram, and then link that program object. Unlike in OpenGL ES 2.0, program objects have a 13-bit namespace that is independent of shader objects. Consequently, up to 8191 objects can be created simultaneously. If any of these objects are deleted by glDeleteProgram, they can be re-created.\nCode 5-3. Definition of the glCreateProgram Function\n GLuint glCreateProgram(void);\nUse the glAttachShader function to attach a shader program to a program object.\nCode 5-4. Definition of the glAttachShader Function\n void glAttachShader(GLuint program, GLuint shader);\nSet program to the return value from glCreateProgram and shader to the shader object.\nBoth the vertex shader and geometry shader are attached from loaded binaries. The reserved fragment shader, on the other hand, does not need to be loaded and is attached by setting shader to GL_DMP_FRAGMENT_SHADER_DMP.\nYou can attach one vertex shader, one geometry shader, and one reserved fragment shader to a single program object. In other words, if a point shader and line shader (both geometry shaders) are attached one after the other, only the line shader will come into effect.\nUse the glLinkProgram function to link a program object. You can link more than one program object. However, if you use glAttachShader to attach a different shader program to a linked program object, you must use glLinkProgram to relink the program object.\nCode 5-5. Definition of the glLinkProgram Function\n void glLinkProgram(GLuint program);\nThe program object fails to be linked when the vertex shader and geometry shader use a total of more than 2048 uniforms.\n",0]
,["Pages/Page_55588669.html#PageId_55588673","5.4. Using Shaders ",
"\n5.4. Using Shaders\nCall the glUseProgram function to apply a linked shader program to the 3D processing pipeline.\nCode 5-6. Definition of the glUseProgram Function\n void glUseProgram(GLuint program);\nThis function allows you to switch between several linked shader programs.\nOpenGL allowed you to call the glValidateProgram function to validate a shader program, but this function does nothing when it is called on a 3DS system.\nCode 5-7. Definition of the glValidateProgram Function\n void glValidateProgram(GLuint program);\n",0]
,["Pages/Page_55588669.html#PageId_55588674","5.5. Detaching Shaders ",
"\n5.5. Detaching Shaders\nYou can use the glDetachShader function to detach shader programs that are no longer necessary.\nCode 5-8. Definition of the glDetachShader Function\n void glDetachShader(GLuint program, GLuint shader);\n",0]
,["Pages/Page_55588669.html#PageId_55588675","5.6. Destroying Shaders ",
"\n5.6. Destroying Shaders\nYou can use the glDeleteShader function to destroy shader objects that are no longer necessary.\nCode 5-9. Definition of the glDeleteShader Function\n void glDeleteShader(GLuint shader);\n",0]
,["Pages/Page_55588669.html#PageId_55588676","5.7. Querying Shaders ",
"\n5.7. Querying Shaders\n",0]
,["Pages/Page_55588669.html#PageId_55588677","5.7.1. Validation ",
"You can query program objects and shader objects to determine whether they are valid or invalid, and to get parameters and other shader-related information.\n5.7.1. Validation\nYou can use the glIsProgram and glIsShader functions to determine whether program objects and shader objects are valid.\nCode 5-10. Definition of the glIsProgram and glIsShader Functions\n GLboolean glIsProgram(GLuint program);\nGLboolean glIsShader(GLuint shader);\nThese functions return a value of GL_TRUE if the program object or shader object passed as an argument is valid and GL_FALSE if it is not.\n",0]
,["Pages/Page_55588669.html#PageId_55588678","5.7.2. Getting Attached Shader Objects ",
"\n5.7.2. Getting Attached Shader Objects\nYou can use the glGetAttachedShaders function to get the shader objects attached to a program object.\nCode 5-11. Definition of the glGetAttachedShaders Function\n void glGetAttachedShaders(GLuint program, GLsizei maxcount, GLsizei* count,\n GLuint* shaders);\nA list of shader objects attached to the program object specified by program is stored in the array specified by shaders. For maxcount, specify the size of the array specified in shaders. A GL_INVALID_VALUE error is generated if program is invalid or maxcount is negative.\nThe count parameter holds the number of shader objects that were saved. This value is not saved if NULL is specified, but you can get the number of attached shader objects by calling the glGetProgramiv function (described later) with GL_ATTACHED_SHADERS passed as an argument.\n",0]
,["Pages/Page_55588669.html#PageId_55588679","5.7.3. Getting Command List Parameters ",
"\n5.7.3. Getting Command List Parameters\nYou can use the glGetProgramiv and glGetShaderiv functions to get parameters for program objects and shader objects.\nCode 5-12. Definition of the glGetProgramiv and glGetShaderiv Functions\n void glGetProgramiv(GLuint program, GLenum pname, GLint* params);\nvoid glGetShaderiv(GLuint shader, GLenum pname, GLint* params);\nThese functions store parameter values in params that correspond to the parameter name specified by pname. A GL_INVALID_ENUM error is generated if an invalid value is specified for pname. A GL_INVALID_VALUE error is generated if an invalid value is specified for program or shader.\nThe following table shows the parameter names that can be specified for pname and the parameters that are stored in params for the glGetProgramiv function.\nTable 5-2. Specifiable Parameter Names and Values Saved for the glGetProgramiv Function\n \t pname\n \t Values Saved in params\n \t GL_DELETE_STATUS\n \t GL_TRUE if the program object is in the \"waiting to be deleted\" state, and GL_FALSE otherwise. A program object in use by glUseProgram transitions to the \"waiting to be deleted\" state when glDeleteProgram is called on it.\n \t GL_LINK_STATUS\n \t GL_TRUE if glLinkProgram has successfully linked the program object, and GL_FALSE otherwise.\n \t GL_VALIDATE_STATUS\n \t The same value as when GL_LINK_STATUS is specified.\n \t GL_INFO_LOG_LENGTH\n \t Always 0.\n \t GL_ATTACHED_SHADERS\n \t The number of shader objects attached to the program object.\n \t GL_ACTIVE_ATTRIBUTES\n \t The number of vertex attributes in the active state.\n \t GL_ACTIVE_ATTRIBUTE_MAX_LENGTH\n \t The number of characters in the longest name of the active vertex attributes. The character count includes the terminating character (NULL).\n \t GL_ACTIVE_UNIFORMS\n \t The number of uniforms in the active state.\n \t GL_ACTIVE_UNIFORM_MAX_LENGTH\n \t The number of characters in the longest name of the active uniforms. The character count includes the terminating character (NULL).\n The following table shows the parameter names that can be specified for pname, and the parameters that are stored in params for the glGetShaderiv function.\nTable 5-3. Specifiable Parameter Names and Values Saved for the glGetShaderiv Function\n \t pname\n \t Values Saved in params\n \t GL_SHADER_TYPE\n \t The shader type.\n \t GL_DELETE_STATUS\n \t GL_TRUE if the shader object is in the \"waiting to be deleted\" state, and GL_FALSE otherwise. The shader object attached to a program object transitions to the \"waiting to be deleted\" state when glDeleteShader is called on it.\n \t GL_COMPILE_STATUS\n \t Always GL_FALSE.\n \t GL_INFO_LOG_LENGTH\n \t Always 0.\n \t GL_SHADER_SOURCE_LENGTH\n \t Always 0.\nCONFIDENTIAL\n",0]
,["Pages/Page_55588680.html","6. Vertex Buffers ",
"6. Vertex Buffers\n",0]
,["Pages/Page_55588680.html#PageId_55588681","6.1. Creating Objects ",
"Vertex buffers store vertex coordinates, colors, and indices, in addition to texture coordinates and other information. Use vertex buffers to process certain kinds of models, such as those with a large number of vertices, in the vertex shader. If you do not use vertex buffers, heavy CPU processing (such as vertex array sorting) could cause a considerable drop in performance.\n6.1. Creating Objects\nUse the glGenBuffers function to create the buffer objects to bind as vertex buffers.\nCode 6-1. Definition of the glGenBuffers Function\n void glGenBuffers(GLsizei n, GLuint* buffers);\n",0]
,["Pages/Page_55588680.html#PageId_55588682","6.2. Specifying Objects ",
"\n6.2. Specifying Objects\nUse the glBindBuffer function to specify the buffer objects to bind as vertex buffers. After this function is called, processing for each type of vertex buffer is run on the specified buffer objects.\nCode 6-2. Definition of the glBindBuffer Function\n void glBindBuffer(GLenum target, GLuint buffer);\nSet target to the vertex buffer type.\nTable 6-1. Vertex Buffer Types\n \t target Value\n \t Vertex Buffer Types\n \t GL_ARRAY_BUFFER\n \t Buffer for vertex coordinates, vertex colors, normals, and so on.\n \t GL_ELEMENT_ARRAY_BUFFER\n \t Buffer used by the glDrawElements function.\n \t GL_VERTEX_STATE_COLLECTION_DMP\n \t Vertex state collection (see 6.6. Vertex State Collections).\n ",0]
,["Pages/Page_55588680.html#PageId_55588683","6.3. Allocating Buffers ",
"\n6.3. Allocating Buffers\nUse the glBufferData function to allocate a buffer.\nCode 6-3. Definition of the glBufferData Function\n void glBufferData(GLenum target, GLsizeiptr size, const void* data,\n GLenum usage);\nFor target specify the value that was passed to glBindBuffer. For data and size, specify the actual data to store and its size in bytes, respectively. The value of usage must be GL_STATIC_DRAW.\nWhen data is 0 (NULL) the buffer is simply allocated and no data is stored.\n",0]
,["Pages/Page_55588680.html#PageId_55588684","6.4. Rewriting Buffers ",
"\n6.4. Rewriting Buffers\nUse the glBufferSubData function to rewrite part of the buffer that was allocated by glBufferData.\nCode 6-4. Definition of the glBufferSubData Function\n void glBufferSubData(GLenum target, GLintptr offset, GLsizeiptr size,\n const void* data);\nFor target, data, and size, specify the values that were passed to glBufferData. For offset, specify the starting index of the section to rewrite.\n",0]
,["Pages/Page_55588680.html#PageId_55588685","6.5. Destroying Buffers ",
"\n6.5. Destroying Buffers\nUse the glDeleteBuffers function to destroy buffers that are no longer necessary.\nCode 6-5. Definition of the glDeleteBuffers Function\n void glDeleteBuffers(GLsizei n, const GLuint* buffers);\n",0]
,["Pages/Page_55588680.html#PageId_55588686","6.6. Vertex State Collections ",
"\n6.6. Vertex State Collections\nBy using vertex state collections, you can bind buffer objects to vertex buffers and set vertex attributes for a group of vertices.\n",0]
,["Pages/Page_55588680.html#PageId_55588687","6.6.1. Creating Vertex State Collections ",
"Vertex state collections share a namespace with buffer objects and can be created, specified, and destroyed using glGenBuffers, glBindBuffer, and glDeleteBuffers, respectively.\n6.6.1. Creating Vertex State Collections\nA vertex state collection operates as a special buffer object. Use the glGenBuffers function to generate objects that can be used as vertex state collections, just as you would for buffer objects.\n",0]
,["Pages/Page_55588680.html#PageId_55588688","6.6.2. Specifying Vertex State Collections ",
"\n6.6.2. Specifying Vertex State Collections\nUse the glBindBuffer function to specify a buffer object to use as a vertex state collection. For target, specify GL_VERTEX_STATE_COLLECTION_DMP. By default, the object with a name of 0 (with a value of 0 passed into buffer) is a vertex state collection.\nAfter this function is called, the vertex state collection records which buffer objects are bound by glBindBuffer as vertex buffers (using GL_ARRAY_BUFFER or GL_ELEMENT_ARRAY_BUFFER), and also records which vertex attribute values are set by glEnableVertexAttribArray, glDisableVertexAttribArray, glVertexAttrib{1234}{fv}, or glVertexAttribPointer. When a buffer object is bound as a vertex buffer, it overwrites any other existing bindings to the same vertex buffer target. Settings continue to be recorded in a vertex state collection until it is switched.\nWhen the vertex state collection is switched, all buffer objects recorded in the new vertex state collection are bound as the new vertex buffers.\n",0]
,["Pages/Page_55588680.html#PageId_55588689","6.6.3. Destroying Vertex State Collections ",
"\n6.6.3. Destroying Vertex State Collections\nAs with a buffer object, you can use the glDeleteBuffers function to destroy vertex state collections. Even if a vertex state collection is destroyed, it does not affect the binding between the vertex buffer target and the buffer objects recorded in that vertex state collection.\nThe glDeleteBuffers function does not immediately destroy a vertex state collection that is in use. A vertex state collection remains in use until it is switched with another one. You cannot destroy the default vertex state collection. Calls to glDeleteBuffers on the default vertex state collection are ignored.\n",0]
,["Pages/Page_55588680.html#PageId_55588690","6.7. Sample Vertex Buffer Usage ",
"\n6.7. Sample Vertex Buffer Usage\nThe following code samples show how to render a single triangle using a vertex buffer and the glDrawElements function. There are three steps: defining the arrays, allocating the buffers, and rendering the triangle.\nCode 6-6. Defining the Arrays\n GLuint triIndexID, triArrayID;\nGLushort triIndex[1 * 3] = { 0, 1, 2 };\nGLfloat triVertex[3 * 4] =\n{\n 0.5f, 0.0f, 0.0f, 1.0f,\n -0.5f, 0.5f, 0.0f, 1.0f,\n -0.5f, -0.5f, 0.0f, 1.0f\n};\nGLfloat triVertexColor[3 * 3] =\n{\n 1.0f, 0.0f, 0.0f,\n 0.0f, 1.0f, 0.0f,\n 0.0f, 0.0f, 1.0f\n};\nCode 6-7. Allocating the Buffers\n glGenBuffers(1, &triArrayID);\nglBindBuffer(GL_ARRAY_BUFFER, triArrayID);\nglBufferData(GL_ARRAY_BUFFER, sizeof(triVertex) + sizeof(triVertexColor), 0,\n GL_STATIC_DRAW);\nglBufferSubData(GL_ARRAY_BUFFER, 0, sizeof(triVertex), triVertex);\nglBufferSubData(GL_ARRAY_BUFFER, sizeof(triVertex), sizeof(triVertexColor),\n triVertexColor);\nglGenBuffers(1, &triIndexID);\nglBindBuffer(GL_ELEMENT_ARRAY_BUFFER, triIndexID);\nglBufferData(GL_ELEMENT_ARRAY_BUFFER, 3 * sizeof(GLushort), triIndex,\nGL_STATIC_DRAW);\nCode 6-8. Rendering With glDrawElements\n glBindBuffer(GL_ARRAY_BUFFER, triArrayID);\nglVertexAttribPointer(0, 4, GL_FLOAT, GL_FALSE, 0, 0);\nglVertexAttribPointer(1, 3, GL_FLOAT, GL_FALSE, 0, (GLvoid*)sizeof(triVertex));\nglBindBuffer(GL_ELEMENT_ARRAY_BUFFER, triIndexID);\nglDrawElements(GL_TRIANGLES, 3, GL_UNSIGNED_SHORT, 0);\n",0]
,["Pages/Page_55588680.html#PageId_55588691","6.8. Restrictions on Vertex Data Placement ",
"\n6.8. Restrictions on Vertex Data Placement\nThe following hardware restrictions apply to the vertex data placement configured by the glVertexAttribPointer function when rendering using vertex data stored in a vertex buffer. A GL_INVALID_OPERATION error is generated when the glDrawArrays or glDrawElements function is called in violation of these restrictions.\nAll vertex data must be aligned to its own data type's size.\n\tThe stride for all vertex data in a single structure must be a multiple of the size of that structure's largest vertex data type. \tIf any padding (beyond the minimum amount required to meet the two aforementioned restrictions) is inserted after the vertex data, it must start at the end of the vertex data and continue until the second-closest 4-byte boundary.\nThe compiler automatically inserts padding, so there will be no conflicts with the first two restrictions even if you do not pay attention to them when coding.\nThe following sample code would conflict with the last restriction, if it did not insert extraPadding2.\nCode 6-9. Sample Code for Working Around Restrictions on Vertex Data Placement\n struct tagVertex\n{\n GLshort position[3];\n GLshort extraPadding1;\n GLshort extraPadding2[2];\n GLshort color[4];\n};\n",0]
,["Pages/Page_55588680.html#PageId_55588692","6.8.1. Restrictions Affecting Only glDrawElements ",
"You cannot mix vertex attributes and indices that use vertex buffers with attributes and indices that do not use them in a single call to the glDrawArrays or glDrawElements function. A GL_INVALID_OPERATION error is generated if they are mixed together.\n6.8.1. Restrictions Affecting Only glDrawElements\nA GL_INVALID_OPERATION error is generated when all of the following conditions are met, due to restrictions associated with the method of vertex array storage.\n\tThe vertex buffer is being used. \tThere are 12 vertex attributes being used. \tAll vertex attributes are being used as vertex arrays.\n(qlEnableVertexAttribArray has been called for all vertex attributes.)\n \tRendering is performed using qlDrawElements.\nWhen all these conditions are present, at least two vertex attributes must be placed as interleaved arrays. In other words, 12 independent arrays cannot be simultaneously used as vertex attributes.\n Note:\n An interleaved array is a group made up of multiple vertex attributes defined as a structure. An individual array made up of one vertex attribute is called an independent array.\nCONFIDENTIAL\n",0]
,["Pages/Page_55588693.html","7. Textures ",
"7. Textures\nIn addition to cube map textures and 2D textures that are pasted onto polygon models, the 3DS system can handle shadow textures used for shadows, gas textures used for rendering gases, and lookup tables used by the reserved fragment shader (these lookup tables cannot be used as textures).\n",0]
,["Pages/Page_55588693.html#PageId_55588694","7.1. Creating Texture Objects ",
"This chapter explains the necessary procedures for using these textures and describes how the native PICA format differs from the OpenGL specifications.\n7.1. Creating Texture Objects\nUse the glGenTextures function to create texture objects to bind as textures.\nCode 7-1. Definition of the glGenTextures Function\n void glGenTextures(GLsizei n, GLuint* textures);\n",0]
,["Pages/Page_55588693.html#PageId_55588695","7.2. Specifying Texture Objects ",
"\n7.2. Specifying Texture Objects\nUse the glBindTexture function to specify a texture object to bind as a texture. After this function is called, the various texture processes are performed using the specified texture object. Texture images loaded for (and the results of processing performed on) a texture object are preserved until the texture object is deleted. Consequently, you can switch texture objects to change textures without reloading texture images.\nCode 7-2. Definition of the glBindTexture Function\n void glBindTexture(GLenum target, GLuint texture);\nSpecify the texture type for target, and the texture object to bind for texture. A GL_INVALID_ENUM error is generated if you set target to a value that is not in the following table.\nTable 7-1. Texture Types\n \t target Value\n \t Type of Texture\n \t GL_TEXTURE_2D\n \t 2D textures, shadow textures, and gas textures.\n \t GL_TEXTURE_CUBE_MAP_POSITIVE_X\n \t Cube map textures.\n \t GL_TEXTURE_CUBE_MAP_NEGATIVE_X\n \t Cube map textures.\n \t GL_TEXTURE_CUBE_MAP_POSITIVE_Y\n \t Cube map textures.\n \t GL_TEXTURE_CUBE_MAP_NEGATIVE_Y\n \t Cube map textures.\n \t GL_TEXTURE_CUBE_MAP_POSITIVE_Z\n \t Cube map textures.\n \t GL_TEXTURE_CUBE_MAP_NEGATIVE_Z\n \t Cube map textures.\n \t GL_LUT_TEXTUREi_DMP (where i is a value between 0 and 31)\n \t Lookup tables (used by the reserved fragment shader).\n \t GL_TEXTURE_COLLECTION_DMP\n \t Texture collections (see 7.9. Texture Collections).\n ",0]
,["Pages/Page_55588693.html#PageId_55588696","7.3. Loading Texture Images ",
"\n7.3. Loading Texture Images\nIn addition to normal texture images (2D textures), the glTexImage2D function loads the following special textures: cube map textures, shadow textures, and gas textures. Partial texture loading with the glTexSubImage2D function is not supported.\nCode 7-3. Definition of the glTexImage2D Function\n void glTexImage2D(GLenum target, GLint level, GLenum internalformat,\n GLsizei width, GLsizei height, GLint border, GLenum format,\n GLenum type, const void* pixels);\nSet target to one of the following values.\nTable 7-2 target Values in glTexImage2D\n \t target Value\n \t Texture Usage\n \t GL_TEXTURE_2D\n \t 2D textures, shadow textures, and gas textures.\n \t GL_TEXTURE_CUBE_MAP_POSITIVE_X\n \t Cube map +X plane textures.\n \t GL_TEXTURE_CUBE_MAP_NEGATIVE_X\n \t Cube map −X plane textures.\n \t GL_TEXTURE_CUBE_MAP_POSITIVE_Y\n \t Cube map +Y plane textures.\n \t GL_TEXTURE_CUBE_MAP_NEGATIVE_Y\n \t Cube map −Y plane textures.\n \t GL_TEXTURE_CUBE_MAP_POSITIVE_Z\n \t Cube map +Z plane textures.\n \t GL_TEXTURE_CUBE_MAP_NEGATIVE_Z\n \t Cube map −Z plane textures.\n Combinations of format and type specify the format of texture images stored in the region given by pixels. The following table lists the formats that can be handled by the 3DS system. The Bytes column indicates the number of bytes per texel. An asterisk (*) that follows the byte count indicates a native PICA format, which has a different byte order than the standard OpenGL format. For more information about the native PICA format, see 7.10. Native PICA Format.\nTable 7-3. Texture Image Formats Specified by format and type\n \t format\n \t type\n \t Format\n \t Bytes\n \t GL_RGBA\n \t GL_UNSIGNED_SHORT_4_4_4_4\n \t RGBA4\n \t 2\n \t GL_RGBA\n \t GL_UNSIGNED_SHORT_5_5_5_1\n \t RGBA5551\n \t 2\n \t GL_RGBA\n \t GL_UNSIGNED_BYTE\n \t RGBA8\n \t 4\n \t GL_RGB\n \t GL_UNSIGNED_SHORT_5_6_5\n \t RGB565\n \t 2\n \t GL_RGB\n \t GL_UNSIGNED_BYTE\n \t RGB8\n \t 3\n \t GL_ALPHA\n \t GL_UNSIGNED_BYTE\n \t A8\n \t 1\n \t GL_ALPHA\n \t GL_UNSIGNED_4BITS_DMP\n \t A4\n \t 0.5\n \t GL_LUMINANCE\n \t GL_UNSIGNED_BYTE\n \t L8\n \t 1\n \t GL_LUMINANCE\n \t GL_UNSIGNED_4BITS_DMP\n \t L4\n \t 0.5\n \t GL_LUMINANCE_ALPHA\n \t GL_UNSIGNED_BYTE\n \t LA8\n \t 2\n \t GL_LUMINANCE_ALPHA\n \t GL_UNSIGNED_BYTE_4_4_DMP\n \t LA4\n \t 1\n \t GL_SHADOW_DMP\n \t GL_UNSIGNED_INT\n \t -\n \t 4\n \t GL_GAS_DMP\n \t GL_UNSIGNED_SHORT\n \t -\n \t 4\n \t GL_HILO8_DMP\n \t GL_UNSIGNED_BYTE\n \t -\n \t 2\n \t GL_RGBA_NATIVE_DMP\n \t GL_UNSIGNED_SHORT_4_4_4_4\n \t RGBA4\n \t 2\n \t GL_RGBA_NATIVE_DMP\n \t GL_UNSIGNED_SHORT_5_5_5_1\n \t RGBA5551\n \t 2\n \t GL_RGBA_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t RGBA8\n \t 4*\n \t GL_RGB_NATIVE_DMP\n \t GL_UNSIGNED_SHORT_5_6_5\n \t RGB565\n \t 2\n \t GL_RGB_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t RGB8\n \t 3*\n \t GL_ALPHA_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t A8\n \t 1\n \t GL_ALPHA_NATIVE_DMP\n \t GL_UNSIGNED_4BITS_DMP\n \t A4\n \t 0.5\n \t GL_LUMINANCE_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t L8\n \t 1\n \t GL_LUMINANCE_NATIVE_DMP\n \t GL_UNSIGNED_4BITS_DMP\n \t L4\n \t 0.5\n \t GL_LUMINANCE_ALPHA_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t LA8\n \t 2*\n \t GL_LUMINANCE_ALPHA_NATIVE_DMP\n \t GL_UNSIGNED_BYTE_4_4_DMP\n \t LA4\n \t 1\n \t GL_SHADOW_NATIVE_DMP\n \t GL_UNSIGNED_INT\n \t -\n \t 4*\n \t GL_GAS_NATIVE_DMP\n \t GL_UNSIGNED_SHORT\n \t -\n \t 4*\n \t GL_HILO8_DMP_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t -\n \t 2*\n R, G, and B represent colors (red, green, and blue, respectively).\nA represents the alpha value.\nL represents the luminance value.\nAs set forth in the OpenGL ES 1.1 specifications, a value of 1 is output for the alpha component if the texture combiner references a texture without an alpha component. This is also true for compressed textures.\nYou can only set format to GL_RGB or GL_RGB_NATIVE_DMP and type to GL_UNSIGNED_BYTE when target is GL_TEXTURE_2D. When format is GL_*_NATIVE_DMP, pixels must specify data in the native PICA format. When GL_GAS_DMP is specified, pixels must be 0 (NULL).\nFor width and height, specify the width and height of the texture image. Both numbers must be powers of 2, from 8 through 1024.\nIf target is GL_TEXTURE_CUBE_MAP_*, width and height must have the same value. Every surface must have the same settings, except for pixels (and target).\nIf pixels is set to 0 (NULL), a region is allocated but image data is not loaded.\nFor internalformat specify the basic internal format. A GL_INVALID_OPERATION error is generated when internalformat and format are not the same value. The following table shows how an image's RGBA components correspond to the components in the basic internal format.\nTable 7-4. Correspondence Between RGBA Formats and Basic Internal Formats\n \t Basic Internal Format\n \t RGBA\n \t Internal Format\n \t GL_ALPHA\n \t A\n \t A\n \t GL_LUMINANCE\n \t R\n \t L\n \t GL_LUMINANCE_ALPHA\n \t R, A\n \t L, A\n \t GL_RGB\n \t R, G, B\n \t R, G, B\n \t GL_RGBA\n \t R, G, B, A\n \t R, G, B, A\n \t GL_HILO8_DMP\n \t R, G\n \t Nx, Ny\n The GL_HILO8_DMP format outputs 0.0 for the B component and 1.0 for the A component.\nUnlike in the OpenGL specifications, level specifies the number of mipmap levels as a negative value. For example, -2 indicates two mipmap levels, and both -1 and 0 indicate one mipmap level. You cannot separately specify which textures to use for each mipmap level. Instead, set pixels to a series of textures, starting with the texture to use as the largest mipmap and ending with the texture to use as the smallest mipmap.\n",0]
,["Pages/Page_55588693.html#PageId_55588697","7.3.1. Formats With 4-Bit Components ",
"You must set border to 0.\n7.3.1. Formats With 4-Bit Components\nTexture formats with a type of GL_UNSIGNED_BYTE_4_4_DMP or GL_UNSIGNED_4BITS_DMP use 4 bits for a single component, and require a special data ordering.\nIn a format with a type of GL_UNSIGNED_BYTE_4_4_DMP, two components are stored in a single byte. You can combine this with a format of GL_LUMINANCE_ALPHA or GL_LUMINANCE_ALPHA_NATIVE_DMP. The luminance is stored in the most-significant 4 bits and the alpha component is stored in the least-significant 4 bits.\nIn a format with a type of GL_UNSIGNED_4BITS_DMP, two texels are stored in a single byte. You can combine this with a format of GL_LUMINANCE, GL_LUMINANCE_NATIVE_DMP, GL_ALPHA, or GL_ALPHA_NATIVE_DMP. Viewed as a row of texels, the first component is stored in the least-significant 4 bits, the second component is stored in the most-significant 4 bits, and so on.\n Warning:\n If you enable a texture format with a type of GL_UNSIGNED_4BITS_DMP (a 4-bit format) at the same time as another texture format (a non-4-bit format that includes ETC1 compressed textures) and then use them as a multitexture, there are restrictions on the placements of the texture.\n If you are placing 4-bit textures in VRAM, you must place 4-bit textures and non-4-bit textures in separate memory. In such cases, VRAM-A and VRAM-B are treated as separate memory. Behavior is undefined when textures having different bit formats are placed in the same memory.\n There is no restriction on texture arrays when 4-bit textures are placed in main memory.\nFigure 7-1. Bit Layout of Texture Formats That Comprise 4-Bit Components\n",0]
,["Pages/Page_55588693.html#PageId_55588698","7.4. Loading Compressed Textures ",
"\n7.4. Loading Compressed Textures\nYou can load compressed image data as texture images. Partial texture loading, with the glCompressedTexSubImage2D function, is not supported.\nCode 7-4. Definition of the glCompressedTexImage2D Function\n void glCompressedTexImage2D(GLenum target, GLint level, GLenum internalformat,\n GLsizei width, GLsizei height, GLint border,\n GLsizei imageSize, const void* data);\nFor target specify the value specified in the glTexImage2D function. However, shadow textures and gas textures cannot use the regions allocated by this function.\nFor width and height, specify the width and height of the texture image. Both numbers must be powers of 2, from 16 through 1024.\nThe values specified for level and border, and the restrictions on cube map textures and mipmap textures, are the same as in the glTexImage2D function.\nThe hardware supports only one compressed texture format: ETC1 (Ericsson Texture Compression). You can set internalformat to either GL_ETC1_RGB8_NATIVE_DMP or GL_ETC1_ALPHA_RGB8_A4_NATIVE_DMP.\nThe ETC1 format takes blocks of 4×4 texels in the 24-bit RGB format and compresses them each into 64 bits. An alpha channel is not supported by GL_ETC1_RGB8_NATIVE_DMP, but it is supported by GL_ETC1_ALPHA_RGB8_A4_NATIVE_DMP, which adds 4 bits of alpha component data for each of the 16 texels.\nA value of 1 is output for the alpha component when the texture combiner references a compressed texture with a format that does not include an alpha channel.\nFor imageSize specify the number of bytes in the image data. If the original texture image has a width of w and a height of h, imageSize can be found using the following equation. The value of blockSize is either 8 when there is no alpha channel or 16 when there is.\nimageSize = (w / 4) * (h / 4) * blockSize\nThe ETC1 format handled by the 3DS system is different from the standard OpenGL specifications. 7.10. Native PICA Format explains the differences between this format and the standard specifications. For more information about formats, see the 3DS Programming Manual: Advanced Graphics.\n",0]
,["Pages/Page_55588693.html#PageId_55588699","7.5. Copying From the Color Buffer ",
"\n7.5. Copying From the Color Buffer\nYou can get (copy) an image of the color buffer as a texture.\nCode 7-5. Definition of the glCopyTexImage2D Function\n void glCopyTexImage2D(GLenum target, GLint level, GLenum internalformat,\n GLint x, GLint y, GLsizei width, GLsizei height,\n GLint border);\nFor target, specify the value specified in the glTexImage2D function. All other argument values are also the same, except for the following differences.\n\tFor internalformat specify either GL_RGB or GL_RGBA, but data cannot be converted from the color buffer format during the copy (only formats with the same pixel sizes are allowed). \tThe values of x and y specify the starting point from which to copy color buffer data (with the origin at the lower-left corner and the positive axes pointing up and right). For width and height, specify the width and height of the region to copy. The values of x and y must be multiples of 8. ",0]
,["Pages/Page_55588693.html#PageId_55588700","7.5.1. Copying From the Color Buffer ",
" \tYou can only specify 0 for level.\n 7.5.1. Copying From the Color Buffer\nYou can also copy a partial texture image region from the color buffer.\nCode 7-6. Definition of the glCopyTexSubImage2D Function\n void glCopyTexSubImage2D(GLenum target, GLint level,\n GLint xoffset, GLint yoffset, GLint x, GLint y,\n GLsizei width, GLsizei height);\nData must be copied to a texture image region that was allocated in advance by the glTexImage2D function.\nThis function is the same as glCopyTexImage2D except that xoffset and yoffset specify the coordinates of the region to be copied to (where the origin is the lower-left corner and the positive axes point up and toward the right), and width and height specify a multiple of 8 (not necessarily a power of two).\n",0]
,["Pages/Page_55588693.html#PageId_55588701","7.5.2. Copying From the Depth Buffer ",
"\n7.5.2. Copying From the Depth Buffer\nIf you call the glEnable function with GL_DEPTH_STENCIL_COPY_DMP passed as an argument, the glCopyTexImage2D and glCopyTexSubImage2D functions copy the content of the depth (and stencil) buffer, rather than the color buffer, to a texture as long as depth/stencil copy operations are enabled.\nThe format of the current depth buffer determines the format of the texture to specify as the copy target. Because the format is not converted during the copy operation, a GL_INVALID_OPERATION error occurs if you attempt to copy data to a texture with an unsupported format. The copied data is identical for both native and non-native texture formats.\nTable 7-5. Depth Buffer Formats and the Corresponding Texture Format and Type\n \t Depth Buffer Format\n \t Texture Format\n \t Texture Type\n \t Components\n \t GL_DEPTH24_STENCIL8_EXT\n \t GL_RGBA\n GL_RGBA_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t R: Stencil\nG: D [23 : 16]\nB: D [15 : 8]\nA: D [7 : 0]\n \t GL_DEPTH_COMPONENT24_OES\n \t GL_RGB\n GL_RGB_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t R: D [23 : 16]\nG: D [15 : 8]\nB: D [7 : 0]\n \t GL_DEPTH_COMPONENT16\n \t GL_HILO8_DMP\n GL_HILO8_DMP_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t R: D [15 : 8]\nG: D [7 : 0]\n Note: The Components column shows which bits of the depth value are set in each component. (For example, Depth [15:8] indicates that bits 8 through 15 of the depth value are set for that component.)\nYou can call the glEnable, glDisable, and glIsEnabled functions and pass GL_DEPTH_STENCIL_COPY_DMP as an argument to respectively enable, disable, and determine the current status of depth and stencil copy operations.\n Note:\n You cannot use the glCopyTexImage2D and glCopyTexSubImage2D functions in block 32 mode. No errors are generated, but images are not transferred properly. For information about block mode settings, see the 3DS Programming Manual: Advanced Graphics.\n",0]
,["Pages/Page_55588693.html#PageId_55588702","7.6. Specifying a Texture as the Render Target ",
"\n7.6. Specifying a Texture as the Render Target\nTo write rendering results directly to a texture (\"render to texture\"), take a texture image allocated by glTexImage2D and bind it to a framebuffer object using the glFramebufferTexture2D function. Because rendering results are written directly to each of the special textures used by shadows and gases, you do not need to use this function to specify the special textures as rendering targets.\nCode 7-7. Definition of the glFramebufferTexture2D Function\n void glFramebufferTexture2D(GLenum target, GLenum attachment, GLenum textarget,\n GLuint texture, GLint level);\nFor target, you can only specify GL_FRAMEBUFFER.\nFor attachment, specify the data to write to the texture. You can specify GL_COLOR_ATTACHMENT0 (color data) or GL_DEPTH_ATTACHMENT (depth data). For textarget, specify the value specified for target in the glTexImage2D function. For texture, specify the texture object to bind to the framebuffer object. For level, can only specify 0.\nWhen a texture is specified as the render target for the depth (and stencil) buffer, Table 7-5 in 7.5.2. Copying From the Depth Buffer shows which texture format to specify for each depth buffer format.\nSpecifying GL_DEPTH_ATTACHMENT as the attachment parameter has the same effect as specifying GL_DEPTH_STENCIL_ATTACHMENT, when you call this function on a depth buffer in the GL_DEPTH24_STENCIL8_EXT format.\n",0]
,["Pages/Page_55588693.html#PageId_55588703","7.7. Loading Lookup Tables ",
"\n7.7. Loading Lookup Tables\nThe glTexImage1D function loads one-dimensional textures in OpenGL, but it is used to load lookup tables on the 3DS system. A lookup table is a one-dimensional table accessed by procedural textures, fragment lighting, fog, and gases. It cannot be used as a texture.\nCode 7-8. Definition of the glTexImage1D Function\n void glTexImage1D(GLenum target, GLint level, GLint internalformat,\n GLsizei width, GLint border, GLenum format, GLenum type,\n const GLvoid *pixels);\nFor target, specify GL_LUT_TEXTUREi_DMP to specify the lookup table to load. The lookup table number is given by i, a number between 0 and one less than the value obtained by passing GL_MAX_LUT_TEXTURES_DMP to pname in the glGetIntegerv function. In other words, you can specify a lookup table number in the range from 0 through 31. GL_LUT_TEXTUREi_DMP is defined as GL_LUT_TEXTURE0_DMP+i.\nYou can only specify 0 for level, GL_FLOAT for type, and GL_LUMINANCEF_DMP for format and internalformat. A GL_INVALID_VALUE error is generated if level is nonzero. A GL_INVALID_ENUM error is generated if type, format, or internalformat is not one of these values.\nFor width, specify the number of table elements, and for pixels, specify the table elements. The maximum value that can be specified for width is 512, which is the same value obtained from the glGetIntegerv function when GL_MAX_LUT_ENTRIES_DMP is passed into pname. However, the various reserved fragment processes have unique restrictions on the number of table elements and on the table elements.\nTo get the ID of the texture object bound to GL_LUT_TEXTUREi_DMP, call glGetIntegerv and specify GL_TEXTURE_BINDING_LUTi_DMP for the pname parameter (where i is a value from 0 through 31).\nUse the glTexSubImage1D function to rewrite a subset of the lookup table data that has been loaded.\nCode 7-9. Definition of the glTexSubImage1D Function\n void glTexSubImage1D(GLenum target, GLint level, GLint xoffset,\n GLsizei width, GLenum format, GLenum type,\n const GLvoid *pixels);\nData must be copied to a lookup table region that was allocated in advance by the glTexImage1D function.\nThis function is identical to glTexImage1D except for xoffset and width, which specify the starting element number and the number of elements, respectively. A GL_INVALID_VALUE error is generated if the sum of width and xoffset exceeds the number of table elements.\nThe glCopyTexImage1D and glCopyTexSubImage1D functions have not been implemented. Texture parameter settings are also unsupported.\n",0]
,["Pages/Page_55588693.html#PageId_55588704","7.8. Destroying Texture Objects ",
"\n7.8. Destroying Texture Objects\nYou can use the glDeleteTextures function to destroy texture objects that are no longer necessary.\nCode 7-10. Definition of the glDeleteTextures Function\n void glDeleteTextures(GLsizei n, const GLuint* textures);\n",0]
,["Pages/Page_55588693.html#PageId_55588705","7.9. Texture Collections ",
"\n7.9. Texture Collections\n",0]
,["Pages/Page_55588693.html#PageId_55588706","7.9.1. Creating Texture Collections ",
"Using texture collections, you can bind a group of texture objects to a texture type in one operation. Texture collections share their namespace with texture objects and can be created, specified, and destroyed using glGenTextures, glBindTexture, and glDeleteTextures, respectively.\n7.9.1. Creating Texture Collections\nA texture collection operates as a special texture object. Consequently, use the glGenTextures function to generate objects that can be used as texture collections, just as you would for texture objects.\n",0]
,["Pages/Page_55588693.html#PageId_55588707","7.9.2. Specifying Texture Collections ",
"\n7.9.2. Specifying Texture Collections\nUse the glBindTexture function to specify a texture object to use as a texture collection. Call this function, and for the target parameter, specify GL_TEXTURE_COLLECTION_DMP. By default, the object with a name of 0 (with a value of 0 passed into texture) is a texture collection.\nAfter this function is called, the texture collection records the texture objects bound by glBindTexture to each texture type (2D texture, cube map texture, and lookup table). The function overwrites any other existing binding to the same texture. Settings recorded in the texture collection remain until the collection is switched.\nWhen the texture collection is switched, all texture objects recorded in the new texture collection are bound as the new textures.\n",0]
,["Pages/Page_55588693.html#PageId_55588708","7.9.3. Destroying Texture Collections ",
"\n7.9.3. Destroying Texture Collections\nTexture collections can be destroyed with the glDeleteTextures function, just like texture objects. Even if a texture collection is destroyed, it does not affect texture bindings to the texture objects recorded in that vertex state collection.\nThe glDeleteTextures function does not immediately destroy a texture collection that is in use. A texture collection remains in use until it is switched with another one. You cannot destroy the default texture collection. Calls to glDeleteTextures on the default texture collection are ignored.\n",0]
,["Pages/Page_55588693.html#PageId_55588709","7.10. Native PICA Format ",
"\n7.10. Native PICA Format\nThe GPU's texture unit supports a different texture format than the OpenGL specifications. The format that is actually supported by the texture unit is called the native PICA format. Because libraries do not convert textures loaded in native PICA format, it is more efficient than the standard format.\n",0]
,["Pages/Page_55588693.html#PageId_55588710","7.10.1. Byte Order Swapping ",
"There are three major differences between this format and the OpenGL specifications.\n\tByte order\nBytes are written in a different order because of the way internal addresses are processed.\n \tV-Flip\nThe two formats use v-axes that run in opposite directions, when placing texels using UV coordinates.\n \tAddressing\nTexels and compressed data blocks are written in different orders, because of differences between linear addressing (OpenGL) and block addressing (native PICA format).\nIn the CTR system, v-flip, addressing, and byte-order conversions are run, in that order, on an uncompressed texture (loaded by glTexImage2D), to convert it from the OpenGL format into the native PICA format. The system converts a compressed texture (loaded by glCompressedTexImage2D) by running V-flip, ETC compression, addressing, and finally, byte-order conversions. Note that v-flip conversion must precede ETC compression.\n7.10.1. Byte Order Swapping\nWhen the format uses more than one byte to represent a single texel and the texture was defined by calling glTexImage2D and passing GL_UNSIGNED_BYTE as the type, the number of bytes swapped (replaced) is the same as the number of bytes that represent a texel.\nData in a compressed format is swapped one block (8 bytes for 4×4 texels) at a time. However, if the format has an alpha channel (the first 8 bytes), the alpha portion is not byte-swapped.\nFigure 7-2. Four-Byte Swap\nFigure 7-3. Three-Byte Swap\nFigure 7-4. Two-Byte Swap\nFigure 7-5. Byte Swap in a Compressed Format (No Alpha Channel)\nFigure 7-6. Byte Swap in a Compressed Format (With Alpha Channel)\n",0]
,["Pages/Page_55588693.html#PageId_55588711","7.10.2. V-Flip Differences ",
"\n7.10.2. V-Flip Differences\nIn the OpenGL specifications, images are converted into data starting with the texel at UV coordinates (0.0, 0.0). In the native PICA format, images are converted into data, starting with the texel at UV coordinates (0.0, 1.0). This is the same for both uncompressed and compressed textures.\n",0]
,["Pages/Page_55588693.html#PageId_55588712","7.10.3. Differences in Addressing ",
"\n7.10.3. Differences in Addressing\n",0]
,["Pages/Page_55588693.html#PageId_55588713","7.10.3.1. Uncompressed Textures ",
"In the OpenGL specifications, texels are stored consecutively in the U direction (linear addressing). In the native PICA format, blocks of texels are stored, consecutively, in the U direction, but texels within those blocks are stored in a zigzag pattern (block addressing).\n7.10.3.1. Uncompressed Textures\nTexels are stored in a zigzag pattern within each block of 8×8 texels (as shown by the red lines in the figure), but the blocks are stored consecutively in the U direction.\nFigure 7-7. Texel Order in an Uncompressed Texture\n",0]
,["Pages/Page_55588693.html#PageId_55588714","7.10.3.2. Compressed Textures ",
"\n7.10.3.2. Compressed Textures\nA compressed texture compresses each block of 4×4 texels. In the OpenGL specifications, blocks are stored consecutively in the U direction (shown by the blue line in the figure). In the native PICA format, blocks are stored in a zigzag pattern in meta-blocks of 2×2 blocks. The meta-blocks themselves are stored consecutively in the U direction (shown by the red line in the figure).\nFigure 7-8. Texel Order in a Compressed Texture\nFor more information about compressed textures, see the 3DS Programming Manual: Advanced Graphics.\nCONFIDENTIAL\n",0]
,["Pages/Page_55588715.html","8. Vertex Shaders ",
"8. Vertex Shaders\nA vertex shader converts coordinates, applies shadows, and performs other operations on vertex attributes that are input. The 3DS system has four vertex processors, each of which can handle vector data comprising four, 24-bit floating-point values. The input vertex data is processed, in parallel, by the four vertex processors. Although the processors can run general-purpose calculations, they cannot read data from or write data to VRAM.\nLike OpenGL ES 2.0, the 3DS system does not make a clear distinction between coordinates, normals, and other attributes stored with vertex data. Attributes determine how a vertex shader processes and outputs its input data. In general, the following types of attributes are input as vertex data for 3D graphics processing.\n\tVertex coordinates \tNormal vectors \tTangent vectors \tTexture coordinates \tVertex color\n",0]
,["Pages/Page_55588715.html#PageId_55588716","8.1. Input Vertex Data ",
"We recommend that you refer to the Vertex Shader Reference as you continue to read this chapter.\n8.1. Input Vertex Data\nVertex data is passed to the vertex shader through input registers that are bound to vertex attribute numbers. Using #pragma bind_symbol, the vertex shader specifies the names and registers for input data.\nCode 8-1. Binding Data Names and Registers (in Shader Assembly)\n #pragma bind_symbol(AttribPosition.xyzw, v0, v0)\nYou cannot bind more than one data name to the same input register. The second and third arguments must also take the same value (or the third argument must be omitted). The application uses the glBindAttribLocation function to bind vertex attribute numbers and data names, the glEnableVertexAttribArray function to enable bound vertex attribute numbers, and glVertexAttribPointer or some other function to input vertex data.\nCode 8-2. Binding Vertex Attribute Numbers and Inputting Vertex Data\n glBindAttribLocation(program, 0, \"AttribPosition\");\nglEnableVertexAttribArray(0);\nglVertexAttribPointer(0, 4, GL_FLOAT, GL_FALSE, 0, pointer);\nThe register number and the vertex attribute number do not have to be the same.\nInput registers that are not bound by #pragma bind_symbol have undefined values. Any input vertex data that is not a GL_FLOAT value is automatically converted into a GL_FLOAT value. However, note that the data is not normalized.\nWhen a vertex buffer is used, the vertex attribute data type and data size combination affects transfer speed of the vertex data. For more information, see 15.13. Effect of Vertex Attribute Combinations on Vertex Data Transfer Speed.\n Warning:\n If vertex data is input through the glVertexAttribPointer function, the fourth argument (specifying whether to normalize values) is ignored because it is not supported by the hardware. The vertex shader must explicitly normalize values because the specified value has no effect.\n You cannot specify GL_FIXED or GL_UNSIGNED_SHORT for the type parameter of the glVertexAttribPointer function. If GL_FLOAT or GL_SHORT is specified, ptr must be a pointer that is 4-byte aligned or 2-byte aligned, respectively.\nWhile a vertex shader is processing a single vertex, it must load vertex data from at least one input register (even a single component), or it may not run properly.\n",0]
,["Pages/Page_55588715.html#PageId_55588717","8.2. Vertex Data Output ",
"\n8.2. Vertex Data Output\nThe vertex shader's processing results are written to the output registers that are mapped to output vertex attributes, and in this way the results are passed on to the next processing stage.\nBecause only a reserved shader can be used for fragment processing, the vertex shader's output vertex attributes are fixed in advance.\nCode 8-3. Mapping Registers to Output Vertex Attributes and Writing to the Registers (in Shader Assembly)\n #pragma output_map(position, o0)\nmov o0, v0\nOutput vertex attributes have the following names. The components that the vertex shader must set for the output attributes are fixed.\nTable 8-1. Output Vertex Attributes\n \t Attribute Name\n \t Output Attribute\n \t Required Components\n \t position\n \t Vertex coordinates.\n \t 4 components: x, y, z, and w.\n \t color\n \t Vertex color.\n \t 4 components: R, G, B, and A.\n \t texture0\n \t Texture coordinate 0.\n \t 2 components: u and v.\n \t texture0w\n \t Texture coordinate 0.\n \t 1 component: w.\n \t texture1\n \t Texture coordinate 1.\n \t 2 components: u and v.\n \t texture2\n \t Texture coordinate 2.\n \t 2 components: u and v.\n \t quaternion\n \t Quaternions.\n \t 4 components: x, y, z, and w.\n \t view\n \t View vector.\n \t 3 components: x, y, and z.\n \t generic\n \t General-purpose attribute.\n \t Any number of components that are used by the geometry shader.\n The vertex shader must write some value to every component (x, y, z, and w) of registers that have been mapped. Dummy values must be written to any unmapped (unused) components in the registers specified by #pragma output_map.\nThe vertex shader forces processing to end when values have been written to all mapped registers. It then moves on to process the next vertex (the end instruction must be called). In other words, after the last attribute data has been written to a register, later instructions might not be executed.\nYou can only write to (each component of) a single output register once while processing a single vertex. Correct operation is not guaranteed if data is written to the same component (of the same register) more than once.\nExcept for the generic attribute, output vertex attributes can be mapped to no more than seven output registers. To map eight or more non-generic output vertex attributes, you must map multiple attributes to a single register. In this case, multiple vertex attributes (up to a total of four components) must be packed into a single register. For example, you could map texture1 and texture2 to o0.xy and o0.zw, respectively.\n",0]
,["Pages/Page_55588715.html#PageId_55588718","8.3. Uniform Settings ",
"\n8.3. Uniform Settings\nWhen #pragma bind_symbol binds data names to non-input registers (floating-point constant registers, Boolean registers, or integer registers), the application can use the glUniform* functions to set values in each register. You can transpose matrices within the glUniformMatrix* functions (convert the matrices from column-major to row-major order), by specifying GL_TRUE for transpose.\nSample code follows for both the vertex shader and the application.\nCode 8-4. How to Bind Data Names to Non-Input Registers (in Shader Assembly)\n #pragma bind_symbol ( ModelViewMatrix , c0 , c3 )\n#pragma bind_symbol ( LoopCounter0 , i1 , i1 )\n#pragma bind_symbol ( bFirst , b2 , b2 )\n#pragma bind_symbol ( Scalar.x , c4, c4 )\nCode 8-5 How to Set Uniforms\n uniform_location = glGetUniformLocation ( program , \"ModelViewMatrix\" );\nGLfloat matrix[4][4];\nglUniform4fv ( uniform_location , 4 , matrix );\nGLfloat scalar_value;\nuniform_location = glGetUniformLocation ( program , \"Scalar\" );\nglUniform1f ( uniform_location , scalar_value );\nuniform_location = glGetUniformLocation ( program , \"bFirst\" );\nglUniform1i ( uniform_location , GL_TRUE );\nGLint loop_setting[3] = { 4 , 0 , 1 } ; // loop_count – 1 , init , step\nuniform_location = glGetUniformLocation ( program , \"LoopCounter0\" );\nglUniform3iv ( uniform_location , loop_setting );\nThe sample code specifies component x for the data name Scalar. Components can be specified this way when binding a floating-point constant register. You must specify components consecutively, in xyzw order. In other words, xy, zw, or yzw can be specified but not xz, yw, or xyw.\nInteger registers are used to control the loop instruction in a shader program. In a register that is 24 bits wide, the loop count is assigned to bits 0 through 7, the initial value is assigned to bits 8 through 15, and the increment value is assigned to bits 16 through 23. The loop instruction initializes the loop counter register to a default value, and then repeatedly executes the instructions between loop and endloop once more than the specified loop count. The loop counter register is incremented only by the increment value each time through the loop.\n",0]
,["Pages/Page_55588715.html#PageId_55588719","8.4. Notes for the Clip Coordinate System ",
"\n8.4. Notes for the Clip Coordinate System\nThe vertex shader outputs Z components in a clip coordinate system that differs from the one used in OpenGL ES.\nOpenGL ES clips coordinates between -Wc and Wc, but the 3DS system clips them between 0 and -Wc (the sign is reversed). To use projective transformation matrices that are compatible with OpenGL ES, applications must convert the range from -Wc to Wc into the range from 0 to -Wc.\nWhen applying such conversions of projection matrices in application code, make the following conversion and set the resulting projection matrix as a uniform.\nCode 8-6. Converting to an OpenGL ES-Compatible Projective Transformation Matrix\n GLfloat projection[16];\nprojection[2] = (projection[2] + projection[3]) * (-0.5f);\nprojection[6] = (projection[6] + projection[7]) * (-0.5f);\nprojection[10] = (projection[10] + projection[11]) * (-0.5f);\nprojection[14] = (projection[14] + projection[15]) * (-0.5f);\nWhen making the same conversion in a vertex shader, apply the projection conversion as follows.\nCode 8-7. Converting to an OpenGL ES-Compatible Projective Transformation Matrix (in Shader Assembly)\n #pragma output_map(position, o0)\n#pragma bind_symbol(attrib_position, v0)\n#pragma bind_symbol(modelview, c0, c3)\n#pragma bind_symbol(projection, c4, c7)\ndef c8, -0.5, -0.5, -0.5, -0.5\n// Model View Transformation\ndp4 r0.x, v0, c0\ndp4 r0.y, v0, c1\ndp4 r0.z, v0, c2\ndp4 r0.w, v0, c3\n// Projective Transformation\ndp4 o0.x, r0, c4\ndp4 o0.y, r0, c5\nmov r1, c6\nadd r1, r1, c7\nmul r1, r1, c8\ndp4 o0.z, r0, r1\ndp4 o0.w, r0, c7\n",0]
,["Pages/Page_55588715.html#PageId_55588720","8.5. Vertex Cache ",
"\n8.5. Vertex Cache\nSome of the vertex data created or processed by the vertex shader is saved in a cache. The vertex shader does not process input vertex data that is determined to be the same as the original vertex data saved in the cache, based on its vertex indices. Instead, the processed data in the cache is sent to the next process. The same vertex data is often processed more than once when it is input using GL_TRIANGLES, but this can be avoided if there is processed vertex data in the cache already.\nThe following conditions must be satisfied to use the vertex cache.\n\tVertex data must be input in a format that accesses vertex indices. In short, glDrawElements must be called to input vertex data. \tInput vertex data must use a vertex buffer.\nThe vertex cache can save 32 vertex data entries. It is implemented with a proprietary algorithm that resembles the functionality of the LRU (least recently used) algorithm.\nWhen the repeatedly accessed vertex data contains no more than 32 vertices, there is a higher chance of a cache hit. But the efficiency of the vertex cache is affected by conditions other than index order, including the usage state of the memory holding the index array, and the length of the shader executing as the vertex shader. For these reasons, the optimal index depends on the content, and there may not be a definitive answer.\n",0]
,["Pages/Page_55588715.html#PageId_55588721","8.6. Querying the Vertex Shader ",
"\n8.6. Querying the Vertex Shader\n",0]
,["Pages/Page_55588715.html#PageId_55588722","8.6.1. Getting Vertex Attribute Information ",
"You can query the vertex shader for information about active vertex attributes and uniforms.\n8.6.1. Getting Vertex Attribute Information\nYou can use the glGetActiveAttrib function to get attribute information for the vertex data input to the vertex shader.\nCode 8-8. Definition of the glGetActiveAttrib Function\n void glGetActiveAttrib(GLuint program, GLuint index, GLsizei bufsize,\n GLsizei* length, GLint* size, GLenum* type, char* name);\nA GL_INVALID_OPERATION error is generated if program specifies an unlinked or otherwise invalid program object.\nFor index specify a value between 0 and one less than the number of vertex attributes obtained from the glGetProgramiv function, when it is called on the program object specified by program, when GL_ACTIVE_ATTRIBUTES is specified for pname. A GL_INVALID_VALUE error is generated if the specified value is negative or greater than or equal to the number of vertex attributes.\nFor bufsize specify the size of the array specified in name. A GL_INVALID_VALUE error is generated if a negative value is specified.\nThe vertex attribute's type is returned in type. The vertex attribute's size is returned in size. This is the number of values indicated by type that are required to represent the vertex attribute.\nThe vertex attribute's name is returned in name. If there are more than bufsize characters in the vertex attribute's name, up to bufsize - 1 characters are stored with a terminating character (NULL) added at the end. The number of characters in name is returned in length (excluding the terminating null character).\n",0]
,["Pages/Page_55588715.html#PageId_55588723","8.6.2. Getting Uniform Information ",
"\n8.6.2. Getting Uniform Information\nYou can use the glGetActiveUniform function to get uniform information registered with a program object. This is not limited to the vertex shader; you can also get information about uniforms used by the geometry shader and reserved fragment shader, which are described later.\nCode 8-9. Definition of the glGetActiveUniform Function\n void glGetActiveUniform(GLuint program, GLuint index, GLsizei bufsize,\n GLsizei* length, GLint* size, GLenum* type, char* name);\nA GL_INVALID_OPERATION error is generated if program specifies an unlinked or otherwise invalid program object.\nFor index, specify a value between 0 and one less than the uniform information count obtained from the glGetProgramiv function when it is called on the program object specified by program and when pname is GL_ACTIVE_UNIFORMS. A GL_INVALID_VALUE error is generated if the specified value is negative or greater than or equal to the uniform information count.\nFor bufsize, specify the size of the array specified in name. A GL_INVALID_VALUE error is generated if a negative value is specified.\nThe type of value for the uniform setting is returned in type. The value's size is returned in size. This is the number of elements indicated by type that are required to represent the uniform setting. For example, GL_FLOAT_VEC4 is stored in type and 4 is stored in size for a 4×4 matrix, such as the modelview matrix.\nThe uniform's name is returned in name. If there are more than bufsize characters in the uniform's name, up to bufsize - 1 characters are returned with a terminating character (NULL) added at the end. The number of characters in name is returned in length (excluding the terminating null character).\n",0]
,["Pages/Page_55588715.html#PageId_55588724","8.6.3. Setting Categories ",
"\n8.6.3. Setting Categories\nThe glGetActiveAttrib and glGetActiveUniform functions can get the following types of values.\nTable 8-2. List of Setting Categories\n \t Type\n \t Type\n \t Number of Components\n \t Major Use\n \t GL_FLOAT\n \t float\n \t 1\n \t Bias and scale values.\n \t GL_FLOAT_VEC2\n \t float\n \t 2\n \t Viewport settings.\n \t GL_FLOAT_VEC3\n \t float\n \t 3\n \t Colors (RGB) and directional vectors.\n \t GL_FLOAT_VEC4\n \t float\n \t 4\n \t Colors (RGBA) and transformation matrices.\n \t GL_INT\n \t int\n \t 1\n \t Mode settings.\n \t GL_INT_VEC3\n \t int\n \t 3\n \t Combiner source input.\n \t GL_BOOL\n \t bool\n \t 1\n \t Enabling and disabling features.\n \t GL_SAMPLER_1D\n \t int\n \t 1\n \t Specifying lookup tables.\n Lists categories that are actually used by uniforms in each shader.\n",0]
,["Pages/Page_55588715.html#PageId_55588725","8.7. Getting and Setting the Values of Multiple Uniforms ",
"\n8.7. Getting and Setting the Values of Multiple Uniforms\nFunctions are provided for getting and setting multiple uniforms, concurrently, on the 3DS system.\nBy calling the glUniformsDMP function, you can concurrently set values in multiple uniforms for the program object that is currently bound.\nCode 8-10. Setting a Group of Uniforms\n void glUniformsDMP(GLuint n, GLint* locations, GLsizei* counts,\n const GLuint* value);\nFor n, specify the number of uniforms to set.\nFor locations, specify a pointer to an array storing n uniform locations (which can be obtained by glGetUniformLocation). For counts, specify a pointer to an array storing the number of elements in the n uniforms. The count parameter of the glUniform* functions corresponds to the number of uniform elements. Fill the array specified by counts with the number of elements to set for each array uniform and 1 for each non-array uniform.\nFor value, specify a pointer to an array storing the values to set in the uniforms. Because each uniform has a different amount of data, the indices for the values to store in value are not necessarily the same as the indices in locations and counts for the corresponding uniforms. You can mix both GLfloat and GLuint data in the uniforms that you set. Store 32-bit GLfloat data for the values to set in GLfloat uniforms.\nThis function does not perform any error-checking. Behavior is undefined if you specify an invalid value for any argument.\nBy calling the glGetUniformsDMP function, you can concurrently get values from multiple uniforms for a specified program object.\nCode 8-11. Getting a Group of Uniforms\n void glGetUniformsDMP(GLuint program, GLuint n, GLint* locations,\n GLsizei* counts, GLuint* params);\nFor program, specify the program object for which to get uniform values.\nFor n, specify the number of uniforms to get.\nFor locations, specify a pointer to an array storing n uniform locations (which can be obtained by glGetUniformLocation). For counts, specify a pointer to an array storing the number of elements in the n uniforms. The number of uniform elements is either 1 (for a non-array uniform) or the number of array elements from which to get values for an array uniform.\nFor params, specify a pointer to an array used to get the uniform values. Because each uniform has a different amount of data, the indices for the values stored in params are not necessarily the same as the indices in locations and counts for the corresponding uniforms. Both GLfloat and GLuint data can be mixed in the uniforms from which values are obtained. The GLfloat values obtained from GLfloat uniforms are stored as 32-bit data in params.\nThis function does not perform any error-checking. Behavior is undefined if you specify an invalid value for any argument.\n",0]
,["Pages/Page_55588715.html#PageId_55588726","8.8. Other Notes and Cautions ",
"\n8.8. Other Notes and Cautions\nYou can access uniform values with the glUniform* and glGetUniform* functions, using uniform locations that can be obtained with the glGetUniformLocation function. By adding an offset to a location, you can specify and access a specific element in an array uniform. For example, the second element of an array uniform is accessed when 1 is added to the uniform's location.\nA uniform's location is fixed when the glLinkProgram function is called and the value originally differs for each program object. The glUniform* functions generate an error if the location is not related to the current program object. The glGetUniform* functions generate an error if the location is related to a program object other than program. However, the glUniform* and glGetUniform* functions do not generate errors if the programs being queried are reserved fragment shaders and the location is specified using a bitwise OR with 0xFFF80000.\nDo not include vertex attributes in #pragma output_map definitions when they do not need to be output. If you do not follow this advice, useless instructions will be required because values must be written to all defined vertex attributes (output registers) when the vertex shader is run. Some output attributes also involve clock instructions for parts of the GPU circuit, and their output can needlessly run down the battery.\nThe maximum number of vertex attributes when rendering without using a vertex buffer is 16, and the maximum number of vertex attributes that can use a vertex buffer is 12. However, if 12 vertex attributes are rendered using a vertex buffer, you must take care about the limitations in vertex data placement (see 6.8.1. Restrictions Affecting Only glDrawElements).\nIn the vertex shader assembler, you can define up to 16 vertex attributes, but if the maximum number as determined according to the above conditions is exceeded, a GL_INVALID_OPERATION error may be generated when the rendering function is called.\nCONFIDENTIAL\n",0]
,["Pages/Page_55588727.html","9. Geometry Shaders ",
"9. Geometry Shaders\nGeometry shaders use vertex shader output to generate primitives and output an arbitrary number of vertices.\nIn OpenGL ES 2.0, you can specify GL_POINTS, GL_LINES, GL_LINE_STRIP, or GL_LINE_LOOP to the glDrawElements or glDrawArrays function to render primitives, but on the 3DS system, you must use the geometry shader to generate non-triangle primitives. Triangles are rendered with GL_TRIANGLES, GL_TRIANGLE_STRIP, and GL_TRIANGLE_FAN because they are included in OpenGL ES 2.0, but multisample rendering is not supported.\nAs discussed in 5. Shader Programs, only the geometry shaders provided by the SDK can be used on the 3DS system. You can use the following geometry shaders.\n\tPoint shaders \tLine shaders \tSilhouette shaders \tCatmull-Clark subdivision shaders \tLoop subdivision shader \tParticle system shader\nThese geometry shaders cannot be used alone. Always load a binary that also has a linked vertex shader, and also make sure that you use the vertex shader. When a geometry shader is used, one of the four vertex processors is used as the geometry processor. The 15th Boolean register (b15) is reserved for the geometry shader.\nFigure 9-1. Processor Configurations When the Geometry Shader Is Used and Not Used\nVertex shader output is used as input data to the geometry shader. Each geometry shader requires specific vertex attributes and a specific input order and can only use certain vertex attributes. The vertex shader must output these values correctly. Data is input to the geometry shader in order, starting with the smallest register number output by the vertex shader. Output vertex attributes are defined by #pragma output_map, but any generic attribute names are only used by the geometry shader, without being handled by some later process (such as fragment processing). Geometry shaders have reserved uniforms.\n",0]
,["Pages/Page_55588727.html#PageId_55588728","9.1. Point Shaders ",
"The point shader, for example, uses reserved uniforms to configure the viewport and other settings. Because all of these reserved uniforms initially have undefined values, they must be set by the application.\n9.1. Point Shaders\n",0]
,["Pages/Page_55588727.html#PageId_55588729","9.1.1. Shader Files ",
"The point shader generates two triangle primitives to draw a square (a point primitive) centered at the specified vertex coordinates, with sides as long as the specified point size. The point sprite shader is a point shader that outputs texture coordinates s, t (shown in the following figure), r (fixed at 0.0), and q (fixed at 1.0), for applying textures to point primitives. Neither shader supports grid adjustments or multisample rendering.\nFigure 9-2. Drawing Point Primitives\n9.1.1. Shader Files\nThe shader file to link with the vertex shader is determined by two things: the number of vertex attributes output for fragment processing that are also not required by the point shader, and the number of texture coordinates output by the point sprite shader.\nPoints: DMP_pointN.obj\nPoint sprites: DMP_pointSpriteN_T.obj\nWhere N is the number of vertex attributes that are not required by the point (or point sprite) shader, and T is the number of texture coordinates.\n",0]
,["Pages/Page_55588727.html#PageId_55588730","9.1.2. Reserved Uniform ",
"\n9.1.2. Reserved Uniform\nThere are reserved uniforms for configuring the viewport, and for enabling or disabling distance attenuation on the point size.\n",0]
,["Pages/Page_55588727.html#Anchor_55588730_ビューポート","Viewport ",
"Viewport\nUse the glUniform2fv function to set the width and height of the viewport in the reserved uniform dmp_Point.viewport.\n",0]
,["Pages/Page_55588727.html#Anchor_55588730_距離減衰","Distance Attenuation ",
"Distance Attenuation\nUse the glUniform1i function to set the reserved uniform for distance attenuation on the point size, dmp_Point.distanceAttenuation. A value of GL_TRUE enables distance attenuation, and GL_FALSE disables it. The point size is multiplied by the clip coordinate Wc when distance attenuation is disabled. This cancels out the division by Wc during the conversion to window coordinates and prevents the size of displayed point primitives from being affected by distance.\nThese reserved uniforms must be set by the application, because they initially have undefined values.\n Table 9-1. Reserved Uniforms for Point Shaders\n \t Reserved Uniform\n \t Type\n \t Value to Set\n \t dmp_Point.viewport\n \t vec2\n \t The viewport using the following equation.\n (1 / viewport.width, 1 / viewport.height)\n \t dmp_Point.distanceAttenuation\n \t bool\n \t Specifies whether the point size is affected by distance attenuation.\n Specify GL_TRUE to enable distance attenuation, and GL_FALSE otherwise.\n ",0]
,["Pages/Page_55588727.html#PageId_55588733","9.1.3. Vertex Shader Settings ",
"\n9.1.3. Vertex Shader Settings\nThe point shader requires vertex coordinates and a point size to render point primitives. The vertex shader outputs the vertex coordinates and then the point size in order (starting with the smallest output register number). This is followed by the texture coordinates for the point sprite shader. If multiple texture coordinate pairs are used, they must be packed two at a time into the xy and zw components of a single output register.\nOutput vertex attributes must be set as follows: position for the vertex coordinates, generic for the point size, and texture0 through texture2 for the texture coordinates.\nIf the vertex shader outputs the vertex color in addition to the vertex coordinates and point size required by the point shader, link the DMP_point1.obj shader file and use the following #pragma output_map statements in the vertex shader.\nCode 9-1. Sample Output Register Settings When the Point Shader Is Used (in Shader Assembly)\n #pragma output_map ( position , o0 )\n#pragma output_map ( generic , o1 )\n#pragma output_map ( color , o2 )\nIf the vertex shader outputs the vertex color in addition to the vertex coordinates, point size, and two texture coordinate pairs required by the point sprite shader, link the DMP_pointSprite1_2.obj shader file and use the following #pragma output_map statements in the vertex shader.\nCode 9-2. Sample Output Register Settings When the Point Sprite Shader Is Used (in Shader Assembly)\n #pragma output_map ( position , o0 )\n#pragma output_map ( generic , o1 )\n#pragma output_map ( texture0 , o2.xy )\n#pragma output_map ( texture1 , o2.zw )\n#pragma output_map ( color , o3 )\nThe point sprite shader outputs texture coordinates for point sprites that replace the input texture coordinates. Note that the vertex shader must write dummy values to the output registers for the texture coordinates that are replaced by the point sprite shader.\n",0]
,["Pages/Page_55588727.html#PageId_55588734","9.1.4. Input Vertex Data ",
"\n9.1.4. Input Vertex Data\nTo generate primitives with the point shader, call the glDrawElements or glDrawArrays function and specify GL_GEOMETRY_PRIMITIVE_DMP for mode.\n",0]
,["Pages/Page_55588727.html#PageId_55588735","9.2. Line Shaders ",
"\n9.2. Line Shaders\nA line shader generates two triangle primitives to draw lines (a line primitive) that connect two points specified by vertex coordinates. You can use a reserved uniform to set the line width. The line shader does not support grid adjustments or multisample rendering.\n Figure 9-3. Drawing Line Primitives\n",0]
,["Pages/Page_55588727.html#PageId_55588736","9.2.1. Shader Files ",
"Coordinates are generated for the four vertices that make a rectangle (parallelogram) from the slope and width of the line segment connecting the specified vertex coordinates. The coordinates of the four vertices are generated from the input vertex coordinates in the Y direction (or the X direction for some line segment slopes).\n9.2.1. Shader Files\nThe shader file to link with the vertex shader is determined by the number of vertex attributes output for fragment processing that are also not required by the line shader.\nVertex coordinates can be specified for separate lines or line strips. Separate lines are drawn using two sets of vertex coordinates per line. The coordinates of the first two vertices are the same for both a line strip and separate lines, but the next separate line is drawn from the coordinates of the second and third vertices. In other words, starting with the third vertex, each vertex is used together with the previous one to draw a single, connected line.\nSeparator lines: DMP_separateLineN.obj\nStrip lines: DMP_stripLineN.obj\nWhere N is the number of vertex attributes not required by the line shader.\n",0]
,["Pages/Page_55588727.html#PageId_55588737","9.2.2. Reserved Uniform ",
"\n9.2.2. Reserved Uniform\nThere is a reserved uniform for setting the line width.\n",0]
,["Pages/Page_55588727.html#Anchor_55588737_ライン幅","Line Width ",
"Line Width\nUse the glUniform4fv function to set the reserved uniform for the line width, dmp_Line.width, using values calculated from both the line width and the width and height of the viewport. The reserved uniform for the line width must be set by the application because its initial value is undefined.\nTable 9-2. Reserved Uniforms for the Line Shader\n \t Reserved Uniform\n \t Type\n \t Value to Set\n \t dmp_Line.width\n \t vec4\n \t The line width is given by the following expression.\n (viewport.width / line.width,\nviewport.height / line.width,\nviewport.width * viewport.height,\n2 / line.width)\n ",0]
,["Pages/Page_55588727.html#PageId_55588739","9.2.3. Vertex Shader Settings ",
"\n9.2.3. Vertex Shader Settings\nThe line shader requires vertex coordinates to render line primitives. The vertex shader outputs the vertex coordinates, starting with the smallest output register number.\nOne output vertex attribute must be set: the position attribute of the vertex coordinates.\nIf the vertex shader outputs the vertex color in addition to the vertex coordinates required by the separate line shader, link the DMP_separateLine1.obj shader file and use the following #pragma output_map statements in the vertex shader.\nCode 9-3. Sample Output Register Settings When a Line Shader Is Used (in Shader Assembly)\n #pragma output_map ( position , o0 )\n#pragma output_map ( color , o1 )\nThe line strip shader works the same way as the separate line shader. Its shader file is DMP_stripLine1.obj.\n",0]
,["Pages/Page_55588727.html#PageId_55588740","9.2.4. Input Vertex Data ",
"\n9.2.4. Input Vertex Data\nTo generate primitives with the line shader, call the glDrawElements or glDrawArrays function and pass GL_GEOMETRY_PRIMITIVE_DMP for the mode parameter.\n",0]
,["Pages/Page_55588727.html#PageId_55588741","9.3. Silhouette Shaders ",
"\n9.3. Silhouette Shaders\nSilhouette shaders generate and render silhouettes around object edges. You can use silhouette edges to render object contours and, when combined with the shadow feature, soft shadows.\n",0]
,["Pages/Page_55588727.html#PageId_55588742","9.3.1. Triangle With Neighborhood ",
"To generate silhouette edges, the silhouette shader needs a primitive called a triangle with neighborhood, or TWN for short.\n9.3.1. Triangle With Neighborhood\nThis section describes one of the triangles that make up the objects used to render silhouette edges.\nThis is called a center triangle. A triangle with neighborhood (TWN) comprises a center triangle and the three (adjacent) triangles that share an edge with it.\nFigure 9-4. Sample Triangles With Neighborhood\nOne TWN comprises the center triangle with vertices 3, 1, and 4, in addition to the three triangles defined by vertices 0, 1, and 3; 2, 4, and 1; and 6, 3, and 4. When vertices 3, 4, and 6 form the center triangle, vertices 3, 1, and 4 represent a triangle in a separate TWN.\nTWNs are used to detect the silhouette edges of center triangles. You can make an object from TWNs to render silhouette edges on it.\n",0]
,["Pages/Page_55588727.html#PageId_55588743","9.3.2. Shader Files ",
"\n9.3.2. Shader Files\nTWN vertices can be input to a silhouette shader in two ways: (1) either as silhouette triangles, one TWN at a time (just like normal triangles), or (2) as continuous silhouette strips of adjoining TWNs.\nSilhouette triangles: DMP_silhouetteTriangle.obj\nSilhouette strips: DMP_silhouetteStrip.obj\n",0]
,["Pages/Page_55588727.html#PageId_55588744","9.3.3. Reserved Uniform ",
"\n9.3.3. Reserved Uniform\nSilhouette shaders have the following reserved uniforms.\n",0]
,["Pages/Page_55588727.html#Anchor_55588744_ポリゴンの表裏判定","Polygon Facing ",
"Polygon Facing\nUse the glUniform1i function to configure how the silhouette shader determines whether a polygon is front-facing (dmp_Silhouette.frontFaceCCW). Specify GL_TRUE or GL_FALSE if GL_CCW or GL_CW, respectively, have been passed to the glFrontFace function for object vertex input.\n",0]
,["Pages/Page_55588727.html#Anchor_55588744_シルエットエッジの幅","Silhouette Edge Width ",
"Silhouette Edge Width\nUse the glUniform2fv function to set the silhouette edge width (dmp_Silhouette.width) to a value calculated by multiplying the normal vector's x direction by a coefficient.\nYou can configure the effect of a vertex's w component (dmp_Silhouette.scaleByW). Multiplying this component by the silhouette edge width disables distance attenuation. Call the glUniform1i function and specify a value of GL_TRUE to multiply by the w component, or specify GL_FALSE to fix the silhouette edge's w component at 1.0.\n",0]
,["Pages/Page_55588727.html#Anchor_55588744_シルエットエッジのカラー","Silhouette Edge Colors ",
"Silhouette Edge Colors\nUse the glUniform4fv function to set the color of the silhouette edge (dmp_Silhouette.color) with R, G, B, and A values.\n",0]
,["Pages/Page_55588727.html#Anchor_55588744_オープンエッジ","Open Edges ",
"Open Edges\nAn open edge is an edge of the center triangle that is not shared with any other triangles. You can configure open edges (dmp_Silhouette.acceptEmptyTriangles) to always be drawn (GL_TRUE) or not (GL_FALSE).\nOpen edges differ from silhouette edges in that they are drawn like line primitives, without using vertex normals. As a result, they may look different from silhouette edges at some angles. Some settings are specific to open edges.\n",0]
,["Pages/Page_55588727.html#Anchor_55588744_オープンエッジの幅","Open Edge Width ",
"Open Edge Width\nUse the glUniform4fv function to set the open edge width (dmp_Silhouette.openEdgeWidth) using values calculated from a specified width and the viewport's width and height (just like a line shader).\n",0]
,["Pages/Page_55588727.html#Anchor_55588744_オープンエッジのカラー","Open Edge Colors ",
"Open Edge Colors\nUse the glUniform4fv function to set the open edge color (dmp_Silhouette.openEdgeColor) using R, G, B, and A values.\n",0]
,["Pages/Page_55588727.html#Anchor_55588744_オープンエッジの視点方向へのバイアス","Open Edge Bias Toward the Viewpoint ",
"Open Edge Bias Toward the Viewpoint\nUse the glUniform1fv function to set the bias toward the viewpoint (dmp_Silhouette.openEdgeDepthBias). A negative value indicates movement toward the viewpoint, and a positive value indicates movement away from it. Normal vectors are not used to generate open edges, so this bias value adjusts their appearance.\n",0]
,["Pages/Page_55588727.html#Anchor_55588744_オープンエッジのw成分の乗算","Multiplying an Open Edge's w Component ",
"Multiplying an Open Edge's w Component\nYou can multiply a vertex's w component by using the open edge’s width and bias values. Use the glUniform1i function to set dmp_Silhouette.openEdgeWidthScaleByW and dmp_Silhouette.openEdgeDepthBiasScaleByW to GL_TRUE or GL_FALSE for each setting.\nThese reserved uniforms must be set by the application because they initially have undefined values.\nTable 9-3. Reserved Uniforms for Silhouette Shaders\n \t Reserved Uniform\n \t Type\n \t Value to Set\n \t dmp_Silhouette.width\n \t vec2\n \t Specifies the silhouette edge width using the following expression.\n (xscale_f,\nxscale_f * viewport.width / viewport.height)\nxscale_f is the factory to multiply by the normal vector's x component.\n \t dmp_Silhouette.scaleByW\n \t bool\n \t Specifies whether to apply a vertex's w component to silhouette edges.\n Specify GL_TRUE to apply the vertex's w component, and GL_FALSE otherwise.\n \t dmp_Silhouette.color\n \t vec4\n \t Specifies the silhouette edge color.\n \t dmp_Silhouette.frontFaceCCW\n \t bool\n \t Specifies how a polygon is determined to be front-facing or back-facing.\n Specify GL_TRUE to set CCW (counterclockwise is front-facing), and GL_FALSE to set CW (clockwise is front-facing).\n \t dmp_Silhouette.acceptEmptyTriangles\n \t bool\n \t Specifies whether to render open edges.\n Specify GL_TRUE to render open edges, and GL_FALSE otherwise.\n \t dmp_Silhouette.openEdgeColor\n \t vec4\n \t Specifies the open edge color.\n \t dmp_Silhouette.openEdgeWidth\n \t vec4\n \t Specifies the open edge width using the following expression.\n (viewport.width / silhouette.width,\nviewport.height / silhouette.width,\nviewport.width / viewport.height,\n2 / silhouette.width)\n \t dmp_Silhouette.openEdgeDepthBias\n \t float\n \t Specifies the open edge bias toward the viewpoint.\n A negative value indicates movement toward the viewpoint, and a positive value indicates movement away from it.\n \t dmp_Silhouette. openEdgeWidthScaleByW\n \t bool\n \t Specifies whether to multiply the open edge width by a vertex's w component.\n Specify GL_TRUE to multiply the open edge width by a vertex's w component, and GL_FALSE otherwise.\n \t dmp_Silhouette. openEdgeDepthBiasScaleByW\n \t bool\n \t Specifies whether to multiply a vertex's w component by the open edge bias toward the viewpoint.\n Specify GL_TRUE to multiply the open edge width by a vertex's w component, and GL_FALSE otherwise.\n ",0]
,["Pages/Page_55588727.html#PageId_55588753","9.3.4. Vertex Shader Settings ",
"\n9.3.4. Vertex Shader Settings\nA silhouette shader requires three items to render silhouette edges: vertex coordinates, vertex colors, and normal vectors. The vertex shader outputs the vertex coordinates, vertex color, and then the normal vector in that order, starting with the smallest output register number.\nThe output vertex attributes must be set using the vertex coordinates in the position attribute, the vertex color in the color attribute, and the normal vector in the generic attribute.\nTo output silhouette triangles, link the DMP_silhouetteTriangle.obj shader file, and use the following #pragma output_map statements in the vertex shader.\nCode 9-4. Sample Output Register Settings When a Silhouette Shader Is Used (in Shader Assembly)\n #pragma output_map ( position , o0 )\n#pragma output_map ( color , o1 )\n#pragma output_map ( generic , o2 )\nThe vertex shader runs a modelview transformation on the normal vectors input from the application. Normalized values must be output for the x and y components. In other words, the vertex shader must output a normal vector that is normalized as follows from the normal vector (nx, ny, nz) in the viewpoint coordinate system.\nThis is shown by the following shader assembly code. The normal vector input from the application is given by aNormal, and the normal vector output to the silhouette shader is given by vNormal.\nCode 9-5. Normalizing Input Normal Vectors for the Silhouette Shader (in Shader Assembly)\n mov TEMP_NORM, CONST_0\ndp3 TEMP_NORM.x, aNormal, MATRIX_ModelView[0]\ndp3 TEMP_NORM.y, aNormal, MATRIX_ModelView[1]\nmul TEMP, TEMP_NORM, TEMP_NORM\nadd TEMP, TEMP.x, TEMP.y\nrsq TEMP, TEMP.x\nmul vNormal, TEMP_NORM, TEMP\nTo output silhouette strips, use the same procedure as you would for silhouette triangles, but link the DMP_silhouetteStrip.obj shader file instead.\n",0]
,["Pages/Page_55588727.html#PageId_55588754","9.3.5. Input Vertex Data ",
"\n9.3.5. Input Vertex Data\nTo render silhouette edges with the silhouette shader, call the glDrawElements or glDrawArrays function, and specify GL_GEOMETRY_PRIMITIVE_DMP for the mode parameter. You must also disable culling by calling glDisable(GL_CULL_FACE) before rendering.\nWe also recommend using the vertex indices to input vertex data to the silhouette shader, in consideration of TWN characteristics. The following description assumes that you are using the vertex indices. If you are not using the vertex indices, you must order the vertex data according to the TWN vertex input rules.\n",0]
,["Pages/Page_55588727.html#PageId_55588755","9.3.6. Silhouette Triangle Indices ",
"\n9.3.6. Silhouette Triangle Indices\nSilhouette triangles are input to the shader, one TWN at a time, using six vertices per TWN.\nVertices are input in the following order.\n \tThe first vertex of the center triangle. \tThe second vertex of the center triangle. \tThe remaining vertex of the adjacent triangle that shares the edge created by the first and second vertices of the center triangle. \tThe third vertex of the center triangle. \tThe remaining vertex of the adjacent triangle that shares the edge created by the first and third vertices of the center triangle. \tThe remaining vertex of the adjacent triangle that shares the edge created by the second and third vertices of the center triangle.\nThe object in the following figure shows how to specify indices.\nFigure 9-5. How to Specify Indices With TWNs\nA setting of CCW is assumed for determining polygon facing.\nHaving selected the center triangles (1,4,3) and (3,4,6) so that they are front-facing, specify the indices as follows.\nTriangle (1,4,3): 1, 4, 2, 3, 0, 6.\nTriangle (3,4,6): 3, 4, 1, 6, 5, 7.\nSilhouette edges are not generated for a degenerate center triangle. No consideration has been given to triangles that have edges shared by three or more triangles.\n",0]
,["Pages/Page_55588727.html#PageId_55588756","9.3.7. Silhouette Strip Indices ",
"\n9.3.7. Silhouette Strip Indices\nSilhouette strips are input to the shader as consecutive TWNs, using six vertices for the first TWN and two vertices for each subsequent TWN. You can expect this to be at least twice as efficient as using silhouette triangles to render the same model.\nSilhouette strips can be used as input when the second and third vertices of a center triangle share an edge with another vertex to form the center triangle of the next TWN. In other words, the last adjacent triangle of a TWN is the center triangle of the next TWN.\nVertices are input in the following order.\n \tThe first six vertices are the same as those used for silhouette triangles. The second and third vertices of the center triangle and the last specified vertex represent the first, second, and third vertices of the next center triangle. \tThe remaining vertex of the adjacent triangle shares the edge created by the first and third vertices of the center triangle. \tThe remaining vertex of the adjacent triangle that shares the edge created by the second and third vertices of the center triangle. \tThe second and third vertices of the center triangle and the vertex specified in step 3 form vertices 1 through 3 of a new center triangle. Repeat steps 2 and 3.\nTo stop entering silhouette strips or to enter the next silhouette strip, give the third vertex of the center triangle as input before, the vertex specified in step 3.\nFigure 9-6. Sample Indices for Silhouette Strips\nGiven (3,8,7) as the final center triangle in the figure above,\nthe silhouette strip indices would be specified in the following order: 1, 4, 2, 3, 0, 8, 6, 7, 5, 7, 9.\nThe number in bold, 7, is the vertex used to indicate the end of the silhouette strip.\nTo continue entering silhouette strips after you have specified the end of one strip, you must first enter the six vertices that start a new TWN. If the first center triangle of the new silhouette strip faces in the opposite direction than the one specified using glFrontFace, enter its first vertex twice. In other words,\nif the silhouette strip in the previous example was initially back-facing, its indices would be specified in the following order: 1, 1, 4, 2, 3, 0, 8, 6, 7, 5, 7, 9.\nThe end of a silhouette strip is specified as a delimiter when inputting multiple silhouette strips. However, if the end of the last silhouette strip is not specified, further input triangles connected to the silhouette strip may cause duplicate silhouette edges to be rendered. The end of each input silhouette strip must be specified, when alpha blending is used on silhouettes.\nNote that a degenerate center triangle is considered to specify the end of a strip.\n",0]
,["Pages/Page_55588727.html#PageId_55588757","9.3.8. Open Edges ",
"\n9.3.8. Open Edges\nAn open edge has already been explained to be an edge of the center triangle that is not shared with any other triangle. Because it does not share an edge with any other triangle, its indices are specified by using the remaining vertex from the center triangle. You can visualize an open edge as an adjacent triangle that has been folded precisely over the center triangle.\nThe indices for the object in Figure 9-5 would be specified as follows, if vertex 0 did not exist.\nSilhouette triangle (1,4,3): 1, 4, 2, 3, 4, 6.\nSilhouette strip: 1, 4, 2, 3, 4, 6, 5, 6, 7.\nOpen edges must be configured differently than silhouette edges. For more information, see 9.3.3. Reserved Uniforms.\n",0]
,["Pages/Page_55588727.html#PageId_55588758","9.3.9. Generating Silhouette Edges ",
"\n9.3.9. Generating Silhouette Edges\nA silhouette edge is rendered by generating a new rectangular polygon on the edge of a front-facing center triangle and a back-facing adjacent triangle in a TWN. Two vertices are added along the normal vectors (n1 and n2) of the two vertices (1 and 2) shared by the center and adjacent triangle to generate a rectangular polygon (from two triangular polygons).\nFigure 9-7. Rectangular Polygon That Forms a Silhouette Edge\nThe coordinates (x', y', z', w') of the vertices to add are calculated by the following equation, where (x, y, z, and w) are the coordinates of a vertex on the center triangle, and (nx, ny, nz) represent a normal vector.\nx' = x + x_factor * nx * w_scale\ny' = y + y_factor * ny * w_scale\nz' = z\nw' = w\nThe reserved uniform for silhouette edge width sets the values applied to x_factor, y_factor, and w_scale.\n",0]
,["Pages/Page_55588727.html#PageId_55588759","9.4. Catmull-Clark Subdivision Shaders ",
"\n9.4. Catmull-Clark Subdivision Shaders\n",0]
,["Pages/Page_55588727.html#PageId_55588760","9.4.1. Subdivision Patches ",
"The Catmull-Clark subdivision shader uses quadrilateral polygons and their surrounding vertices to split groups of vertices into smooth polygons. Note: The word subdivision always indicates Catmull-Clark subdivision within this section.\n9.4.1. Subdivision Patches\nTo subdivide polygons, give the shader a set of polygons comprising only quadrilaterals (a Catmull-Clark subdivision patch, or subdivision patch for short). To subdivide polygons, give the shader a Loop subdivision patch (referred to later simply as a subdivision patch). A subdivision patch is made up of the target (center) quadrilateral and the group of quadrilaterals that share edges formed by that center quadrilateral's four vertices.\nFigure 9-8. Sample Catmull-Clark Subdivision Patch\nA subdivision patch can only be applied to a polygon model that is entirely made up of quadrilaterals.\nEach vertex in the central quadrilateral usually forms four edges, like vertices 5, 6, and 10. However, a vertex that forms three or five edges like vertex 9 is called an extraordinary point, and its edge count is called its valence. A subdivision patch can only have one extraordinary point. It must be in the central quad and have a valence from 3 through 12.\nIf a subdivision patch's central quad has an extraordinary point, you must start specifying indices from that point. Any other subdivision patches with central quads that have the same extraordinary point must be entered consecutively. Holes will appear in the mesh in some cases.\n",0]
,["Pages/Page_55588727.html#PageId_55588761","9.4.2. Shader Files ",
"\n9.4.2. Shader Files\nThe shader file to link with the vertex shader is determined by the number of vertex attributes output for fragment processing that are not required by the subdivision shader.\nSubdivision: DMP_subdivisionN.obj\nWhere N is the number of vertex attributes (1 through 6) that are not required by the subdivision shader.\n",0]
,["Pages/Page_55588727.html#PageId_55588762","9.4.3. Reserved Uniform ",
"\n9.4.3. Reserved Uniform\nSubdivision shaders have the following reserved uniforms.\n",0]
,["Pages/Page_55588727.html#Anchor_55588762_細分割レベル","Subdivision Level ",
"Subdivision Level\nUse the glUniform1f function to set the subdivision level (dmp_Subdivision.level), which controls how finely the shader subdivides polygons. Higher levels (larger numbers) indicate finer subdivision. At the smallest value of 0, a single vertex is added to the center of the subdivision patch, and the patch's original vertex coordinates are adjusted.\n",0]
,["Pages/Page_55588727.html#Anchor_55588762_クォータニオンの使用","Using Quaternions ",
"Using Quaternions\nThe subdivision shader generates new output vertices that have interpolated vertex attributes for their attributes that are not vertex coordinates. Because quaternions must be subdivided in a particular way, the shader must be notified when it is given quaternions.\nUse the glUniform1i function to either enable (GL_TRUE) or disable (GL_FALSE) quaternions (dmp_Subdivision.fragmentLightingEnabled).\nThese reserved uniforms must be set by the application because they initially have undefined values.\nTable 9-4. Reserved Uniforms for the Catmull-Clark Subdivision Shader\n \t Reserved Uniform\n \t Type\n \t Value to Set\n \t dmp_Subdivision.level\n \t float\n \t Specifies the subdivision level.\n 0 (The lowest subdivision level).\n1\n2 (The highest subdivision level).\n \t dmp_Subdivision.fragmentLightingEnabled\n \t bool\n \t Specifies whether to use quaternions required for fragment lighting.\n Specify GL_TRUE to use quaternions., and GL_FALSE otherwise.\n ",0]
,["Pages/Page_55588727.html#PageId_55588765","9.4.4. Vertex Shader Settings ",
"\n9.4.4. Vertex Shader Settings\nThe subdivision shader requires one vertex attribute: the vertex coordinates. The vertex shader outputs the vertex coordinates, starting with the smallest output register number.\nTwo output vertex attributes must be set: the position attribute of the vertex coordinates and one other attribute.\nIf the vertex shader outputs the vertex color in addition to the vertex coordinates required by the subdivision shader, link the DMP_subdivision1.obj shader file and use the following #pragma output_map statements in the vertex shader.\nCode 9-6. Sample Output Register Settings When the Catmull-Clark Subdivision Shader Is Used (in Shader Assembly)\n #pragma output_map ( position , o0 )\n#pragma output_map ( color , o1 )\nWhen quaternions are required for fragment lighting, they must be output to the register with the smallest number following the register that outputs the vertex coordinates.\nCode 9-7. Sample Output Register Settings When the Subdivision Shader Uses Quaternions (in Shader Assembly)\n #pragma output_map ( position , o0 )\n#pragma output_map ( quaternion , o1 )\n#pragma output_map ( color , o2 )\n",0]
,["Pages/Page_55588727.html#PageId_55588766","9.4.5. Input Vertex Data ",
"\n9.4.5. Input Vertex Data\nTo use Catmull-Clark subdivision, call the glDrawElements function, and specify GL_GEOMETRY_PRIMITIVE_DMP for the mode parameter. You cannot use the glDrawArrays function. Vertex indices must also be used through the vertex buffer.\n",0]
,["Pages/Page_55588727.html#PageId_55588767","9.4.6. Subdivision Patch Indices ",
"\n9.4.6. Subdivision Patch Indices\nThe number of vertices in a subdivision patch depends on the valence (from 3 through 12) of its extraordinary point. A subdivision patch does not have a fixed number of input vertices.\nEnter the size of the subdivision patch first. Specify the size of the subdivision patch as the number of vertices that it includes. To find this number, double the extraordinary point's valence, and add 8. A subdivision patch's size must be in the range from 14 through 32. It is 16 when the patch does not have an extraordinary point. Behavior is undefined for input patch sizes larger than 32.\nSpecify a subdivision patch's indices in the following order.\n \tThe four vertices in the central quad (following the order specified when glFrontFace was called, starting from the extraordinary point, if it exists). \tThe vertices around the central quad (following the order specified when glFrontFace was called).\nAssuming that polygons with a counterclockwise (CCW) winding are front-facing, the indices for the subdivision patch in Figure 9-8 would be specified in the following order.\nSubdivision Patch Indices: 18, 9, 5, 6, 10, 8, 4, 0, 1, 2, 3, 7, 11, 17, 16, 15, 14, 13, 12\nThe first number, 18, is the size of the subdivision patch.\nIt is followed by the subdivision patch for the central quad, with vertex 9.\n",0]
,["Pages/Page_55588727.html#PageId_55588768","9.5. Loop Subdivision Shader ",
"\n9.5. Loop Subdivision Shader\n",0]
,["Pages/Page_55588727.html#PageId_55588769","9.5.1. Subdivision Patches ",
"The Loop subdivision shader uses triangular polygons and their surrounding vertices to split groups of vertices into smooth polygons. The word subdivision always indicates Loop subdivision within this section.\n9.5.1. Subdivision Patches\nTo subdivide polygons, give the shader a Loop subdivision patch (referred to later simply as a subdivision patch). A subdivision patch comprises the target (center) triangle and the group of vertices that share edges with that triangle's three vertices.\nFigure 9-9. Sample Loop Subdivision Patch\nEach vertex in the center triangle forms a number of edges that is called its valence.\nIn Figure 9-9, vertices 0, 1, and 2 indicate the center triangle. Vertex 0 has a valence of 6, vertex 1 has a valence of 7, and vertex 2 has a valence of 6. A subdivision patch allows each vertex to have a valence between 3 and 12. The center triangle's three vertices must have a total valence of 29 or less.\nYou can increase the number of virtual vertices to include a vertex with a valence of 2 (a vertex that only shares an edge with other vertices in the center triangle) in a subdivision patch.\n",0]
,["Pages/Page_55588727.html#PageId_55588770","9.5.2. Shader Files ",
"\n9.5.2. Shader Files\nThe number of output registers configured to send non-valence vertex attributes to the subdivision shader determines which shader file to link with the vertex shader. There must be at least one output register because vertex coordinates are required output.\nSubdivision: DMP_subdivisionN.obj\nWhere N is the number of output registers (1 through 4) configured with vertex attributes, other than the valence.\n",0]
,["Pages/Page_55588727.html#PageId_55588771","9.5.3. Reserved Uniform ",
"\n9.5.3. Reserved Uniform\nThe Loop subdivision shader has the same reserved uniforms as the Catmull-Clark subdivision shader. Although new vertices are not added when the subdivision level is 0, the subdivision patch's original vertex coordinates are adjusted.\n",0]
,["Pages/Page_55588727.html#PageId_55588772","9.5.4. Vertex Shader Settings ",
"\n9.5.4. Vertex Shader Settings\nThe subdivision shader requires two attributes: the vertex coordinates and the valence. The vertex shader outputs these attributes, starting at the smallest output register number. The vertex coordinates are first, followed by any other vertex attributes that exist, and finally, followed by the valence.\nTwo output vertex attributes must be set: the vertex coordinates using the position attribute, and the valence using the generic attribute.\nIf the vertex shader outputs the vertex color in addition to the vertex coordinates required by the subdivision shader, link the DMP_loopSubdivision2.obj shader file (because the vertex coordinates are also included in the number of output registers) and use the following #pragma output_map statements in the vertex shader.\nCode 9-8. Sample Output Register Settings When the Loop Subdivision Shader Is Used (in Shader Assembly)\n #pragma output_map ( position , o0 )\n#pragma output_map ( color , o1 )\n#pragma output_map ( generic, o2 )\nWhen quaternions are required for fragment lighting, they must be output to the register with the smallest number following the register that outputs the vertex coordinates. Because the number of non-valence output registers is restricted to 4 or less, multiple attributes must be packed into a single register, when there are five or more vertex attributes other than the valence. Quaternions, however, cannot be packed with other vertex attributes.\nCode 9-9. Sample Output Register Settings When the Subdivision Shader Uses a Large Number of Vertex Attributes (in Shader Assembly)\n #pragma output_map ( position , o0 )\n#pragma output_map ( quaternion , o1 )\n#pragma output_map ( color , o2 )\n#pragma output_map ( texture0 , o3.xy )\n#pragma output_map ( texture1 , o3.zw )\n#pragma output_map ( generic , o4 )\n",0]
,["Pages/Page_55588727.html#PageId_55588773","9.5.5. Input Vertex Data ",
"\n9.5.5. Input Vertex Data\nTo use Loop subdivision, call the glDrawElements function and, for the mode parameter, pass GL_GEOMETRY_PRIMITIVE_DMP. You cannot use the glDrawArrays function. Vertex indices must also be used through the vertex buffer.\n",0]
,["Pages/Page_55588727.html#PageId_55588774","9.5.6. Subdivision Patch Indices ",
"\n9.5.6. Subdivision Patch Indices\nThe number of vertices in a subdivision patch depends on the total valence of the center triangle. A subdivision patch does not have a fixed number of input vertices.\nEnter the size of the subdivision patch first. Specify the size of the subdivision patch as three plus the total valence of the vertices that form the center triangle.\nSpecify a subdivision patch's indices in the following order.\n \tThe indices of the three vertices (v0, v1, and v2) that form the center triangle (following the order specified when glFrontFace was called). \tAll vertices that share an edge with v0 (in any order, although, the same order must also be used by any other subdivision patch that includes the same vertex). \tAll vertices that share an edge with v1 (in any order, although, the same order must also be used by any other subdivision patch that includes the same vertex). \tAll vertices that share an edge with v2 (in any order, although, the same order must also be used by any other subdivision patch that includes the same vertex). \tA fixed value of 12 and the center triangle's three vertices (12, v0, v1, and v2). \tA vertex (e00) that forms a triangle with v0 and v2 and is not in the center triangle. \tA vertex (e10) that forms a triangle with v0 and v1 and is not in the center triangle. \tA vertex (e20) that forms a triangle with v1 and v2 and is not in the center triangle. \tThe vertex that shares an edge with v0 and is next to e00, in counterclockwise order. \tThe vertex that shares an edge with v1 and is next to e10, in counterclockwise order. \tThe vertex that shares an edge with v2 and is next to e20, in counterclockwise order. \tThe vertex that shares an edge with v0 and is next to e10, in clockwise order. \tThe vertex that shares an edge with v1 and is next to e20, in clockwise order. \tThe vertex that shares an edge with v2 and is next to e00, in clockwise order.\nAssuming that polygons with a counterclockwise (CCW) winding are front-facing, the indices for the subdivision patch in Figure 9-8 would be specified in the following order.\nSubdivision Patch Indices:\n22, 0, 1, 2, 1, 2, 12, 3, 4, 5, 2, 0, 5, 6, 7, 8, 9, 0, 1, 9, 10, 11, 12,\n12, 0, 1, 2, 12, 5, 9, 3, 6, 10, 4, 11, 8\nThe number 22 on the first line is the size of the subdivision patch. The number 12 on the second line is a fixed value.\nWhen another subdivision patch uses v0 in its center triangle, the vertices that share an edge with it must be specified in the same order: (1, 2, 12, 3, 4, 5). The same applies to v1 and v2.\nAlthough some vertices will be specified more than once when specifying a subdivision patch, they are read from the cache after vertex processing, and do not actually impose a performance penalty.\n",0]
,["Pages/Page_55588727.html#PageId_55588775","9.6. Particle System Shaders ",
"\n9.6. Particle System Shaders\nParticle system shaders are used by particle systems that render a large number of point sprites (particles) along a Bézier curve.\nParticles are rendered along a Bézier curve that is defined by four control points input to the shader. Each control point is randomly placed within its own bounding box, changing the Bézier curve.\n",0]
,["Pages/Page_55588727.html#PageId_55588776","9.6.1. Shader Files ",
"A particle's color, size, angle of texture coordinate rotation, and other attributes are interpolated based upon its position on the Bézier curve.\n9.6.1. Shader Files\nLink the vertex shader to the correct particle system shader file, based on the features that you want to support.\nParticle system shader: DMP_particleSystem_X_X_X_X.obj\nWhere each X represents a value of 0 or 1 that controls a particle system feature. These features are, in order, particle time clamping, texture coordinate rotation, the use of RGBA components or the alpha component alone, and output of texture coordinate 2. A feature is not necessarily disabled by a value of 0, nor enabled by a value of 1. Refer to the following table to determine which shader file to link.\nTable 9-5. Shader Filenames and the Particle System Features They Support\n \t Filename\n \t Time Clamping\n \t Texture Coordinate Rotation\n \t RGBA Colors\n \t Texture coordinate 2.\n \t *_0_0_0_0.obj\n \t Yes\n \t Yes\n \t (Alpha only)\n \t No\n \t *_0_0_0_1.obj\n \t Yes\n \t Yes\n \t (Alpha only)\n \t Yes\n \t *_0_0_1_0.obj\n \t Yes\n \t Yes\n \t Yes\n \t No\n \t *_0_0_1_1.obj\n \t Yes\n \t Yes\n \t Yes\n \t Yes\n \t *_0_1_0_0.obj\n \t Yes\n \t None\n \t (Alpha only)\n \t No\n \t *_0_1_0_1.obj\n \t Yes\n \t None\n \t (Alpha only)\n \t Yes\n \t *_0_1_1_0.obj\n \t Yes\n \t None\n \t Yes\n \t No\n \t *_0_1_1_1.obj\n \t Yes\n \t None\n \t Yes\n \t Yes\n \t *_1_0_0_0.obj\n \t None\n \t Yes\n \t (Alpha only)\n \t No\n \t *_1_0_0_1.obj\n \t None\n \t Yes\n \t (Alpha only)\n \t Yes\n \t *_1_0_1_0.obj\n \t None\n \t Yes\n \t Yes\n \t No\n \t *_1_0_1_1.obj\n \t None\n \t Yes\n \t Yes\n \t Yes\n \t *_1_1_0_0.obj\n \t None\n \t None\n \t (Alpha only)\n \t No\n \t *_1_1_0_1.obj\n \t None\n \t None\n \t (Alpha only)\n \t Yes\n \t *_1_1_1_0.obj\n \t None\n \t None\n \t Yes\n \t No\n \t *_1_1_1_1.obj\n \t None\n \t None\n \t Yes\n \t Yes\n Each asterisk (*) in the table stands for DMP_particleSystem.\n",0]
,["Pages/Page_55588727.html#PageId_55588777","9.6.2. Reserved Uniform ",
"\n9.6.2. Reserved Uniform\nParticle system shaders have the following reserved uniforms.\n",0]
,["Pages/Page_55588727.html#Anchor_55588777_カラー","Color ",
"Color\nUse the glUniformMatrix4fv function to set the particle color (dmp_PartSys.color). A 4x4 matrix represents the particle color of the first, second, third, and fourth control point, using an RGBA value in each of the corresponding rows. This setting is valid only for a shader file that uses RGBA colors (DMP_particleSystem_X_X_1_X.obj).\nPerformance is worse when RGBA colors are used than when alpha components alone are used. If you are not using color components, we recommend that you link to a shader file that uses alpha components alone (DMP_particleSystem_X_X_0_X.obj).\n",0]
,["Pages/Page_55588727.html#Anchor_55588777_アスペクト","Aspect ",
"Aspect\nUse the glUniformMatrix4fv function to set the particle aspect (dmp_PartSys.aspect) with a 4x4 matrix. Each row corresponds to a control point (from 1 through 4) and configures the particle size, texture coordinate rotation, texture coordinate scaling, and alpha component.\n",0]
,["Pages/Page_55588727.html#Anchor_55588777_サイズ","Size ",
"Size\nSet the particle size (the first aspect column) to a value of 1.0 or greater.\nUse the glUniform2fv function to set the minimum and maximum particle size (dmp_PartSys.pointSize). Use the glUniform2fv function to set the reciprocal of the viewport's width and height (dmp_PartSys.viewport) because particle rendering does not account for the screen size. If distance attenuation is being applied to the particle size, call the glUniform3fv function to set the distance attenuation factor (dmp_PartSys.distanceAttenuation), and specify the attenuation coefficients that calculate the attenuated size, as shown in the following equation.\nderived_size is the size with distance attenuation applied, size is the original size, and d is the distance from the viewpoint.\n",0]
,["Pages/Page_55588727.html#Anchor_55588777_テクスチャ座標","Texture Coordinates ",
"Texture Coordinates\nYou can set the rotation (the second aspect column) and scaling (the third aspect column) of texture coordinates at each control point.\nThe particle system outputs two texture coordinates: 0 and 2. Both texture coordinates support rotations, but only texture coordinate 2 supports scaling. The linked shader file enables or disables these settings, in addition to texture coordinate 2 output.\nRotations are specified in radians. Clockwise rotations are specified by positive values.\nTexture coordinate 0 is output with the particle's lower-left, lower-right, upper-left, and upper-right corners at (0,0), (1,0), (0,1), and (1,1) respectively. Texture coordinate 2 is output with the particle's lower-left, lower-right, upper-left, and upper-right corners at (-1,-1), (1,-1), (-1,1), and (1,1) respectively.\nGiven a rotation of A and a scaling value of R, texture coordinate 0 is calculated as follows.\nLower-left = ( 0.5×(1.0 + (-cosA + sinA)), 0.5×(1.0 + (-cosA - sinA)) )\nLower-right = ( 0.5×(1.0 + (cosA + sinA)), 0.5×(1.0 + (-cosA + sinA)) )\nUpper-left = ( 0.5×(1.0 + (-cosA - sinA)), 0.5×(1.0 + (cosA - sinA)) )\nUpper-right = ( 0.5×(1.0 + (cosA - sinA)), 0.5×(1.0 + (cosA + sinA)) )\nTexture coordinate 2 is calculated as follows.\nLower-left = ( R×(-cosA + sinA), R×(-cosA - sinA) )\nLower-right = ( R×(cosA + sinA), R×(-cosA + sinA) )\nUpper-left = ( R×(-cosA - sinA), R×(cosA - sinA) )\nUpper-right = ( R×(cosA - sinA), R×(cosA + sinA) )\n",0]
,["Pages/Page_55588727.html#Anchor_55588777_アルファ成分","Alpha Component ",
"Alpha Component\nSet the particle's alpha component (the fourth aspect column) as a value in the range from 0.0 to 1.0. This setting is used with the linked shader files that only use the alpha component (DMP_particleSystem_X_X_0_X.obj).\n",0]
,["Pages/Page_55588727.html#Anchor_55588777_発生数","Emission Count ",
"Emission Count\nUse the glUniform1fv function to set the maximum particle emission count (dmp_PartSys.countMax). Set this value to one less than the actual number of particles you want to emit. You must set a value of 0.0 or greater. However, because the shader program implementation limits the maximum number of emitted particles to 255, no more than 255 particles will be emitted, even when this reserved uniform is set to a value greater than 256.\n",0]
,["Pages/Page_55588727.html#Anchor_55588777_実行時間と速度","Execution Time and Speed ",
"Execution Time and Speed\nA particle system has a concept of time. Use the glUniform1fv function to set the particle system time (dmp_PartSys.time) to the current time. This current time is randomly converted into each particle's execution time, during which the particle travels from the first control point to the fourth control point. A particle is emitted at the first control point when its execution time is 0.0, and reaches the fourth control point when its execution time is 1.0.\nIf you link to a shader file that clamps the execution time (DMP_particleSystem_0_X_X_X.obj), particles with an execution time of 1.0 or greater cease to be rendered. Consequently, particles will cease to be emitted at some point, if the application simply lets time pass without resetting the execution time.\nIf you link to a shader file that does not clamp the execution time (DMP_particleSystem_1_X_X_X.obj), the execution time loops between 0.0 and 1.0. In other words, particles that reach the fourth control point are re-emitted from the first control point.\nUse the glUniform1fv function to set the particle speed (dmp_PartSys.speed).\n",0]
,["Pages/Page_55588727.html#Anchor_55588777_ランダム値","Random Values ",
"Random Values\nThe position of control points within their bounding boxes and the execution time of particles are determined using a function that generates pseudorandom numbers. The application can specify a random seed and coefficient to use in this random function.\nThe implementation of the random function is similar to the following algorithm for a pseudorandom number generator.\nUse the glUniform4fv function to set the random seed (dmp_PartSys.randSeed) with an array of values (the x, y, and z components of the Bézier curve and the particle execution time, in that order) corresponding to X0 in Equation 9-8.\nFor the random function's coefficients (dmp_PartSys.randomCore), use the glUniform4fv function to specify the values a, b, m, and 1/m in Equation 9-8.\nThese reserved uniforms must be set by the application because they initially have undefined values.\nTable 9-6. Reserved Uniforms for Particle System Shaders\n \t Reserved Uniform\n \t Type\n \t Value to Set\n \t dmp_PartSys.color\n \t mat4\n \t The color at each control point.\n (R, G, B, A) * 4vec\n Each component has a value in the range from 0.0 through 1.0.\n \t dmp_PartSys.aspect\n \t mat4\n \t The aspect of each control point.\n (particle_size, rotation_angle, scale, alpha) * 4vec\n Where particle_size is 1.0 or greater and alpha is in the range from 0.0 through 1.0.\n \t dmp_PartSys.time\n \t float\n \t The current particle system time.\n \t dmp_PartSys.speed\n \t float\n \t The speed of particle movement.\n 0.0 or greater\n \t dmp_PartSys.countMax\n \t float\n \t A number that is one less than the maximum number of particles to emit.\n 0.0 or greater\n \t dmp_PartSys.randSeed\n \t vec4\n \t The random seed for each of the random functions.\n (Multiplicand with the Bézier curve's x component, multiplicand with the Bézier curve's y component, multiplicand with the Bézier curve's z component, multiplicand with the particle execution time).\n \t dmp_PartSys.randomCore\n \t vec4\n \t The random function's coefficients.\n (a, b, m, 1/m)\n \t dmp_PartSys.distanceAttenuation\n \t vec3\n \t The distance attenuation factor.\n \t dmp_PartSys.viewport\n \t vec2\n \t The viewport using the following equation.\n (1 / viewport.width, 1 / viewport.height)\n \t dmp_PartSys.pointSize\n \t vec2\n \t The minimum and maximum particle size.\n Each has a value of 0.0 or greater.\n ",0]
,["Pages/Page_55588727.html#PageId_55588786","9.6.3. Vertex Shader Settings ",
"\n9.6.3. Vertex Shader Settings\nA particle system shader requires a 4×4 matrix that contains the following values converted into clip coordinates: a single control point's vertex coordinates and the radius for the x, y, and z components of its bounding box (centered on said vertex coordinates). Starting at the smallest output register number, the vertex shader outputs the vertex coordinates followed by the first, second, third, and fourth rows of the conversion matrix, in that order.\nTwo output vertex attributes must be set: the vertex coordinates using the position attribute and the converted matrix using the generic attribute.\nCode 9-10. Sample Output Register Settings When a Particle System Shader Is Used (in Shader Assembly)\n #pragma output_map ( position , o0 )\n#pragma output_map ( generic , o1 )\n#pragma output_map ( generic , o2 )\n#pragma output_map ( generic , o3 )\n#pragma output_map ( generic , o4 )\nThe following equation shows the conversion into clip coordinates. The radii for the bounding box's x, y, and z components are: Rx, Ry, and Rz respectively. The projection matrix is Mproj and the modelview matrix is Mmodelview.\nThis is shown by the following shader assembly code. aBoundingBox is a vector with the x, y, and z components of the radius input by the application, and vBoundingBox1 through vBoundingBox4 represent the matrix output to the particle system shader. The bounding box's radius is input as attributes in this sample code, but because the particle system shader requires data for only four vertices, you can use an implementation that sets all of this along with the vertex coordinates in uniforms.\nCode 9-11. Bounding Box Radius and Clip Coordinate Conversion (in Shader Assembly)\n mov TEMP_BOX[0], CONST_0\nmov TEMP_BOX[1], CONST_0\nmov TEMP_BOX[2], CONST_0\nmov TEMP_BOX[3], CONST_0\nmov TEMP_BOX[0].x, aBoundingBox.x\nmov TEMP_BOX[1].y, aBoundingBox.y\nmov TEMP_BOX[2].z, aBoundingBox.z\nm4x4 TEMP_MAT, MATRIX_Project, MATRIX_ModelView\nm4x4 TEMP_MAT, TEMP_MAT, TEMP_BOX\nmov vBoundingBox1, TEMP_MAT[0]\nmov vBoundingBox2, TEMP_MAT[1]\nmov vBoundingBox3, TEMP_MAT[2]\nmov vBoundingBox4, TEMP_MAT[3]\n",0]
,["Pages/Page_55588727.html#PageId_55588787","9.6.4. Input Vertex Data ",
"\n9.6.4. Input Vertex Data\nCalls from the glDrawArrays function are not supported. When using a particle system shader, call the glDrawElements function, and specify GL_GEOMETRY_PRIMITIVE_DMP for the mode parameter.\nCONFIDENTIAL\n",0]
,["Pages/Page_55588788.html","10. Rasterization ",
"10. Rasterization\nEven primitives created by geometry shaders, such as the point and line shader, are all ultimately handled as triangles. The generated triangles are culled, clipped, converted into window coordinates, and finally rasterized into a collection of fragments. Unlike in OpenGL ES 2.0, the scissor test is performed at this stage. Further processing affects the generated fragments.\nAll processing after rasterization is implemented as a fixed pipeline, so there are fixed vertex attributes that can be assigned to fragments. The following are the major vertex attributes.\n\tWindow coordinates \tDepth values \tTexture coordinates and partial differential values \tQuaternions \tView vectors ",0]
,["Pages/Page_55588788.html#PageId_55588789","10.1. Culling ",
" \tVertex colors (rasterized from absolute values).\n10.1. Culling\n",0]
,["Pages/Page_55588788.html#PageId_55588790","10.1.1. Determining the Front and Back Faces ",
"The order in which vertices are specified for the generated triangles (polygons) determines which face, front or back, is facing the viewer. Culling is a feature that determines whether to rasterize a polygon based on whether it is back-facing.\n10.1.1. Determining the Front and Back Faces\nA polygon's front and back faces are determined by the order in which its triangles' vertices are specified in window coordinates. Either a clockwise (CW) or a counterclockwise (CCW) winding can be considered to be front-facing. You can specify this using the glFrontFace function.\nCode 10-1. Definition of the glFrontFace Function\n void glFrontFace(GLenum mode);\nFor mode, specify GL_CW for clockwise winding or GL_CCW for counterclockwise winding. A counterclockwise winding (GL_CCW) is configured by default.\n",0]
,["Pages/Page_55588788.html#PageId_55588791","10.1.2. How to Use ",
"\n10.1.2. How to Use\nCulling is used in the same way it is used in OpenGL\n",0]
,["Pages/Page_55588788.html#Anchor_55588791_カリングの有効・無効","Enabling or Disabling Culling ",
"Enabling or Disabling Culling\nTo enable or disable culling, call glEnable or glDisable, respectively, and specify GL_CULL_FACE for cap. Call the glIsEnabled function, and specify GL_CULL_FACE for cap to get the current setting. Culling is disabled by default.\n",0]
,["Pages/Page_55588788.html#Anchor_55588791_カリング面の指定","Specifying the Culled Face ",
"Specifying the Culled Face\nUse the glCullFace function to specify which face is not rasterized (the culled face).\nCode 10-2. Definition of the glCullFace Function\n void glCullFace(GLenum mode);\nYou can choose from the following mode values to specify the culled face.\nTable 10-1. Specifying the Culled Face\n \t Value\n \t Culled Face\n \t GL_FRONT\n \t Front\n \t GL_BACK (default)\n \t Back\n \t GL_FRONT_AND_BACK\n \t Both\n",0]
,["Pages/Page_55588788.html#PageId_55588794","10.2. Clipping ",
"\n10.2. Clipping\nClipping is a feature that clips (removes) primitives from the regions (clip volumes) at which the view volume intersects the half-spaces defined by the specified clipping planes. 3DS clipping features correspond to the clipping features in OpenGL ES 1.1, but they are all controlled using reserved uniforms. The clipping implementation also generates new vertices and triangles for each clipped triangle, splitting it into multiple triangles.\n",0]
,["Pages/Page_55588788.html#PageId_55588795","10.2.1. Reserved Uniform ",
"Because GPU vertex processing converts coordinates using 24-bit floating-point numbers, clipping may not be performed correctly at the far clipping plane, if the ratio of the near clipping plane to the far clipping plane is large. Either configure the clip volume to keep the ratio of the near clipping plane to the far clipping plane small, or avoid placing polygons near the far clipping plane, whenever possible.\n10.2.1. Reserved Uniform\nThe following reserved uniforms are used for clipping.\n",0]
,["Pages/Page_55588788.html#Anchor_55588795_クリッピングの有効・無効","Enabling or Disabling Clipping ",
"Enabling or Disabling Clipping\nTo enable clipping, set the reserved uniform dmp_FragOperation.enableClippingPlane to GL_TRUE, using the glUniform1i function. This setting is disabled (set to GL_FALSE) by default.\n",0]
,["Pages/Page_55588788.html#Anchor_55588795_クリップ平面","Clipping Plane ",
"Clipping Plane\nTo specify the clipping plane, set four coefficients in the reserved uniform dmp_FragOperation.clippingPlane using the glUniform4f function. If p1, p2, p3, and p4 are the four coefficients, the clip volume is a collection of points that satisfy the following equation.\nBecause these coefficients must be defined in the clip coordinate system, you must specify values that have been through a modelview transformation and perspective projection for the clipping plane used by the OpenGL ES standard. Unlike the OpenGL ES specifications, the 3DS system clips Z coordinates between 0 and -Wc. OpenGL ES-compatible matrices cannot be used unchanged for perspective projections. All coefficients are set to 0.0 by default.\n Note:\n For cautions related to using OpenGL ES-compatible matrices with projection transformations, see 8.4. Notes for the Clip Coordinate System.\nTable 10-2. Reserved Uniforms Used for Clipping\n \t Reserved Uniform\n \t Type\n \t Value to Set\n \t dmp_FragOperation.enableClippingPlane\n \t bool\n \t Enables or disables clipping.\n GL_TRUE\n GL_FALSE (default)\n \t dmp_FragOperation.clippingPlane\n \t vec4\n \t Specifies the four coefficients for the clipping plane.\n This is (0.0, 0.0, 0.0, 0.0) by default.\n",0]
,["Pages/Page_55588788.html#PageId_55588798","10.3. Transformation to Window Coordinates ",
"\n10.3. Transformation to Window Coordinates\nA polygon model's vertices are transformed from object coordinates into window coordinates through the following four steps.\n\tModelview transformation:\nTransforms object coordinates into eye coordinates.\n \tProjection transformation:\nTransforms eye coordinates into clip coordinates. Note that the converted values (Zc) are between -Wc and 0.\n \tPerspective division:\nTransforms clip coordinates into normalized device coordinates using w values.\n \tViewport transformation:\nTransforms normalized device coordinates into window coordinates following the viewport settings.\nThe vertex shader (and geometry shader) must output vertex attributes in clip coordinates. As a result, the vertex shader generally performs modelview and projection transformations internally. Perspective division and viewport transformation determine the position of each generated triangle in the display region. These triangles are converted into fragments during rasterization, and are then used by processes such as fragment lighting.\n",0]
,["Pages/Page_55588788.html#PageId_55588799","10.3.1. Configuring the Viewport ",
"The following figure illustrates the process of transforming coordinates.\nFigure 10-1. Process of Transforming Vertices From Object Coordinates Into Window Coordinates\n10.3.1. Configuring the Viewport\nNormalized device coordinates are transformed into window coordinates by the following equation.\nXw, Yw, and Zw are window coordinates.\nXd, Yd, and Zd are normalized device coordinates.\npx and py are the viewport's width and height.\nox and oy are the viewport's center point.\nn and f are the depth values of the near and far planes, respectively, in clip space.\nYou can use the glDepthRangef function to set the values applied to n and f in this equation.\nCode 10-3. Definition of the glDepthRangef Function\n void glDepthRangef(GLclampf zNear, GLclampf zFar);\nThe values set for zNear and zFar are clamped between 0.0 and 1.0. By default, zNear is 0.0 and zFar is 1.0.\nPx, Py, Ox, and Oy can all be calculated from the viewport settings. The glViewport function configures the viewport.\nCode 10-4. glViewport Function\n void glViewport(GLint x, GLint y, GLsizei width, GLsizei height);\nFor x and y specify the coordinates of the viewport's starting point (lower-left corner). A GL_INVALID_VALUE error is generated if a negative value is specified. If the specified value is not a multiple of 4, processing efficiency drops (to half for an even number and to one third for an odd number). In this case, extend the viewport so that it is a multiple of 4, adjust the perspective projection matrix so that it renders correctly on the extended viewport, and then apply the scissor test to avoid rendering unnecessary regions.\nFor width and height, specify the width and height of the viewport. A GL_INVALID_VALUE error is generated if a negative value is specified. The maximum width and height are both 1024.\n",0]
,["Pages/Page_55588788.html#PageId_55588800","10.3.1.1. A Viewport Larger Than 1023×1016 Causes Incorrect Rendering ",
"The value of width is applied to px, height is applied to py, (x + width) / 2 is applied to ox, and (y + height) / 2 is applied to oy.\n10.3.1.1. A Viewport Larger Than 1023×1016 Causes Incorrect Rendering\nA hardware bug prevents images from being rendered properly if the following conditions are met when the glViewport function sets the viewport.\n\tWhen width is greater than 1023, entire polygons will not be rendered if they contain pixels whose window x-coordinate is greater than the x parameter to the glViewport function by at least 1023 (taking the left side of the window to have an x-coordinate of 0). \tWhen height is greater than 1016, the GPU will hang if any polygon contains pixels whose window y-coordinate is greater than the y parameter to the glViewport function by at least 1016 (taking the bottom of the window to have a y-coordinate of 0).\nTo work around this hardware bug, avoid rendering pixels at the problematic coordinates by setting the viewport’s width and height to values that are not greater than 1023 and 1016, respectively.\nWhen using the render-to-texture technique with a 1024×1024 texture, you must only render to a 1023×1016 region, and you must adjust texture coordinates so that only a 1023×1016 texture region is valid for use.\nIf you want to render to the entire 1024×1024 region, you must keep the viewport’s width from exceeding 1023 and its height from exceeding 1016, while changing the viewport’s offset to render the texture in sections. For example, the bug described in this section does not occur if you split rendering into the following four function calls: glViewport(0, 0, 512, 512), glViewport(512, 0, 512, 512), glViewport(0, 512, 512, 512), and glViewport(512, 512, 512, 512).\nYou cannot work around this bug by using the scissor test to prevent the pixels at the problematic coordinates from being rendered.\n",0]
,["Pages/Page_55588788.html#PageId_55588801","10.3.2. Polygon Offset ",
"\n10.3.2. Polygon Offset\nPolygon offset is a feature that adds an offset to depth values when polygons are rasterized and converted into fragments. This resolves the situation in which a lack of depth value resolution prevents the fragments' front-to-back order from being determined, as for overlapping coplanar polygons.\n",0]
,["Pages/Page_55588788.html#Anchor_55588801_ポリゴンオフセットの有効・無効","Enabling or Disabling Polygon Offset ",
"Enabling or Disabling Polygon Offset\nTo enable or disable polygon offset, call glEnable or glDisable, respectively, and specify GL_POLYGON_OFFSET_FILL for cap. Call the glIsEnabled function, and specify GL_POLYGON_OFFSET_FILL for cap to get the current setting. Polygon offset is disabled by default.\n",0]
,["Pages/Page_55588788.html#Anchor_55588801_オフセット値の指定","Specifying Offset Values ",
"Specifying Offset Values\nYou can use the glPolygonOffset function to specify the offset given to depth values when polygon offset is enabled.\nCode 10-5. Definition of the glPolygonOffset Function\n void glPolygonOffset(GLfloat factor, GLfloat units);\nOpenGL uses the values of factor and units to determine the offset, but the 3DS system uses only units. The value of factor is set, but it is irrelevant to the offset.\nThe offset value is set with the product of units and the minimum (fixed) value at which a difference in the depth value appears in window coordinates. Because the z-value for vertex coordinates is implemented as a 24-bit floating-point number after vertex processing, a units value that is not a multiple of 128 has no effect when a polygon’s z-value is close to 1.0. To get a definite effect, set units to a multiple of 128.\nDepth values are written to the depth buffer after an offset is added to them.\n",0]
,["Pages/Page_55588788.html#PageId_55588804","10.3.3. W-Buffer ",
"\n10.3.3. W-Buffer\nThe w-buffer is a feature that calculates depth values in window coordinates without using a perspective projection. You can control the w-buffer with the following reserved uniforms. Settings made by the glDepthRangef function are disabled when the w-buffer is enabled.\n",0]
,["Pages/Page_55588788.html#Anchor_55588804_デプス値に対するスケール","Depth Value Scale ",
"Depth Value Scale\nDepth values are calculated by the following equation, when the w-buffer is enabled.\nZw = -scalew × Zc\nZw is the depth value in window coordinates, Zc is the z-value in clip coordinates, and scalew is the scale value. This scale value is a floating-point number set by the reserved uniform dmp_FragOperation.wScale. As long as it is not 0.0, the w-buffer is enabled. Set the scale so that Zw stays in the range from 0.0 through 1.0.\nTable 10-3. Reserved Uniforms Used for the W-Buffer\n \t Reserved Uniform\n \t Type\n \t Setting Value\n \t dmp_FragOperation.wScale\n \t float\n \t Specifies the depth value scale.\n 0.0 by default.\n ",0]
,["Pages/Page_55588788.html#Anchor_55588804_ポリゴンオフセットが有効になっている場合","When Polygon Offset Is Enabled ",
"When Polygon Offset Is Enabled\nIf both the w-buffer and polygon offset features are enabled, the value obtained by multiplying Wc (the w value in clip coordinates) times units, as specified by the glPolygonOffset function, is applied to as the polygon offset value.\n",0]
,["Pages/Page_55588788.html#PageId_55588807","10.4. Scissor Test ",
"\n10.4. Scissor Test\n",0]
,["Pages/Page_55588788.html#PageId_55588808","10.4.1. How to Use ",
"The scissor test is a feature that rejects fragments outside the specified range of window coordinates to reduce the number of fragments handled by further processing.\n10.4.1. How to Use\nAlthough its position in the pipeline is different, the process has the same specifications as the OpenGL scissor test.\n",0]
,["Pages/Page_55588788.html#Anchor_55588808_シザーテストの有効・無効","Enabling or Disabling the Scissor Test ",
"Enabling or Disabling the Scissor Test\nTo enable or disable the scissor test, call glEnable or glDisable respectively, and specify GL_SCISSOR_TEST for cap. Call the glIsEnabled function and specify GL_SCISSOR_TEST for cap to get the current setting. The scissor test is disabled by default. When it is disabled, fragments are not rejected.\n",0]
,["Pages/Page_55588788.html#Anchor_55588808_シザーボックスの指定","Specifying the Scissor Box ",
"Specifying the Scissor Box\nUse the glScissor function to specify the range (scissor box) through which fragments are allowed to pass.\nCode 10-6. Definition of the glScissor Function\n void glScissor(GLint x, GLint y, GLsizei width, GLsizei height);\nx and y specify the starting point (lower-left corner) of the scissor box in window coordinates. width and height specify the width and height of the scissor box. A GL_INVALID_VALUE error is generated if width or height is 0. Because there are no default values, always specify the scissor box when the scissor test is enabled.\nThe scissor box includes the fragment at the starting coordinates, but not the fragment with an x-coordinate of (x + width) or a y-coordinate of (y + height).\n",0]
,["Pages/Page_55588788.html#PageId_55588811","10.5. Rasterization ",
"\n10.5. Rasterization\nPolygon rasterization (fragment generation) by the PICA graphics core adheres to the following rules.\n\tThe center coordinates of the pixel (x + 0.5, y + 0.5; x and y are integers) must lie inside the polygon. \tAccording to the lower-left rule, a fragment is generated if the lower or left edge of the polygon passes through the center coordinate of a pixel. If the upper or right edge passes through the center coordinate, no fragment is generated.\n Note: This assumes that the x-axis is negative toward the left and the y-axis is negative toward the lower edge.\nFigure 10-2 shows how the lower-left rule is applied based on the rasterization results of two polygons. The two polygons are the one defined by the three vertices (5.5, 0.5), (5.5, 5.5), and (0.5, 5.5), and the one defined by the three vertices (5.5, 0.5), (0.5, 0.5), and (0.5, 5.5). The former is shown in red, while the latter is shown in blue.\nFigure 10-2. Example Demonstrating Rasterization Rules (the Lower-Left Rule)\nThe rasterization rules are applied as follows.\nOn the boundary of the two polygons—because the left edge of the red polygon passes through the center of the pixels, these pixels are colored red.\nBecause the blue polygon passes through the center of pixels with a center coordinate x value of 0.5, these pixels are colored blue.\nBecause the right side of the red polygon passes through the center of the pixels with a center coordinate x value of 5.5, these pixels are not colored.\nBecause the bottom of the blue polygon passes through the center of the pixels with a center coordinate y value of 0.5, these pixels are colored blue.\nAnd, because the top of the red polygon passes through the center of the pixels with a center coordinate y value of 5.5, these pixels are not colored.\nCONFIDENTIAL\n",0]
,["Pages/Page_55588812.html","11. Texture Processing ",
"11. Texture Processing\n",0]
,["Pages/Page_55588812.html#PageId_55588813","11.1. Texture Unit ",
"The 3DS system can perform texture processing operations equivalent to those in OpenGL ES 2.0, but there are several CTR-specific restrictions.\n11.1. Texture Unit\nFour texture units (TEXTURE0 through TEXTURE3) are installed on the 3DS system, but each one can handle different types of textures. A rasterizer can independently output up to three sets of texture coordinates to the texture units. To output texture coordinates to all four texture units, TEXTURE2 or TEXTURE3 must share the same texture coordinates with other texture units.\nTable 11-1. Textures Supported by the Texture Units\n \t Texture\nUnit\n \t 2D\nTextures\n \t Cube Map\nTextures\n \t Shadow\nTextures\n \t Projection\nTextures\n \t Procedural\nTextures\n \t TEXTURE0\n \t ✓\n \t ✓\n \t ✓\n \t ✓\n \t\n \t TEXTURE1\n \t ✓\n \t\n \t\n \t\n \t\n \t TEXTURE2\n \t ✓\n \t\n \t\n \t\n \t\n \t TEXTURE3\n \t\n \t\n \t\n \t\n \t ✓\n ",0]
,["Pages/Page_55588812.html#PageId_55588814","11.1.1. Texture Coordinate Input ",
"One- and three-dimensional textures are not supported. Cube map textures, shadow textures, projection textures, and other textures that require the w component can only be processed by TEXTURE0. TEXTURE3 is a unit used exclusively for procedural textures. GL_MAX_COMBINED_TEXTURE_IMAGE_UNITS defines the number of installed texture units to be 4.\n11.1.1. Texture Coordinate Input\nOnly texture unit 0 can accept the w component as input; the other texture units only allow two input components, u and v. A projection texture is simply a 2D texture with its w component enabled, but care must be taken with the coordinates output by the vertex shader because the generated UV coordinates are divided by the w component.\nTo send texture coordinates from the vertex shader, map output registers to the attribute names texture0, texture0w, texture1, and texture2. texture0w must be output for cube map textures, shadow textures, projection textures, and anything else that requires the w component. If it is not output, texture coordinate 0's output is undefined. texture0w output is ignored for textures that do not require the w component.\nTable 11-2. Attribute Names and Corresponding Texture Coordinates\n \t Attribute Name\n \t Attributes Sent From the Vertex Shader\n \t texture0\n \t UV components of texture coordinate 0.\n \t texture0w\n \t W component of texture coordinate 0.\n \t texture1\n \t UV components of texture coordinate 1.\n \t texture2\n \t UV components of texture coordinate 2.\n ",0]
,["Pages/Page_55588812.html#PageId_55588815","11.1.1.1. Texture Coordinate Precision ",
"11.1.1.1. Texture Coordinate Precision\nWithin a texture unit, texture coordinates are represented as 16-bit values combining an integer and decimal component. The number of decimal bits decreases as the absolute value of the integer component increases.\nThe accuracy of texture sampling depends on the decimal bit precision. A texture can be sampled, optimally, if there are enough decimal bits to represent its width and height in texels. Allocate another six decimal bits for bilinear filtering.\n",0]
,["Pages/Page_55588812.html#PageId_55588816","11.1.2. How to Use ",
"\n11.1.2. How to Use\nTo enable or disable a texture unit in OpenGL, you would specify an argument such as TEXTURE_2D to glEnable and glDisable. To do this on the 3DS system, however, set the reserved uniform dmp_Texture[i].samplerType (where i is the texture unit number). Behavior is undefined if you call glEnable or glDisable using GL_TEXTURE_2D as an argument.\nTo disable a texture unit, call glUniform1i, and pass GL_FALSE to set a reserved uniform value. Each texture unit is enabled by different reserved uniform settings.\nTable 11-3. Settings for the Reserved Uniforms dmp_Texture[i].samplerType\n \t Reserved Uniform\n \t Value to Set\n \t Supported Textures\n \t dmp_Texture[0].samplerType\n \t GL_FALSE\n \t Disabled (default).\n \t GL_TEXTURE_2D\n \t 2D textures.\n \t GL_TEXTURE_CUBE_MAP\n \t Cube map textures.\n \t GL_TEXTURE_SHADOW_2D_DMP\n \t Shadow textures.\n \t GL_TEXTURE_SHADOW_CUBE_DMP\n \t Cube map shadow textures.\n \t GL_TEXTURE_PROJECTION_DMP\n \t Projection textures.\n \t dmp_Texture[1].samplerType\n \t GL_FALSE\n \t Disabled (default).\n \t GL_TEXTURE_2D\n \t 2D textures.\n \t dmp_Texture[2].samplerType\n \t GL_FALSE\n \t Disabled (default).\n \t GL_TEXTURE_2D\n \t 2D textures.\n \t dmp_Texture[3].samplerType\n \t GL_FALSE\n \t Disabled (default).\n \t GL_TEXTURE_PROCEDURAL_DMP\n \t Procedural textures.\n Texture units 0 and 1 both have fixed texture coordinate input: texture coordinates 0 and 1, respectively. Texture units 2 and 3 must have their texture coordinate input specified by the reserved uniform dmp_Texture[i].texcoord (where i is the texture unit number 2 or 3). Each texture unit allows different reserved uniform settings. When all four texture units are used, texture unit 2 or 3 must share its input texture coordinates with another texture unit.\nColor values input to the texture combiner are undefined when the texture combiner accesses a disabled texture unit.\nTable 11-4.Settings for the Reserved Uniforms dmp_Texture[i].texcoord\n \t Reserved Uniform\n \t Value to Set\n \t Texture Coordinate Used as Input\n \t dmp_Texture[2].texcoord\n \t GL_TEXTURE1\n \t Texture coordinate 1.\n \t GL_TEXTURE2\n \t Texture coordinate 2 (default).\n \t dmp_Texture[3].texcoord\n \t GL_TEXTURE0\n \t Texture coordinate 0 (default).\n \t GL_TEXTURE1\n \t Texture coordinate 1.\n \t GL_TEXTURE2\n \t Texture coordinate 2.\n Unless they are configured using reserved uniforms, texture unit settings apply to the texture unit specified by the glActiveTexture function.\nCode 11-1. Definition of the glActiveTexture Function\n void glActiveTexture(GLenum texture);\nFor texture you can specify GL_TEXTURE0, GL_TEXTURE1, or GL_TEXTURE2. A GL_INVALID_ENUM error is generated when some other value, or GL_TEXTURE3 (texture unit 3), is specified.\nReserved uniforms are used for all procedural texture settings.\n",0]
,["Pages/Page_55588812.html#PageId_55588817","11.1.3. Specifying Textures to Use ",
"\n11.1.3. Specifying Textures to Use\nTo specify the texture to use with a texture unit, first specify the texture unit using the glActiveTexture function, and then specify the texture object using the glBindTexture function.\nIf you want to use a different texture for each texture unit, call the glActiveTexture and glBindTexture functions, as shown in the following sample code.\nCode 11-2. How to Specify a Texture for Each Texture Unit\n // Texture Unit0\nglActiveTexture(GL_TEXTURE0);\nglBindTexture(GL_TEXTURE_2D, imageTexID);\n// Texture Unit1\nglActiveTexture(GL_TEXTURE1);\nglBindTexture(GL_TEXTURE_2D, bumpTexID);\n",0]
,["Pages/Page_55588812.html#PageId_55588818","11.1.4. Texture Parameters ",
"\n11.1.4. Texture Parameters\nUse the glTexParameter* functions to add parameters to a texture, such as the texture wrapping mode and filters.\nCode 11-3. Definition of the glTexParameter* Functions\n void glTexParameterf(GLenum target, GLenum pname, GLfloat param);\nvoid glTexParameterfv(GLenum target, GLenum pname, const GLfloat* params);\nvoid glTexParameteri(GLenum target, GLenum pname, GLint param);\nvoid glTexParameteriv(GLenum target, GLenum pname, const GLint* params);\nThe glTexParameteri function adds parameters passed in as integers and the glTexParameterf function adds parameters passed in as floating-point numbers. Functions with names that end in v are used to add parameters that must be passed as vectors (arrays).\nFor target, specify the value specified in functions such as glTexImage2D. pname is the name of the parameter to add and param is the parameter value. Unique parameters have been added for 3DS.\nThe 3DS system gets up to eight texels for a single fragment when filtering.\nTable 11-5. Parameters Added for pname\n \t pname\n \t int / float / vector\n \t Added Parameter\n \t GL_TEXTURE_WRAP_S\n \t int\n \t Wrapping mode in the S direction (Table 11-6).\n \t GL_TEXTURE_WRAP_T\n \t int\n \t Wrapping mode in the S direction (Table 11-6).\n \t GL_TEXTURE_MIN_FILTER\n \t int\n \t Minification filter (Table 11-7).\n \t GL_TEXTURE_MAG_FILTER\n \t int\n \t Magnification filter (Table 11-8).\n \t GL_TEXTURE_BORDER_COLOR\n \t vec4(float)\n \t The border color to use when the wrapping mode is GL_CLAMP_TO_BORDER (each component is between 0.0 and 1.0).\n \t GL_TEXTURE_LOD_BIAS\n \t float\n \t LOD bias (-16 through 16.0. The default is 0.0.) Default value: 0.0\n \t GL_TEXTURE_MIN_LOD\n \t int\n \t Minimum LOD (-1000 by default.)\n \t GL_GENERATE_MIPMAP\n \t int(bool)\n \t Automatic mipmap texture generation (GL_FALSE by default). For more information, see 11.1.4.1. Automatic Generation of Mipmap Textures.\n The following values specify the wrapping mode in the S and T directions.\nTable 11-6. Specifying the Wrapping Mode in the S and T Directions\n \t param\n \t Description\n \t GL_REPEAT\n \t Repeat (default).\n \t GL_MIRRORED_REPEAT\n \t Flip and repeat.\n \t GL_CLAMP_TO_EDGE\n \t Use the color at the edge of the texture image for texture coordinates that are not in the range from 0.0 through 1.0.\n \t GL_CLAMP_TO_BORDER\n \t Use the border color for texture coordinates that are not in the range from 0.0 through 1.0.\n The following values specify the filters to use when rendering texture images that have been scaled down.\nTable 11-7. Specifying Minification Filters\n \t param\n \t Description\n \t GL_NEAREST\n \t Use the color of the nearest texel (default).\n \t GL_LINEAR\n \t Use bilinear sampling (the average of four samples) to determine the color.\n \t GL_NEAREST_MIPMAP_NEAREST\n \t Select the nearest mipmap texture, and then use the color of the nearest texel.\n \t GL_NEAREST_MIPMAP_LINEAR\n \t Select two mipmap texture levels, and then interpolate between the nearest colors at each level.\n \t GL_LINEAR_MIPMAP_NEAREST\n \t Select the nearest mipmap texture, and then use bilinear sampling to determine the color.\n \t GL_LINEAR_MIPMAP_LINEAR\n \t Select two mipmap texture levels, and then interpolate between the bilinearly sampled colors at each level.\n The following values specify the filters to use when rendering texture images that have been scaled up.\nTable 11-8. Specifying Magnification Filters\n \t param\n \t Description\n \t GL_NEAREST\n \t Use the color of the nearest texel (default).\n \t GL_LINEAR\n \t Use bilinear sampling (the average of four samples) to determine the color.\n ",0]
,["Pages/Page_55588812.html#PageId_55588819","11.1.4.1. Automatic Generation of Mipmap Textures ",
"11.1.4.1. Automatic Generation of Mipmap Textures\nWhen the texture parameter GL_GENERATE_MIPMAP is GL_TRUE and a value of -2 or less is passed to the level parameter in the glTexImage2D, glCopyTexImage2D, or glCopyTexSubImage2D function, mipmap textures are automatically generated for all but the lowest mipmap level. However, texture formats (combinations of the format and type parameters) apply the following restrictions to the minimum width and height of mipmap textures that are automatically generated.\nTable 11-9. Minimum Width and Height of Automatically Generated Mipmap Textures\n \t Format\n \t format\n \t type\n \t Minimum Width and Height\n \t RGBA4\n \t GL_RGBA\nGL_RGBA_NATIVE_DMP\n \t GL_UNSIGNED_SHORT_4_4_4_4\n \t 64\n \t RGBA5551\n \t GL_RGBA\nGL_RGBA_NATIVE_DMP\n \t GL_UNSIGNED_SHORT_5_5_5_1\n \t 64\n \t RGBA8\n \t GL_RGBA\nGL_RGBA_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t 32\n \t RGB565\n \t GL_RGB\nGL_RGB_NATIVE_DMP\n \t GL_UNSIGNED_SHORT_5_6_5\n \t 64\n \t RGB8\n \t GL_RGB\nGL_RGB_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t 32\n Even if two textures have the same width and height, the range of values that can be specified for level differ, if the texture formats have different minimum values. A GL_INVALID_OPERATION error is generated when you specify the automatic generation of a mipmap texture smaller than the minimum size. For a texture that is 128×128 texels, for example, you can specify a level of -2 or -3 when the format is RGB8, but you can only specify -2 for level when the format is RGB565.\nWhen enabled, automatically generated mipmap textures take priority. The mipmap texture data loaded with a texture image is ignored.\nWith automatic generation enabled, the level parameter of the glCopyTexImage2D and glCopyTexSubImage2D functions must have the same value as was passed to the level parameter of the glTexImage2D function when the texture was loaded. A GL_INVALID_OPERATION error is generated if these values differ. However, even if a nonzero value is passed to level, data is copied to the texture with the lowest level, and the mipmap texture is not replaced.\nWith automatic generation disabled, the level parameter of the glCopyTexImage2D and glCopyTexSubImage2D functions must be 0, regardless of the value that was passed to the level parameter of the glTexImage2D function when the texture was loaded. A GL_INVALID_OPERATION error is generated if a nonzero value is specified.\n",0]
,["Pages/Page_55588812.html#PageId_55588820","11.1.4.2. Cautions for Specifying GL_NEAREST as the Filter ",
"\n11.1.4.2. Cautions for Specifying GL_NEAREST as the Filter\nIf you set the GL_TEXTURE_MIN_FILTER and GL_TEXTURE_MAG_FILTER texture parameters to GL_NEAREST and use a texture image with vertical or horizontal lines (including color boundaries that form straight lines), when the texture is applied to polygons that place those stripes horizontally or perpendicular to the screen's scan lines, the straight lines may appear as broken lines.\nThis phenomenon is caused by the computational precision of texture coordinate interpolation within polygons. Consider a row of fragments that is horizontal or perpendicular to the scan lines. If each fragment samples texels in a single vertical or horizontal line, the straight lines on the texture image are rendered unchanged on the polygons. If each of these fragments sample around the boundary between two adjacent rows of texels, however, each fragment will sample a different texel row due to texture coordinate errors. As a result, the straight line on the texture image is rendered as a broken line on the polygons. You can work around this issue by adjusting the texture coordinates and rendering area so that each fragment samples the texel centers, just as when a rectangular polygon with texture coordinates of 0 and 1 at its edges is rendered using the same size as a texture.\n",0]
,["Pages/Page_55588812.html#PageId_55588821","11.1.4.3. Cautions for Specifying GL_XXX_MIPMAP_LINEAR as a Minification Filter ",
"\n11.1.4.3. Cautions for Specifying GL_XXX_MIPMAP_LINEAR as a Minification Filter\nTrilinear filtering is enabled when GL_XXX_MIPMAP_LINEAR is set for the texture parameter GL_TEXTURE_MIN_FILTER. This interpolates colors from two mipmap texture levels and then renders with the resulting color. However, this interpolation is subject to errors caused by computational precision. For example, even if two colors with the same component values are processed, a different color could be rendered due to these minor errors.\nColors are interpolated when, as a result of LOD calculations, they must be obtained from two mipmap texture levels. Colors are not interpolated when they are obtained from only one mipmap texture level. A texture is slightly darker where colors are interpolated than where colors are not. This causes differences in hue to be rendered as edges along mipmap level boundaries.\nYou can mitigate this effect by using fixed component values in a texel color. For example, a texture in the GL_RGB format has a fixed alpha component of 1.0, but even this fixed component value decreases slightly for texels that have been interpolated by trilinear filtering. Nonetheless, this component retains its value of 1.0 in texels that have not been interpolated. To correct a texture color, multiply it by the change in alpha value caused by interpolation, and then add this product to the texture color.\nTexture combiners can make this correction, as shown in the following sample code. Below is an example of the setting.\nCode 11-4. Compensating for Trilinear Filtering\n glUniform1i(glGetUniformLocation(program, \"dmp_TexEnv[0].combineRgb\"),\n GL_MULT_ADD_DMP);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[0].operandRgb\"),\n GL_SRC_COLOR, GL_ONE_MINUS_SRC_ALPHA, GL_SRC_COLOR);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[0].srcRgb\"),\n GL_TEXTURE0, GL_TEXTURE0, GL_TEXTURE0);\nWherever colors are not interpolated, these settings result in a product of 0 and output the original texture color. Wherever colors are interpolated, these settings multiply the texture color and the difference in the alpha component, add this product to the texture color, and finally output the corrected color. You can only apply these settings to the first texture combiner (combiner 0) because they specify a texture color for every input source.\nIf you are using a texture format without fixed component values, such as GL_RGBA, prepare a separate texture with fixed component values and use that as a multitexture to perform the correction. This texture must have the same size, number of mipmap levels, and UV input values as the original texture. We recommend using an ETC1-compressed texture for data size and cache efficiency reasons.\n",0]
,["Pages/Page_55588812.html#PageId_55588822","11.1.4.4. Getting Texture Level (Mipmap) Parameters ",
"\n11.1.4.4. Getting Texture Level (Mipmap) Parameters\nYou can use the following functions to get parameters for each mipmap level of textures bound to texture units that are currently active. However, you cannot get information about procedural textures because the texture unit for procedural textures (GL_TEXTURE3) cannot be specified to the glActiveTexture function.\nCode 11-5. Getting Texture Level Parameters\n void glGetTexLevelParameterfv(GLenum target, GLint level, GLenum pname,\n GLfloat *params);\nvoid glGetTexLevelParameteriv(GLenum target, GLint level, GLenum pname,\n GLint *params);\nBoth functions get the same values, even though they are saved as different types.\nSpecify the type of texture in target. You can specify the following values.\nTable 11-10. Specifying the Types of Textures to Get Texture Level Parameters For\n \t target Value\n \t Type of Texture\n \t GL_TEXTURE_2D\n \t 2D texture (including shadow and gas textures).\n \t GL_TEXTURE_CUBE_MAP_POSITIVE_X\n \t Cube-map texture (in the positive X direction).\n \t GL_TEXTURE_CUBE_MAP_NEGATIVE_X\n \t Cube-map texture (in the negative X direction).\n \t GL_TEXTURE_CUBE_MAP_POSITIVE_Y\n \t Cube-map texture (in the positive Y direction).\n \t GL_TEXTURE_CUBE_MAP_NEGATIVE_Y\n \t Cube-map texture (in the negative Y direction).\n \t GL_TEXTURE_CUBE_MAP_POSITIVE_Z\n \t Cube-map texture (in the positive Z direction).\n \t GL_TEXTURE_CUBE_MAP_NEGATIVE_Z\n \t Cube-map texture (in the negative Z direction).\n Specify the mipmap level to get in level. When level is 0, you will get the parameter for the texture at the lowest mipmap level (the largest texture). You can get the next level and the one after that by specifying 1 and 2, respectively, for level.\nSpecify the type of parameter to get in pname. The following table shows how the value of pname corresponds to the parameter stored in params.\nTable 11-11. Specifying Texture Level Parameters to Get\n \t pname Value\n \t Parameters\n \t GL_TEXTURE_WIDTH\n \t Texture width (in texels).\n \t GL_TEXTURE_HEIGHT\n \t Texture height (in texels).\n \t GL_TEXTURE_DEPTH\n \t Unsupported; has a fixed value of 0.\n \t GL_TEXTURE_INTERNAL_FORMAT\n \t A texture’s internal format.\n \t GL_TEXTURE_BORDER\n \t Unsupported; has a fixed value of 0.\n \t GL_TEXTURE_RED_SIZE\n \t Number of bits in the red component (per texel).\n \t GL_TEXTURE_GREEN_SIZE\n \t Number of bits in the green component (per texel).\n \t GL_TEXTURE_BLUE_SIZE\n \t Number of bits in the blue component (per texel).\n \t GL_TEXTURE_ALPHA_SIZE\n \t Number of bits in the alpha component (per texel).\n \t GL_TEXTURE_LUMINANCE_SIZE\n \t Number of bits in the luminance component (per texel).\n \t GL_TEXTURE_INTENSITY_SIZE\n \t Number of bits in the intensity component (per texel). Only for shadow textures.\n \t GL_TEXTURE_DEPTH_SIZE\n \t Number of bits in the depth component (per texel). Only for shadow textures.\n \t GL_TEXTURE_DENSITY1_SIZE_DMP\n \t Number of bits (per texel) of density information (density value 1) that does not account for intersections. Only for gas textures.\n \t GL_TEXTURE_DENSITY2_SIZE_DMP\n \t Number of bits (per texel) of density information (density value 2) that does account for intersections. Only for gas textures.\n \t GL_TEXTURE_COMPRESSED\n \t Whether a texture is compressed.\n Specify GL_TRUE to compress the texture, and GL_FALSE otherwise. Compressed texture\nGL_FALSE: Uncompressed texture\n \t GL_TEXTURE_COMPRESSED_IMAGE_SIZE\n \t Number of bytes in the texture at the mipmap level specified by level. Only for compressed textures. Specifying an uncompressed texture results in a GL_INVALID_OPERATION error.\n The following table shows how the number of bits in each component for a single texel corresponds to the internal format of textures obtained when GL_TEXTURE_INTERNAL_FORMAT is specified for pname.\nTable 11-12. Internal Formats and the Number of Bits in Components Making Up Each Texel\n \t Internal Format\n \t Red\n \t Green\n \t Blue\n \t Alpha\n \t Luminance\n \t Intensity\n \t Depth\n \t Density 1\n \t Density 2\n \t GL_RGBA4\n \t 4\n \t 4\n \t 4\n \t 4\n \t\n \t\n \t\n \t\n \t\n \t GL_RGB5_A1\n \t 5\n \t 5\n \t 5\n \t 1\n \t\n \t\n \t\n \t\n \t\n \t GL_RGBA\n \t 8\n \t 8\n \t 8\n \t 8\n \t\n \t\n \t\n \t\n \t\n \t GL_RGB565\n \t 5\n \t 6\n \t 5\n \t\n \t\n \t\n \t\n \t\n \t\n \t GL_RGB\n \t 8\n \t 8\n \t 8\n \t\n \t\n \t\n \t\n \t\n \t\n \t GL_ALPHA\n \t\n \t\n \t\n \t 8\n \t\n \t\n \t\n \t\n \t\n \t GL_ALPHA4_EXT\n \t\n \t\n \t\n \t 4\n \t\n \t\n \t\n \t\n \t\n \t GL_LUMINANCE\n \t\n \t\n \t\n \t\n \t 8\n \t\n \t\n \t\n \t\n \t GL_LUMINANCE4_ EXT\n \t\n \t\n \t\n \t\n \t 4\n \t\n \t\n \t\n \t\n \t GL_LUMINANCE_ALPHA\n \t\n \t\n \t\n \t 8\n \t 8\n \t\n \t\n \t\n \t\n \t GL_LUMINANCE4_ALPHA4_EXT\n \t\n \t\n \t\n \t 4\n \t 4\n \t\n \t\n \t\n \t\n \t GL_SHADOW_DMP\n \t\n \t\n \t\n \t\n \t\n \t 8\n \t 24\n \t\n \t\n \t GL_GAS_DMP\n \t\n \t\n \t\n \t\n \t\n \t\n \t\n \t 16\n \t 16\n \t GL_HILO8_DMP\n \t 8\n \t 8\n \t\n \t\n \t\n \t\n \t\n \t\n \t\n \t GL_ETC1_RGB8_NATIVE_DMP\n \t 8\n \t 8\n \t 8\n \t\n \t\n \t\n \t\n \t\n \t\n \t GL_ETC1_ALPHA_RGB8_A4_NATIVE_DMP\n \t 8\n \t 8\n \t 8\n \t 4\n \t\n \t\n \t\n \t\n \t\n A GL_INVALID_ENUM error is generated when an invalid value is specified for target and pname. A GL_INVALID_VALUE error is generated when the mipmap level specified for level has not been loaded.\n",0]
,["Pages/Page_55588812.html#PageId_55588823","11.1.5. Effects of Texture Settings on Performance ",
"\n11.1.5. Effects of Texture Settings on Performance\nGraphics performance is affected by texture format and size, and by various settings. The following is a list of common tendencies.\n\tCompressed textures are processed the fastest, followed by formats that use a small number of bytes per texel. \tProcessing speed increases as the size decreases. \tContention for memory access causes processing to slow as the number of textures used simultaneously increases. \tThe following pairs of minification filter settings are each processed at the same speed:GL_NEAREST and GL_LINEARGL_NEAREST_MIPMAP_NEAREST and GL_LINEAR_MIPMAP_NEARESTGL_NEAREST_MIPMAP_LINEAR and GL_LINEAR_MIPMAP_LINEAR. However, because GL_NEAREST(_XXX) fetches 1 texel per pixel and GL_LINEAR(_XXX) fetches 4 texels per pixel, GL_NEAREST(_XXX) uses less memory. \tIt is faster to apply scaled-down textures using mipmaps. Even when mipmaps are used, however, the processing load depends on the filter settings. GL_*_MIPMAP_LINEAR may entail approximately twice the processing load of GL_*_MIPMAP_NEAREST. \tAlthough the GL_NEAREST and GL_LINEAR magnification filters have nearly the same performance, GL_NEAREST is slightly faster in some cases. \tGas and shadow textures cannot use mipmaps and are processed more slowly than normal textures. \tShadow textures make use of special filters for shadows, so there is a processing load comparable to trilinear filtering (when GL_*_MIPMAP_LINEAR is set as the minification filter for normal textures). The processing load is around twice that for a normal texture (excluding trilinear filtering). \tThere are no differences caused by conditions for setting procedural textures. They are faster to process than normal 2D textures. \tWhen multiple textures are used, they are processed more quickly if they are all placed in VRAM-A or VRAM-B, rather than split between the two. \tTextures created to match the upward direction of the framebuffer can sometimes process faster than textures created to match the upward direction of the rendering results. This is because the direction in which fragments are generated matches the direction in which textures are loaded, improving the texture cache hit rate. Flipping a texture vertically has no effect on performance. Fragments are processed horizontally in 8x8-pixel units, whereas textures are loaded in 8x4-texel units. Note also that the short sides of the LCD screen on the 3DS are used as the top and bottom.\n",0]
,["Pages/Page_55588812.html#PageId_55588824","11.1.6. Texture Cache ",
"\n11.1.6. Texture Cache\nThere is a 256-byte L1 texture cache and an 8-KB L2 texture cache. Within a cache, only compressed textures (in the ETC format) are handled unchanged. All other textures (including those in the alpha ETC format) are converted into a 32-bit format.\nThere is a separate L1 cache for each texture unit, but the L2 cache is shared by all texture units.\nThere is a 5-cycle penalty for missing the L1 cache and, instead, getting data from the L2 cache. There is an additional penalty of approximately 30 cycles for missing the L2 cache and instead getting data from VRAM. However, the hardware is implemented to prefetch texels to hide these delays.\n",0]
,["Pages/Page_55588812.html#PageId_55588825","11.2. Combiners ",
"\n11.2. Combiners\nThere are six (texture) combiners installed on the 3DS system. They can combine the primary and secondary colors for fragment lighting, in addition to the colors output by texture units, such as the texture color, vertex color, and constant color. If you have experience developing applications for the Nintendo GameCube and Wii, you can more easily understand this effect if you imagine it as combining color and alpha values by using the TEV.\nOpenGL ES 1.1 uses TexEnv for combiner settings, but the 3DS system uses reserved uniforms. The following table shows the reserved uniforms that correspond to TexEnv parameters.\nTable 11-13. Reserved Uniforms Corresponding to TexEnv Parameters in OpenGL ES 1.1\n \t TexEnv\n \t Reserved Uniform\n \t Setting\n \t COMBINE_RGB\n \t dmp_TexEnv[i].combineRgb\n \t Color combiner function.\n \t COMBINE_ALPHA\n \t dmp_TexEnv[i].combineAlpha\n \t Alpha combiner function.\n \t SRCn_RGB\n \t dmp_TexEnv[i].srcRgb\n \t Color source.\n \t SRCn_ALPHA\n \t dmp_TexEnv[i].srcAlpha\n \t Alpha source.\n \t OPERANDn_RGB\n \t dmp_TexEnv[i].operandRgb\n \t Color operands.\n \t OPERANDn_ALPHA\n \t dmp_TexEnv[i].operandAlpha\n \t Alpha operands.\n \t RGB_SCALE\n \t dmp_TexEnv[i].scaleRgb\n \t Color scaling value.\n \t ALPHA_SCALE\n \t dmp_TexEnv[i].scaleAlpha\n \t Alpha scaling value.\n \t TEXTURE_ENV_COLOR\n \t dmp_TexEnv[i].constRgba\n \t Constant color (with an alpha component).\n Where n is the source (from 0 through 2) and i is the combiner number (from 0 through 5).\nEach combiner processes its three source inputs as operands in its combiner function, multiplies the calculated result by a scaling value, and clamps the value between 0.0 and 1.0 before outputting it. Also, 0.0 to 1.0 are clamped before input is computed in the combiner, and the clamped value is a result of a rasterized absolute value for the vertex color (primary color).\nColor operations are performed on all components (red, green, and blue) using a single setting, and alpha operations are performed on the alpha component using a separate setting. Color values input to the texture combiner are undefined when the texture combiner accesses a disabled texture unit.\nFigure 11-1. Combiner Options\nA combiner accepts three input sources. Each input source must be one of the following types (a single type can be used for more than one input).\n\tTexture color output by a texture unit. \tConstant color. \tPrimary color. \tPrimary color for fragment lighting. \tSecondary color for fragment lighting. \tOutput from the previous combiner stage (except for combiner 0). ",0]
,["Pages/Page_55588812.html#PageId_55588826","11.2.1. Reserved Uniforms for Combiner Functions ",
" \tOutput from the previous combiner buffer stage (except for combiner 0).\n Figure 11-2. Combiner Input Sources\n11.2.1. Reserved Uniforms for Combiner Functions\nThere are two reserved uniforms for combiner functions: dmp_TexEnv[i].combineRgb and dmp_TexEnv[i].combineAlpha. Use the glUniform1i function to set a value at the reserved uniform location obtained by glGetUniformLocation.\nThe following values are used to set the reserved uniforms for combiner functions. The same values can be set for both dmp_TexEnv[i].combineRgb and dmp_TexEnv[i].combineAlpha.\nIf GL_DOT3_RGBA is set for the combiner, it must have the same combiner function (GL_DOT3_RGBA) for both the color (combineRgb) and alpha (combineAlpha) components.\nTable 11-14. Reserved Uniform Values That Can Be Set for Combiner Functions\n \t Value\n \t Combiner Function\n \t GL_REPLACE\n \t Src0 (default)\n \t GL_MODULATE\n \t Src0 * Src1\n \t GL_ADD\n \t Src0 + Src1\n \t GL_ADD_SIGNED\n \t Src0 + Src1 - 0.5\n \t GL_INTERPOLATE\n \t Src0 * Src2 + Src1 * (1 - Src2)\n \t GL_SUBTRACT\n \t Src0 - Src1\n \t GL_DOT3_RGB\n \t 4 * ((Src0_Red - 0.5) * (Src1_Red - 0.5) + (Src0_Green - 0.5) * (Src1_Green - 0.5) + (Src0_Blue - 0.5) * (Src1_Blue - 0.5))\n \t GL_DOT3_RGBA\n \t 4 * ((Src0_Red - 0.5) * (Src1_Red - 0.5) + (Src0_Green - 0.5) * (Src1_Green - 0.5) + (Src0_Blue - 0.5) * (Src1_Blue - 0.5))\n \t GL_ADD_MULT_DMP\n \t (Src0 + Src1) * Src2 Note: The sum is clamped between 0.0 and 1.0 before it is multiplied.\n \t GL_MULT_ADD_DMP\n \t (Src0 * Src1) + Src2\n ",0]
,["Pages/Page_55588812.html#PageId_55588827","11.2.2. Reserved Uniforms for the Input Sources ",
"\n11.2.2. Reserved Uniforms for the Input Sources\nThere are two reserved uniforms for the input sources: dmp_TexEnv[i].srcRgb and dmp_TexEnv[i].srcAlpha.\nUse the glUniform3i function to set values at the reserved uniform location obtained by the glGetUniformLocation function. Source 0 is first, followed by source 1 and source 2, respectively.\nThe following values are used to set the reserved uniforms for the input sources. The same values can be set for dmp_TexEnv[0].srcRgb and dmp_TexEnv[0].srcAlpha, and for dmp_TexEnv[i].srcRgb and dmp_TexEnv[i].srcAlpha.\n Warning:\n Every combiner except for combiner 0 must have GL_CONSTANT, GL_PREVIOUS, or GL_PREVIOUS_BUFFER_DMP specified as one of its three input sources.\nTable 11-15. Reserved Uniform Values That Can Be Set for Input Sources\n \t Value\n \t Input Source\n \t GL_TEXTURE0\n \t Texture color from texture unit 0.\n \t GL_TEXTURE1\n \t Texture color from texture unit 1.\n \t GL_TEXTURE2\n \t Texture color from texture unit 2.\n \t GL_TEXTURE3\n \t Texture color from texture unit 3.\n \t GL_CONSTANT\n \t Constant color. (This is the default for combiner 0 and is set by dmp_TexEnv[i].constRgba.)\n \t GL_PRIMARY_COLOR\n \t Primary color (vertex color).\n \t GL_PREVIOUS\n \t Output from the previous combiner stage. (Cannot be set for combiner 0. Default for combiners other than 0.)\n \t GL_PREVIOUS_BUFFER_DMP\n \t Output from the previous combiner buffer stage. (Cannot be set for combiner 0.)\n \t GL_FRAGMENT_PRIMARY_COLOR_DMP\n \t Primary color for fragment lighting.\n \t GL_FRAGMENT_SECONDARY_COLOR_DMP\n \t Secondary color for fragment lighting.\n ",0]
,["Pages/Page_55588812.html#PageId_55588828","11.2.3. Reserved Uniforms for Operands ",
"\n11.2.3. Reserved Uniforms for Operands\nThere are two reserved uniforms for operands: dmp_TexEnv[i].operandRgb and dmp_TexEnv[i].operandAlpha.\nUse the glUniform3i function to set values at the reserved uniform location obtained by the glGetUniformLocation function. Source 0 is first, followed by source 1 and source 2, respectively.\nThe following values are used to set the reserved uniforms for operands.\nTable 11-16. Reserved Uniform Values That Can Be Set for Operands\n \t Value\n \t Operands\n \t GL_SRC_COLOR\n \t Color. (Cannot be set for operandAlpha. operandAlpha is the default.)\n \t GL_ONE_MINUS_SRC_COLOR\n \t 1 - Color. (Cannot be set for operandAlpha.)\n \t GL_SRC_ALPHA\n \t Alpha. (operandAlpha is the default.)\n \t GL_ONE_MINUS_SRC_ALPHA\n \t 1 - Alpha\n \t GL_SRC_R_DMP\n \t Color_Red\n \t GL_ONE_MINUS_SRC_R_DMP\n \t 1 - Color_Red\n \t GL_SRC_G_DMP\n \t Color_Green\n \t GL_ONE_MINUS_SRC_G_DMP\n \t 1 - Color_Green\n \t GL_SRC_B_DMP\n \t Color_Blue\n \t GL_ONE_MINUS_SRC_B_DMP\n \t 1 - Color_Blue\n ",0]
,["Pages/Page_55588812.html#PageId_55588829","11.2.4. Reserved Uniforms for Scaling Values ",
"\n11.2.4. Reserved Uniforms for Scaling Values\nThere are two reserved uniforms for scaling values: dmp_TexEnv[i].scaleRgb and dmp_TexEnv[i].scaleAlpha.\nUse the glUniform1f function to set a value at the reserved uniform location obtained by the glGetUniformLocation function\nThe following values are used to set the reserved uniforms for scaling.\nTable 11-17. Reserved Uniform Values That Can Be Set for Scaling\n \t Value\n \t Scaling\n \t 1.0\n \t Unchanged combiner output (default).\n \t 2.0\n \t Double the combiner output (clamp between 0.0 and 1.0).\n \t 4.0\n \t Quadruple the combiner output (clamp between 0.0 and 1.0).\n ",0]
,["Pages/Page_55588812.html#PageId_55588830","11.2.5. Reserved Uniforms for Constant Colors ",
"\n11.2.5. Reserved Uniforms for Constant Colors\nThere is one uniform for constant colors: dmp_TexEnv[i].constRgba. Use the glUniform4f function to set values at the reserved uniform location obtained by the glGetUniformLocation function. The first value is R, followed by G, B, and A, respectively.\nBy default, 0.0 is set for the R, G, B, and A values in the reserved uniforms for constant colors.\n",0]
,["Pages/Page_55588812.html#PageId_55588831","11.2.6. Sample Combiner Settings ",
"\n11.2.6. Sample Combiner Settings\nIn the following example, combiner 2 references the primary color and outputs (renders) it unchanged.\nTo output only the primary color from the combiners without being affected by anything else, combiner 2's input source 0 is set to the primary color (GL_PRIMARY_COLOR), operand 0 is set to the unchanged input source color (GL_SRC_COLOR), the combiner function is configured to output input source 0 unchanged (GL_REPLACE), and the scale is set to 1.0. With these settings, the combiner outputs only the primary color and combiners 0 and 1 do not affect the output results.\nThis is illustrated by the following connection diagram.\nFigure 11-3. Sample Combiner Settings 1\nThe following sample code shows how to make these settings in a program.\nCode 11-6. Code for Sample Combiner Settings 1\n glUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[2].srcRgb\"),\n GL_PRIMARY_COLOR, GL_PREVIOUS, GL_PREVIOUS);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[2].srcAlpha\"),\n GL_PRIMARY_COLOR, GL_PREVIOUS, GL_PREVIOUS);\nglUniform1i(glGetUniformLocation(program, \"dmp_TexEnv[2].combineRgb\"),\n GL_REPLACE);\nglUniform1i(glGetUniformLocation(program, \"dmp_TexEnv[2].combineAlpha\"),\n GL_REPLACE);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[2].operandRgb\"),\n GL_SRC_COLOR, GL_SRC_COLOR, GL_SRC_COLOR);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[2].operandAlpha\"),\n GL_SRC_ALPHA, GL_SRC_ALPHA, GL_SRC_ALPHA);\nglUniform1f(glGetUniformLocation(program, \"dmp_TexEnv[2].scaleRgb\"), 1.0);\nglUniform1f(glGetUniformLocation(program, \"dmp_TexEnv[2].scaleAlpha\"), 1.0);\nIn a more complex example, combiner 1 is configured to add output from texture 0 and texture 1, and combiner 2 is configured to multiply the result by the primary color.\nFigure 11-4. Sample Combiner Settings 2\nCode 11-7. Code for Sample Combiner Settings 2\n glUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[1].srcRgb\"),\n GL_TEXTURE0, GL_TEXTURE1, GL_PREVIOUS);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[1].srcAlpha\"),\n GL_TEXTURE0, GL_TEXTURE1, GL_PREVIOUS);\nglUniform1i(glGetUniformLocation(program, \"dmp_TexEnv[1].combineRgb\"),\n GL_ADD);\nglUniform1i(glGetUniformLocation(program, \"dmp_TexEnv[1].combineAlpha\"),\n GL_ADD);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[1].operandRgb\"),\n GL_SRC_COLOR, GL_SRC_COLOR, GL_SRC_COLOR);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[1].operandAlpha\"),\n GL_SRC_ALPHA, GL_SRC_ALPHA, GL_SRC_ALPHA);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[2].srcRgb\"),\n GL_PREVIOUS, GL_PRIMARY_COLOR, GL_PREVIOUS);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[2].srcAlpha\"),\n GL_PREVIOUS, GL_PRIMARY_COLOR, GL_PREVIOUS);\nglUniform1i(glGetUniformLocation(program, \"dmp_TexEnv[2].combineRgb\"),\n GL_MODULATE);\nglUniform1i(glGetUniformLocation(program, \"dmp_TexEnv[2].combineAlpha\"),\n GL_MODULATE);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[2].operandRgb\"),\n GL_SRC_COLOR, GL_SRC_COLOR, GL_SRC_COLOR);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[2].operandAlpha\"),\n GL_SRC_ALPHA, GL_SRC_ALPHA, GL_SRC_ALPHA);\n",0]
,["Pages/Page_55588812.html#PageId_55588832","11.3. Combiner Buffers ",
"\n11.3. Combiner Buffers\n",0]
,["Pages/Page_55588812.html#PageId_55588833","11.3.1. Reserved Uniforms for Combiner Buffers ",
"In the CTR system, each combiner, except for the last one (combiner 5), has a combiner buffer configured in parallel to it. A combiner buffer can select the output of the previous combiner stage or combiner buffer stage as its input source. By preserving the output of a previous combiner buffer stage, a combiner buffer can allow a later combiner stage's input to come from the output of a combiner stage earlier than the previous one.\n11.3.1. Reserved Uniforms for Combiner Buffers\nBecause combiner buffer 0 has no input, its initial value is a constant color set by the reserved uniform dmp_TexEnv[0].bufferColor. Use the glUniform4f function to set the constant color at the reserved uniform location obtained by the glGetUniformLocation function. The first value is R, followed by G, B, and A, respectively.\nBy default, 0.0 is set as the R, G, B, and A values of combiner buffer 0's constant color.\nYou can choose output from either the previous combiner stage or the previous combiner buffer stage as the input source to combiner buffers 1 through 4. Input from the color and alpha components can be selected separately through the reserved uniforms dmp_TexEnv[i].bufferInput (where i is 1 through 4). Use the glUniform2i function to set a value at the reserved uniform location obtained by glGetUniformLocation. The color components are first, followed by the alpha component.\nThe following reserved uniform values are used to configure the combiner buffer input sources.\nTable 11-18. Reserved Uniform Values That Can Be Set for Combiner Buffer Input Sources\n \t Value\n \t Input Source\n \t GL_PREVIOUS\n \t Output from the previous combiner stage (default).\n \t GL_PREVIOUS_BUFFER_DMP\n \t Output from the previous combiner buffer stage.\n ",0]
,["Pages/Page_55588812.html#PageId_55588834","11.3.2. Sample Combiner Buffer Settings ",
"\n11.3.2. Sample Combiner Buffer Settings\nThe following sample settings demonstrate how to connect combiner buffers so that texture 0 is multiplied with the primary color for fragment lighting, texture 1 is multiplied with the secondary color for fragment lighting, and then these results are added together and output.\nFigure 11-5. Sample Combiner Buffer Settings\nThe following sample code shows how to make these settings in a program.\nCode 11-8. Sample Code for Setting Up the Combiner Buffers\n // Combiner 0\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[0].srcRgb\"),\n GL_TEXTURE0, GL_FRAGMENT_PRIMARY_COLOR_DMP, GL_PREVIOUS);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[0].srcAlpha\"),\n GL_TEXTURE0, GL_PREVIOUS, GL_PREVIOUS);\nglUniform1i(glGetUniformLocation(program, \"dmp_TexEnv[0].combineRgb\"),\n GL_MODULATE);\nglUniform1i(glGetUniformLocation(program, \"dmp_TexEnv[0].combineAlpha\"),\n GL_REPLACE);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[0].operandRgb\"),\n GL_SRC_COLOR, GL_SRC_COLOR, GL_SRC_COLOR);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[0].operandAlpha\"),\n GL_SRC_ALPHA, GL_SRC_ALPHA, GL_SRC_ALPHA);\n// CombinerBuffer 1\nglUniform2i(glGetUniformLocation(program, \"dmp_TexEnv[1].bufferInput\"),\n GL_PREVIOUS, GL_PREVIOUS);\n// Combiner 1\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[1].srcRgb\"),\n GL_TEXTURE1, GL_FRAGMENT_SECONDARY_COLOR_DMP, GL_PREVIOUS);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[1].srcAlpha\"),\n GL_TEXTURE1, GL_PREVIOUS, GL_PREVIOUS);\nglUniform1i(glGetUniformLocation(program, \"dmp_TexEnv[1].combineRgb\"),\n GL_MODULATE);\nglUniform1i(glGetUniformLocation(program, \"dmp_TexEnv[1].combineAlpha\"),\n GL_REPLACE);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[1].operandRgb\"),\n GL_SRC_COLOR, GL_SRC_COLOR, GL_SRC_COLOR);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[1].operandAlpha\"),\n GL_SRC_ALPHA, GL_SRC_ALPHA, GL_SRC_ALPHA);\n// Combiner 2\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[2].srcRgb\"),\n GL_PREVIOUS_BUFFER_DMP, GL_PREVIOUS, GL_PREVIOUS);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[2].srcAlpha\"),\n GL_PREVIOUS_BUFFER_DMP, GL_PREVIOUS, GL_PREVIOUS);\nglUniform1i(glGetUniformLocation(program, \"dmp_TexEnv[2].combineRgb\"),\n GL_ADD);\nglUniform1i(glGetUniformLocation(program, \"dmp_TexEnv[2].combineAlpha\"),\n GL_REPLACE);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[2].operandRgb\"),\n GL_SRC_COLOR, GL_SRC_COLOR, GL_SRC_COLOR);\nglUniform3i(glGetUniformLocation(program, \"dmp_TexEnv[2].operandAlpha\"),\n GL_SRC_ALPHA, GL_SRC_ALPHA, GL_SRC_ALPHA);\n",0]
,["Pages/Page_55588812.html#PageId_55588835","11.4. Procedural Textures ",
"\n11.4. Procedural Textures\nProcedural textures are unlike conventional textures, in that they determine texel colors procedurally rather than by referencing images. Procedural textures are most effective when used for perfectly regular patterns and patterns that are regular, but have some randomness. Because they do not access texture images, they avoid memory access conflicts and reduce data (content) sizes.\nTexture unit 3 is exclusively used for procedural textures, and is also the only texture unit that can handle procedural textures. Even though procedural textures calculate texel colors, they are similar to normal textures, in that they still determine which texel colors correspond to UV texture coordinates.\n",0]
,["Pages/Page_55588812.html#PageId_55588836","11.4.1. Procedural Texture Unit ",
"As a part of the reserved fragment shaders, procedural textures have parameters that are configured through reserved uniforms by the glUniform* functions.\n11.4.1. Procedural Texture Unit\nThe procedural texture unit comprises three computational components. In order of process flow, these are random number generation, clamping, and mapping. Random number generation adds noise to UV texture coordinates; clamping determines wrapping and mirror symmetry for patterns; and mapping calculates texel colors from UV coordinates.\nFigure 11-6. Structure of the Procedural Texture Unit\nInput texture coordinates are processed as shown in the figure above. Use the following procedure to set parameters to get the desired image.\n \tEnable Procedural Textures \tSet shared RGBA mode or independent alpha mode.\nThis corresponds to settings for G(u, v) and F(g). \tSelect a basic shape.\nThis corresponds to the selection in G(u, v). \tSet the basic color.\nColors are set in color lookup tables. This corresponds to settings for Color(f). \tSet the relationship between the basic shape and color lookup table.\nThis determines how the basic shape in step 3 corresponds to the color lookup table in step 4. This corresponds to settings for F(g). \tSelect random number parameters.\nIf necessary, enable random numbers and determine the size of their effect. If not necessary, disable them. This corresponds to random number generation. The UV coordinates are made into absolute values and output to the clamp portion, regardless of whether they are enabled or disabled. \tConfigure wrapping and symmetry.\nThis corresponds to clamp settings.\n",0]
,["Pages/Page_55588812.html#PageId_55588837","11.4.2. Enable Procedural Textures ",
"\n11.4.2. Enable Procedural Textures\nTo enable or disable texture unit 3, which is used by procedural textures, set a value for the dmp_Texture[3].samplerType reserved uniform. Note that using glActiveTexture to select a unit or glEnable to select a texture type results in an error.\nUsing glUniform1i, set the value to GL_TEXTURE_PROCEDURAL_DMP to enable texture unit 3 or to GL_FALSE to disable it. The only supported texture type is GL_TEXTURE_PROCEDURAL_DMP.\nCode 11-9. Enabling Procedural Textures\n glUniform1i(\n glGetUniformLocation(s_PgID, \"dmp_Texture[3].samplerType\"),\n GL_TEXTURE_PROCEDURAL_DMP);\n",0]
,["Pages/Page_55588812.html#PageId_55588838","11.4.3. Setting Shared RGBA Mode or Independent Alpha Mode ",
"\n11.4.3. Setting Shared RGBA Mode or Independent Alpha Mode\nChoose whether the alpha component is mapped using the same functions as the RGB components (shared RGBA mode) or using functions that are set separately for the alpha component (independent alpha mode). If independent alpha mode is selected, two G functions and two F functions must be set for mapping. Shared RGBA mode is easier to set if you just want to output an image to see what it looks like.\nUsing the glUniform1i function, set the reserved uniform dmp_Texture[3].ptAlphaSeparate to GL_FALSE to select shared RGBA mode or to GL_TRUE to select independent alpha mode. Shared RGBA mode is the default.\nFigure 11-7. Mapping in Independent Alpha Mode\n",0]
,["Pages/Page_55588812.html#PageId_55588839","11.4.4. Selecting a Basic Shape ",
"\n11.4.4. Selecting a Basic Shape\nA procedural texture's basic shape is determined by the G function during mapping.\nThe basic shapes in Table 11-19 use the following color assignments.\nTable 11-19. G Function Settings, Selected Functions, and Basic Shapes\n \t Value\n \t Selected Function\n \t Basic Shape\n \t GL_PROCTEX_U_DMP (default)\n \t u\n \t\n \t GL_PROCTEX_V_DMP\n \t v\n \t\n \t GL_PROCTEX_U2_DMP\n \t u2\n \t\n \t GL_PROCTEX_V2_DMP\n \t v2\n \t\n \t GL_PROCTEX_ADD_DMP\n \t (u + v) / 2\n \t\n \t GL_PROCTEX_ADD2_DMP\n \t (u2 + v2) / 2\n \t\n \t GL_PROCTEX_ADDSQRT2_DMP\n \t sqrt(u2 + v2)\n \t\n \t GL_PROCTEX_MIN_DMP\n \t min(u, v)\n \t\n \t GL_PROCTEX_MAX_DMP\n \t max(u, v)\n \t\n \t GL_PROCTEX_RMAX_DMP\n \t ((u + v) / 2 + sqrt(u2 + v2)) / 2\n \t\n The basic shape is rendered using texture coordinates between -1.0 and 1.0 (with 0.0 at the center) and GL_MIRRORED_REPEAT specified for wrapping.\nThe reserved uniforms used for selection with the G function are dmp_Texture[3].ptRgbMap for RGB components and dmp_Texture[3].ptAlphaMap for the alpha component. Use the glUniform1i function to set each of these reserved uniforms.\ndmp_Texture[3].ptAlphaMap settings are only valid in independent alpha mode. Choose the shape that is closest to the desired texture image.\nFor example, choose GL_PROCTEX_U_DMP and GL_PROCTEX_V_DMP for wood grain or GL_PROCTEX_ADDSQRT2_DMP for the annual rings in a tree.\n",0]
,["Pages/Page_55588812.html#PageId_55588840","11.4.5. Color Lookup Table Settings ",
"\n11.4.5. Color Lookup Table Settings\nColor lookup tables are used to convert values calculated by the G and F functions into actual texel colors, and can set each of the RGBA components separately.\nThe content of the color lookup table depends on whether LOD is used.\nFigure 11-8. Color Lookup Table Differences Caused by the Use of LOD\nWhen LOD is not used, multiple color tables can be stored as partial arrays in a color lookup table. By changing the offset and table width, you can also render textures with different coloring from the same calculation results.\nA color lookup table can hold up to 512 elements. You can store the color table in the first 256 elements and the differences between the color table values in the last 256 elements. Because the delta values must start at the 257th element, the number of color lookup table entries is defined as the sum of 256, the number of color lookup table elements that are actually referenced, and the starting offset to the color tables. The number of elements is calculated for the last color table.\nUse the glUniform1i function to set the color table width that is actually referenced (dmp_Texture[3].ptTexWidth). Set this value to a power of 2 that is no greater than 128. Use the glUniform1i function to set the color table's starting offset (dmp_Texture[3].ptTexOffset) to an integer between 0 and 128.\nWhen LOD is used, the color table width and offset must be 128 and 0 respectively. The level of detail determines which color table is actually referenced. The maximum number of elements in a color lookup table is fixed at 512.\nTable 11-20. Level of Detail and Corresponding Color Tables\n \t Level of Detail\n \t Starting Position\n \t Width\n \t 0\n \t 0\n \t 128\n \t 1\n \t 128\n \t 64\n \t 2\n \t 192\n \t 32\n \t 3\n \t 224\n \t 16\n \t 4\n \t 240\n \t 8\n \t 5\n \t 248\n \t 4\n \t 6\n \t 252\n \t 2\n As explained in 7.7. Loading Lookup Tables, lookup tables are loaded from arrays by calls to the glTexImage1D function. Prepare an array of floating-point numbers with as many elements as the color lookup table, storing values 0.0 through 1.0 for the color table elements (T) in the first half of the array and the differences between the first half’s 256 elements (ΔT) in the last half of the array. The following equations calculate the elements and delta values, given size as the number of elements in the color table, offset as the starting offset, Ci as each element, and func as the conversion function.\nSet the last difference value to either 0.0 or the difference between the color table's last element and the convergence value.\nYou can use the following code to call the glTexImage1D function and configure 512 as the maximum number of elements in a color lookup table, data as the array storing the color lookup table, and 0 as the number of the lookup table to set.\nCode 11-10. Loading a Color Lookup Table\n glTexImage1D(GL_LUT_TEXTURE0_DMP, 0, GL_LUMINANCEF_DMP, 512, 0,\n GL_LUMINANCEF_DMP, GL_FLOAT, data);\nA reserved uniform for each component specifies which lookup table number to use for the component’s loaded color lookup table. In this context, GL_LUT_TEXTURE0_DMP, GL_LUT_TEXTURE1_DMP, and GL_LUT_TEXTURE2_DMP indicate lookup table numbers 0, 1, and 2 respectively. Note that they do not specify the name (ID) of the texture nor GL_LUT_TEXTUREi_DMP directly. They are set by the glUniform1i function.\nTable 11-21. Reserved Uniforms That Specify Color Lookup Tables\n \t Reserved Uniform\n \t Value to Set\n \t dmp_Texture[3].ptSamplerR\n \t Specifies the lookup table number to use as the color lookup table for the red component.\n \t dmp_Texture[3].ptSamplerG\n \t Specifies the lookup table number to use as the color lookup table for the green component.\n \t dmp_Texture[3].ptSamplerB\n \t Specifies the lookup table number to use as the color lookup table for the blue component.\n \t dmp_Texture[3].ptSamplerA\n \t Specifies the lookup table number to use as the color lookup table for the alpha component.\n Each reserved uniform specifies a value between 0 and 31.\nThe value set for dmp_Texture[3].ptSamplerA is ignored in independent alpha mode.\nYou can apply the same minification filters to a procedural texture's color lookup tables as you can to a normal texture. Choose a value from the following table to set the reserved uniform dmp_Texture[3].ptMinFilter with the glUniform1i function.\nTable 11-22. Color Lookup Table Filters\n \t Value to Set\n \t Filter to Apply\n \t GL_NEAREST\n \t Nearest in the UV directions without LOD.\n \t GL_LINEAR\n \t Linear in the UV directions without LOD (default).\n \t GL_NEAREST_MIPMAP_NEAREST\n \t Nearest in the UV directions with the nearest LOD.\n \t GL_NEAREST_MIPMAP_LINEAR\n \t Nearest in the UV directions with a linear LOD.\n \t GL_LINEAR_MIPMAP_NEAREST\n \t Linear in the UV directions with the nearest LOD.\n \t GL_LINEAR_MIPMAP_LINEAR\n \t Linear in the UV directions with a linear LOD.\n You can apply an LOD bias when referencing the color lookup table. Use the glUniform1f function to set the reserved uniform dmp_Texture[3].ptTexBias to a value from 0.0 through 6.0. This is disabled with a value of 0.0, and has a default value of 0.5.\n",0]
,["Pages/Page_55588812.html#PageId_55588841","11.4.6. Setting the Relationship Between the Basic Shape and Color Lookup Table ",
"\n11.4.6. Setting the Relationship Between the Basic Shape and Color Lookup Table\nThe F function configures how the G function, which selects the basic shape, corresponds to the color lookup table, which sets the basic color. The F function uses a lookup table to map output from the G function (0.0 through 1.0) into lookup values for the color lookup table (0.0 through 1.0). The lookup table has 256 elements. A mapping table is stored in the first 128 elements and the differences between the mapping table values are stored in the last 128 elements. Because the mapping table configures the relationship between shapes and colors, by changing the mapping table you can render textures that have different appearances even though they use the same shape and color lookup table. By modifying the F function, a wide variety of outputs is possible. For example, you could use a simple F function to calculate results, such as F(x)=x or F(x)=x2, or you could have the F function use discontinuous values and operate like an index.\nLike color lookup tables, the mapping table for the F function is loaded from an array into a lookup table by a call to the glTexImage1D function. Prepare an array of floating-point numbers with as many elements (256) as the mapping table, storing the mapping table elements (0.0 through 1.0) in the first half of the array and the differences between those elements in the last half of the array. The following equations calculate the mapping table elements and delta values, assuming Fi is a mapping table element and func is the conversion function.\nYou can use the following code to call the glTexImage1D function and configure 256 as the number of mapping table elements, data as the array storing the mapping table, and 0 as the number of the lookup table to set.\nCode 11-11. Loading a Mapping Table\n glTexImage1D(GL_LUT_TEXTURE0_DMP, 0, GL_LUMINANCEF_DMP, 256, 0,\n GL_LUMINANCEF_DMP, GL_FLOAT, data);\nReserved uniforms specify which lookup tables that contain mapping tables to use as F functions. In this context, GL_LUT_TEXTURE0_DMP, GL_LUT_TEXTURE1_DMP, and GL_LUT_TEXTURE2_DMP indicate lookup table numbers 0, 1, and 2 respectively. Note that they do not specify the name (ID) of the texture nor GL_LUT_TEXTUREi_DMP directly. They are set by the glUniform1i function.\nTable 11-23. Reserved Uniforms That Specify Mapping Tables\n \t Reserved Uniform\n \t Value to Set\n \t dmp_Texture[3].ptSamplerRgbMap\n \t Specifies the lookup table number to use as the F function for RGB values.\n \t dmp_Texture[3].ptSamplerAlphaMap\n \t Specifies the lookup table number to use as the F function for alpha values.\n Each reserved uniform specifies a value between 0 and 31.\ndmp_Texture[3].ptSamplerAlphaMap is only valid in independent alpha mode.\n",0]
,["Pages/Page_55588812.html#PageId_55588842","11.4.7. Selecting Random Number Parameters ",
"\n11.4.7. Selecting Random Number Parameters\nAs a random element in a procedural texture, noise can be added to the UV texture coordinates that are input to the G function. Noise affects the basic shape. When the G function is GL_PROCTEX_U_DMP and noise affects U texture coordinates, for example, it becomes possible to render wood grain with natural warping. Ordinarily, wood grain can only be rendered in a straight line.\nTo enable or disable noise, set a value in the reserved uniform dmp_Texture[3].ptNoiseEnable. Using glUniform1i, specify GL_TRUE to enable it, and GL_FALSE to disable it.\nCode 11-12. Enabling Noise\n glUniform1i(\n glGetUniformLocation(s_PgID, \"dmp_Texture[3].ptNoiseEnable\"), GL_TRUE);\nThe function that adds noise is a black box, but it can be controlled through three parameters from the application: the frequency (F), the phase (P), and the amplitude (A). The F parameter adjusts the speed of the fluctuations (noise) so that large values create jagged waves and small values create gentle undulations. The P parameter changes the starting location of the noise. When rendering a texture of the ocean surface, for example, you can represent changing waves by modifying only the P parameter. When increased, the A parameter magnifies the effect of the noise and further destroys the basic shape.\nThe three parameters F, P, and A can each be set separately for the U and V components.\nTable 11-24. Reserved Uniforms for Noise\n \t Reserved Uniform\n \t Value to Set\n \t dmp_Texture[3].ptNoiseU\n \t Specifies the F, P, and A parameters for the u-component. The range from -8.0 through 8.0 is clamped only for parameter A.\n (F-parameter, P-parameter, A-parameter)\n These are (0.0, 0.0, 0.0) by default.\n \t dmp_Texture[3].ptNoiseV\n \t Specifies the F, P, and A parameters for the v component. The range from -8.0 through 8.0 is clamped only for parameter A.\n (F-parameter, P-parameter, A-parameter)\n These are (0.0, 0.0, 0.0) by default.\n Apart from the noise parameters, you can control changes in the continuity of random numbers (called noise modulation) in the function that adds noise. Noise modulation (a continuous noise function) is specified by a lookup table called the noise modulation table. The noise function takes a noise modulation table and uses it to create natural noise values from the discrete values that arise from calculations alone. A suitable continuous noise function, such as 3x2 - 2x3, generates values that change gradually when x is near 0.0 and 1.0.\nLike the color lookup table, the noise modulation table is loaded from an array by a call to the glTexImage1D function. The noise function takes a noise modulation table and uses it to create natural noise values from the discrete values that arise from calculations alone. The following equations calculate the noise modulation table elements and delta values, assuming Ni is a table element and func is the conversion function.\nYou can use the following code to call the glTexImage1D function and configure 256 as the number of noise modulation table elements, data as the array storing the noise modulation table, and 0 as the number of the lookup table to set.\nCode 11-13 Loading a Noise Modulation Table\n glTexImage1D(GL_LUT_TEXTURE0_DMP, 0, GL_LUMINANCEF_DMP, 256, 0,\n GL_LUMINANCEF_DMP, GL_FLOAT, data);\nReserved uniforms specify which lookup tables that contain noise modulation tables to use as continuous noise functions. In this context, GL_LUT_TEXTURE0_DMP, GL_LUT_TEXTURE1_DMP, and GL_LUT_TEXTURE2_DMP indicate lookup table numbers 0, 1, and 2 respectively. Note that they do not specify the name (ID) of the texture nor GL_LUT_TEXTUREi_DMP directly. They are set by the glUniform1i function.\nTable 11-25. Reserved Uniforms That Specify Noise Modulation Tables\n \t Reserved Uniform\n \t Value to Set\n \t dmp_Texture[3].ptSamplerNoiseMap\n \t Specifies the lookup table number to use as the continuous noise function.\n 0 to 31\n To illustrate the effects of the three noise parameters F, P, and A on the output results, consider the difference between a procedural texture that is rendered as a concentric circle when it is unaffected by noise, and the same texture when its parameters are changed in both the U and V directions. These are rendered with F(x)=x as the continuous noise function.\nFigure 11-9 shows the effect of changing only the A parameter. The other parameters, F and P, are set to 0.3 and 0.0 respectively. Although the waves become more prominent as A gets larger, note that most points along the circumference of the circle are unaffected.\nFigure 11-9. Effect of the A Parameter\nFigure 11-10 shows the effect of changing only the F parameter. The other parameters, A and P, are set to 0.3 and 0.0 respectively. Note that as F gets larger, the frequency of the noise (fluctuations) increases, and the affected locations along the circumference of the circle get closer to each other. Also note that the absolute value is used for the F parameter when calculating noise, so reversing the sign does not change the result.\nFigure 11-10. Effect of the F Parameter\nFigure 11-11 shows the effect of changing only the P parameter. The other parameters, A and F, are both set to 0.3. Note that when P changes, only the shape of the noise changes. By modifying the P parameter, you can change a procedural texture so that it appears animated.\nIf you set the P parameter equal to a large value while using it to animate a texture, small changes to the P parameter will affect the shape of the noise. This is caused by the accuracy of calculations in the hardware. For example, if you animate a texture by adding a constant value to the P parameter every frame (which changes the noise), you must restore the P parameter to a small value before it gets too large. One characteristic of the F and P parameters is that when they are both positive and have a product that is a multiple of 16, they have the same effect as when the P parameter is 0.0. In other words, by changing the P parameter back to 0 when the product of the P and F parameters is 16, you can maintain the animation’s continuity. However, you may not get the same shape when the P parameter is 0.0 and when the product of the F and P parameters is 16, if the F parameter is large.\nIf the values for the F and A parameters are fixed, and the P parameter varies in a range where the sign does not change for the phase |u| + u (or for the phase |v| + v), you can get the same noise result for X + 16 when F×P is some arbitrary value X. However, depending on the accuracy of the noise calculation process, you may not be able to get the same random value this way if you set a large value for the F parameter. Also, if you set a large value for the P parameter, changes in small values may not get reflected in the noise result.\nFigure 11-11. Effect of the P Parameter\n",0]
,["Pages/Page_55588812.html#PageId_55588843","11.4.8. Wrapping and Symmetry Settings ",
"\n11.4.8. Wrapping and Symmetry Settings\nProcedural textures have a feature equivalent to the wrapping mode that can be set for normal textures. This feature is called clamp calculation, and can be configured with a number of dedicated modes, such as pulse and zero-clamp. There are also shift calculations that shift blocks of texture coordinates that have the same integer values during wrapping.\nClamp calculations use the clamp mode to determine how to convert texture coordinates that are less than 0.0 or greater than 1.0 into values between 0.0 and 1.0.\nTable 11-26. Clamp Modes\n \t Clamping Mode\n \t Coordinate Clamping\n \t GL_SYMMETRICAL_REPEAT_DMP\n \t\n \t GL_MIRRORED_REPEAT\n \t\n \t GL_PULSE_DMP\n \t\n \t GL_CLAMP_TO_EDGE\n(default)\n \t\n \t GL_CLAMP_TO_ZERO_DMP\n \t\n You can set different clamp modes for the U and V texture coordinates. To set these modes, call the glUniform1i function on the reserved uniform dmp_Texture[3].ptClampU or dmp_Texture[3].ptClampV.\nGL_SYMMETRICAL_REPEAT_DMP lines up the same image on a grid. GL_MIRRORED_REPEAT uses a mirror reflection at even-numbered values. GL_PULSE_DMP uses the pixel closest to the edge of the texture for each pixel that is used for rendering. GL_CLAMP_TO_EDGE uses a texture's internal image for values between -1.0 and 1.0. Outside of that range, it uses pixels at the edge of the texture. GL_CLAMP_TO_ZERO_DMP uses the texture image for values between -1.0 and 1.0 (excluding these two values). Outside of that range, it uses the image at coordinate 0 (including the values -1.0 and 1.0).\nShift calculations determine the shift coordinates based on the shift mode. The shift width depends on the clamp mode. This process is applied before clamp calculations, allowing you to avoid rendering the same image over and over.\nTable 11-27. Shift Modes\n \t Shift Mode\n \t Shift Calculation\n \t Shift Width\n \t GL_NONE_DMP\n(default)\n \t No shift calculation.\n \t None.\n \t GL_ODD_DMP\n \t Shifts coordinates when their integer value changes from an odd number to an even number.\n \t 1.0 for GL_MIRRORED_REPEAT only; 0.5 otherwise.\n \t GL_EVEN_DMP\n \t Shifts coordinates when their integer value changes from an even number to an odd number.\n \t 1.0 for GL_MIRRORED_REPEAT only; 0.5 otherwise.\n You can set different shift modes for the U and V texture coordinates. To set these modes, call the glUniform1i function on the reserved uniform dmp_Texture[3].ptShiftU or dmp_Texture[3].ptShiftV.\nFigure 11-12. How Shift Modes and Clamping Modes Affect Shift Widths\nCONFIDENTIAL\n",0]
,["Pages/Page_55588844.html","12. Reserved Fragment Shaders ",
"12. Reserved Fragment Shaders\nA reserved fragment shader can process lighting and other effects for fragments output from the previous shader.\nAs explained in 5. Shader Programs, the reserved fragment shader does not need to be loaded from a binary. To use it, attach it to the same program object as the vertex shader and geometry shader, using the special name GL_DMP_FRAGMENT_SHADER_DMP. The reserved fragment shader is a collection of fragment processing features. These features include the following.\n\tFragment lighting \tShadows \tFog \tGas ",0]
,["Pages/Page_55588844.html#PageId_55588845","12.1. Fragment Operation Mode ",
" \tMiscellaneous (alpha test, w buffer)\nThere are reserved uniforms for each fragment process. These reserved uniforms have default values and must be set, as necessary, by the application.\n12.1. Fragment Operation Mode\nReserved fragment processing replaces the standard OpenGL fragment pipeline (from the alpha test onward) with independent processing that can handle the special rendering passes required by shadows and gas. To switch this fragment operation mode, set the reserved uniform (dmp_FragOperation.mode) to the desired mode using the glUniform1i function.\nTable 12-1. Fragment Operation Modes\n \t Fragment Operation Mode\n \t Pipeline\n \t GL_FRAGOP_MODE_GL_DMP (default)\n \t Standard OpenGL fragment pipeline (standard mode).\n \t GL_FRAGOP_MODE_SHADOW_DMP\n \t Fragment pipeline for the shadow accumulation pass (shadow mode).\n \t GL_FRAGOP_MODE_GAS_ACC_DMP\n \t Fragment pipeline for rendering density information (gas mode).\n ",0]
,["Pages/Page_55588844.html#PageId_55588846","12.2. Fragment Lighting ",
"\n12.2. Fragment Lighting\nThe 3DS system uses fragment lighting, which calculates the primary and secondary colors for each fragment, rather than for each vertex. It also has the following features: bump mapping, which references a texture to perturb normal vectors; shadows, which involve the creation of shadow textures and color calculations; and attenuation, which is calculated from the distance to a spotlight or some other light.\nThe primary and secondary colors are determined by first combining multiple functions that output lookup table values based on the dot product of two vectors, and then using bitwise AND/OR operations to combine those output values. With 3DS, although you cannot fully customize the method that combines the lookup tables and vectors for the dot products, you can select different configurations from preset settings.\nNintendo expects the CTR system to use eye coordinates for several vectors, because lighting equations are considered to be in eye coordinates. Although eye coordinates are not necessarily required, all the vectors that are used must be in the same coordinate system.\nTo use fragment lighting, you must set dmp_FragmentLighting.enabled to GL_TRUE with the glUniform1i function and enable at least one light. In the vertex shader, the normal vector, view vector, and tangent vector (when required for lighting) must be converted into a quaternion and output as a single vertex attribute.\nTable 12-2. Enabling and Disabling Fragment Lighting\n \t Reserved Uniform\n \t Type\n \t Setting Value\n \t dmp_FragmentLighting.enabled\n \t bool\n \t Specify GL_TRUE to enable lighting, and GL_FALSE otherwise.\n GL_FALSE is the default.\n ",0]
,["Pages/Page_55588844.html#PageId_55588847","12.2.1. Quaternion Conversion ",
"12.2.1. Quaternion Conversion\nLighting calculations require all vectors to use the same coordinate system. In other words, bump mapping (explained later) must convert the perturbation normals referenced in a texture from surface-local coordinates (with the vertex at the origin and the normal vector along the positive z-axis) into eye coordinates.\nThe following 3x3 rotation matrix converts surface-local coordinates into eye coordinates. It comprises a normal, tangent, and binormal vector, and can be converted into a quaternion (Qx, Qy, Qz, Qw).\n(E represents the eye coordinates, T is the tangent, N is the normal, B is the binormal, and S represents the surface-local coordinates.)\nFragment lighting is implemented to generate a quaternion for each fragment from a quaternion for each vertex, rather than generate a vector for each fragment from the normal, tangent, and binormal (which can be calculated from the normal and tangent) vectors input for each vertex. The quaternion is converted into the original rotation matrix during fragment light processing. To use fragment lighting, you must convert each vector into a valid quaternion in the vertex shader.\n",0]
,["Pages/Page_55588844.html#Anchor_55588847_法線のみのクォータ二オン生成について","Generating Normal-Only Quaternions ",
"Generating Normal-Only Quaternions\nThe CTR-SDK sample demos include a vertex shader (l_position_view_quaternion) that generates quaternions using only the vertex normal information. To generate a quaternion that transforms normals in surface-local coordinates to normals in perspective coordinates, the vertex shader uses the half angle vector between the unit normal vector (0, 0, 1) and a normal vector that has been transformed to perspective coordinates (Nx, Ny, Nz) as the axis of rotation, and derives a quaternion that performs 180° rotations.\nFigure 12-1. The Rotation Axis and Angle of a Quaternion\nFor an axis of rotation (α, β, γ) and an angle of rotation θ, the derived quaternion Q would be computed as follows.\n Note:\n In the vertex shader, the real component of the quaternion is set to the w component.\nFor a θ value of 180°, this becomes:\nQ = ( 0; α, β, γ )\nTaking the half-angle vector as the axis of rotation, this becomes:\nThe orientation of the half-angle vector is undefined only when (Nx, Ny, Nz) is (0, 0, –1). In this case, assume that (α, β, γ) = (1, 0, 0).\n",0]
,["Pages/Page_55588844.html#PageId_55588849","12.2.2. Lighting Overview ",
"\n12.2.2. Lighting Overview\n3DS fragment lighting always calculates the primary and secondary colors.\nThe primary color is calculated first by accumulating each light's effect (with shadows, spotlight attenuation, and distance attenuation applied) on a fragment's ambient and diffuse light. The fragment's emissive light and the effect of the scene's ambient light is then added to this accumulated value. This becomes the fragment's base color.\nThe secondary color is calculated by accumulating each light's effect on a fragment's second specular light with shadows, spotlight attenuation, and distance attenuation applied. This color is mainly used for fragment highlights.\nLike OpenGL, an object's color is determined from its ambient, diffuse, emissive, and specular light. Unlike OpenGL, however, lighting uses per-fragment calculations and second specular light, making it possible to calculate the specular light in a variety of ways. The second specular light in particular can be used to represent materials with colors that change depending on the angle.\nFigure 12-2. Fragment Lighting\n",0]
,["Pages/Page_55588844.html#PageId_55588850","12.2.3. Scene Settings ",
"\n12.2.3. Scene Settings\nFragment lighting can handle scene sizes from -216 through 215. Do not allow the distance between the viewpoint and any fragment or light in the scene to be greater than or equal to 216.\nThe scene affects fragments through its ambient light (the global ambient light). To specify the scene's global ambient light, set the reserved uniform dmp_FragmentLighting.ambient to an RGBA color using the glUniform4fv function.\nTable 12-3. Reserved Uniforms for Scene Settings\n \t Reserved Uniform\n \t Type\n \t Setting Value\n \t dmp_FragmentLighting.ambient\n \t vec4\n \t Specifies the scene's global ambient light (R, G, B, A).\n Each component has a value between 0.0 and 1.0.\n This is (0.2, 0.2, 0.2, 1.0) by default.\n ",0]
,["Pages/Page_55588844.html#PageId_55588851","12.2.4. Material Settings ",
"\n12.2.4. Material Settings\nMaterial settings can be described simply as settings that use color information, such as the ambient and specular light, to represent a fragment's materials and texture. Specify material-related settings in the reserved uniforms dmp_FragmentMaterial.*.\n12.2.7. Equations for the Primary Color and 12.2.8. Equations for the Secondary Color explain how the settings are used in lighting calculations.\nTable 12-4. Reserved Uniforms for Material Settings\n \t Reserved Uniform\n \t Type\n \t Setting Value\n \t dmp_FragmentMaterial.ambient\n \t vec4\n \t Specifies the ambient light (R, G, B, A). Each component has a value between 0.0 and 1.0.\n This is (0.2, 0.2, 0.2, 1.0) by default.\n \t dmp_FragmentMaterial.diffuse\n \t vec4\n \t Specifies the ambient light (R, G, B, A). Each component has a value between 0.0 and 1.0.\n This is (0.8, 0.8, 0.8, 1.0) by default.\n \t dmp_FragmentMaterial.emission\n \t vec4\n \t Specifies the ambient light (R, G, B, A). Each component has a value between 0.0 and 1.0.\n This is (0.0, 0.0, 0.0, 1.0) by default.\n \t dmp_FragmentMaterial.specular0\n \t vec4\n \t Specifies specular light 0 (R, G, B, A). Each component has a value between 0.0 and 1.0.\n This is (0.0, 0.0, 0.0, 1.0) by default.\n \t dmp_FragmentMaterial.specular1\n \t vec4\n \t Specifies specular light 1 (R, G, B, A). Each component has a value between 0.0 and 1.0.\n This is (0.0, 0.0, 0.0, 1.0) by default.\n \t dmp_FragmentMaterial.samplerXX\n (XX=DO,D1,RR,RG,RB,FR)\n \t int\n \t Specifies the lookup tables to use for lighting calculations.\n This is a number from 0 through 31 for each factor.\n ",0]
,["Pages/Page_55588844.html#PageId_55588852","12.2.5. Light Settings ",
"\n12.2.5. Light Settings\nThere are two types of light settings. One configures the effect of light on a material and the other configures the light itself. Fragment lighting can handle eight lights. Specify light-related settings in the reserved uniforms dmp_FragmentLightSource[i].* (where i is the light number).\n12.2.7. Equations for the Primary Color and 12.2.8. Equations for the Secondary Color explain how the settings are used in lighting calculations.\nTable 12-5. Reserved Uniforms for Light Settings\n \t Reserved Uniform\n \t Type\n \t Setting Value\n \t *.enabled\n \t bool\n \t Enables or disables a light.\n Specify GL_TRUE to enable a light, and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.ambient\n \t vec4\n \t Specifies an ambient light (R, G, B, A). Each component has a value between 0.0 and 1.0.\n This is (0.0, 0.0, 0.0, 0.0) by default.\n \t *.diffuse\n \t vec4\n \t Specifies a diffuse light (R, G, B, A). Each component has a value between 0.0 and 1.0.\n By default, only light number 0 is (1.0, 1.0, 1.0, 1.0).\nAll of the others are (0.0, 0.0, 0.0, 0.0).\n \t *.specular0\n \t vec4\n \t Specifies specular light 0 (R, G, B, A). Each component has a value between 0.0 and 1.0.\n By default, only light number 0 is (1.0, 1.0, 1.0, 1.0).\nAll of the others are (0.0, 0.0, 0.0, 0.0).\n \t *.specular1\n \t vec4\n \t Specifies specular light 1 (R, G, B, A). Each component has a value between 0.0 and 1.0.\n This is (0.0, 0.0, 0.0, 0.0) by default.\n \t *.position\n \t vec4\n \t Specifies the position of a light source (x, y, z, w).\n The vector does not need to be normalized. The w component is used to distinguish between directional (0.0) and positional light sources.\n This is (0.0, 0.0, 1.0, 0.0) by default.\n \t *.spotDirection\n \t vec3\n \t Specifies a spotlight direction (x, y, z).\n The vector does not need to be normalized.\n (0.0, 0.0, -1.0) by default.\n \t *.shadowed\n \t bool\n \t Specifies whether a light is affected by shadows.\n Specify GL_TRUE to set light to be affected by shadows, and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.geomFactor0\n \t bool\n \t Specifies whether to use geometry factor 0 in lighting calculations.\n Specify GL_TRUE to use geometry factor 0 in lighting calculations, and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.geomFactor1\n \t bool\n \t Specifies whether to use geometry factor 1 in lighting calculations.\n Specify GL_TRUE to use geometry factor 1 in lighting calculations, and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.twoSideDiffuse\n \t bool\n \t Specifies whether to use two-sided lighting.\n Specify GL_TRUE to use two-sided lighting, and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.spotEnabled\n \t bool\n \t Specifies whether to attenuate a spotlight based on its light distribution.\n Specify GL_TRUE to attenuate a spotlight based on its light distribution, and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.distanceAttenuationEnabled\n \t bool\n \t Specifies whether to attenuate a light over distance.\n Specify GL_TRUE to attenuate a spotlight based on its light distribution, and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.distanceAttenuationBias\n \t float\n \t Specifies the distance attenuation bias.\n 0.0 by default.\n \t *.distanceAttenuationScale\n \t float\n \t Specifies the distance attenuation scale.\n 1.0 by default.\n \t *.samplerXX\n (XX=SP,DA)\n \t int\n \t Specifies the lookup table to use for calculating spotlight attenuation and distance attenuation.\n This is a number between 0 and 31 for each factor.\n The table uses asterisks (*) to indicate dmp_FragmentLightSource[i]. i specifies the light number (0 to 7).\n",0]
,["Pages/Page_55588844.html#PageId_55588853","12.2.6. Lighting Environment ",
"\n12.2.6. Lighting Environment\nThe reserved uniforms dmp_LightEnv.* configure settings related to general lighting, including shadow texture selection, bump map settings, and lookup table input.\n12.2.7. Equations for the Primary Color and 12.2.8. Equations for the Secondary Color explain how the settings are used in lighting calculations.\nTable 12-6. Reserved Uniforms for the Lighting Environment\n \t Reserved Uniform\n \t Type\n \t Setting Value\n \t *.absLutInputXX\n (XX=D0,D1,RR,RG,RB,FR,SP)\n \t bool\n \t Specifies whether to convert lookup table input for each factor into absolute values.\n Specify GL_TRUE to convert lookup table input for each factor into absolute values, and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.lutInputXX\n (XX=D0,D1,RR,RG,RB,FR,SP)\n \t int\n \t Specifies the cosine of the angle between two vectors to use as lookup table input for each factor.\n GL_LIGHT_ENV_NH_DMP: The normal and half vectors (default).\nGL_LIGHT_ENV_VH_DMP: The view and half vectors.\nGL_LIGHT_ENV_NV_DMP: The normal and view vectors.\nGL_LIGHT_ENV_LN_DMP: The light and normal vectors.\nGL_LIGHT_ENV_SP_DMP: The light and spotlight vectors.\nGL_LIGHT_ENV_CP_DMP: The tangent vector and the projection of the half vector onto the tangent plane.\n \t *.lutScaleXX\n (XX=D0,D1,RR,RG,RB,FR,SP)\n \t float\n \t Specifies the scale to apply to each factor's lookup table output. After applying a scale value to the value output by the lookup table, clamping is performed within a range from -2.0 through 2.0.\n 0.25\n0.5\n1.0 (default)\n2.0\n4.0\n8.0\n \t *.shadowSelector\n \t int\n \t Specifies the texture unit to use for shadows.\n GL_TEXTURE0 (default)\nGL_TEXTURE1\nGL_TEXTURE2\nGL_TEXTURE3\n \t *.bumpSelector\n \t int\n \t Specifies the texture unit to use for bump maps.\n GL_TEXTURE0 (default)\nGL_TEXTURE1\nGL_TEXTURE2\nGL_TEXTURE3\n \t *.bumpMode\n \t int\n \t Specifies the perturbation mode for normal and tangent vectors.\n GL_LIGHT_ENV_BUMP_NOT_USED_DMP: No bump mapping (default).\nGL_LIGHT_ENV_BUMP_AS_BUMP_DMP: Perturb normal vectors.\nGL_LIGHT_ENV_BUMP_AS_TANG_DMP: Perturb tangent vectors.\n \t *.bumpRenorm\n \t bool\n \t Specifies whether to regenerate the third component of the normal vector.\n Specify GL_TRUE to regenerate the third component of the normal vector, and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.config\n \t int\n \t Specifies the configuration for each factor.\n GL_LIGHT_ENV_LAYER_CONFIG0_DMP (default)\nGL_LIGHT_ENV_LAYER_CONFIG1_DMP\nGL_LIGHT_ENV_LAYER_CONFIG2_DMP\nGL_LIGHT_ENV_LAYER_CONFIG3_DMP\nGL_LIGHT_ENV_LAYER_CONFIG4_DMP\nGL_LIGHT_ENV_LAYER_CONFIG5_DMP\nGL_LIGHT_ENV_LAYER_CONFIG6_DMP\nGL_LIGHT_ENV_LAYER_CONFIG7_DMP\n \t *.invertShadow\n \t bool\n \t Specifies whether to invert the shadow term (1.0 - shadow).\n Specify GL_TRUE to invert the shadow term, and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.shadowPrimary\n \t bool\n \t Specifies whether to apply shadows to the primary color.\n Specify GL_TRUE to apply shadows to the primary color, and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.shadowSecondary\n \t bool\n \t Specifies whether to apply shadows to the secondary color.\n Specify GL_TRUE to apply shadows to the primary color, and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.shadowAlpha\n \t bool\n \t Specifies whether to apply shadows to the alpha component.\n Specify GL_TRUE to apply shadows to the primary color, and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.fresnelSelector\n \t int\n \t Specifies the output mode for the Fresnel factor.\n GL_LIGHT_ENV_NO_FRESNEL_DMP (default)\nGL_LIGHT_ENV_PRI_ALPHA_FRESNEL_DMP\nGL_LIGHT_ENV_SEC_ALPHA_FRESNEL_DMP\nGL_LIGHT_ENV_PRI_SEC_ALPHA_FRESNEL_DMP\n \t *.clampHighlights\n \t bool\n \t Specifies whether to clamp the specular output value.\n Specify GL_TRUE to clamp the specular output value, and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.lutEnabledD0\n \t bool\n \t Specifies whether to apply output values from the lookup table for distribution 0 (D0).\n Specify GL_TRUE to apply output values from the lookup table for distribution 0 (D0), and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.lutEnabledD1\n \t bool\n \t Specifies whether to apply output values from the lookup table for distribution 1 (D1).\n Specify GL_TRUE to apply output values from the lookup table for distribution 0 (D0), and GL_FALSE otherwise. GL_FALSE is the default.\n \t *.lutEnabledRefl\n \t bool\n \t Specifies whether to apply output values from the lookup tables for reflection (RR, RG, RB).\n Specify GL_TRUE to apply output values from the lookup table for distribution 0 (D0), and GL_FALSE otherwise. GL_FALSE is the default.\n The table uses asterisks (*) to indicate dmp_LightEnv.\n",0]
,["Pages/Page_55588844.html#PageId_55588854","12.2.7. Equations for the Primary Color ",
"\n12.2.7. Equations for the Primary Color\nThe following formula summarizes how the primary color is calculated.\nColorprimary = ∑((Diffuse × DPLN × Shadow + Ambient) × Spot × DistAtt + Ambientglobal + Emission\nThe following paragraphs provide more information about how each term is calculated.\nThe product of the material and light color components is applied to the diffuse and ambient light.\nDiffuse = Diffusematerial × Diffuselight\nAmbient = Ambientmaterial × Ambientlight\nThe diffuse light is affected by shadows and the angle of incident light.\nDPLN = max { 0, L∙N } or abs ( L∙N )\nThe effect of the angle of incident light is given by DPLN in the equation. This is the dot product of a normalized light vector and normal vector. For one-sided lighting this is the larger of 0 and the dot product, and for two-sided lighting this is the absolute value of the dot product. To enable one-sided or two-sided lighting, set dmp_FragmentLightSource[i].twoSideDiffuse to GL_FALSE or GL_TRUE respectively.\nThe effect of shadow attenuation is represented by Shadow in the equation. If either dmp_LightEnv.shadowPrimary or dmp_FragmentLightSource[i].shadowed is GL_FALSE, shadows have no effect and a value of 1.0 is applied. If either reserved uniform is GL_TRUE, the value used is sampled from the texture unit specified by dmp_LightEnv.shadowSelector. If dmp_LightEnv.invertShadow is GL_TRUE, the value used is actually the sampled value subtracted from 1.0. If any texture other than a shadow texture is bound to the specified texture unit, color components are applied unchanged to the sampled value.\nThe effect of spotlights is represented by Spot in the equation. You can set a lookup table for each light and configure whether it is used. Use dmp_FragmentLightSource[i].spotEnabled to configure whether spotlights are used and dmp_FragmentLightSource[i].samplerSP to set the lookup tables used by spotlights. Use dmp_FragmentLightSource[i].spotDirection to set the spotlight direction vector. A value of GL_LIGHT_ENV_SP_DMP is usually specified as the lookup table input for spotlights (dmp_LightEnv.lutInputSP).\nThe effect of distance attenuation is represented by DistAtt in the equation. Directional light sources are unaffected by distance attenuation.\nThe reserved uniform dmp_FragmentLightSource[i].distanceAttenuationEnabled configures whether distance attenuation is applied. This can be controlled for each light. However, the effect of distance attenuation is disabled (1.0 is applied) when dmp_LightEnv.config is GL_LIGHT_ENV_LAYER_CONFIG7_DMP.\nThe lookup table to use is set by dmp_FragmentLightSource[i].samplerDA. Lookup table input is affected by the values set for the scale (dmp_FragmentLightSource[i].distanecAttenuationScale) and bias (dmp_FragmentLightSource[i].distanceAttenuationBias).\nThis section has so far described the effect of each light on a fragment's primary color. This effect is calculated only for valid lights and then it is added to the material's emissive color and the global ambient color, which are unaffected by lights, to compute the fragment's final primary color.\nThe global ambient light is the product of the material's ambient light and the scene's ambient light.\nAmbientglobal = Ambientmaterial × Ambientscene\nWhen light source 0 is disabled (dmp_FragmentLightSource[0].enabled is GL_FALSE), however, a value of 0.0 is applied to the global ambient light and the material's emissive light.\n",0]
,["Pages/Page_55588844.html#PageId_55588855","12.2.8. Equations for the Secondary Color ",
"\n12.2.8. Equations for the Secondary Color\nThe following formula summarizes how the secondary color is calculated.\nColorsecondary = ∑((Specular0 + Specular1) × f × Shadow × Spot × DistAtt)\nThe following paragraphs provide more information about how each term is calculated.\nApart from the fact that dmp_LightEnv.shadowSecondary settings are used instead of dmp_LightEnv.shadowPrimary settings, Shadow, Spot, and DistAtt are each calculated in the same way that they are calculated for the primary color.\nSpecular0 and Specular1 are each calculated as follows.\nSpecular0 = Specular0material × Specular0light × Distribution0 × Geometry0\nSpecular1 = ReflectionRGB × Specular1light × Distribution1 × Geometry1\nThe specular term is usually calculated as the product of the specular property of a material, the specular color of a light, a distribution function, and a geometry factor. Some settings allow output from lookup tables that define different reflections for each color to be applied to the term that corresponds to a material's specular light 1. By adjusting the reflection and distribution lookup tables, you can represent fragments with a variety of different textures.\nDistribution functions (factors) are represented by Distribution0 and Distribution1 in the equation. The distribution functions are configured through lookup tables for distribution 0 (D0) and distribution 1 (D1).\nThe reserved uniform dmp_LightEnv.lutEnabledD0 (lutEnabledD1) controls whether these functions are used. The reserved uniform dmp_FragmentLightSource[i].samplerD0 (samplerD1) specifies the lookup table number to use. The reserved uniforms dmp_LightEnv.lutInputD0 and dmp_LightEnv.lutInputD1 specify the lookup table input. dmp_LightEnv.absLutInputD0 and dmp_LightEnv.absLutInputD1 specify the absolute value of the input. Lookup table output accounts for the scale values specified by dmp_LightEnv.lutScaleD0 and dmp_LightEnv.lutScaleD1.\nReflections are represented by ReflectionRGB in the equation. They use lookup tables (RR, RG, RB) to set functions that calculate the reflection for each RGB component instead of a material's specular light 1. The reserved uniform dmp_LightEnv.lutEnabledRefl controls whether this feature is used. If GL_FALSE is specified, the material's specular color 1 is applied. The reserved uniform dmp_FragmentLightSource[i].samplerRR (samplerRG, samplerRB) specifies the lookup table number to use. The reserved uniforms dmp_LightEnv.lutInputXX specify the lookup table input and dmp_LightEnv.absLutInputXX specify the absolute value of the input (where XX is RR, RG, or RB).\nGeometry factors are represented by Geometry0 and Geometry1 in the equation. They are used by the Cook-Torrance lighting model. The reserved uniforms dmp_FragmentLightSource[i].geomFactor0 and dmp_FragmentLightSource[i].geomFactor1 control whether they are used. A value of 1.0 is applied for a setting of GL_FALSE and an approximation of the geometry factors used in the Cook-Torrance lighting model is applied for a setting of GL_TRUE.\nf is a function that uses the dot product of the normalized light vector and normal vector to determine whether lighting is enabled for a fragment. A value of 1.0 is always applied when dmp_LightEnv.clampHighlights is set to GL_FALSE. This is used to represent translucent objects that allow light to pass through them to areas where it would not otherwise reach. If GL_TRUE is specified, a value of 0.0 is applied when the dot product is 0.0 or less and a value of 1.0 is applied when the dot product is greater than 0.0. In this case, unlit areas have no specular light.\nA fragment's final secondary color is calculated from the effect of each valid light's effect on it.\n",0]
,["Pages/Page_55588844.html#PageId_55588856","12.2.9. Alpha Component Lighting ",
"\n12.2.9. Alpha Component Lighting\nThe previous sections have explained how to calculate the primary and secondary colors, with the alpha component fixed at 1.0. Fragment lighting allows you to apply Fresnel factors and shadows to the alpha component.\nFresnel factors were originally intended to be used as lookup tables (FR) for Fresnel reflections in translucent objects, but by replacing the alpha component with lookup table output, they can also be used for other purposes.\nThe reserved uniform dmp_FragmentLightSource[i].samplerFR specifies the number of the lookup table to use for the Fresnel factors. The reserved uniform dmp_LightEnv.lutInputFR specifies the lookup table input and dmp_LightEnv.absLutInputFR specifies the absolute values of the input. If multiple lights have been enabled, the light with the largest number has its light vector used in the dot product that is input to the lookup table. The reserved uniform dmp_LightEnv.fresnelSelector uses the following values to control the extent to which Fresnel factors are applied.\nTable 12-7. Scope of the Applied Fresnel Factor\n \t Value\n \t Applies To\n \t GL_LIGHT_ENV_NO_FRESNEL_DMP (default)\n \t Nothing.(Fx the alpha component at 1.0.)\n \t GL_LIGHT_ENV_PRI_ALPHA_FRESNEL_DMP\n \t Only the alpha component for the primary color.\n \t GL_LIGHT_ENV_SEC_ALPHA_FRESNEL_DMP\n \t Only the alpha component for the secondary color.\n \t GL_LIGHT_ENV_PRI_SEC_ALPHA_FRESNEL_DMP\n \t The alpha component for the primary and secondary colors.\n The Fresnel factor is also applied to the alpha component for shadows. The shadow alpha component is multiplied with the applied alpha component when a value of GL_TRUE is specified for dmp_LightEnv.shadowAlpha, which controls the effect on the shadow alpha component. If dmp_LightEnv.invertShadow is GL_TRUE, the shadow alpha value is subtracted from 1.0 (as it is for colors) before being multiplied.\n",0]
,["Pages/Page_55588844.html#PageId_55588857","12.2.10. Creating and Specifying Lookup Tables ",
"\n12.2.10. Creating and Specifying Lookup Tables\nLighting equations use the following eight types of lookup tables.\n\tReflections (three types: RR, RG, and RB) \tDistribution factors (two types: D0 and D1) \tFresnel factors (FR) \tSpotlights (SP) \tDistance attenuation of light\nThe lookup tables for reflections (RR, RG, RB), distribution factors (D0, D1), and Fresnel factors are all material settings, and are common to all lights. The lookup tables for spotlights (SP) and the distance attenuation of light (DA) can be set differently for each light.\nThe layer configuration (dmp_LightEnv.config) can control which lookup tables are used for each term in the secondary color's lighting equation, except for the distance attenuation of light.\nTable 12-8. Lookup Tables and Number of Cycles for Each Layer Configuration\n \t Layer Configuration\n \t Rr\n \t Rg\n \t Rb\n \t D0\n \t D1\n \t Fr\n \t Sp\n \t Cycles\n \t GL_LIGHT_ENV_LAYER_CONFIG0_DMP\n \t RR\n \t RR\n \t RR\n \t D0\n \t -\n \t -\n \t SP\n \t 1\n \t GL_LIGHT_ENV_LAYER_CONFIG1_DMP\n \t RR\n \t RR\n \t RR\n \t -\n \t -\n \t FR\n \t SP\n \t 1\n \t GL_LIGHT_ENV_LAYER_CONFIG2_DMP\n \t RR\n \t RR\n \t RR\n \t D0\n \t D1\n \t -\n \t -\n \t 1\n \t GL_LIGHT_ENV_LAYER_CONFIG3_DMP\n \t -\n \t -\n \t -\n \t D0\n \t D1\n \t FR\n \t -\n \t 1\n \t GL_LIGHT_ENV_LAYER_CONFIG4_DMP\n \t RR\n \t RG\n \t RB\n \t D0\n \t D1\n \t -\n \t SP\n \t 2\n \t GL_LIGHT_ENV_LAYER_CONFIG5_DMP\n \t RR\n \t RG\n \t RB\n \t D0\n \t -\n \t FR\n \t SP\n \t 2\n \t GL_LIGHT_ENV_LAYER_CONFIG6_DMP\n \t RR\n \t RR\n \t RR\n \t D0\n \t D1\n \t FR\n \t SP\n \t 2\n \t GL_LIGHT_ENV_LAYER_CONFIG7_DMP\n \t RR\n \t RG\n \t RB\n \t D0\n \t D1\n \t FR\n \t SP\n \t 4\n The table shows which lookup tables are used to get values for a reflection's RGB components, the distribution factors, the Fresnel factors, and the spotlight term. A value of 1.0 is applied to the lighting equation for any cell that contains a hyphen (-). In other words, the corresponding term disappears from the equation. The Cycles column shows the number of hardware cycles required for lighting calculations. To speed up lighting calculations, choose a layer configuration that minimizes this number.\n Warning:\n When only write access to the color buffer has been configured (when the glColorMask function has set a value of GL_TRUE for all color buffer components and the glDisable function has disabled GL_BLEND and GL_COLOR_LOGIC_OP), layer configurations from GL_LIGHT_ENV_LAYER_CONFIG4_DMP through GL_LIGHT_ENV_LAYER_CONFIG6_DMP require three rather than two cycles to process a single pixel.\n A setting of GL_LIGHT_ENV_LAYER_CONFIG7_DMP disables the effect of distance attenuation when the primary and secondary color are calculated.\nFor example, when GL_LIGHT_ENV_LAYER_CONFIG0_DMP is set, all of the reflection RGB components are obtained from the RR lookup table, the distribution 0 values are obtained from the D0 lookup table, and the spotlight values are obtained from the SP lookup table. A fixed value of 1.0 is applied for distribution 1 and the Fresnel factor.\nAs explained in 7.7. Loading Lookup Tables, lookup tables are prepared by the glTexImage1D function. The lookup tables used for fragment lighting have a fixed width of 512 elements. The first 256 elements store the lookup table's sampling values and the last 256 elements store the differences between each of the sampling values.\nThe order that sampling values are stored in depends on whether lookup table input is between 0.0 and 1.0 or between –1.0 and 1.0. The reserved uniforms dmp_LightEnv.absLutInputXX (where XX is D0, D1, RR, RG, RB, FR, or SP) set the range of input values from 0.0 to 1.0 when GL_TRUE is specified or from –1.0 to 1.0 when GL_FALSE is specified.\nProcedures for getting sampling values are explained next, followed by the corresponding procedures for storing them.\nIf the range of input values is between 0.0 and 1.0, each input value is multiplied by 256 and then clamped to 255. The integer portion of this number is the index for getting values. The index is first used to get a sampling value from the lookup table, and then it is incremented by 256 to get a difference value. The difference value is multiplied by the fractional portion of the input value, and then added to the original sampling value to find the final sampling value.\nCode 12-1. Procedure to Get Sampling Values for Input Between 0.0 and 1.0 (Pseudocode)\n index=min(floor(input * 256), 255);\nsamplingValue=LUT[index] + LUT[index + 256] * (input * 256 - index);\nIf the range of input values is between –1.0 and 1.0, each input value is multiplied by 128 and then its integer portion is converted to a two's complement index. The index is first used to get a sampling value from the lookup table, and then it is incremented by 256 to get a difference value. The difference value is multiplied by the fractional portion of the input value, and then added to the original sampling value to find the final sampling value.\nCode 12-2. Procedure to Get Sampling Values for Input Between –1.0 and 1.0 (Pseudocode)\n if (input < 0.0) {\n flooredInput=floor(input * 128);\n index = 255 + flooredInput;\n samplingValue=LUT[index] + LUT[index + 256] * (input * 128 - flooredInput);\n} else {\n index=min(floor(input * 128), 127);\n samplingValue=LUT[index] + LUT[index + 256] * (input * 128 - index);\n}\nFigure 12-3 shows the order that sampling values are stored in.\nFigure 12-3. Order of Sampling Values Stored in the Lookup Table\nLookup tables are usually created by storing the sampling value that results from dividing the index by 256 or 128. The difference between each sampling value and the next is stored 256 indices later. Note that the lookup table is discontinuous for input values between –1.0 and 1.0.\nThe following pseudocode samples illustrate this process using func as the function that calculates sampling values from input values.\nCode 12-3. Creating a Lookup Table for Input Between 0.0 and 1.0 (Pseudocode)\n for (i = 0; i < 256; i++) LUT[i] = func((float) i / 256.0f);\nfor (i = 0; i < 255; i++) LUT[i + 256] = LUT[i + 1] - LUT[i];\nLUT[511] = func(1.0f) - LUT[255] * 16.0f / 15.0f;\nThe difference value for an input of 1.0 (the last difference element) is multiplied by 16.0/15.0 because the GPU has a fractional precision of 4 bits. If the original difference value were stored, the highest input value would not produce the sampling value for 1.0.\nCode 12-4. Creating a Lookup Table for Input Between –1.0 and 1.0 (Pseudocode)\n for (i = 0; i < 128; i++) {\n LUT[i] = func((float) i / 128.0f);\n LUT[255 - i] = func((float) (i + 1) * -1.0f / 128.0f);\n}\nfor (i= 0; i < 127; i++) {\n LUT[i + 256] = LUT[i + 1] - LUT[i];\n LUT[i + 384] = LUT[i + 129] - LUT[i + 128];\n}\nLUT[383] = func(1.0f) - LUT[127] * 16.0f / 15.0f;\nLUT[511] = LUT[0] - LUT[255];\nIn this example, the value is multiplied by 16.0/15.0 because, if the original difference value were stored, the highest input value would not produce the sampling value for 1.0.\nAs explained in the lighting equations, after you have created a lookup table and bound it to a texture type for lookup tables, load it with the glTexImage1D function and set its lookup table number in a reserved uniform to allow calculations to access it. Note that the glUniform1i function sets the lookup table number, not the texture ID, in the reserved uniform.\nCode 12-5. Specifying the Reflection Lookup Table (for the R Component)\n glBindTexture(GL_LUT_TEXTURE2_DMP, lutTextureID);\nglTexImage1D(GL_LUT_TEXTURE2_DMP, 0, GL_LUMINANCEF_DMP, 512, 0,\n GL_LUMINANCEF_DMP, GL_FLOAT, LUT);\nglUniform1i(glGetUniformLocation(progID, \"dmp_LightEnv. samplerRR\"), 2);\n",0]
,["Pages/Page_55588844.html#PageId_55588858","12.2.11. Lookup Table Input ",
"\n12.2.11. Lookup Table Input\nAll terms (D0, D1, RR, RG, RB, FR, SP), except for the distance attenuation of light (DA), take as input the cosine of the angle between two vectors (the dot product of two normalized vectors).\nThe input vectors are the normal vector (N), light vector (L), view vector (V), half vector (H), tangent vector (T), binormal vector (B), spotlight direction vector, and the projection of the half vector onto the tangent plane.\nFigure 12-4. Vectors Used as Lookup Table Input\nTo specify the values used as lookup table input for each factor, set the reserved uniforms for the lighting environment, dmp_LightEnv.lutInputXX (where XX is D0, D1, RR, RG, RB, FR, or SP), to one of the following values using the glUniform1i function. The cosine of the angle between the two specified vectors is used as lookup table input for each factor.\nTable 12-9. Settings for Lookup Table Input\n \t Value\n \t Input Vector Pair\n \t GL_LIGHT_ENV_NH_DMP\n \t The normal and half vectors (default).\n \t GL_LIGHT_ENV_VH_DMP\n \t The view and half vectors.\n \t GL_LIGHT_ENV_NV_DMP\n \t The normal and view vectors.\n \t GL_LIGHT_ENV_LN_DMP\n \t The light and normal vectors.\n \t GL_LIGHT_ENV_SP_DMP\n \t The inverse light vector and the spotlight vector (cannot be used with RR, RG, RB, and FR).\n \t GL_LIGHT_ENV_CP_DMP\n \t The tangent vector and the projection of the half vector onto the tangent plane (cannot be used with RR, RG, RB, and FR).\n ",0]
,["Pages/Page_55588844.html#PageId_55588859","12.2.12. Distance Attenuation of Light ",
"\n12.2.12. Distance Attenuation of Light\nThe following equation calculates input values for the distance attenuation of light.\nfposition is the fragment position and lposition is the light position. Both are expected to use eye coordinates. Position only has meaning for a point light source. It means nothing for a directional light source.\nThis equation shows that the distance between a fragment and a light is multiplied by a scale value, and then added to a bias value to calculate an input value for the distance attenuation of light.\nCreate lookup tables that take input between 0.0 and 1.0 (absolute values). Use the glUniform1f function to set the scale value in the reserved uniform dmp_FragmentLightSource[i].distanceAttenuationScale and the bias value in dmp_FragmentLightSource[i].distanceAttenuationBias.\nWhen GL_LIGHT_ENV_LAYER_CONFIG7_DMP is specified as the layer configuration’s setting value, you must disable distance attenuation. Set dmp_FragmentLightSource[i].distanceAttenuationEnabled to GL_FALSE.\n",0]
,["Pages/Page_55588844.html#PageId_55588860","12.2.13. Texture Combiner Settings ",
"\n12.2.13. Texture Combiner Settings\nThe primary and secondary colors calculated by fragment lighting can each be used as an input source to a texture combiner.\nTo use them, set the reserved uniforms for the input source (dmp_TexEnv[i].srcRgb and dmp_TexEnv[i].srcAlpha) to GL_FRAGMENT_PRIMARY_COLOR_DMP for the primary color, and to GL_FRAGMENT_SECONDARY_COLOR_DMP for the secondary color.\nThe output value is (0.0, 0.0, 0.0, 1.0) when lighting is disabled (when dmp_FragmentLighting.enabled is GL_FALSE).\n",0]
,["Pages/Page_55588844.html#PageId_55588861","12.3. Bump Mapping ",
"\n12.3. Bump Mapping\n",0]
,["Pages/Page_55588844.html#PageId_55588862","12.3.1. Reserved Uniform ",
"Bump mapping is a feature of fragment lighting that perturbs (alters) a fragment's normal and tangent vectors according to a normal map that is input as a texture. Bump mapping can make an object appear to have shadows caused by surface irregularities. This allows you to render a simple model that looks complex but actually has a small polygon count.\n12.3.1. Reserved Uniform\nThe following reserved uniforms are used for bump mapping.\n",0]
,["Pages/Page_55588844.html#Anchor_55588862_法線マップ","Normal Maps ",
"Normal Maps\nFor the normal map texture for bump mapping (dmp_LightEnv.bumpSelector), specify the texture unit to which the texture is bound. A normal map texture is created with the x, y, and z components of the perturbation vectors encoded in the R, G, and B components, respectively. A vector value of -1.0 is encoded as the minimum luminance (0 in an 8-bit format), and 1.0 is encoded as the maximum luminance (255 in an 8-bit format).\n",0]
,["Pages/Page_55588844.html#Anchor_55588862_摂動方法","Perturbation Mode ",
"Perturbation Mode\nTo enable bump mapping, set the perturbation mode (dmp_LightEnv.bumpMode) to any value other than GL_LIGHT_ENV_BUMP_NOT_USED_BUMP. The following table shows the perturbation modes.\nTable 12-10. Perturbation Modes\n \t Perturbation Mode\n \t Perturbed Vectors\n \t GL_LIGHT_ENV_BUMP_NOT_USED_BUMP\n \t None.\n \t GL_LIGHT_ENV_BUMP_AS_BUMP_DMP\n \t Normal vectors (bump mapping).\n \t GL_LIGHT_ENV_BUMP_AS_TANG_DMP\n \t Tangent vectors (tangent mapping).\n ",0]
,["Pages/Page_55588844.html#Anchor_55588862_法線の再計算機能","Normal Recalculation ",
"Normal Recalculation\nIf dmp_LightEnv.bumpRenorm is set to GL_TRUE to enable recalculation of normal vectors, the z-component of the normal vector is not obtained from the B component sampled from a texture. Instead, the z-component is recalculated from the x-component and the y-component.\n Note: If the expression inside the square root is negative, the result is 0.\nIn most cases, recalculating values yields better results than sampling them from a texture. This recalculation feature must be enabled, if bump mapping (of normals) uses a texture that only has R and G components (GL_HILO8_DMP). However, if you have selected tangent mapping as the perturbation mode for use with a technique such as anisotropic reflections, we recommend that you avoid using this feature. This is because tangent mapping (for fragment lighting) expects input perturbation tangents that do not have a z-component. If the recalculation feature is enabled, nonzero values may be generated in the z-component.\nIf recalculation is disabled, the perturbation normal vectors sampled from the texture are not normalized before they are used. Make sure that you normalize values before storing them in a texture. Re-enable normal calculation when point sampling (GL_NEAREST) is not configured as the texture filter mode because filtering can cause the non-normalized values to be used as the perturbation normals.\nTable 12-11. Reserved Uniforms Used for Bump Mapping\n \t Reserved Uniform\n \t Type\n \t Setting Value\n \t dmp_LightEnv.bumpSelector\n \t int\n \t Specifies the texture unit to use for the normal map.\n GL_TEXTURE0 (default)\nGL_TEXTURE1\nGL_TEXTURE2\nGL_TEXTURE3\n \t dmp_LightEnv.bumpMode\n \t int\n \t Specifies the perturbation mode for normal and tangent vectors.\n GL_LIGHT_ENV_BUMP_NOT_USED_DMP: No bump mapping (default).\nGL_LIGHT_ENV_BUMP_AS_BUMP_DMP: Perturb normal vectors.\nGL_LIGHT_ENV_BUMP_AS_TANG_DMP: Perturb tangent vectors.\n \t dmp_LightEnv.bumpRenorm\n \t bool\n \t Specifies whether to regenerate the third component of the normal vector.\n Specify GL_TRUE to regenerate the third component of the normal vector, and GL_FALSE otherwise. GL_FALSE is the default.\n",0]
,["Pages/Page_55588844.html#PageId_55588866","12.4. Shadows ",
"\n12.4. Shadows\n",0]
,["Pages/Page_55588844.html#PageId_55588868","12.4.1. Shadow Accumulation Pass ",
"3DS shadows are rendered in two passes. First, the shadow accumulation pass creates a shadow buffer (that contains the scene's depth values taking the light source as the origin), which is then referenced by the shadow lookup pass to cast shadows. The shadow intensity information collected, along with the depth values in the first pass, allows you to represent soft shadows.\n12.4.1. Shadow Accumulation Pass\nThe shadow accumulation pass requires that the fragment operation mode (dmp_FragOperation.mode) be switched to shadow mode (GL_FRAGOP_MODE_SHADOW_DMP), and that the shadow information (depth values and shadow intensity) be stored in a shadow texture (with a format of GL_SHADOW_DMP and a type of GL_UNSIGNED_INT). Note that only texture unit 0 (GL_TEXTURE0) can write shadow information to a shadow texture. Also note that mipmaps cannot be applied to shadow textures.\nWhen the fragment pipeline switches to shadow mode, shadow information is output to the attachment point for the color buffer rather than the depth or stencil buffer. As a result, a shadow texture must be attached to the color buffer's attachment point (GL_COLOR_ATTACHMENT0). Render targets attached to depth and stencil attachment points are ignored in shadow mode. The alpha and stencil tests are skipped.\nYou can use the following procedure to create a shadow texture and specify a render target.\nCode 12-6. Creating a Shadow Texture and Specifying a Render Target\n glActiveTexture(GL_TEXTURE0);\nglGenTextures(1, &shadowTexID);\nglBindTexture(GL_TEXTURE_2D, shadowTexID);\nglTexImage2D(GL_TEXTURE_2D, 0, GL_SHADOW_DMP, shadowWidth, shadowHeight, 0,\n GL_SHADOW_DMP, GL_UNSIGNED_INT, 0);\nglGenFramebuffers(1, &shadowFboID);\nglBindFramebuffer(GL_FRAMEBUFFER, shadowFboID);\nglFramebufferTexture2D(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_TEXTURE_2D,\n shadowTexID, 0);\nShadow information is accumulated using the coordinate system of the light source. Shadow information comprises the depth from the light source (the depth values) and the shadow intensity. When rendering shadows, there are no shadows wherever the G component of the color information is 1.0 (the R, B, and A components have no effect), there are opaque hard shadows wherever the G component of the color information is 0.0, and there are non-opaque shadows (soft shadows) everywhere else.\nWhen opaque hard shadows are rendered, only the shadow depth values are updated. The shadow intensity does not change. The depth of a fragment and its corresponding pixel in the shadow buffer are compared (using GL_LESS). If the fragment has a smaller value, the depth value in the shadow buffer is updated.\nWhen non-opaque soft shadows are rendered, only the shadow intensity is updated. The shadow depth values do not change. The depth of a fragment and its corresponding pixel in the shadow buffer are compared (using GL_LESS). If the fragment has a smaller value, the shadow intensity is also compared (using GL_LESS) and then, if the fragment still has a smaller value, the shadow intensity in the shadow buffer is updated.\nWhen you initialize the color buffer in the shadow accumulation pass you must set the clear color to (1.0, 1.0, 1.0, 1.0) by using the glClearColor function and specify GL_COLOR_BUFFER_BIT to the glClear function. Note that you must set all color components (R, G, B, and A)—not just the G component—equal to 1.0 in the clear color.\nThe shadow lookup pass is processed in eye coordinates, so the depth values must be created by using linear interpolation in eye space. (In most cases this differs from OpenGL, which uses non-linear relationships.) As a result, you must set a value in the reserved uniform for the w-buffer's scale factor (dmp_FragOperation.wScale), using the glUniform1f function. This has an initial value of 0.0, which results in the same non-linear relationship as OpenGL. Depth values have a lower valid precision around the far clipping plane, when the near clipping plane is close to the viewpoint. To use linear interpolation, given f as the clip value for the far clipping plane, set the scale factor to 1.0/f for a perspective projection or to 1.0 for an orthographic projection.\nAn object casts a hard shadow if it is rendered with a G color component of 0.0. This is generally implemented by disabling textures, and then implementing a vertex shader that outputs those vertex colors with G components of 0.0. Every other rendered color is treated as a soft shadow.\nSeveral rendering passes may be necessary to accumulate the required shadow information. Information for non-opaque shadows must be accumulated after information for opaque shadows. Results are not guaranteed if information is accumulated in the opposite order or in alternating order. When light sources do not move, you can generate shadow textures more efficiently by rendering motionless objects alone to a shadow texture ahead of time. Simplifying object shapes and decreasing polygon counts are also effective ways to improve performance.\nTexture unit 0 and the texture combiners are configured as follows, when shadows are rendered using a vertex shader implementation that outputs vertex colors.\nCode 12-7. Sample Settings for Texture Units and Texture Combiners When Rendering Shadows\n glUniform1i(glGetUniformLocation(progID, \"dmp_Texture[0].samplerType\"),\n GL_FALSE);\nglUniform1i(glGetUniformLocation(progID, \"dmp_TexEnv[0].combineRgb\"),\n GL_REPLACE);\nglUniform1i(glGetUniformLocation(progID, \"dmp_TexEnv[0].combineAlpha\"),\n GL_REPLACE);\nglUniform3i(glGetUniformLocation(progID, \"dmp_TexEnv[0].operandRgb\"),\n GL_SRC_COLOR, GL_SRC_COLOR, GL_SRC_COLOR);\nglUniform3i(glGetUniformLocation(progID, \"dmp_TexEnv[0].operandAlpha\"),\n GL_SRC_ALPHA, GL_SRC_ALPHA, GL_SRC_ALPHA);\nglUniform3i(glGetUniformLocation(progID, \"dmp_TexEnv[0].srcRgb\"),\n GL_PRIMARY_COLOR, GL_PRIMARY_COLOR, GL_PRIMARY_COLOR);\nglUniform3i(glGetUniformLocation(progID, \"dmp_TexEnv[0].srcAlpha\"),\n GL_PRIMARY_COLOR, GL_PRIMARY_COLOR, GL_PRIMARY_COLOR);\n",0]
,["Pages/Page_55588844.html#PageId_55588869","12.4.2. Shadow Lookup Pass ",
"\n12.4.2. Shadow Lookup Pass\nThe shadow lookup pass requires that the fragment operation mode be switched to normal mode (GL_FRAGOP_MODE_GL_DMP), and that texture unit 0 (GL_TEXTURE0) be configured to reference the shadow texture that has accumulated the shadow information. To reference a shadow texture, you must set the reserved uniform dmp_TexEnv[0].samplerType to GL_TEXTURE_SHADOW_2D_DMP (which specifies shadow textures), and bind the texture that has accumulated shadow information to GL_TEXTURE_2D. When fragment lighting is enabled, its primary and secondary colors are used as texture combiner input. Otherwise, vertex colors and output from texture unit 0 are used.\nThe reserved uniform dmp_Texture[0].perspectiveShadow indicates whether a perspective projection or orthographic projection was applied when the shadow accumulation pass was run. Specify GL_TRUE for perspective projection or GL_FALSE for orthographic projection.\nCode 12-8. Sample Settings When Fragment Lighting Is Disabled\n glUniform1i(glGetUniformLocation(progID, \"dmp_Texture[0].samplerType\"),\n GL_TEXTURE_SHADOW_2D_DMP);\nglUniform1i(glGetUniformLocation(progID, \"dmp_Texture[0].perspectiveShadow\"),\n GL_TRUE);\nglUniform1i(glGetUniformLocation(progID, \"dmp_TexEnv[0].combineRgb\"),\n GL_MODULATE);\nglUniform1i(glGetUniformLocation(progID, \"dmp_TexEnv[0].combineAlpha\"),\n GL_MODULATE);\nglUniform3i(glGetUniformLocation(progID, \"dmp_TexEnv[0].operandRgb\"),\n GL_SRC_COLOR, GL_SRC_COLOR, GL_SRC_COLOR);\nglUniform3i(glGetUniformLocation(progID, \"dmp_TexEnv[0].operandAlpha\"),\n GL_SRC_ALPHA, GL_SRC_ALPHA, GL_SRC_ALPHA);\nglUniform3i(glGetUniformLocation(progID, \"dmp_TexEnv[0].srcRgb\"),\n GL_TEXTURE0, GL_PRIMARY_COLOR, GL_PRIMARY_COLOR);\nglUniform3i(glGetUniformLocation(progID, \"dmp_TexEnv[0].srcAlpha\"),\n GL_TEXTURE0, GL_PRIMARY_COLOR, GL_PRIMARY_COLOR);\nglUniform1i(glGetUniformLocation(progID, \"dmp_FragOperation.mode\"),\n GL_FRAGOP_MODE_GL_DMP);\nglActiveTexture(GL_TEXTURE0);\nglBindTexture(GL_TEXTURE_2D, shadowTexID);\nBoth 3DS and OpenGL reference shadow information from textures. The texture unit compares texture coordinates and shadow information obtained from texels. For this to work properly, the correct texture coordinates must be specified. Note that a shadow texture is referenced using texture coordinates (s/q, t/q, r/q) in OpenGL, and (s/r, t/r, r - bias) on 3DS. The texture transformation matrix has already been applied to texture coordinates (s, t, r, q), and the reserved uniform dmp_Texture[0].shadowZBias specifies the bias value. If shadow information is accumulated by an orthographic projection, texture coordinates s and t are referenced directly. A perspective projection, however, requires adjustments to the texture transformation matrix and bias value.\nThe following texture transformation matrix and bias value could be used to compare the texture coordinates and shadow information accumulated by a perspective projection.\n",0]
,["Pages/Page_55588844.html#Anchor_55588869_透視投影の場合","Texture Transformation Matrix and Bias for Perspective Projection ",
"Texture Transformation Matrix and Bias for Perspective Projection\n",0]
,["Pages/Page_55588844.html#Anchor_55588869_平行投影の場合","Texture Transformation Matrix and Bias for Parallel Projection ",
"Texture Transformation Matrix and Bias for Parallel Projection\nn is the clip value at the near clipping plane, f is the clip value at the far clipping plane, and r and t are the right and top side values of the frustum.\nWhen calculating the value that is compared to the shadow buffer depth (that is, r – bias ), if the texture coordinate r is not within the range from 0.0 to 1.0, r is clamped to between 0.0 and 1.0 before bias is subtracted. To compare values correctly, specify a bias of 0 for any objects placed beyond the far clipping plane in the coordinate system of the light source used during the shadow accumulation pass.\nYou can create a texture transformation matrix with the following OpenGL code.\nCode 12-9. Creating a Texture Transformation Matrix With OpenGL Code\n glMatrixMode(GL_TEXTURE);\nglLoadIdentity();\n// (glFrustum(-r, r, -t, t, n, f))\nglFrustumf(r/n, -3r/n, t/n, -3t/n, 1.0f, 0.0f);\nglScalef(-1.0f/(f-n), -1.0f/(f-n), -1.0f/(f-n));\n// (glOrtho(-r, r, -t, t, n, f))\nglOrthof(-3r, r, -3t, t, 2n-f, f);\nBy setting the border color and texture wrapping mode, you can control the sampling results for texture coordinates that are less than 0.0 or greater than 1.0. The texture wrapping mode guarantees that when GL_CLAMP_TO_BORDER is configured for the s and t texture coordinates, out-of-range sampling values are set to the border color (which must have a value of 0.0 or 1.0 for all components). Sampling results are undefined in the current implementation if the wrapping mode is not GL_CLAMP_TO_BORDER or if the border color is neither (0.0, 0.0, 0.0, 0.0) nor (1.0, 1.0, 1.0, 1.0).\nCode 12-10. Sample Settings for the Wrapping Mode and Border Color\n glBindTexture(GL_TEXTURE_2D, shadowTexID);\nGLfloat bcolor[] = {1.0f, 1.0f, 1.0f, 1.0f};\nglTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, bcolor);\nglTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);\nglTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);\nglTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);\nglTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);\nglTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_LOD, 0);\nIf the depth values in light source coordinates are greater than the depth values in shadow texels, the shadow intensity will be 0.0 after the comparison. Otherwise, the shadow intensity in the shadow texels is used. Texture units output the shadow intensity as a value for each RGBA component.\n",0]
,["Pages/Page_55588844.html#PageId_55588872","12.4.3. Omnidirectional Shadow Mapping ",
"\n12.4.3. Omnidirectional Shadow Mapping\nYou can combine cube mapping with shadow textures to implement omnidirectional shadow mapping.\nThis implementation requires that you bind a cube map texture (one of six types: GL_TEXTURE_CUBE_MAP_POSITIVE_{X,Y,Z} or GL_TEXTURE_CUBE_MAP_NEGATIVE_{X,Y,Z}) to the texture unit during the shadow accumulation pass, render six times, and then specify GL_TEXTURE_SHADOW_CUBE_DMP as the texture to reference during the shadow lookup pass (dmp_Texture[0].samplerType). Also, you must specify GL_CLAMP_TO_EDGE as the texture coordinate wrapping mode to use in both the S and T directions.\n",0]
,["Pages/Page_55588844.html#PageId_55588873","12.4.4. Soft Shadows Using the Silhouette Shader ",
"\n12.4.4. Soft Shadows Using the Silhouette Shader\nShadow textures contain depth values and shadow intensity. The sampling value output by a texture unit that references a shadow texture is either the color black (0.0, 0.0, 0.0), within a shadow region, or the shadow intensity saved in the shadow texture (between 0.0 and 1.0), outside of a shadow region. You can represent soft shadows by setting this shadow intensity to the appropriate values.\nThe 3DS silhouette shader can render silhouette primitives (edges) as soft shadow regions, whose shadow intensity changes gradually until they disappear. Opaque hard shadows are rendered before the silhouette shader is used for rendering. Set the G component of the color at the silhouette edge to 1.0, and set the G component of the vertex color output by the vertex shader to 0.0. All other settings are the same as those used to render opaque hard shadows. This renders the rectangle for a silhouette edge so that its G component gradually changes from 0.0 on the object side to 1.0 on the outer side.\nDuring the shadow accumulation pass for these soft shadow regions, you can apply additive modulation to the shadow intensity, according to the relative distance to an object. This is called an attenuation factor, and it can adjust the width of the soft shadow region through the reserved uniforms for the soft shadow bias (dmp_FragOperation.penumbraBias) and scale (dmp_FragOperation.penumbraScale). By adjusting these values, you can represent more natural soft shadows that narrow close to the object.\nThe attenuation factor is calculated by the following equation. The equation uses Zfrag to represent a fragment's depth value and Zrec to represent the depth value stored in a shadow texture. The shadow intensity is not attenuated properly if objects have not already been rendered and depth values have not already been saved in a shadow texture.\n",0]
,["Pages/Page_55588844.html#PageId_55588874","12.4.5. Handling Shadow Artifacts ",
"\n12.4.5. Handling Shadow Artifacts\n",0]
,["Pages/Page_55588844.html#PageId_55588875","12.4.5.1. Self-Shadow Aliasing ",
"12.4.5.1. Self-Shadow Aliasing\nVarious problems can occur during multiple-pass shadow rendering. For example, by accidentally casting a shadow on itself (called self-shadow aliasing) a fragment can cause a moiré pattern to be rendered. This occurs when depth values from the shadow accumulation pass are slightly smaller than depth values from the shadow lookup pass (measured from the light source). Unnatural shadows such as these are called shadow artifacts.\nOne way to suppress shadow artifacts is to apply a negative offset (bias) to depth values during the shadow lookup pass. You can set the bias value in the reserved uniform dmp_Texture[0].shadowZBias.\nCode 12-11. Sample Settings to Suppress Self-Shadow Aliasing\n glUniform1f(glGetUniformLocation(progID, \"dmp_Texture[0].shadowZBias\"),\n 1.2f*n/(f-n));\n",0]
,["Pages/Page_55588844.html#Anchor_55588875_シルエットのシャドウアーティファクト","Silhouette Shadow Artifacts ",
"Silhouette Shadow Artifacts\nAs explained earlier, the shadow intensity in a shadow texture is output as a sampling value for any location that is determined to be outside of a shadow region during the shadow lookup pass. Although non-shadow regions usually have a shadow intensity of 1.0 and no brightness attenuation, the soft shadow regions rendered by the silhouette shader can have an output shadow intensity that is not 1.0, and thereby have brightness attenuation. This causes shadow artifacts to occur for some objects.\nThe method used to suppress self-shadow aliasing does not work for silhouette shadow artifacts, but these artifacts can still be suppressed through adjustments to shadow settings and texture combiner settings.\nBecause the main cause of these artifacts is the brightness attenuation of a light source by soft shadows on a parallel plane, the shadow texture output (shadow attenuation) can be calculated by using the following equation.\nShadowAttenuation = 1.0 – f (1.0 – ShadowIntensity)\nWhere f is a function that returns a value close to 0.0 when the fragment normals are perpendicular to the light source and 1.0 when the fragment normals are parallel to the light source.\nThis equation yields a shadow attenuation of 1.0 (shadows have no effect) when a fragment's normals are nearly perpendicular to the light source and yields the shadow intensity itself when the normals are nearly parallel to the light source.\nThis shadow attenuation term can be implemented by using the Fresnel factor (lookup table FR) as the f function, reserved uniforms for the lighting environment (dmp_LightEnv.shadowAlpha and dmp_LightEnv.invertShadow) as the inverse of the shadow intensity, and texture combiner settings as the inverse of the final value.\nLighting is configured as follows. Note that the Fresnel factor affects only the alpha component. The alpha component is multiplied by the texture combiners. The f function is configured to return the square of the input value.\nCode 12-12. Sample (Lighting) Settings to Suppress Silhouette Shadow Artifacts\n glUniform1i(glGetUniformLocation(progID, \"dmp_FragmentLighting.enabled\"),\n GL_TRUE);\n// ..other code..\nglUniform1i(glGetUniformLocation(progID, \"dmp_LightEnv.lutInputFR\"),\n GL_LIGHT_ENV_LN_DMP);\nglUniform1i(glGetUniformLocation(progID, \"dmp_LightEnv.config\"),\n GL_LIGHT_ENV_LAYER_CONFIG1_DMP);\nglUniform1i(glGetUniformLocation(progID, \"dmp_LightEnv.fresnelSelector\"),\n GL_LIGHT_ENV_PRI_SEC_ALPHA_FRESNEL_DMP);\nglUniform1i(glGetUniformLocation(progID, \"dmp_LightEnv.shadowAlpha\"),\n GL_TRUE);\nglUniform1i(glGetUniformLocation(progID, \"dmp_LightEnv.invertShadow\"),\n GL_TRUE);\nGLfloat lut[512];\nint j;\nmemset(lut, 0, sizeof(lut));\nfor (j = 1; j < 128; j++)\n{\n lut[j] = powf((float)j/127.0f, 2.0f);\n lut[j+255] = lut[j] - lut[j-1];\n}\nglTexImage1D(GL_LUT_TEXTURE0_DMP, 0, GL_LUMINANCEF_DMP, 512, 0,\n GL_LUMINANCEF_DMP, GL_FLOAT, lut);\nglUniform1i(glGetUniformLocation(progID, \"dmp_FragmentMaterial.samplerFR\"), 0);\nIn this code, f (1.0 – ShadowIntensity) is output as the fragment's primary and secondary alpha component.\nThe final shadow attenuation factor is calculated next from the texture combiner settings and is multiplied by the fragment's primary color. Note that the fragment's primary alpha value is inverted here (GL_ONE_MINUS_SRC_ALPHA).\nCode 12-13. Sample (Texture Combiner) Settings to Suppress Silhouette Shadow Artifacts\n glUniform1i(glGetUniformLocation(progID, (\"dmp_TexEnv[0].combineRgb\"),\n GL_MODULATE);\nglUniform1i(glGetUniformLocation(progID, (\"dmp_TexEnv[0].combineAlpha\"),\n GL_REPLACE);\nglUniform3i(glGetUniformLocation(progID, (\"dmp_TexEnv[0].operandRgb\"),\n GL_SRC_COLOR, GL_ONE_MINUS_SRC_ALPHA, GL_SRC_COLOR);\nglUniform3i(glGetUniformLocation(progID, (\"dmp_TexEnv[0].operandAlpha\"),\n GL_SRC_ALPHA, GL_SRC_ALPHA, GL_SRC_ALPHA);\nglUniform3i(glGetUniformLocation(progID, (\"dmp_TexEnv[0].srcRgb\"),\n GL_FRAGMENT_PRIMARY_COLOR_DMP, GL_FRAGMENT_PRIMARY_COLOR_DMP,\n GL_PRIMARY_COLOR);\nglUniform3i(glGetUniformLocation(progID, (\"dmp_TexEnv[0].srcAlpha\"),\n GL_PRIMARY_COLOR, GL_PRIMARY_COLOR, GL_PRIMARY_COLOR);\n",0]
,["Pages/Page_55588844.html#PageId_55588876","12.4.6. Reserved Uniform ",
"\n12.4.6. Reserved Uniform\nThe following table lists the reserved uniforms used for shadows.\nTable 12-12. Reserved Uniforms Used for Shadows\n \t Reserved Uniform\n \t Type\n \t Setting Value\n \t dmp_Texture[0].samplerType\n \t int\n \t Specifies the type of texture to reference. The following two types can be used for shadows.\n GL_TEXTURE_SHADOW_2D_DMP\n GL_TEXTURE_SHADOW_CUBE_DMP\n \t dmp_Texture[0].perspectiveShadow\n \t bool\n \t Specifies whether a perspective projection has been applied as the projection transformation.\n Specify GL_TRUE if a perspective projection has been applied as the projection transformation, and GL_FALSE otherwise. GL_TRUE is the default.\n \t dmp_Texture[0].shadowZBias\n \t float\n \t Specifies the bias value for the negative offset to apply to depth values during the lookup pass.\n 0.0 (default)\n \t dmp_FragOperation.mode\n \t int\n \t Specifies the fragment operation mode. Specifies shadow mode during the accumulation pass and standard mode during the lookup pass.\n GL_FRAGOP_MODE_GL_DMP (standard mode)\nGL_FRAGOP_MODE_SHADOW_DMP (shadow mode)\n \t dmp_FragOperation.wScale\n \t float\n \t Specifies the scale factor for the w-buffer.\n 0.0 (default)\n \t dmp_FragOperation.penumbraScale\n \t float\n \t Specifies the scale value for soft shadows.\n 1.0 (default)\n \t dmp_FragOperation.penumbraBias\n \t float\n \t Specifies the bias value for soft shadows.\n 0.0 (default)\n ",0]
,["Pages/Page_55588844.html#PageId_55588877","12.4.7. Checking Shadow Texture Content ",
"\n12.4.7. Checking Shadow Texture Content\nTo check the image that was rendered to a shadow texture, attach the shadow texture to the current color buffer, and then read the texel data by calling the glReadPixels function and specifying GL_RGBA for the format parameter and GL_UNSIGNED_BYTE for the type parameter. When accessed through a u32 pointer, the least-significant 8 bits of texel data store the shadow intensity, and the most-significant 24 bits store the depth value. Shift this to the right by 8 bits to get only the depth value.\nBecause shadow textures store an 8-bit shadow intensity and a 24-bit depth value, and both of these represent values in the range from 0.0 to 1.0, the shadow intensity and depth value have different precisions.\nThe shadow intensity takes a value between 0x00 and 0xFF, where 0xFF indicates the absence of a soft shadow region and all other values indicate the presence of a soft shadow region. In other words, a value of 0x00 indicates a shadow intensity of 0.0 and 0xFF indicates a shadow intensity of 1.0.\nThe depth value is scaled so that 0x000000 is the near value and 0xFFFFFF is the far value. In other words, a value of 0x000000 indicates a depth value of 0.0 and 0xFFFFFF indicates a depth value of 1.0. Note that this scaling is uniform if the w buffer is enabled during the shadow accumulation pass. For more information, see 10.3.3. W-Buffer.\nA shadow intensity of 0xFF and a depth value of 0xFFFFFF are written to the shadow texture wherever there are no shadows. A shadow intensity of 0xFF and a depth value other than 0xFFFFFF are written to the shadow texture wherever there are only hard shadows. A shadow intensity other than 0xFF and a depth value other than 0xFFFFFF are used in regions that have both hard and soft shadows.\nTable 12-13. Types of Shadows and the Values Written to Shadow Textures\n \t Shadow Type\n \t Shadow Intensity\n \t Depth Values\n \t No shadows\n \t 0xFF\n \t 0xFFFFFF\n \t Only hard shadows\n \t 0xFF\n \t Value other than 0xFFFFFF\n \t Soft shadows\n \t Value other than 0xFF\n \t Value other than 0xFFFFFF\n",0]
,["Pages/Page_55588844.html#PageId_55588878","12.5. Fog ",
"\n12.5. Fog\nAlthough 3DS fog has nearly the same features as fog defined in OpenGL ES 1.1, the effect of 3DS fog is determined by projection-corrected depth values, whereas the effect of OpenGL fog is determined by the distance from the viewpoint. Another difference is that fog coefficients are specified by lookup tables. There are also no settings for fog properties, such as the beginning, end, and density.\nAs in OpenGL, the following equation determines a fragment's color after fog has been applied.\nColor = f × Cfragment + (1 - f) × Cfog\n",0]
,["Pages/Page_55588844.html#PageId_55588879","12.5.1. Reserved Uniform ",
"Where f is the fog coefficient (0.0–1.0).\n12.5.1. Reserved Uniform\nThe following reserved uniforms are used for fog.\n",0]
,["Pages/Page_55588844.html#Anchor_55588879_フォグモード","Fog Mode ",
"Fog Mode\nTo enable fog, specify a fog mode (dmp_Fog.mode) of GL_FOG using the glUniform1i function.\nTo disable it, specify GL_FALSE.\n",0]
,["Pages/Page_55588844.html#Anchor_55588879_フォグカラー","Fog Color ",
"Fog Color\nUse the glUniform3f function to set the fog color (dmp_Fog.color). Only the RGB components are set. (The alpha component is not.)\n",0]
,["Pages/Page_55588844.html#Anchor_55588879_フォグ係数","Fog Coefficients ",
"Fog Coefficients\nFog coefficients are specified by a lookup table that takes depth values in window coordinates as input. Call the glUniform1i function to set the reserved uniform dmp_Fog.sampler to the lookup table number to use.\nSpecify the table's number rather than GL_LUT_TEXTUREi_DMP.\n",0]
,["Pages/Page_55588844.html#Anchor_55588879_入力デプス値の反転","Whether to Invert Input Depth Value ",
"Whether to Invert Input Depth Value\nYou can choose whether to invert input values (changing z to 1 - z) for the fog coefficient lookup table. To invert values, set the reserved uniform dmp_Fog.zFlip to GL_TRUE, using the glUniform1i function.\nTable 12-14. Reserved Uniforms Used for Fog\n \t Reserved Uniform\n \t Type\n \t Setting Value\n \t dmp_Fog.mode\n \t int\n \t Specifies the mode for processing the fog pipeline (the fog mode).\n GL_FOG\nGL_GAS_DMP\nGL_FALSE (default)\n \t dmp_Fog.color\n \t vec3\n \t Specifies the fog color. There is no alpha component.\n Each component has a value between 0.0 and 1.0.\n (0.0, 0.0, 0.0) by default.\n \t dmp_Fog.zFlip\n \t bool\n \t Specifies whether to invert the depth values used as input to the lookup table for fog coefficients.\n GL_TRUE\n GL_FALSE (default)\n \t dmp_Fog.sampler\n \t int\n \t Specifies the lookup table to use for fog coefficients.\n 0 to 31\n ",0]
,["Pages/Page_55588844.html#PageId_55588884","12.5.2. Creating and Specifying Lookup Tables ",
"\n12.5.2. Creating and Specifying Lookup Tables\nA lookup table for fog coefficients takes input values between 0.0 and 1.0, and has a fixed width of 256. Like other lookup tables, it stores the output values in the first 128 elements and the differences between output values in the last 128 elements.\nYou must be careful about how you convert input values, when you create lookup tables to implement OpenGL fog coefficients on a 3DS system. The input depth values are specified in window coordinates, with the near clipping plane at the minimum value (0.0) and the far clipping plane at the maximum value (1.0). For a perspective projection, however, these depth values have a non-linear relationship with depth values in eye coordinates. As a result, lookup table output must be calculated using depth values that were converted from window coordinates into eye coordinates.\nConsidering that input in window coordinates (between 0.0 and 1.0) is mapped to clip coordinates (between 0.0 and –1.0), the following equation is used to convert values into eye coordinates. Note that the sign of the input is reversed.\n(Xe Ye Ze We) = (0.0 0.0 -Zw 1.0) × Mprojection-1\nFog coefficients are a function of the distance to a fragment from the origin in eye coordinates. Input to this function can be approximated as the distance -Ze / We between the xy plane and the fragment in eye coordinates.\nThis is shown by the following sample code. FogCoef is the fog coefficient function.\nCode 12-14. Creating a Fog Lookup Table\n float Fog_LUT[256], Fog_c[128 + 1];\nint i;\nMatrix44 invPM;\nVector4 v_eye, v_clip(0.0f, 0.0f, 0.0f, 1.0f);\nMTX44Inverse(&invPM, &projMatrix);\nVector4 v0(invPM.m[0]);\nVector4 v1(invPM.m[1]);\nVector4 v2(invPM.m[2]);\nVector4 v3(invPM.m[3]);\nfor (i = 0; i <= 128; i++) {\n v_clip.z = -(static_cast<f32>(i)) / 128;\n v_eye.x = VEC4Dot(&v0, &v_clip); v_eye.y = VEC4Dot(&v1, &v_clip);\n v_eye.z = VEC4Dot(&v2, &v_clip); v_eye.w = VEC4Dot(&v3, &v_clip);\n Fog_c[i] = -(v_eye.z / v_eye.w);\n}\nfor (i = 0; i < 128; i++) {\n Fog_LUT[i] = FogCoef(Fog_c[i]);\n Fog_LUT[128 + i]= FogCoef(Fog_c[i + 1]) - FogCoef(Fog_c[i]);\n}\n Note:\n OpenGL fog is affected by the eye coordinates' z value, but 3DS fog is affected by depth values that have been corrected with a perspective projection. As a result, fog changes when the near and far clipping planes change, and the same lookup table can produce different effects depending on whether the w-buffer or a normal depth buffer is used. For more information, see 10.3.3. W-Buffer.\n",0]
,["Pages/Page_55588844.html#PageId_55588885","12.6. Gas Rendering ",
"\n12.6. Gas Rendering\n",0]
,["Pages/Page_55588844.html#PageId_55588886","12.6.1. Polygon Object Rendering Pass ",
"Gas rendering uses the fog feature, configured in gas mode, to render gaseous bodies from density information, which is itself generated based on depth values of polygon objects. Gaseous bodies require three rendering passes: the polygon object rendering pass, which generates depth values for the polygon objects; the density rendering pass, which accumulates density information in a gas texture; and the shading pass, which renders gaseous bodies by referencing a gas texture.\n12.6.1. Polygon Object Rendering Pass\nThis pass renders polygon objects as usual, generating depth information that is used to determine where the polygon objects and gaseous bodies intersect. The content of the depth buffer is the only rendering result that is used by the next pass.\n",0]
,["Pages/Page_55588844.html#PageId_55588887","12.6.2. Density Rendering Pass ",
"\n12.6.2. Density Rendering Pass\nThis pass uses point sprites, or some other means, to render the smallest units comprising a gaseous body and gas particles (a texture with a density pattern), and it accumulates a gas's depth information in the color buffer.\n",0]
,["Pages/Page_55588844.html#Anchor_55588887_カラーバッファとガステクスチャの準備","Preparing Color Buffers and Gas Textures ",
"Preparing Color Buffers and Gas Textures\nPrepare a color buffer with an internal format of GL_GAS_DMP and a texture (gas texture), both of which are used to copy the rendering results. Create the color buffer with the same size as the depth buffer, but the gas texture must have a width and height (in texels) that are both powers of 2, and GL_UNSIGNED_SHORT must be specified for type. Because gas textures always use point sampling, minification and magnification filters have no effect. Use GL_NEAREST as the filter setting. Note that mipmaps cannot be applied to gas textures.\nCode 12-15. Preparing a Color Buffer and Gas Texture for Gas Rendering\n // Generating Object\nglGenFramebuffers(1, &gasAccFboID);\nglGenTextures(1, &gasTexID);\n// Renderbuffer & Framebuffer\nglGenRenderbuffers(1, &gasAccColorID);\nglBindRenderbuffer(GL_RENDERBUFFER, gasAccColorID);\nglRenderbufferStorage(GL_RENDERBUFFER, GL_GAS_DMP, GAS_ACC_WIDTH,\n GAS_ACC_HEIGHT);\nglBindFramebuffer(GL_FRAMEBUFFER, gasAccFboID);\nglFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_RENDERBUFFER,\n gasAccColorID);\n// Gaseous Texture\nglBindTexture(GL_TEXTURE_2D, gasTexID);\nglTexImage2D(GL_TEXTURE_2D, 0, GL_GAS_DMP, GAS_TEX_WIDTH, GAS_TEX_HEIGHT, 0,\n GL_GAS_DMP, GL_UNSIGNED_SHORT, 0);\nglTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);\nglTexParameterf(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);\n",0]
,["Pages/Page_55588844.html#Anchor_55588887_ガスパーティクルのレンダリング","Rendering Gas Particles ",
"Rendering Gas Particles\nWhen the fragment pipeline is switched and gas mode (GL_FRAGOP_MODE_GAS_ACC_DMP) is configured as the fragment operation mode, density information is rendered to a color buffer. Two types of density information are accumulated in the color buffer: information that is simply accumulated (D1), and information that accounts for intersections with polygon offsets (D2).\nDisable the depth test, the depth mask, blending, and fog to prevent the content of the depth buffer from being updated. However, do not change the comparison function that was used for the depth test, when polygon objects were rendered.\nClear only one buffer, the color buffer to which density information is rendered. The depth buffer must not be cleared.\nThe R component of the gas particles to be rendered (Df) is accumulated in the color buffer as density information. D1 and D2 are updated to D1' and D2' by the following equations. The equation for D2' depends on the comparison function for the depth test.\nZb is the depth value stored in the depth buffer, and Zf is the depth value of the fragment.\nD1 accumulates fragment (gas particle) density information unchanged. D2 accumulates fragment density information that has been multiplied by the attenuation coefficient EZ in the depth direction and also by the difference between the depth values in the depth buffer and fragment. The EZ coefficient is a floating-point number set by the glUniform1f function in the reserved uniform dmp_Gas.deltaZ.\nCode 12-16. Rendering Gas Particles\n // Change to gas accumulation mode.\nglBindFramebuffer(GL_FRAMEBUFFER, gasAccFboID);\nglUniform1i(glGetUniformLocation(progID, \"dmp_FragOperation.mode\"),\n GL_FRAGOP_MODE_GAS_ACC_DMP);\nglDisable(GL_DEPTH_TEST);\nglDepthMask(GL_FALSE);\nglDisable(GL_BLEND);\nglUniform1i(glGetUniformLocation(progID, \"dmp_Fog.mode\"), GL_FALSE);\n// Colorbuffer Clear\nglClearColor(0.0f, 0.0f, 0.0f, 0.0f);\nglClear(GL_COLOR_BUFFER_BIT);\n// Set dmp_Gas.deltaZ.\nglDepthFunc(GL_LESS);\nglUniform1f(glGetUniformLocation(progID, \"dmp_Gas.deltaZ\"), 50.0f);\n",0]
,["Pages/Page_55588844.html#Anchor_55588887_ガステクスチャへのコピー","Copying Information to a Gas Texture ",
"Copying Information to a Gas Texture\nAfter all gas particles have been rendered, the density information accumulated in the color buffer is copied to a gas texture.\nThe gas texture is not usually allocated with the same size as the color buffer (the width and height must be powers of 2), so the glCopyTexSubImage2D function partially copies color buffer data. The gas texture is required for the next pass.\nCode 12-17. Coping Information to a Gas Texture\n // Bind and copy to gaseous texture.\nglBindTexture(GL_TEXTURE_2D, gasTexID);\nglCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, GAS_ACC_WIDTH,\n GAS_ACC_HEIGHT);\n",0]
,["Pages/Page_55588844.html#PageId_55588891","12.6.3. Shading Pass ",
"\n12.6.3. Shading Pass\nThis pass shades gaseous bodies by referencing the density information accumulated in a gas texture, and then blends the shading results with the color buffer contents from the polygon object rendering pass.\nGaseous bodies are shaded using the fog feature configured in gas mode. Special settings are required for fog input from the texture combiners.\n",0]
,["Pages/Page_55588844.html#Anchor_55588891_フォグへの入力（テクスチャコンバイナの設定）","Fog Input (Texture Combiner Settings) ",
"Fog Input (Texture Combiner Settings)\nFog takes two inputs: the RGBA components from the second-to-last texture combiner (though only the R component is actually used), and the density information from the input source, specified as input source 2 (the third component of srcRgb and srcAlpha) to the last texture combiner (the specified texture unit must sample the gas texture). Because output from the last texture combiner is ignored, the number of usable texture combiner levels is reduced by one.\nUltimately, fog output is blended with the contents of the color buffer. Because fog outputs alpha values that account for intersections with polygon objects, blending both outputs based on the fog's alpha values allows gaseous bodies to be rendered in the correct order front-to-back.\nFigure 12-5.Fog Input and Blending With the Color Buffer\n",0]
,["Pages/Page_55588844.html#Anchor_55588891_ガスモード時のフォグの動作","Fog Operations in Gas Mode ",
"Fog Operations in Gas Mode\nIn gas mode, fog shades gaseous bodies based on input information.\nThe RGB components of the shading results are determined by shading lookup tables. To determine the alpha component, the gas attenuation value (dmp_Gas.attenuation) is multiplied by density information that accounts for intersections (D2), and the result is looked up in the lookup table specified by the fog coefficient (dmp_Fog.sampler).\nFigure 12-6. Fog in Gas Mode\n",0]
,["Pages/Page_55588844.html#Anchor_55588891_シェーディング参照テーブル","Shading Lookup Tables ",
"Shading Lookup Tables\nShading lookup tables accept either density information or shading intensity as input, and give the RGB components of the shading results as output.\nShading lookup tables are specified separately for each component using tables generated with width set to 16. Both the input and output values are between 0.0 and 1.0. As with other lookup tables, the output values are stored in the first eight elements, the differences between output values are stored in the last eight elements, and the output values are interpolated using the delta values.\nTo specify a lookup table, set its number in the following reserved uniforms, using the glUniform1i function.\nTable 12-15. Reserved Uniforms Used to Specify Shading Lookup Tables\n \t Reserved Uniform\n \t Type\n \t Setting Value\n \t dmp_Gas.samplerTR\ndmp_Gas.samplerTG\ndmp_Gas.samplerTB\n \t int\n \t Specifies the lookup tables to use as the shading lookup tables (for the R, G, and B components).\n 0 to 31\n The following are examples of shading lookup tables and their implementations.\nFigure 12-7. A Shading Lookup Table\nCode 12-18. Creating Shading Lookup Tables\n // Define\nGLfloat shading_color[3 * 9] = {\n 0.00f, 0.00f, 0.00f,\n 0.20f, 0.15f, 0.05f,\n 0.60f, 0.25f, 0.15f,\n 0.90f, 0.35f, 0.20f,\n 0.92f, 0.60f, 0.15f,\n 0.95f, 0.85f, 0.05f,\n 1.00f, 0.95f, 0.00f,\n 1.00f, 1.00f, 1.00f,\n 1.00f, 1.00f, 1.00f\n};\nGLfloat samplerTR[16], samplerTG[16], samplerTB[16];\n// Table\nfor(int i = 0; i < 8; i++) {\n // Shading color value\n samplerTR[i] = shading_color[3*i + 0];\n samplerTG[i] = shading_color[3*i + 1];\n samplerTB[i] = shading_color[3*i + 2];\n // Difference of shading color value\n samplerTR[8 + i] = shading_color[3*(i + 1) + 0] - shading_color[3*i + 0];\n samplerTG[8 + i] = shading_color[3*(i + 1) + 1] - shading_color[3*i + 1];\n samplerTB[8 + i] = shading_color[3*(i + 1) + 2] - shading_color[3*i + 2];\n}\n// Texture\nglBindTexture(GL_LUT_TEXTURE1_DMP, samplerTR_ID);\nglTexImage1D(GL_LUT_TEXTURE1_DMP, 0, GL_LUMINANCEF_DMP, 16, 0,\n GL_LUMINANCEF_DMP, GL_FLOAT, samplerTR);\nglBindTexture(GL_LUT_TEXTURE2_DMP, samplerTG_ID);\nglTexImage1D(GL_LUT_TEXTURE2_DMP, 0, GL_LUMINANCEF_DMP, 16, 0,\n GL_LUMINANCEF_DMP, GL_FLOAT, samplerTG);\nglBindTexture(GL_LUT_TEXTURE3_DMP, samplerTB_ID);\nglTexImage1D(GL_LUT_TEXTURE3_DMP, 0, GL_LUMINANCEF_DMP, 16, 0,\n GL_LUMINANCEF_DMP, GL_FLOAT, samplerTB);\n// Set Uniform\nglUniform1i(glGetUniformLocation(progID, \"dmp_Gas.samplerTR\"), 1);\nglUniform1i(glGetUniformLocation(progID, \"dmp_Gas.samplerTG\"), 2);\nglUniform1i(glGetUniformLocation(progID, \"dmp_Gas.samplerTB\"), 3);\n",0]
,["Pages/Page_55588844.html#Anchor_55588891_密度情報によるシェーディング","Shading Based on Density Information ",
"Shading Based on Density Information\nFor shading that is based on density information alone (without accounting for the effect of shadows cast by light), set the reserved uniform dmp_Gas.colorLutInput to GL_GAS_DENSITY_DMP, using the glUniform1i function to select density information, as the shading lookup table input.\nAs explained for the density rendering pass, two types of density information are stored in a gas texture. One type of density information does not account for intersections with polygon objects (D1), and the other type does (D2). You can set a value in the reserved uniform dmp_Gas.shadingDensitySrc to select the density information to use.\nSpecify GL_GAS_PLAIN_DENSITY_DMP for D1 and GL_GAS_DEPTH_DENSITY_DMP for D2, using the glUniform1i function. Even if you choose D1 as input to the shading lookup tables, fog in gas mode outputs alpha components calculated from density information that accounts for intersections (D2). By using alpha values while blending, polygon objects and gaseous bodies can be rendered with the correct front-to-back ordering.\nLookup table input uses values between 0.0 and 1.0. You can multiply the density information by the reciprocal of the maximum density value to keep values in this range. The reciprocal of the maximum value can be calculated automatically. By setting the reserved uniform dmp_Gas.autoAcc to GL_TRUE, you can use the glUniform1i function to calculate the reciprocal from the maximum D1 value in the density rendering pass.\nWhen GL_FALSE is specified, the value set in the reserved uniform dmp_Gas.accMax by the glUniform1f function is used as the reciprocal of the maximum value.\n",0]
,["Pages/Page_55588844.html#Anchor_55588891_シェーディング強度によるシェーディング","Shading Based on the Shading Intensity ",
"Shading Based on the Shading Intensity\nTo use shading that is calculated from the shading intensity (accounting for the effect of shadows cast by light), set the reserved uniform dmp_Gas.colorLutInput to GL_GAS_LIGHT_FACTOR_DMP, using the glUniform1i function. This selects the shading intensity as input to the shading lookup table.\nThe shading intensity is the total of two calculated values: the planar shading intensity (IG) and the view shading intensity (IS). IG and IS are defined as follows.\nig = r × (1.0 - lightAtt × d1)\nIG = (1.0 - ig) × lightMin + ig × lightMax\nis = LZ × (1.0 - scattAtt × d1)\nIS = (1.0 - is) × scattMin + is × scattMax\nd1 is the result of multiplying the density information that does not account for intersections with polygon objects (D1), by the reciprocal of the maximum density. This mechanism is similar to the one used for shading based on density information. r is the R component output from the texture combiner that is used as fog input. lightAtt, lightMin, and lightMax are coefficients of the planar shading intensity. LZ is the light direction along the z-axis in eye coordinates. scattAtt, scattMin, and scattMax are coefficients of the view shading intensity. These all take values between 0.0 and 1.0.\nThe planar shading intensity is proportional to r and (1.0 – lightAtt × d1), as its equation shows. It approaches lightMax when r increases and lightMin when d1 increases. Likewise, the view shading intensity is proportional to LZ and (1.0 – scattAtt × d1). It approaches scattMax when LZ increases and scattMin when d1 increases.\nNote that the shading intensity input to a shading lookup table is larger for gaseous bodies that are less dense. This emulates the real behavior of light, which penetrates through the thin (low-density) areas of a gaseous body, and is absorbed in the thick (high-density) areas. Accordingly, lightMin and scattMin represent the shading intensity when the effect of light is small, and lightMax and scattMax represent the shading intensity when the effect of light is large. lightAtt and scattAtt set the ratio of light attenuation caused by density. Note that, depending on the values set in the shading lookup tables, lightMin is not necessarily less than lightMax (and scattMin is not necessarily less than scattMax). Also note that alpha values are determined by fog coefficients whose input is proportional to density.\nCoefficients for the planar shading intensity (lightMin, lightMax, and lightAtt) are set, as a group, in the reserved uniform dmp_Gas.lightXY. Coefficients for the view shading intensity (scattMin, scattMax, and scattAtt), and the light direction along the z-axis in eye coordinates (LZ), are set as a group in the reserved uniform dmp_Gas.lightZ. The minimum value, maximum value, and attenuation are set in that order. They are followed by LZ for the view shading intensity.\nThe following code shows how to set the shading intensity coefficients.\nCode 12-19. Setting Shading Intensity Coefficients\n GLfloat lightXY[3], lightZ[4];\nGLfloat lightMin, lightMax, lightAtt;\nGLfloat scattMin, scattMax, scattAtt, LZ;\n//...\n// lightXY\nlightXY[0] = lightMin;\nlightXY[1] = lightMax;\nlightXY[2] = lightAtt;\n// lightZ\nlightZ[0] = scattMin;\nlightZ[1] = scattMax;\nlightZ[2] = scattAtt;\nlightZ[3] = LZ;\n// Set Uniform\nglUniform3fv(glGetUniformLocation(progID, \"dmp_Gas.lightXY\"), 1, lightXY);\nglUniform4fv(glGetUniformLocation(progID, \"dmp_Gas.lightZ\"), 1, lightZ);\n",0]
,["Pages/Page_55588844.html#Anchor_55588891_アルファ値のシェーディング","Alpha Shading ",
"Alpha Shading\nThe results of shading the alpha component are determined by output from the lookup table for fog coefficients (dmp_Fog.sampler), given the product of the gas attenuation (dmp_Gas.attenuation) and the density information that accounts for intersections (D2) as input.\nFog coefficients are normally specified using a function that approaches 0.0 when the gas density is low, and 1.0 when the gas density is high.\nCode 12-20. Setting Fog Coefficients\n // Fog Factor\nfor(int i = 0; i < 128; i++) {\n fogTable[i]= 1.0f - exp(-8.0f * i / 128.0f);\n}\nfor(int i = 0; i < 128; i++) {\n fogTable[128 + i] = fogTable[i + 1] - fogTable[i];\n}\nfogTable[255] = 0;\n// Set LUT\nglGenTextures(1, &fogLUT_ID);\nglBindTexture(GL_LUT_TEXTURE0_DMP, fogLUT_ID);\nglTexImage1D(GL_LUT_TEXTURE0_DMP, 0, GL_LUMINANCEF_DMP, 256, 0,\n GL_LUMINANCEF_DMP, GL_FLOAT, fogTable);\nThe following sample code sets uniforms required by the shading pass and renders a quad (polygon), to which a gas texture has been applied.\nCode 12-21. Setting Uniforms for the Shading Pass\n // Bind Framebuffer(Colorbuffer)\nglBindFramebuffer(GL_FRAMEBUFFER, renderFboID);\n// Bind Gas Texture\nglActiveTexture(GL_TEXTURE0);\nglBindTexture(GL_TEXTURE_2D, gasTexID);\nglUniform1i(glGetUniformLocation(progID, \"dmp_Texture[0].samplerType\"),\n GL_TEXTURE_2D);\n// Set TextureCombiner #5\nglUniform3i(glGetUniformLocation(progID, \"dmp_TexEnv[5].srcRgb\"),\n GL_PREVIOUS, GL_PREVIOUS, GL_TEXTURE0);\nglUniform3i(glGetUniformLocation(progID, \"dmp_TexEnv[5].srcAlpha\"),\n GL_PREVIOUS, GL_PREVIOUS, GL_TEXTURE0);\n// Set uniform for gas shading mode.\nglUniform1i(glGetUniformLocation(progID, \"dmp_Fog.sampler\"), 0);\nglUniform1i(glGetUniformLocation(progID, \"dmp_Gas.autoAcc\"), GL_FALSE);\nglUniform1i(glGetUniformLocation(progID, \"dmp_Gas.samplerTR\"), 1);\nglUniform1i(glGetUniformLocation(progID, \"dmp_Gas.samplerTG\"), 2);\nglUniform1i(glGetUniformLocation(progID, \"dmp_Gas.samplerTB\"), 3);\nglUniform1i(glGetUniformLocation(progID, \"dmp_Gas.shadingDensitySrc\"),\n GL_GAS_DEPTH_DENSITY_DMP);\nglUniform1i(glGetUniformLocation(progID, \"dmp_Gas.colorLutInput\"),\n GL_GAS_DENSITY_DMP);\nglUniform1f(glGetUniformLocation(progID, \"dmp_Gas.accMax\"), 1.0f/6.0f);\nglUniform4fv(glGetUniformLocation(progID, \"dmp_Gas.lightZ\"), 1, gasLightZ);\nglUniform3fv(glGetUniformLocation(progID, \"dmp_Gas.lightXY\"), 1, gasLightXY);\n// Change to gas shading mode.\nglUniform1i(glGetUniformLocation(progID, \"dmp_FragOperation.mode\"),\n GL_FRAGOP_MODE_GL_DMP);\nglUniform1i(glGetUniformLocation(progID, \"dmp_Fog.mode\"), GL_GAS_DMP);\nglEnable(GL_BLEND);\nglBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);\nglDisable(GL_DEPTH_TEST);\nglDepthMask(GL_FALSE);\nCode 12-22. Rendering a Quad (Polygon) With a Gas Texture Applied\n // Gaseous Shading\n// Texture Coord\nfloat u0 = 0.0f;\nfloat v0 = 0.0f;\nfloat u1 = (GAS_ACC_WIDTH * 1.0f) / (GAS_TEX_WIDTH * 1.0f);\nfloat v1 = (GAS_ACC_HEIGHT * 1.0f) / (GAS_TEX_HEIGHT * 1.0f);\nGLfloat texCoord[8]= {u0, v0, u0, v1, u1, v1, u1, v0};\n// Vertex\nGLushort quadIndex[6] = {0, 1, 2, 0, 2, 3};\nGLfloat vertex[16] = {\n -1.0f, -1.0f, 0.0f, 1.0f,\n -1.0f, 1.0f, 0.0f, 1.0f,\n 1.0f, 1.0f, 0.0f, 1.0f,\n 1.0f, -1.0f, 0.0f, 1.0f\n};\n// Set Array\nGLfloat quadIndexID, quadVertexID, quadTexCoordID;\nglGenBuffers(1, &quadIndexID);\nglBindBuffer(GL_ELEMENT_ARRAY_BUFFER, quadIndexID);\nglBufferData(GL_ELEMENT_ARRAY_BUFFER, 6 * sizeof(GLushort), &quadIndex,\n GL_STATIC_DRAW);\nglGenBuffers(1, &quadVertexID);\nglBindBuffer(GL_ARRAY_BUFFER, quadVertexID);\nglBufferData(GL_ARRAY_BUFFER, 16 * sizeof(GLfloat), &vertex, GL_STATIC_DRAW);\nglGenBuffers(1, &quadTexCoordID);\nglBindBuffer(GL_ARRAY_BUFFER, quadTexCoordID);\nglBufferData(GL_ARRAY_BUFFER, 8 * sizeof(GLfloat), &texCoord, GL_STATIC_DRAW);\n// Draw Quad\nglEnableVertexAttribArray(0);\nglEnableVertexAttribArray(1);\nglBindBuffer(GL_ARRAY_BUFFER, quadVertexID);\nglVertexAttribPointer(0, 4, GL_FLOAT, GL_FALSE, 0, 0);\nglBindBuffer(GL_ARRAY_BUFFER, quadTexCoordID);\nglVertexAttribPointer(1, 2, GL_FLOAT, GL_FALSE, 0, 0);\nglBindBuffer(GL_ELEMENT_ARRAY_BUFFER, quadIndexID);\nglDrawElements(GL_TRIANGLES, 6, GL_UNSIGNED_SHORT, 0);\n",0]
,["Pages/Page_55588844.html#PageId_55588898","12.6.4. Reserved Uniform ",
"\n12.6.4. Reserved Uniform\nThe following table lists the reserved uniforms used for gas.\nTable 12-16. Reserved Uniforms Used for Gas\n \t Reserved Uniform\n \t Type\n \t Setting Value\n \t dmp_FragOperation.mode\n \t int\n \t Specifies the fragment operation mode. Specify gas mode in the density rendering pass. Specify standard mode in any other pass.\n GL_FRAGOP_MODE_GL_DMP (standard mode)\nGL_FRAGOP_MODE_SHADOW_DMP (shadow mode)\n \t dmp_Fog.mode\n \t int\n \t Specifies the mode for processing the fog pipeline. Specify gas mode in the shading pass. Disable fog in the density rendering pass.\n Specify GL_GAS_DMP for gas mode in the shading pass, and GL_FALSE to disable fog in the density rendering pass.\n \t dmp_Fog.sampler\n \t int\n \t Specifies the lookup table to use for fog coefficients. Fog coefficients are used to calculate alpha values for gaseous bodies.\n 0 to 31\n \t dmp_Gas.deltaZ\n \t float\n \t Specifies the attenuation coefficient EZ in the depth direction.\n 10.0 (default)\n \t dmp_Gas.autoAcc\n \t bool\n \t Specifies whether to automatically calculate the reciprocal of the maximum density value.\n Specify GL_TRUE to automatically calculate the reciprocal of the maximum density value, and GL_FALSE otherwise. GL_TRUE is the default.\n \t dmp_Gas.accMax\n \t float\n \t Specifies the reciprocal of the maximum density value.\n 0.0 or more\n 1.0 (default)\n \t dmp_Gas.samplerTR\n dmp_Gas.samplerTG\n dmp_Gas.samplerTB\n \t int\n \t Specifies the shading lookup table for each RGB component.\n 0 to 31\n \t dmp_Gas.shadingDensitySrc\n \t int\n \t Specifies the density information to use for shading.\n Specify GL_GAS_PLAIN_DENSITY_DMP (default) or GL_GAS_DEPTH_DENSITY_DMP.\n \t dmp_Gas.colorLutInput\n \t int\n \t Specifies whether the density or shading intensity is given as input to the shading lookup tables.\n Specify GL_GAS_LIGHT_FACTOR_DMP (default) or GL_GAS_DENSITY_DMP.\n \t dmp_Gas.lightXY\n \t vec3\n \t Specifies factors used to control planar shading: the minimum intensity, maximum intensity, and density attenuation.\n (lightMin, lightMax, lightAtt)\n Each control value is between 0.0 and 1.0.\n The default value is (0.0, 0.0, 0.0).\n \t dmp_Gas.lightZ\n \t vec4\n \t Specifies factors used to control view shading: the minimum intensity, maximum intensity, density attenuation, and effect in the view direction.\n (scattMin, scattMax, scattAtt, LZ)\n Each control value is between 0.0 and 1.0.\n The default value is (0.0, 0.0, 0.0, 0.0).\n \t dmp_Gas.attenuation\n \t float\n \t Specifies the density attenuation coefficient to use when calculating alpha values for shading.\n 0.0 or greater\n 1.0 (default)\nCONFIDENTIAL\n",0]
,["Pages/Page_55588899.html","13. Per-Fragment Operations ",
"13. Per-Fragment Operations\nPer-fragment operations update the pixel data stored at a fragment's window coordinates in the framebuffer, based on the lighting results and other related data.\nThis does not mean that all fragments are taken into account. Unnecessary fragments are rejected by tests, such as the alpha test, and then existing pixel data and colors are combined through blending and logical operations.\nThe 3DS specifications differ from the OpenGL ES 2.0 specifications in the following ways.\n\tThe scissor test is run during rasterization on a CTR system. \tThe glSampleCoverage function has not been implemented because multisampling is not supported. \tThe alpha test provides features equivalent to those in OpenGL ES 1.1, but it is controlled entirely through reserved uniforms. \tThe glStencil*Separate functions have not been implemented because the stencil test does not distinguish between front-facing polygons and back-facing polygons. \tLogical operators (GL_LOGIC_OP) cannot be selected for blending. \tGL_SRC_ALPHA_SATURATE can be selected for the destination in the glBlendFunc* blending functions, even though this is not allowed by the OpenGL ES specifications. \tDithering is not applied. ",0]
,["Pages/Page_55588899.html#PageId_55588900","13.1. Alpha Test ",
" \tLogical operations are implemented to correspond with the OpenGL ES 1.1 specifications.\n13.1. Alpha Test\nThe alpha test compares a fragment's alpha value with a reference value, to determine whether to pass the fragment to the next process or reject it.\n",0]
,["Pages/Page_55588899.html#PageId_55588901","13.1.1. Reserved Uniform ",
"The 3DS alpha test has features that correspond to the alpha test in OpenGL ES 1.1, but they are all controlled by reserved uniforms.\n13.1.1. Reserved Uniform\nThe following reserved uniforms are used for the alpha test.\n",0]
,["Pages/Page_55588899.html#Anchor_55588901_アルファテストの有効・無効","Enabling or Disabling the Alpha Test ",
"Enabling or Disabling the Alpha Test\nTo enable the alpha test, set the reserved uniform dmp_FragOperation.enableAlphaTest to GL_TRUE, using the glUniform1i function. Disabled (set to GL_FALSE) by default.\n",0]
,["Pages/Page_55588899.html#Anchor_55588901_比較方法","Comparison Function ",
"Comparison Function\nUse the glUniform1i function to set the comparison function in the reserved uniform dmp_FragOperation.alphaTestFunc. You can specify one of the following eight values.\nTable 13-1. Comparison Functions for the Alpha Test\n \t Value\n \t Comparison\n \t GL_NEVER\n \t Always rejected (never passes).\n \t GL_ALWAYS (default)\n \t Always passes.\n \t GL_LESS\n \t Passes if the alpha value is less than the reference value.\n \t GL_LEQUAL\n \t Passes if the alpha value is less than, or equal to, the reference value.\n \t GL_EQUAL\n \t Passes if the alpha value is equal to the reference value.\n \t GL_GEQUAL\n \t Passes if the alpha value is greater than, or equal to, the reference value.\n \t GL_GREATER\n \t Passes if the alpha value is greater than the reference value.\n \t GL_NOTEQUAL\n \t Passes if the alpha value is not equal to the reference value.\n ",0]
,["Pages/Page_55588899.html#Anchor_55588901_参照値","Reference Value ",
"Reference Value\nUse the glUniform1f function to set the reference value for the alpha test in the reserved uniform dmp_FragOperation.alphaRefValue. The reference value is clamped between 0.0 and 1.0 when it is used for comparisons.\nThe following table lists the reserved uniforms used for the alpha test.\nTable 13-2. Reserved Uniforms Used for the Alpha Test\n \t Reserved Uniform\n \t Type\n \t Setting Value\n \t dmp_FragOperation.enableAlphaTest\n \t bool\n \t Specifies whether to enable or disable the alpha test.\n GL_TRUE\n GL_FALSE (default)\n \t dmp_FragOperation.alphaRefValue\n \t float\n \t Specifies the reference value to use with the alpha test.\n 0.0–1.0\n 0.0 by default.\n \t dmp_FragOperation.alphaTestFunc\n \t int\n \t Specifies the comparison function for the alpha test.\n GL_NEVER\n GL_ALWAYS (default)\nGL_LESS\nGL_LEQUAL\nGL_EQUAL\nGL_GEQUAL\nGL_GREATER\nGL_NOTEQUAL\n",0]
,["Pages/Page_55588899.html#PageId_55588905","13.2. Stencil Test ",
"\n13.2. Stencil Test\nThe stencil test compares a reference value with the value stored at a fragment's window coordinates in the stencil buffer to determine whether to pass the fragment to the next process or reject it.\n",0]
,["Pages/Page_55588899.html#PageId_55588906","13.2.1. How to Use ",
"The glStencilFuncSeparate and glStencilOpSeparate functions have not been implemented because the 3DS system does not distinguish between front-facing polygons and back-facing polygons.\n13.2.1. How to Use\nThe stencil test is used in the same way it is used in OpenGL.\n",0]
,["Pages/Page_55588899.html#Anchor_55588906_ステンシルテストの有効・無効","Enabling or Disabling the Stencil Test ",
"Enabling or Disabling the Stencil Test\nTo enable or disable the stencil test, call the glEnable or glDisable function, passing GL_STENCIL_TEST as the cap parameter. To get the current setting, call the glIsEnabled function, passing GL_STENCIL_TEST as the cap parameter. The stencil test is disabled by default. When it is disabled, the stencil buffer is not changed and fragments are not rejected. If a stencil buffer has not been bound to the framebuffer, the stencil test is treated as if it were disabled.\n",0]
,["Pages/Page_55588899.html#Anchor_55588906_比較方法、参照値、マスク","Comparison Function, Reference Value, and Mask ",
"Comparison Function, Reference Value, and Mask\nThe stencil test has three elements: the comparison function, reference value, and mask, which are specified by the glStencilFunc function.\nCode 13-1. Definition of the glStencilFunc Function\n void glStencilFunc(GLenum func, GLint ref, GLuint mask);\nFor the ref parameter, specify an integer to use as the reference value during comparisons. The reference value is handled as an unsigned number when it is compared by the stencil test, and it is clamped between 0 and 255 because the 3DS stencil buffer is always 8-bit. A value of 0 is used by default.\nFor the mask parameter, specify the mask. Bitwise AND operations apply the mask to both the reference value and the value in the stencil buffer to get the values to use during comparisons. A value of 0xFFFFFFFF (all 1s) is used by default.\nFor the func parameter, specify the comparison function. You can specify one of the following eight values.\nTable 13-3. Comparison Functions for the Stencil Test\n \t Value\n \t Comparison\n \t GL_NEVER\n \t Always rejected (never passes).\n \t GL_ALWAYS (default)\n \t Always passes.\n \t GL_LESS\n \t Passes if the reference value is less than the value in the stencil buffer.\n \t GL_LEQUAL\n \t Passes if the reference value is less than, or equal to, the value in the stencil buffer.\n \t GL_EQUAL\n \t Passes if the reference value is equal to the value in the stencil buffer.\n \t GL_GEQUAL\n \t Passes if the reference value is greater than, or equal to, the value in the stencil buffer.\n \t GL_GREATER\n \t Passes if the reference value is greater than the value in the stencil buffer.\n \t GL_NOTEQUAL\n \t Passes if the reference value is not equal to the value in the stencil buffer.\n ",0]
,["Pages/Page_55588899.html#Anchor_55588906_テスト結果への対処","Handling Test Results ",
"Handling Test Results\nUsing the glStencilOp function, you can specify how the contents of the stencil buffer are modified as a result of both the stencil test (which compares the reference value with the stencil buffer) and the depth test, which will be described later.\nCode 13-2. Definition of the glStencilOp Function\n void glStencilOp(GLenum fail, GLenum zfail, GLenum zpass);\nThe fail parameter specifies how the contents of the stencil buffer are modified when a fragment is rejected by the stencil test. You can specify one of the following eight values.\nTable 13-4. Modifying Stencil Buffer Contents\n \t Value\n \t Stencil Buffer Content\n \t GL_KEEP (default)\n \t Keep the current value.\n \t GL_ZERO\n \t Set the value equal to 0.\n \t GL_REPLACE\n \t Replace the value with the reference value.\n \t GL_INCR\n \t Increment the value by 1. This is never greater than 255.\n \t GL_DECR\n \t Decrement the value by 1. This is never less than 0.\n \t GL_INVERT\n \t Bitwise invert the value.\n \t GL_INCR_WRAP\n \t Increment the value by 1. This wraps to 0 after 255.\n \t GL_DECR_WRAP\n \t Decrement the value by 1. This wraps to 255 after 0.\n The zfail parameter configures how the contents of the stencil buffer are modified when a fragment is rejected by the depth test, and the zpass parameter configures how the contents of the stencil buffer are modified when a fragment passes the depth test. For these parameters, specify one of the eight values that is valid for the fail parameter, earlier.\n",0]
,["Pages/Page_55588899.html#PageId_55588910","13.3. Early Depth Test ",
"\n13.3. Early Depth Test\n",0]
,["Pages/Page_55588899.html#PageId_55588911","13.3.1. How to Use ",
"The depth test compares a fragment's depth value with the value stored at the fragment's window coordinates in the depth buffer to determine whether to pass the fragment to the next process or reject it. As explained for the stencil buffer, the results of the depth test can also change the content of the stencil buffer.\n13.3.1. How to Use\nThe depth test is used in the same way it is used in OpenGL.\n",0]
,["Pages/Page_55588899.html#Anchor_55588911_デプステストの有効・無効","Enabling or Disabling the Depth Test ",
"Enabling or Disabling the Depth Test\nTo enable or disable the depth test, call the glEnable or glDisable function, specifying GL_DEPTH_TEST for the cap parameter. To get the current setting, call the glIsEnabled function, specifying GL_DEPTH_TEST for the cap parameter. The depth test is disabled by default. When it is disabled, the depth buffer is not modified, and fragments are not rejected. If a depth buffer has not been bound to the framebuffer, the depth test is treated as if it were disabled.\n",0]
,["Pages/Page_55588899.html#Anchor_55588911_比較方法","Comparison Function ",
"Comparison Function\nUse the glDepthFunc function to set the comparison function.\nCode 13-3. Definition of the glDepthFunc Function\n void glDepthFunc(GLenum func);\nFor func, specify one of the following eight values.\nTable 13-5. Comparison Functions for the Depth Test\n \t Value\n \t Comparison\n \t GL_NEVER\n \t Always rejected (never passes).\n \t GL_ALWAYS\n \t Always passes.\n \t GL_LESS (default)\n \t Passes if the depth value is less than the value in the depth buffer.\n \t GL_LEQUAL\n \t Passes if the depth value is less than, or equal to, the value in the depth buffer.\n \t GL_EQUAL\n \t Passes if the depth value is equal to the value in the depth buffer.\n \t GL_GEQUAL\n \t Passes if the depth value is greater than, or equal to, the value in the depth buffer.\n \t GL_GREATER\n \t Passes if the depth value is greater than the value in the depth buffer.\n \t GL_NOTEQUAL\n \t Passes if the depth value is not equal to the value in the depth buffer.\n If a fragment is rejected as a result of the comparison, the depth buffer is not modified. If a fragment passes, its depth value overwrites the value in the depth buffer.\n",0]
,["Pages/Page_55588899.html#PageId_55588914","13.4. Blending ",
"\n13.4. Blending\n",0]
,["Pages/Page_55588899.html#PageId_55588915","13.4.1. How to Use ",
"Blending combines a fragment's color (the source color) with the color stored at the fragment's window coordinates in the framebuffer (the destination color). The blended result is passed to the next process as the fragment's color.\n13.4.1. How to Use\nBlending is used in the same way it is used in OpenGL.\n",0]
,["Pages/Page_55588899.html#Anchor_55588915_ブレンディングの有効・無効","Enabling or Disabling Blending ",
"Enabling or Disabling Blending\nTo enable or disable blending, call the glEnable or glDisable function, specifying GL_BLEND for cap. To get the current setting, call the glIsEnabled function, specifying GL_BLEND for cap. Blending is disabled by default. When it is disabled, blending is skipped.\nBlending is considered to be disabled, either when a color buffer has not been bound to the framebuffer or when logical operations (described later) are enabled.\n",0]
,["Pages/Page_55588899.html#Anchor_55588915_ブレンディングの計算式（合成方法）","Blend Equations ",
"Blend Equations\nYou can use the glBlendEquation or glBlendEquationSeparate function to specify how a fragment's color (the source color) is combined with a framebuffer color (the destination color).\nCode 13-4. Definitions of the glBlendEquation* Functions\n void glBlendEquation(GLenum mode);\nvoid glBlendEquationSeparate(GLenum modeRGB, GLenum modeAlpha);\nThe glBlendEquation function allows you to specify equations for both the RGB and alpha components by using the mode parameter.\nThe glBlendEquationSeparate function allows you to specify equations for the RGB and alpha components, separately, using the modeRGB and modeAlpha parameters, respectively. All of these parameters accept the following five values.\nTable 13-6. Blending Equations\n \t Value\n \t RGB Components\n \t Alpha Component\n \t GL_FUNC_ADD (default)\n \t R = Rs * Sr + Rd * Dr\nG = Gs * Sg + Gd * Dg\nB = Bs * Sb + Bd * Db\n \t A = As * Sa + Ad * Da\n \t GL_FUNC_SUBTRACT\n \t R = Rs * Sr - Rd * Dr\nG = Gs * Sg - Gd * Dg\nB = Bs * Sb - Bd * Db\n \t A = As * Sa - Ad * Da\n \t GL_FUNC_REVERSE_SUBTRACT\n \t R = Rd * Sr - Rs * Dr\nG = Gd * Sg - Gs * Dg\nB = Bd * Sb - Bs * Db\n \t A = Ad * Sa - As * Da\n \t GL_MIN\n \t R = min( Rs, Rd )\nG = min( Gs, Gd )\nB = min( Bs, Bd )\n \t A = min( As, Ad )\n \t GL_MAX\n \t R = max( Rs, Rd )\nG = max( Gs, Gd )\nB = max( Bs, Bd )\n \t A = max( As, Ad )\n Rs, Gs, Bs, and As represent the source color components.\nRd, Gd, Bd, and Ad represent the destination color components.\nSr, Sg, Sb, and Sa represent the source blend factors.\nDr, Dg, Db, and Da represent the destination blend factors.\nGL_LOGIC_OP, which can be specified in OpenGL ES 2.0, is not supported.\n",0]
,["Pages/Page_55588899.html#Anchor_55588915_ソースとディスティネーションの重み係数","Source and Destination Blend Factors ",
"Source and Destination Blend Factors\nYou can use the glBlendFunc or glBlendFuncSeparate function to specify the blend factors to apply to the source and destination.\nCode 13-5. Definitions of the glBlendFunc* Functions\n void glBlendFunc(GLenum sfactor, GLenum dfactor);\nvoid glBlendFuncSeparate(GLenum srcRGB, GLenum dstRGB, GLenum srcAlpha,\n GLenum dstAlpha);\nThe glBlendFunc function allows you to specify both source and destination blend factors using sfactor and dfactor. The glBlendFuncSeparate function allows you to specify the blend factors for the source RGB and alpha components using srcRGB and srcAlpha, and the blend factors for the destination RGB and alpha components using dstRGB and dstAlpha. All of these parameters accept the following 15 values.\nTable 13-7. Source and Destination Blend Factors\n \t Value\n \t Color Blend Factor\n (Sr, Sg, Sb) or (Dr, Dg, Db)\n \t Alpha Blend Factor\n Sa or Da\n \t GL_ZERO (default for dstRGB and dstAlpha)\n \t (0, 0, 0)\n \t 0\n \t GL_ONE (default for srcRGB and srcAlpha)\n \t (1, 1, 1)\n \t 1\n \t GL_SRC_COLOR\n \t (Rs, Gs, Bs)\n \t As\n \t GL_ONE_MINUS_SRC_COLOR\n \t (1, 1, 1) - (Rs, Gs, Bs)\n \t 1 - As\n \t GL_DST_COLOR\n \t (Rd, Gd, Bd)\n \t Ad\n \t GL_ONE_MINUS_DST_COLOR\n \t (1, 1, 1) - (Rd, Gd, Bd)\n \t 1 - Ad\n \t GL_SRC_ALPHA\n \t (As, As, As)\n \t As\n \t GL_ONE_MINUS_SRC_ALPHA\n \t (1, 1, 1) - (As, As, As)\n \t 1 - As\n \t GL_DST_ALPHA\n \t (Ad, Ad, Ad)\n \t Ad\n \t GL_ONE_MINUS_DST_ALPHA\n \t (1, 1, 1) - (Ad, Ad, Ad)\n \t 1 - Ad\n \t GL_CONSTANT_COLOR\n \t (Rc, Gc, Bc)\n \t Ac\n \t GL_ONE_MINUS_CONSTANT_COLOR\n \t (1, 1, 1) - (Rc, Gc, Bc)\n \t 1 - Ac\n \t GL_CONSTANT_ALPHA\n \t (Ac, Ac, Ac)\n \t Ac\n \t GL_ONE_MINUS_CONSTANT_ALPHA\n \t (1, 1, 1) - (Ac, Ac, Ac)\n \t 1 - Ac\n \t GL_SRC_ALPHA_SATURATE\n \t (f, f, f)\n \t 1\n Rs, Gs, Bs, and As represent the source color.\nRd, Gd, Bd, and Ad represent the destination color.\nRc, Gc, Bc, and Ac represent the constant color.\nf = min(As, 1 - Ad).\nThe 3DS system also allows you to specify GL_SRC_ALPHA_SATURATE for the destination.\n",0]
,["Pages/Page_55588899.html#Anchor_55588915_定数カラー","Constant Color ",
"Constant Color\nYou can specify the constant color using the glBlendColor function.\nCode 13-6. Definition of glBlendColor Function\n void glBlendColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha);\nSpecify each of the RGB and alpha components using the red, green, blue, and alpha parameters. Specify a floating-point number between 0.0 and 1.0 for each component. By default, they are all 0.0 (0.0, 0.0, 0.0, 0.0).\n",0]
,["Pages/Page_55588899.html#PageId_55588921","13.5. Logical Operations ",
"\n13.5. Logical Operations\n",0]
,["Pages/Page_55588899.html#PageId_55588922","13.5.1. How to Use ",
"The final per-fragment operation is the logical operation, which is applied to fragment colors and framebuffer colors for an image. The result of the logical operation is written at the fragment's window coordinates in the framebuffer.\n13.5.1. How to Use\nLogical operations are used in the same way they are used in OpenGL ES 1.1.\n",0]
,["Pages/Page_55588899.html#Anchor_55588922_論理演算の有効・無効","Enabling or Disabling Logical Operations ",
"Enabling or Disabling Logical Operations\nTo enable or disable logical operations, call the glEnable or glDisable function, specifying GL_COLOR_LOGIC_OP for the cap parameter. To get the current setting, call the glIsEnabled function, specifying GL_COLOR_LOGIC_OP for the cap parameter. Logical operations are disabled by default. When disabled, logical operations are skipped, but fragment colors are written to the framebuffer. Blending is disabled when logical operations are enabled.\n",0]
,["Pages/Page_55588899.html#Anchor_55588922_演算方法","Operation ",
"Operation\nUse the glLogicOp function to specify the logical operation to be performed on the fragment color (the source) and the framebuffer color (the destination). To get the current setting, call the glGetIntegerv function, specifying GL_LOGIC_OP_MODE as the pname parameter. The default setting is GL_COPY.\nCode 13-7. Definition of the glLogicOp Function\n void glLogicOp(GLenum opcode);\nSpecify the logical operation for both the RGB and alpha components, by using the opcode parameter.\nTable 13-8. Logical Operations\n \t Value\n \t Operation\n \t C Notation\n \t GL_CLEAR\n \t 0\n \t 0\n \t GL_AND\n \t s ∧ d\n \t s & d\n \t GL_AND_REVERSE\n \t s ∧ ￢ d\n \t s & ~d\n \t GL_COPY by default.\n \t s\n \t s\n \t GL_AND_INVERTED\n \t ￢ s ∧ d\n \t ~s & d\n \t GL_NOOP\n \t d\n \t d\n \t GL_XOR\n \t s xor d\n \t s ^ d\n \t GL_OR\n \t s ∨ d\n \t s | d\n \t GL_NOR\n \t ￢ (s ∨ d)\n \t ~(s | d)\n \t GL_EQUIV\n \t ￢ (s xor d)\n \t ~(s ^ d)\n \t GL_INVERT\n \t ￢ d\n \t ~d\n \t GL_OR_REVERSE\n \t s ∨ ￢ d\n \t s | ~d\n \t GL_COPY_INVERTED\n \t ￢ s\n \t ~s\n \t GL_OR_INVERTED\n \t ￢ s ∨ d\n \t ~s | d\n \t GL_NAND\n \t ￢ (s ∧ d)\n \t ~(s & d)\n \t GL_SET\n \t all 1\n \t pow(2, n) - 1\n Where n is the number of bits in each component.\n",0]
,["Pages/Page_55588899.html#PageId_55588925","13.6. Masking the Framebuffer ",
"\n13.6. Masking the Framebuffer\nYou can apply a masking operation to the color (RGBA), stencil, and depth values that are written to the framebuffer by per-fragment operations. These values can be configured by the glColorMask, glStencilMask, and glDepthMask functions, respectively.\nCode 13-8. Framebuffer Masking Functions\n void glColorMask(GLboolean red, GLboolean green, GLboolean blue,\n GLboolean alpha);\nvoid glStencilMask(GLuint mask);\nvoid glDepthMask(GLboolean flag);\nYou can specify whether to allow (GL_TRUE) or deny (GL_FALSE) the writing of each RGBA color value and depth value. These values are all set to GL_TRUE by default.\nYou can modify the stencil masking value. This masking value is independent of the masking value for the stencil test. It is 0xFFFFFFFF (all 1s) by default.\nCONFIDENTIAL\n",0]
,["Pages/Page_55588926.html","14. Framebuffer Operations ",
"14. Framebuffer Operations\nFrame buffer operations read, copy, and otherwise manipulate framebuffer data.\nThe features supported by the 3DS system are restricted as follows.\n\tThe glDrawBuffer function has not been implemented. \tThe glReadBuffer function has not been implemented. \tThe glDrawPixels function has not been implemented. \tThe glReadPixels function can get the contents of the color, depth, and stencil buffers. \tThe glCopyPixels function has not been implemented. ",0]
,["Pages/Page_55588926.html#PageId_55588927","14.1. Reading Pixels ",
" \tThe glPixelStore* functions have not been implemented.\n14.1. Reading Pixels\nYou can use the glReadPixels function to take the contents of a render buffer (the color, depth, or stencil buffer) bound to the framebuffer and place it into memory. You can also read the contents of the depth and stencil buffers into memory, using a combined format.\nCode 14-1. glReadPixels Function\n void glReadPixels(GLint x, GLint y, GLsizei width, GLsizei height,\n GLenum format, GLenum type, void* pixels);\nFor x and y specify the starting coordinates (the lower-left corner) of the rectangular region to capture. For width and height, specify the width and height of the rectangular region. A GL_INVALID_VALUE error is generated if any of these values are negative. A GL_INVALID_OPERATION error is generated when the sum of x and width exceeds the render buffer width, or when the sum of y and height exceeds the render buffer height.\nUsing a combination of format and type, you can specify the format of the image to capture.\nTable 14-1. Formats That Can Be Specified for Reading\n \t format\n \t type\n \t Format\n \t Bits\n \t GL_RGBA\n \t GL_UNSIGNED_BYTE\n \t RGBA8\n \t 32\n \t GL_UNSIGNED_SHORT_4_4_4_4\n \t RGBA4\n \t 16\n \t GL_UNSIGNED_SHORT_5_5_5_1\n \t RGBA5551\n \t 16\n \t GL_RGB\n \t GL_UNSIGNED_BYTE\n \t RGB8\n \t 24\n \t GL_UNSIGNED_SHORT_5_6_5\n \t RGB565\n \t 16\n \t GL_BGRA\n \t GL_UNSIGNED_BYTE\n \t BGRA8\n \t 32\n \t GL_UNSIGNED_SHORT_4_4_4_4\n \t BGRA4\n \t 16\n \t GL_UNSIGNED_SHORT_5_5_5_1\n \t BGRA5551\n \t 16\n \t GL_LUMINANCE_ALPHA\n \t GL_UNSIGNED_BYTE\n \t LA8\n \t 16\n \t GL_LUMINANCE\n \t GL_UNSIGNED_BYTE\n \t L8\n \t 8\n \t GL_ALPHA\n \t GL_UNSIGNED_BYTE\n \t A8\n \t 8\n \t GL_DEPTH_COMPONENT\n \t GL_UNSIGNED_INT\n \t Depth\n \t 32\n \t GL_UNSIGNED_INT24_DMP\n \t Depth\n \t 24\n \t GL_UNSIGNED_SHORT\n \t Depth\n \t 16\n \t GL_UNSIGNED_BYTE\n \t Depth\n \t 8\n \t GL_STENCIL_INDEX\n \t GL_UNSIGNED_BYTE\n \t Stencil\n \t 8\n \t GL_DEPTH24_STENCIL8_EXT\n \t GL_UNSIGNED_INT\n \t Depth + Stencil\n \t 32\n A GL_INVALID_ENUM error is generated if you specify a combination that is not in the table.\nThe captured image is stored in pixels, using the specified format.\nImages taken from the color buffer are stored in linear format, using a pixel byte order that is OpenGL-compliant.\nImages read from the depth buffer are also stored in linear format, but the pixel data is ordered starting with the least-significant bits. For example, an image captured using a 24-bit format is stored in memory one byte at a time, starting with the least-significant byte and ending with the most-significant byte.\nImages read from the stencil buffer are stored in linear format, with one byte per pixel.\nImages read from the depth and stencil buffers in a combined format are also stored in linear format, but the pixel data is ordered starting with the least-significant bits. 24 bits of depth data are stored first, one byte at a time, from the least-significant to the most-significant byte, followed by 8 bits of stencil data.\nA GL_INVALID_FRAMEBUFFER_OPERATION error is generated if an error related to the framebuffer occurs (for example, if a color buffer has not been bound to the framebuffer). A GL_OUT_OF_MEMORY error is generated when temporary memory fails to be allocated within the library.\n",0]
,["Pages/Page_55588926.html#PageId_55588928","14.2. Copying Pixels ",
"\n14.2. Copying Pixels\nThe glCopyPixels function does not support copying data to other framebuffers.\nYou can call the glCopyTexImage2D and glCopyTexSubImage2D functions to copy the content of the color buffer to a texture. For more information, see 7.5. Copying From the Color Buffer.\n",0]
,["Pages/Page_55588926.html#PageId_55588929","14.3. Texture Rendering ",
"\n14.3. Texture Rendering\nYou can bind a texture to the framebuffer as a render buffer. For more information, see 7.6. Specifying a Texture as the Render Target.\n",0]
,["Pages/Page_55588926.html#PageId_55588930","14.4. Clearing the Framebuffer ",
"\n14.4. Clearing the Framebuffer\nIn OpenGL, the scissor test and masking affect how the glClear function clears (every buffer bound to) the framebuffer. On a 3DS system, however, the scissor test and masking are not included in the graphics pipeline and have no effect.\nYou can bind three render buffers to the framebuffer, as explained in 3.4.1. Render Buffer Allocation: a color buffer (which could be a texture or a buffer used to render gas), a depth buffer, and a stencil buffer. Index colors are not supported.\nTo clear a buffer, call the glClear function with the buffer's corresponding bit specified as an argument.\nCode 14-2. Definition of the glClear Function\n void glClear(GLbitfield mask);\nYou can specify the following values as a bit field in mask. To clear multiple buffers simultaneously, specify the bit field using bitwise OR operations.\nTable 14-2. Specifying the Buffers to Clear\n \t mask\n \t Buffer to Clear\n \t GL_COLOR_BUFFER_BIT\n \t Color buffer.\n \t GL_DEPTH_BUFFER_BIT\n \t Depth buffer.\n \t GL_STENCIL_BUFFER_BIT\n \t Stencil buffer. This must be specified together with GL_DEPTH_BUFFER_BIT.\n ",0]
,["Pages/Page_55588926.html#PageId_55588931","14.4.1. Clear Color ",
"You cannot clear the 3DS stencil buffer alone because it is combined with the depth buffer (GL_DEPTH24_STENCIL8_EXT).\n14.4.1. Clear Color\nYou can use the glClearColor function to specify the color to use when clearing the color buffer.\nCode 14-3. Specifying the Clear Color\n void glClearColor(GLclampf red, GLclampf green, GLclampf blue, GLclampf alpha);\nEach of the specified color components is clamped between 0.0 and 1.0. By default, all components are set to 0.0.\n",0]
,["Pages/Page_55588926.html#PageId_55588932","14.4.2. Clear Depth ",
"\n14.4.2. Clear Depth\nYou can use the glClearDepthf function to specify the depth value to use when clearing the depth buffer.\nCode 14-4. Specifying the Clear Depth\n void glClearDepthf(GLclampf depth);\nThe specified values are clamped between 0.0 and 1.0. The default value is 1.0.\n",0]
,["Pages/Page_55588926.html#PageId_55588933","14.4.3. Clear Stencil ",
"\n14.4.3. Clear Stencil\nYou can use the glClearStencil function to specify the value to use when clearing the stencil buffer.\nCode 14-5. Specifying the Clear Stencil\n void glClearStencil(GLint s);\nSpecify a value between 0 and 255 to use for clear operations. The default value is 0.\nCONFIDENTIAL\n",0]
,["Pages/Page_55588934.html","15. Other ",
"15. Other\n",0]
,["Pages/Page_55588934.html#PageId_55588935","15.1. glFinish and glFlush Functions ",
"15.1. glFinish and glFlush Functions\nYou can use the glFinish and glFlush functions to explicitly instruct the library to execute all 3D commands issued by gl functions to that point.\nCode 15-1. Definitions of the glFinish and glFlush Functions\n void glFinish(void);\nvoid glFlush(void);\nThe glFinish function is the same as the glFlush function on a 3DS system.\n",0]
,["Pages/Page_55588934.html#PageId_55588936","15.2. Differences From OpenGL ES ",
"\n15.2. Differences From OpenGL ES\nMost features and functions follow the OpenGL ES specifications, but there are some differences. For example, some OpenGL ES functions and features have not been implemented, and others have restrictions.\nTable 15-1. Differences by Feature\n \t Features\n \t Differences\n \t GL_ALPHA_TEST\n \t Reserved uniforms are used to control the alpha test.\n \t GL_CLIP_PLANEi\n \t Reserved uniforms are used to control clipping.\n \t GL_COLOR_LOGIC_OP\n \t Logical operations are not present in OpenGL ES 2.0, but the equivalent features from OpenGL ES 1.1 have been implemented for 3DS.\n \t GL_DITHER\n \t Dithering is not supported.\n \t GL_FOG\n \t Reserved uniforms are used to control fog. Fog coefficients are set by a lookup table that takes depth values in window coordinates as input, rather than by the distance from the viewpoint.\n \t GL_INDEX_LOGIC_OP\n \t Index colors are not supported.\n \t GL_LIGHTING\n \t Reserved uniforms are used to control lighting. 3DS lighting is a per-fragment operation.\n \t GL_POLYGON_OFFSET_FILL\n GL_POLYGON_OFFSET_LINE\n GL_POLYGON_OFFSET_POINT\n \t Only GL_POLYGON_OFFSET_FILL is supported for controlling polygon offsets.\n \t GL_SAMPLE_COVERAGE\n GL_SAMPLE_ALPHA_TO_COVERAGE\n \t Multisampling is not supported.\n \t GL_SCISSOR_TEST\n \t The scissor test is run as a subprocess of rasterization.\n \t GL_TEXTURE_2D\n GL_TEXTURE_CUBE_MAP\n \t Reserved uniforms are used to control texture units.\n Table 15-2. Differences by Function\n \t Functions\n \t Differences\n \t Shader Functions\n \t glBufferData\n \t Only GL_STATIC_DRAW can be specified for usage.\n \t glCompileShader\n \t This function is not implemented.\n \t glCreateProgram\n \t This function uses a (13-bit) namespace that is independent of shader objects.\n \t glCreateShader\n \t This function uses a namespace that is independent of program objects.\n \t glDrawArrays, glDrawElements\n \t GL_POINTS, GL_LINES, GL_LINE_STRIP, and GL_LINE_LOOP are not supported. You must use the geometry shader to render points and lines.\n \t glGetProgramInfoLog\n \t This function is not implemented.\n \t glGetShaderInfoLog\n \t This function is not implemented.\n \t glGetShaderPrecisionFormat\n \t This function is not implemented.\n \t glLineWidth\n \t This function is not implemented.\n \t glReleaseShaderCompiler\n \t This function is not implemented.\n \t glShaderBinary\n \t Only GL_PLATFORM_BINARY_DMP can be specified for binaryformat.\n \t glShaderSource\n \t This function is not implemented.\n \t glValidateProgram\n \t This function does nothing when it is called.\n \t glVertexAttribPointer\n \t Neither GL_FIXED nor GL_UNSIGNED_SHORT can be specified for type. The data pointer must be 4-byte aligned when specifying GL_FLOAT for type, or 2-byte aligned when specifying GL_SHORT. This function does not support normalization specified via arguments.\n \t Viewport Functions\n \t glViewport\n \t You cannot specify a negative value for x or y.\n \t Texture Functions\n \t glActiveTexture\n \t An error is generated when GL_TEXTURE3 is specified.\n \t glCompressedTexImage2D\n \t The width and height must be specified as powers of 2 (from 16 through 1024).\n \t glCompressedTexSubImage2D\n \t This function is not implemented.\n \t glGenerateMipmap\n \t This function is not implemented.\n \t glTexImage1D\n glTexSubImage1D\n \t These functions are used to load lookup tables. One-dimensional textures are not supported.\n \t glTexImage2D\n \t The width and height must be specified as powers of 2 (from 8 through 1024).\n \t glTexSubImage2D\n \t This function is not implemented.\n \t Render Buffer Functions\n \t glClear\n \t Index colors are not supported. The stencil buffer must be cleared at the same time as the depth buffer. This function is unaffected by the scissor test and masking.\n \t glCopyPixels\n \t This function is not implemented.\n \t glDrawBuffer\n \t This function is not implemented.\n \t glDrawPixels\n \t This function is not implemented.\n \t glFramebufferRenderbuffer\n \t Because the stencil buffer is combined with the depth buffer, GL_DEPTH_STENCIL_ATTACHMENT must be specified as the attachment point for the stencil buffer.\n \t glFramebufferTexture2D\n \t The color buffer is the only supported attachment point.\n \t glPixelStorei\n \t This function is not implemented.\n \t glReadBuffer\n \t This function is not implemented.\n \t glRenderbufferStorage\n \t There are restrictions on which formats can be specified. The only format that can be used for the stencil buffer is GL_DEPTH24_STENCIL8_EXT.\n \t glSampleCoverage\n \t This function is not implemented.\n \t glStencilFuncSeparate\n glStencilOpSeparate\n glStencilMaskSeparate\n \t This function is not implemented.\n \t Blending Functions\n \t glBlendEquation\n glBlendEquationSeparate\n \t GL_LOGIC_OP, which can be specified in OpenGL ES 2.0, is not supported.\n \t glBlendFunc\n glBlendFuncSeparate\n \t GL_SRC_ALPHA_SATURATE can also be specified for the destination.\n \t Miscellaneous\n \t glFinish\n \t This function does the same thing as glFlush.\n \t glFlush\n \t There is no difference until glFinish is called.\n \t glHint\n \t This function is not implemented.\n \t glPolygonOffset\n \t The value set for factor is ignored.\n ",0]
,["Pages/Page_55588934.html#PageId_55588937","15.3. Improving Buffer Access Performance ",
"\n15.3. Improving Buffer Access Performance\nWhen you are not using either the color, depth, or stencil buffer, you can explicitly disable that buffer's features to reduce unnecessary processing. Because the same buffer is used for the 3DS depth and stencil buffers, however, settings to access either one have the same performance as settings to access both.\n",0]
,["Pages/Page_55588934.html#PageId_55588938","15.3.1. Write Access to the Color Buffer ",
"The following sections describe the conditions for accessing each buffer. Avoid these conditions to prevent unnecessary accesses from occurring.\n15.3.1. Write Access to the Color Buffer\nThese accesses occur when the glColorMask function specifies a value of GL_TRUE for any component.\n",0]
,["Pages/Page_55588934.html#PageId_55588939","15.3.2. Read Access to the Color Buffer ",
"\n15.3.2. Read Access to the Color Buffer\nThese accesses occur when write accesses to the color buffer occur and any of the following conditions are met.\n\tGL_BLEND has been enabled by the glEnable function. \tA call of the glColorMask function has not specified the same values for all components. \tGL_COLOR_LOGIC_OP has been enabled by the glEnable function.\n",0]
,["Pages/Page_55588934.html#PageId_55588940","15.3.3. Write Access to the Depth Buffer ",
"\n15.3.3. Write Access to the Depth Buffer\nThese accesses occur when GL_DEPTH_TEST has been enabled by the glEnable function and GL_TRUE has been specified by the glDepthMask function.\n",0]
,["Pages/Page_55588934.html#PageId_55588941","15.3.4. Read Access to the Depth Buffer ",
"\n15.3.4. Read Access to the Depth Buffer\nThese accesses occur when GL_DEPTH_TEST has been enabled by the glEnable function.\n",0]
,["Pages/Page_55588934.html#PageId_55588942","15.3.5. Write Access to the Stencil Buffer ",
"\n15.3.5. Write Access to the Stencil Buffer\nThese accesses occur when GL_STENCIL_TEST has been enabled by the glEnable function, and a nonzero masking setting is used in the glStencilMask function.\n",0]
,["Pages/Page_55588934.html#PageId_55588943","15.3.6. Read Access to the Stencil Buffer ",
"\n15.3.6. Read Access to the Stencil Buffer\nThese accesses occur when GL_STENCIL_TEST has been enabled by the glEnable function.\n",0]
,["Pages/Page_55588934.html#PageId_55588944","15.4. Improving CPU Performance ",
"\n15.4. Improving CPU Performance\nYou may be able to improve processing speed by paying attention to the following points when you implement your application.\n\tLink as many vertex shader objects together as possible. You can link multiple shader objects together to generate a single shader binary. It is less resource intensive to switch between shader objects when they are linked to the same shader binary than when they are linked to different shader binaries. \tEnsure that applications keep the uniform location values obtained by the glGetUniformLocation function, and use them repeatedly. Location values are static after the glLinkProgram function is called. They do not change until glLinkProgram is called again. \tDo not call the nngxSplitDrawCmdlist function unnecessarily because it generates a split command each time it is called. For example, when the nngxTransferRenderImage function is called, it generates a split command internally. Calling nngxSplitDrawCmdlist immediately afterwards would generate an unnecessary command. \tUse a vertex buffer, whenever possible, to send vertex attribute data to a shader. If you do not use a vertex buffer, the CPU accumulates vertex data in the 3D command buffer, and CPU processing increases significantly. \tUse a texture collection or a vertex state collection when the same texture or vertex buffer is used repeatedly in a particular rendering pass. By binding all textures and setting all vertex arrays at the same time, you can reduce the cost of function calls. \tWhen executing the same shader object with different uniform settings, it is less resource intensive to attach that shader object to multiple program objects (each with its own uniform settings) and then switch between the program objects than it is to switch between many uniform settings on a single program object. The reason is that uniform values are saved for each program object. \tDo not delete and regenerate lookup table objects frequently. After the lookup table data is loaded by the glTexImage1D function, it is converted into an internal hardware format each time the lookup table object is used.\n",0]
,["Pages/Page_55588934.html#PageId_55588945","15.5. Using Vertex Buffers ",
"\n15.5. Using Vertex Buffers\nWhen a vertex buffer is used, the GPU geometry pipeline loads vertex data. When a vertex buffer is not used, however, the CPU sorts the vertex arrays according to the vertex index arrays, converts all vertex data into 24-bit floating-point values, and then fills the command buffer. This process imposes a considerably high processing load on the CPU, and also decreases the efficiency with which the GPU geometry pipeline loads data. This also requires a larger command buffer. All vertex data is converted into 24-bit floating-point numbers for the x, y, z, and w components when it is loaded into the command buffer, which must be able to hold at least 12 bytes multiplied by both the number of vertices and the number of vertex attributes.\n",0]
,["Pages/Page_55588934.html#PageId_55588946","15.5.1. Data Structure for Vertex Arrays ",
"Vertex buffers can be processed more quickly when they are placed in VRAM than when they are placed in main (device) memory. Splitting them between VRAM and main memory results in the same speed as placing them in main memory.\n15.5.1. Data Structure for Vertex Arrays\nA vertex array can either be structured as an interleaved array, which is an array of structures that contain multiple vertex attributes, or as an independent array, which is an array of single vertex attributes.\nWhen you use a vertex buffer, interleaved arrays are more efficient at loading vertex data than independent arrays. The time spent loading vertex data is often hidden by the time spent for processing later on, such as in the vertex shader and during rasterization. When the vertex buffer is placed in main memory, however, by making the loading of data more efficient, you can sometimes reduce the cost of data access and speed up processing.\n",0]
,["Pages/Page_55588934.html#PageId_55588947","15.6. Getting the Starting Address for Each Buffer Type ",
"\n15.6. Getting the Starting Address for Each Buffer Type\nYou can get the starting addresses of the data regions allocated for each texture object, vertex buffer object, and render buffer object.\nThe GPU can directly access all obtained addresses. You cannot get the address of the copy region that is generated by the driver.\nCode 15-2. Getting the Starting Address for Each Buffer Type\n void glGetTexParameteriv(GLenum target, GLenum pname, GLint* params);\nvoid glGetBufferParameteriv(GLenum target, GLenum pname, GLint* params);\nvoid glGetRenderbufferParameteriv(GLenum target, GLenum pname, GLint* params);\nTo get the texture address, call the glGetTexParameteriv function and specify GL_TEXTURE_DATA_ADDR_DMP as the pname parameter.\nTo get the vertex buffer address, call the glGetBufferParameteriv function and specify GL_BUFFER_DATA_ADDR_DMP as the pname parameter.\nTo get the render buffer address, call the glGetRenderbufferParameteriv function and specify GL_RENDERBUFFER_DATA_ADDR_DMP as the pname parameter.\nThese functions allow you to get various types of information, depending on the value passed in as the pname parameter. For more information about the information that can be obtained, see the CTR-SDK API Reference.\n",0]
,["Pages/Page_55588934.html#PageId_55588948","15.7. Number of Bytes Loaded for Various Data Types ",
"\n15.7. Number of Bytes Loaded for Various Data Types\n",0]
,["Pages/Page_55588934.html#PageId_55588949","15.7.1. Vertex Buffers ",
"This section describes the number of bytes that the GPU loads from vertex buffers, textures, and command buffers in a single operation.\n15.7.1. Vertex Buffers\nThe number of bytes loaded concurrently from a vertex buffer depends on the order of the vertex indices.\nThe glDrawElements function loads 16 vertex indices concurrently from an index array, sorts them, and then loads data from the vertex array in the same order as the sorted indices. Consecutive data is loaded from the vertex array for any consecutive vertex indices.\nMultiple vertex attributes are loaded as an interleaved array from a vertex array if: (1) the array contains vertex attributes and has been enabled by the glEnableVertexAttribArray function, and (2) the driver interprets the array as an interleaved array (a vertex array that combines multiple vertex attributes) based on the information specified by the glVertexAttribPointer function.\nBurst reads of up to 256 bytes are used to load consecutive data from a vertex array. If there are more than 256 bytes to be loaded, they are read 256 bytes at a time. Data is read at least 16 bytes at a time, even for non-consecutive data.\nThe glDrawArrays function processes data just like an index array, using consecutive numbers starting at 0.\n",0]
,["Pages/Page_55588934.html#PageId_55588950","15.7.2. Textures ",
"\n15.7.2. Textures\nThe number of bytes that are loaded at one time depends on the texture format. The following table shows the number of bytes that are transferred from VRAM, avoiding the texture cache.\nTable 15-3. Texture Format and Loading Units\n \t format\n \t type\n \t Bytes\n \t GL_RGBA\n GL_RGBA_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t 128\n \t GL_UNSIGNED_SHORT_5_5_5_1\n \t 64\n \t GL_UNSIGNED_SHORT_4_4_4_4\n \t 64\n \t GL_RGB\n GL_RGB_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t 96\n \t GL_UNSIGNED_SHORT_5_6_5\n \t 64\n \t GL_LUMINANCE_ALPHA\n GL_LUMINANCE_ALPHA_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t 64\n \t GL_UNSIGNED_BYTE_4_4_DMP\n \t 32\n \t GL_LUMINANCE\n GL_LUMINANCE_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t 32\n \t GL_UNSIGNED_4BITS_DMP\n \t 16\n \t GL_ALPHA\n GL_ALPHA_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t 32\n \t GL_UNSIGNED_4BITS_DMP\n \t 16\n \t GL_HILO8_DMP\n GL_HILO8_DMP_NATIVE_DMP\n \t GL_UNSIGNED_BYTE\n \t 64\n \t GL_ETC1_RGB8_NATIVE_DMP\n \t -\n \t 128\n \t GL_ETC1_ALPHA_RGB8_A4_NATIVE_DMP\n \t -\n \t 32\n ",0]
,["Pages/Page_55588934.html#PageId_55588951","15.7.3. Command Buffer ",
"\n15.7.3. Command Buffer\nA command buffer loads 128 bytes at a time.\n",0]
,["Pages/Page_55588934.html#PageId_55588952","15.8. Block-Shaped Noise Is Rendered on Some Pixels ",
"\n15.8. Block-Shaped Noise Is Rendered on Some Pixels\nData in the 3DS framebuffer is processed 4×4 pixels at a time. These blocks of pixels are called block addresses and are also used to manage the framebuffer cache. Tag information in the cache is cleared at several times, including when the glFinish, glFlush, or glClear function is called; when the framebuffer-related GPU state (NN_GX_STATE_FRAMEBUFFER, NN_GX_STATE_FBACCESS) is validated; and when the command list is split by the nngxSplitDrawCmdlist function. Cache tags are initialized with their default value of 0x3FFF after tag information in the cache is cleared. Consequently, any pixels that you attempt to render immediately afterward at the same block address as the default cache tag value (0x3FFF) will mistakenly hit the cache. As a result, an incorrect color will be applied to the pixels.\nBlock addresses are assigned consecutively, beginning at 0, in 16-pixel blocks from the starting address of the framebuffer (the color buffer, depth buffer, and stencil buffer). Because addresses are assigned to data that has been laid out in the GPU render format, pixel locations in a rendered image correspond to different block addresses in block 8 mode and in block 32 mode.\nThe problem described in this section is triggered by pixels that are assigned a block address of 0x3FFF. The problem does not occur when the total number of framebuffer blocks is less than or equal to 0x3FFF (in other words, when the total number of framebuffer pixels is less than or equal to 0x3FFF×16, or 262,128 pixels). This is equivalent to a 512×512 rectangle). This problem also does not occur when there are no read accesses on the color buffer, depth buffer, or stencil buffer.\n Note:\n Cache tag information is also cleared when a value of 1 is written to GPU register 0x0110.\n For more information about how to access GPU registers directly and how to control the GPU state, block mode, and framebuffer read access, see the 3DS Programming Manual: Advanced Graphics.\n",0]
,["Pages/Page_55588934.html#PageId_55588953","15.8.1. Relationship Between Pixels and Block Addresses ",
"\n15.8.1. Relationship Between Pixels and Block Addresses\nAs mentioned previously, block addresses begin at 0 and are assigned in ascending order, 16 pixels at a time, from the starting addresses of the color buffer and depth/stencil buffer, which are laid out in the GPU render format. Unlike the origin for the glViewport function, the buffer addresses start with the pixels at the upper-left corner of the image to render. Note, too, that the image width (the horizontal direction) corresponds to the shorter edge of the LCD.\n",0]
,["Pages/Page_55588934.html#PageId_55588954","15.8.1.1. Block 8 Mode ",
"Because there are different ways to assign addresses, the block mode changes which block address in the cache corresponds to the pixels on an image.\n15.8.1.1. Block 8 Mode\nBlock address 0 corresponds to the 4×4 block of pixels at the upper-left corner of the rendered image; block address 1 corresponds to the 4×4 block of pixels immediately to the right of block address 0; block address 2 corresponds to the 4×4 block of pixels immediately below block addresses 0; and block address 3 corresponds to the 4×4 block of pixels immediately below block address 1. Block addresses increase to the right 8×8 pixels at a time. After they reach the edge of the image, they continue from the left edge of the image on the next row.\nFigure 15-1. Block Address Assignments in Block 8 Mode\nThe value of N in the figure is calculated by taking one-quarter the width of the framebuffer (in pixels) and multiplying it by two.\n",0]
,["Pages/Page_55588934.html#PageId_55588955","15.8.1.2. Block 32 Mode ",
"\n15.8.1.2. Block 32 Mode\nIn block 32 mode, addresses are assigned in metablocks of 32×32 pixels. Metablock address 0 corresponds to the 32×32-pixel region at the upper-left corner of the rendered image, and metablock address 1 corresponds to the 32×32-pixel region immediately to its right. Metablock addresses increase to the right 32×32 pixels at a time. After they reach the edge of the image, they continue from the left edge of the image on the next row.\nAs the following figure shows, the block addresses of the pixels are arranged in a zigzag pattern within each metablock, starting with the 4×4-pixel block at the upper-left corner. To find the block address of a single pixel in the image, multiply its metablock address by 0x40, and then add its block address within the metablock.\nFigure 15-2. Block Address Assignments in Block 32 Mode\nThe left side of the figure shows the block addresses (in hexadecimal) for pixels within a metablock. The right side of the figure shows the metablock addresses for the entire image.\n",0]
,["Pages/Page_55588934.html#PageId_55588956","15.8.2. Workaround #1 ",
"\n15.8.2. Workaround #1\nThis problem does not occur when the framebuffer has no more than 262,128 pixels (the product of its width and height). In other words, you can work around this problem by using a framebuffer that is no larger than necessary.\nNote that the problem does not occur with a framebuffer that has the same size as one of the LCDs—240×400 (96,000 pixels) or 240×320 (76,800 pixels)—because the total number of pixels does not exceed 262,128.\nAlthough you specify the framebuffer size with the glRenderbufferStorage function, if you allocate a large framebuffer and only use a part (240×400 region) of it, you can avoid this problem by using no more than the minimum necessary size for the allocated framebuffer region.\n",0]
,["Pages/Page_55588934.html#PageId_55588957","15.8.3. Workaround #2 ",
"\n15.8.3. Workaround #2\nYou can work around this problem by adjusting the size of the framebuffer so that the problematic pixels at block address 0x3FFF are located outside of the rendering region.\nFor example, when you allocate a 480×800 framebuffer (as you would to apply 2×2 antialiasing in block 8 mode), block address 0x3FFF is assigned to the 4×4-pixel region whose upper-left corner is located at pixel coordinates of (124, 548). If you were to extend the size of the framebuffer by 32 pixels to 512×800, however, block address 0x3FFF would be assigned to the 4×4-pixel region whose upper-left corner is located at pixel coordinates (508, 508). By configuring the viewport to display only the 480×800 region on the left side of the framebuffer, you can avoid these problematic pixels.\nOne disadvantage of this method is that it requires you to allocate a framebuffer that is larger than necessary, which wastes VRAM. However, it is a simple workaround that only involves adjusting the framebuffer size.\nFor more more information about how differences in the block mode and framebuffer size affect the pixel coordinates to which block address 0x3FFF is assigned, see 15.8.1. Relationship Between Pixels and Block Addresses.\n",0]
,["Pages/Page_55588934.html#PageId_55588958","15.8.4. Workaround #3 ",
"\n15.8.4. Workaround #3\nYou can work around this problem by rendering several pixels that are not at block address 0x3FFF to change the content of cache tags immediately after they have been cleared.\nTo change the content of the cache tags, you must render four pixels at specific block addresses. When both the color buffer and depth/stencil buffer have been configured to be read, these four pixels must each have a different block address, for which the lower three bits are all 1 (0x7). When only one buffer (either the color buffer or the depth/stencil buffer) has been configured to be read, these four pixels must each have a different block address, for which the lower four bits are all 1 (0xF).\nFor example, assume that pixels are rendered at the following block addresses immediately after cache tags are cleared: 0x00, 0x01, 0x0F, 0x02, 0x1F, 0x03, 0x0F, 0x2F, and 0x3F.\nBlock addresses 0x00 and 0x01 do not count because their lower four bits are not 0xF.\nBlock address 0x0F is only counted once, even though pixels are rendered there twice. In this example, the workaround is only effective after pixels have been rendered at block addresses 0x0F, 0x1F, 0x2F, and 0x3F. If pixels at block address 0x3FFF are rendered before the pixels at block address 0x3F, the problem would occur.\nYou can work around this problem by rendering dummy polygons, with pixels that meet these conditions, immediately after cache tags are cleared, given the following caveats. The following are valid dummy pixels.\n\tPixels that fail the depth test, stencil test, or alpha test. If you use settings that cause dummy pixels to always fail these tests (for example, by specifying GL_NEVER for the depth test function), make sure that you restore the original depth test function when you resume normal rendering. Note that the cache flush command (a command that writes to register 0x111) would be required at this time. \tPixels that do not affect the color buffer when they are rendered because of alpha blend settings. \tThe following are not valid dummy pixels. Pixels that are clipped by the view volume or user-defined clipping planes. \tPixels that are dropped by the scissor test. ",0]
,["Pages/Page_55588934.html#PageId_55588959","15.8.4.1. Block 8 Mode ",
" \tPixels that are dropped by the early depth test.\n 15.8.4.1. Block 8 Mode\nWhen you render a dummy polygon to work around this problem, you must choose pixels at block addresses whose lower four bits (or lower three bits) are all 1. If you look at how block addresses are arranged in block 8 mode, the lower four bits of the block address follow the same 32×8-pixel pattern repeated horizontally, and the lower three bits of the block address follow the same 16×8-pixel pattern repeated horizontally. However, depending on the framebuffer width, these patterns may be shifted horizontally by eight pixels, for every eight pixels vertically.\nFigure 15-3. Block Address Ordering in Block 8 Mode\nThe following table shows rectangle sizes that meet the conditions for a dummy polygon. Note that the rectangle with the smallest possible area must be placed so that the pixels at its four corners have block addresses that meet the necessary conditions.\nTable 15-4. Rectangles to Render as Dummy Polygons in Block 8 Mode\n \t Rectangle Shape/Conditions\n \t Lower 4 Bits Are All 1\n \t Lower 3 Bits Are All 1\n \t Rectangle with the smallest possible area (cannot be placed anywhere).\n \t 94×1\n \t 46×1\n \t Rectangle that can be placed anywhere.\n \t 125×5\n29×29\n \t 61×5\n13×29\n ",0]
,["Pages/Page_55588934.html#PageId_55588960","15.8.4.2. Block 32 Mode ",
"\n15.8.4.2. Block 32 Mode\nWhen you render a dummy polygon to work around this problem, you must choose pixels at block addresses whose lower four bits (or lower three bits) are all 1. If you look at how block addresses are arranged in block 32 mode, the lower four bits of the block address follow the same 32×32-pixel pattern repeated horizontally, and the lower three bits of the block address follow the same 32×16-pixel pattern repeated horizontally.\nFigure 15-4. Block Address Ordering in Block 32 Mode\nThe following table shows rectangle sizes that meet the conditions for a dummy polygon. Note that the rectangle with the smallest possible area must be placed so that the pixels at its four corners have block addresses that meet the necessary conditions.\nTable 15-5. Rectangles to Render as Dummy Polygons in Block 32 Mode\n \t Rectangle Shape/Conditions\n \t Lower 4 Bits Are All 1\n \t Lower 3 Bits Are All 1\n \t Rectangle with the smallest possible area (cannot be placed anywhere).\n \t 46×1\n14×14\n \t 46×1\n14×6\n \t Rectangle that can be placed anywhere.\n \t 61×13\n29×29\n \t 61×5\n29×13\n",0]
,["Pages/Page_55588934.html#PageId_55588961","15.9. Lines Are Rendered in Error and the Region Following the Framebuffer Is Corrupted ",
"\n15.9. Lines Are Rendered in Error and the Region Following the Framebuffer Is Corrupted\nWhen rendering an extremely small polygon that has a right edge close to the window’s x-coordinate of 0, the system sometimes renders lines in error. This phenomenon is caused by coordinate wraparound when the pixel's x-coordinate becomes negative, due to calculation errors on polygon pixel generation. Because of the extremely large x-coordinate value, the system renders a polygon with extremely elongated dimensions in the positive x direction.\nThis phenomenon can also cause corruption in the memory region outside of the rendering memory region. The wrapped x-coordinate becomes 1023, and the system generates pixels from (0, y) through (1023, y), regardless of the size of the rendering region. In other words, the system generates pixels with x-coordinates from 0 through 1023, even when the rendering region size is set to a width smaller than 1024, such as 256×256. The framebuffer is accessed at the addresses calculated from the pixel's (x, y) coordinates and the width of the rendering region, but the raw x-coordinate is used for address calculation, even when it is outside of the rendering region's width. Consequently, depending on the y-coordinate value, the system might write pixel color data to memory addresses following the last address of the rendering region.\n Note:\n Memory corruption does not happen when the rendering region is cropped by using a scissor test. We recommend configuring a scissor test to avoid possible memory corruption. Conducting a scissor test does not entail any penalty in terms of GPU performance.\nThe conditions under which this phenomenon occurs depend solely on the window coordinates. For any set of window coordinates, the problem either always occurs or never occurs. This problem occurs in relation to the view volume or polygons generated as a result of clipping, so it occurs even when the original polygon itself is large, provided that the polygon protrudes beyond the edge of the screen when the window’s x-coordinate is 0, producing an extremely small area contained within the view volume.\nOne workaround for this issue is to adjust the x-coordinate in the vertex shader. The clipping x-coordinate calculated by the vertex shader is clipped from -w to w, so x values close to -w indicate a vertex close to the screen edge at the window’s x-coordinate of 0. You can avoid the erroneous lines by moving any such vertices—vertices located close to the edge of the screen where the x-coordinate is 0—away from the edge of the screen by adjusting the x value in the -w direction. This workaround only changes the vertex coordinate by no more than one pixel, so it has almost no effect on rendering results.\nWhen processing this in the vertex shader, handle the x value after applying a projection transform (the value written to the output register as the vertex coordinate x value) as follows.\nCode 15-3. Processing to Avoid Rendering Lines in Error\n if ( -w < x && x < -w * (1-epsilon) ) x = -w;\nThese x and w values are the x and w values of the vertex coordinate after the projection transformation. The epsilon value is a variable for adjustment that is to be specified as appropriate for the scene you are rendering.\nThe following code section is a sample implementation of a vertex shader. Instructions starting from mul are included to avoid displaying lines in error.\nCode 15-4. Workaround Implementation Example\n // v0 : position attribute\n// o0 : output for position\n// c0-c3 : modelview matrix\n// c4-c7 : projection matrix\n// c8 : (1 - epsilon, 1, any, any)\nm4x4 r0, v0, c0 // modelview transformation\nm4x4 r1, r0, c4 // projection transformation\nmul r2.xy, -r1.w, c8.xy // r2.x = -w * (1-epsilon), r2.y = -w\ncmp 2, 4, r1.xx, r2.xy //\nifc 1, 1, 1 // if ((x < -w * (1-epsilon)) && (x > -w))\n mov r1.x, -r1.w // x = -w;\nendif\nmov o0, r1\n",0]
,["Pages/Page_55588934.html#PageId_55588962","15.10. Changing the Priority of Operations Within a Driver ",
"\n15.10. Changing the Priority of Operations Within a Driver\nThe CPU in the 3DS has two cores, one dedicated to running applications, and the other dedicated to controlling the processes of the system’s devices.\nThe core for the system controls multiple devices, and GPU processing has a high priority among these devices. So heavy graphics processing can affect processing for other devices. In such cases, you can use the nngxSetInternalDriverPrioMode function to set the GPU to a lower priority and minimize the effect on other devices.\nCode 15-5. Changing the Priority of GPU Driver Operations\n void nngxSetInternalDriverPrioMode(nngxInternalDriverPrioMode mode);\nSelect from the values below for the mode parameter.\nTable 15-6. Internal Operation Priorities\n \t Definition\n \t Description\n \t NN_GX_INTERNAL_DRIVER_PRIO_MODE_HIGH\n \t High priority (default)\n \t NN_GX_INTERNAL_DRIVER_PRIO_MODE_LOW\n \t Low priority\n Note that lowering the priority for the GPU reduces the performance effect on other devices, but also reduces graphics performance.\n",0]
,["Pages/Page_55588934.html#PageId_55588963","15.11. Functions That Allocate Internal Buffers Within the Library ",
"\n15.11. Functions That Allocate Internal Buffers Within the Library\n",0]
,["Pages/Page_55588934.html#PageId_55588964","15.11.1. nngxValidateState, glDraw* ",
"The library implicitly allocates internal buffers for some of the gl and nngx functions.\n15.11.1. nngxValidateState, glDraw*\nIf a reference table (LUT) is being used, the library allocates an internal buffer.\nWhen glTexture1D is called and the library is notified that a reference table will be used, the next time nngxValidateState or glDraw* is executed, an intermediate buffer is allocated for loading the reference table. However, when a 3D command is generated directly, and a buffer for the reference table is specified, no intermediate buffer is allocated.\nNote that the allocated region is freed by glDeleteTexture or nngxFinalize.\n",0]
,["Pages/Page_55588934.html#PageId_55588965","15.11.2. Command Lists, Display Lists, Textures, and Similar Objects ",
"\n15.11.2. Command Lists, Display Lists, Textures, and Similar Objects\nFunctions for allocating command lists, display lists, textures, and other objects allocate internal buffers for each object. These buffers are maintained until the corresponding object has been destroyed using nngxDelete* or glDelete*.\nThis applies to the following functions.\nnngxGenCmdlists, nngxGenDisplaybuffers, glCreateProgram, glCreateShader, glGenBuffers, glGenRenderbuffers, glGenTextures, and others.\n",0]
,["Pages/Page_55588934.html#PageId_55588966","15.12. Analyzing Causes of GPU Hangs ",
"\n15.12. Analyzing Causes of GPU Hangs\nThe data returned from a call to the nngxGetCmdlistParameteri function when passing NN_GX_CMDLIST_HW_STATE in the pname parameter includes a number of bits that indicate whether the hardware is busy. This data can be helpful in analyzing the cause of problems in the operation of the hardware, such as when the GPU hangs.\nWhen the hardware is malfunctioning, the likely cause is modules that are stuck in the busy state. For modules that work in sequence, such as the triangle setup > rasterization > texture unit modules, the busy signal propagates from the last module through to the first module in the chain. When a sequence of modules is busy, the last module in the chain is the most likely cause. However, the per-fragment operation module indicated by bit 6 in the returned data can be stuck in the busy state due to invalid data from a previous module, so the most likely cause could also be an earlier module.\nRoughly speaking, busy signals propagate from rasterizing and pixel processing marked by bits 0 through 7, or from geometry processing marked by bits 8 through 16.\nRasterizing and pixel processing covers the sequence of modules of triangle setup > rasterization > texture unit > fragment lighting > texture combiner > per-fragment operation, with busy signals from modules later in the chain propagating to modules earlier in the chain. In other words, busy signals propagate in order from bit 5 to bit 0.\nBit 6 is also a per-fragment operation module busy signal. However, although this does propagate to bits 0 and 1, it does not propagate to bits 2, 3, or 4.\nBit 7 indicates a busy signal from the early depth test module, which occurs when the system is waiting for the early depth buffer to clear (GPU built-in memory). This busy signal does not propagate to other modules.\nA busy signal in the triangle setup module does not propagate to earlier modules (the vertex cache or geometry generator). In other words, no busy signals propagate between rasterizing and pixel processing and geometry processing.\nGeometry processing takes place as follows. Vertex input process module (which loads command buffers and vertex arrays) ⇒ vertex processor ⇒ post vertex cache (in that order). Busy signals from modules later in the chain propagate to modules earlier in the chain. In other words, busy signals propagate in this order: bit 16 ⇒ (bit 11, bit 12, bit 13, bit 14) ⇒ bit 8 ⇒ bit 9. Bits 11, 12, 13, and 14 correspond to the busy states of vertex processors 0, 1, 2, and 3, but because each vertex processor is allocated in parallel with the vertex loading module and post-vertex cache, a busy signal from the post-vertex cache propagates to one or more of the four vertex processors. The signal might not propagate to all four of the vertex processors.\nThis description applies to the situation when the geometry shader is disabled. When it is enabled, vertex processor 0, which is the geometry shader processor, comes after the post-vertex cache. In this case, a busy signal from the geometry shader processor propagates to the post-vertex cache, but it does not propagate to any earlier modules. In other words, a busy signal would propagate in this order: bit 11 > bit 16. A busy signal arising from the post-vertex cache does propagate to earlier modules (vertex processors 1, 2, and 3). In other words, a busy signal would propagate in this order: bit 11 ⇒ bit 16, and bit 16 ⇒ (bit 12, bit 13, bit 14) ⇒ bit 8 ⇒ bit 9.\nThe post-vertex cache, indicated by bit 16, outputs a busy signal when it is filled to capacity with vertex data. If the cache cannot output this data to the next module for some reason, such as when the next module is not responding, vertex data fills the post-vertex cache to capacity. If the geometry shader is disabled, the next module is the triangle setup module. If the geometry shader is enabled, the next module is the geometry processor (vertex processor 0).\nThe cause of the GPU hanging when the texture unit indicated by bit 2 is busy, can be attributed to a bug in the hardware that occurs when simultaneously using textures, both in and not in, a 4-bit format as a multitexture.\nIf the GPU hangs due to an incorrect load array setting (the vertex attribute data load unit to the GPU), bit 8 enters a busy state.\nIf the GPU hangs due to the vertex shader’s output of NaN (or the geometry shader’s output pf NaN when the geometry shader is being used), the rasterization module and the triangle setup (bit 0 and bit 1) enter a busy state.\n Note:\n For more information about the nngxGetCmdlistParamerteri function and the bits obtained with NN_GX_CMDLIST_HW_STATE, see 4.1.10. Getting Command List Parameters.\n",0]
,["Pages/Page_55588934.html#PageId_55588967","15.12.1. Hardware State When the GPU Hangs ",
"\n15.12.1. Hardware State When the GPU Hangs\nThe following table provides examples of the hardware state obtained for NN_GX_CMDLIST_HW_STATE when the GPU hangs, and the related cause.\nTable 15-7. Hardware States and Causes of GPU Hangs\n \t Hardware State\n \t Cause of GPU Hang\n \t 0x00011303\n0x00011F03\n0x00012F03\n \t CPU destroyed content of vertex buffer while GPU was operating.\n \t 0x00010103\n0x0001011B\n0x00014303\n \t GPU bug caused content of vertex buffer in VRAM to be discarded in the middle of rendering.\n For information, see 15.9. Lines Are Rendered in Error and the Region Following the Framebuffer Is Corrupted.\n \t 0x00010107\n0x00011307\n0x00012307\n \t Hang due to multitexture hardware bug.\n Conflict with texture addressing of 128-byte alignment restrictions.\n For information, see 7.3.1. Formats With 4-Bit Components.\n \t 0x00000100\n \t Bit 8 in both PICA registers 0x0229 and 0x0253 was not set correctly.\n These registers must be set again when rendering using a GL function after rendering using a non-GL function.\n Unused elements of the load array (bits 31 to 28 of register 0x0205 + n * 3) are not set to 0.\n \t 0x00007300\n \t PICA register 0x0289 was set to use the geometry shader, but the system instead executed a vertex shader that does not use the geometry shader.\n \t 0x00000000\n \t Related register for the command buffer address jump function was not set properly.\n While in standby for command request execution, nngxFlush3DCommandPartially was called prior to completing configuration of the related register of the address jump function.\n \t 0x00000001\n0x00000002\n0x00000003\n \t NaN was output from the vertex shader.\n Note:\n For more information about how to set a PICA register, see 3DS Programming Manual: Advanced Graphics.\n",0]
,["Pages/Page_55588934.html#PageId_55588968","15.13. Effect of Vertex Attribute Combinations on Vertex Data Transfer Speed ",
"\n15.13. Effect of Vertex Attribute Combinations on Vertex Data Transfer Speed\nWhen a vertex buffer is used, the vertex attribute data type and data size combination (the type and size parameters in the glVertexAttribPointer function) affects the speed of vertex data transfer.\nVertex attribute data stored in the vertex buffer is grouped together as one or multiple vertex attributes and loaded to the GPU. The load array is the unit used in loading the vertex data.\n Note:\n For more information about load arrays, see 3DS Programming Manual: Advanced Graphics.\nWhen the GPU transfers load arrays, it determines whether to perform a read-ahead transfer based on the combination of vertex attribute data types and data sizes comprising the load array. If a read-ahead transfer can be performed, the vertex data transfer speed is faster.\nRead-ahead transfer is performed when data meets the requirements of the conditional equation shown below.\n(Attribute Number Types Other Than GL_FLOAT + Attribute Number Whose Data Size Is 1) <= (GL_FLOAT Type Attribute Number Whose Data Size Is 4 + GL_FLOAT Type Attribute Number Whose Data Size Is 3 / 2)\nThe data size of Attribute Number Types Other Than GL_FLOAT and the data type of Attribute Number Whose Data Size Is 1 are arbitrary. Vertex attributes applying multiple conditions are counted according to each of those conditions. For instance, for GL_BYTE type vertex attributes whose data size is 1, both the attribute type other than GL_FLOAT and the attribute number whose data size is 1 are counted.\nIf the conditions for read-ahead transfer are matched, transfer speed depends on the data volume of load arrays. The smaller the data volume, the faster the transfer speed. If the volume of vertex data is the same, transfer speed depends on the number of attributes included in the load array. The fewer the load array attributes, the faster the transfer speed.\n",0]
,["Pages/Page_55588934.html#PageId_55588969","15.14. Vertex Array Address Alignment ",
"\n15.14. Vertex Array Address Alignment\nEfficiency of vertex array transfer processing can be improved by keeping vertex array address alignment to 32 bytes when rendering with the use of a vertex buffer. The vertex array address is a value comprised of the vertex buffer address and an offset specified by the glVertexAttribPointer function (a value specified in ptr).\nThe extent to which speed is improved in comparison to a vertex array address whose alignment has not been kept within 32 bytes depends on the vertex attribute type, size, location of vertex array storage, and the content of the vertex index. There is no guarantee that this method will be effective. In addition, even if transfer processing performance improves, it will not necessarily improve performance of the overall system unless vertex array transfer processing is causing a noticeable bottleneck.\n",0]
,["Pages/Page_55588934.html#PageId_55588970","15.15. GPU Hangs When Multitexture Is Used ",
"\n15.15. GPU Hangs When Multitexture Is Used\n Note:\n On SNAKE hardware, the GPU does not hang when using multitextures. However, note that hangs can still occur when the application is running on CTR.\nThe GPU may experience a hang when both of the following conditions are met.\n\tMultiple textures are used. \tThere is a considerable difference in performance between texture units.\nProcedural texture is not included in these conditions. This phenomenon does not occur when one normal texture is used simultaneously with a procedural texture.\nThis phenomenon can be avoided by taking the following steps.\n\tUse only one texture. \tFor all textures used, apply the GL_XXX_MIPMAP_LINEAR setting (the setting for trilinear filter use) to the GL_TEXTURE_MIN_FILTER texture parameter.\nIn addition, the following methods are recommended as a means of mitigating this phenomenon.\n\tSet the GL_TEXTURE_MIN_FILTER texture parameter to the GL_XXX_MIPMAP_LINEAR setting (the setting for trilinear filter use) for a portion of the textures to be used.\nThe problem can be avoided completely by setting all textures to the trilinear filter setting, but the phenomenon’s occurrence can be mitigated by using the setting on just a portion of the textures. \tPlace all textures to be used at the same time in the same VRAM. \tReduce the number of textures used.\nBecause the occurrence of this phenomenon is dependent on timing, changing texture settings as listed below can also avoid the problem much of the time. However, in some cases, the frequency of occurrence could actually worsen.\n\tChange the size of textures. \tChange texture formats. \tChange the filter mode of the textures. \tChange the storage locations of textures, switching between VRAM-A, VRAM-B, and FCRAM (device memory).\nFor a description of how to check whether this phenomenon is the cause of a hang, see 15.12. Analyzing Causes of GPU Hangs.\nHowever, note that the same type of hangs can also be caused by being in conflict with restrictions governing the storage location of the 4-bit texture format, making it difficult to determine the cause for certain.\n",0]
,["Pages/Page_55588934.html#PageId_55588971","15.16. GL_INTERPOLATE Calculations of the Texture Combiner ",
"\n15.16. GL_INTERPOLATE Calculations of the Texture Combiner\nThe texture combiner’s GL_INTERPOLATE expression is src0 * src2 + src1 * (1 – src2). If src2 is 1 or 0, you would expect the result of the calculation to be the same value as either src0 or src1. However, because of how the specifications have been implemented, if src2 is 0 and src0 < src1, the output result is not the value of src1 as you would expect, but rather something that is one unit less bright than src1.\nTo avoid this problem, you must combine GL_MODULATE and GL_MULT_ADD_DMP and perform the calculation in a two-stage combiner. Alternatively, if you change the src2 operand to GL_ONE_MINUS_* and switch src0 and src1, you may be able to reduce the fragments for which this problem arises.\nCONFIDENTIAL\n",0]
];
for (var x = 0; x < Search.additionalData.length; x++) Search.indexData.push(Search.additionalData[x]);
