Search.additionalData = [
["history.html","Revision History ",
"Revision History\n",0]
,["history.html","Version 1.0 (2015-01-15)",
"Version 1.0 (2015-01-15)\n",0]
,["history.html","Additions and Changes ",
"Additions and Changes\n\tInitial version.\nCONFIDENTIAL\n",0]
,["title.html","3DS Programming Manual",
"3DS Programming Manual: Dynamic Stereoscopy\nVersion 1.0\n\tNintendo Confidential\n\tThe content of this document is highly confidential and should be handled accordingly.\nThe content of this document cannot be duplicated, copied, reprinted, transferred, distributed, or loaned in whole or in part without the prior approval of Nintendo.\nThis document contains confidential and proprietary information of Nintendo and is also protected under the copyright laws of the United States and foreign countries.No part of this document may be released, distributed, transmitted or reproduced in any form or by any electronic or mechanical means, including information storage and retrieval systems, without permission in writing from Nintendo.\n© 2014–2015 Nintendo. All rights reserved.\nCTR-06-0352-002-A\nAll company and product names in this document are the trademarks or registered trademarks of their respective companies.\n",0]
,["Pages/Page_71309519.html","1. Introduction ",
"1. Introduction\n This document explains what dynamic stereoscopy is and how to implement it. It starts by describing the theoretical framework that dynamic stereoscopy is built on.\n It then details how to relax the theoretical constraints to ensure comfortable viewing and allow artistic control over rendering.\n Finally, it describes the provided API.\n",0]
,["Pages/Page_71309519.html#PageId_71309535","1.1. Scope of This Document ",
"1.1. Scope of This Document\nThis document deals only with perspective projections to generate images from 3D scenes. These perspective projections are well known to graphics developers. This document assumes that readers are already familiar with the perspective projection matrix.\n",0]
,["Pages/Page_71309519.html#PageId_71309736","1.2. Terminology Used in This Document ",
"1.2. Terminology Used in This Document\nThis document uses the following terminology.\n \tReal space\nThe distance from the player to the upper LCD and other factors relevant to the stereoscopic display on CTR/SNAKE. To differentiate between these distances and the space in the application, this document uses the term real space to indicate the space that contains the player and the CTR/SNAKE system. \tVirtual space\nThe space created in the application, as opposed to real space. \tViewport\nThe area in real space where the rendered images are displayed. This area lies in the plane of the screen. Most often, a game uses the entire upper screen as the viewport. \tWindow\nThe representation of the viewport in virtual space. It is a cross-section of the camera viewing volume that lies on the base plane/window plane. It is the window through which the user sees the virtual world when looking at the viewport. Objects lying on the window plane are seen by the user as being exactly at screen distance. Objects farther from the camera are seen as being behind the screen, and nearer objects are seen as being in front of the screen. The virtual screen is also defined as the representation of the screen in virtual space. \tBase camera\nThe monoscopic camera that the application sets and creates according to the scene. This information is required in order to calculate cameras for the left eye and the right eye of the user. \tEye camera\nA camera used to compute the image for the left or right eye.\nFigure 1-1. Virtual Space\nFigure 1-2. Real Space\nCONFIDENTIAL\n",0]
,["Pages/Page_71309783.html","2. Ideal Image Capture and Display ",
"2. Ideal Image Capture and Display\n",0]
,["Pages/Page_71309783.html#PageId_71309793","2.1. Ideal Monoscopic Effect ",
"2.1. Ideal Monoscopic Effect\nWhen computing an image from a 3D scene using a perspective matrix, a central projection is used to transform 3D points into 2D points. (See Figure 2-1.)\nWith a monoscopic effect, the projection plane can be placed at any distance from the center of the camera. The window content is rescaled to match the viewport size.\nWhen a picture is displayed in a viewport, the ideal viewing position is at the point where the viewport has the same angular appearance as the window from the center of projection. The only degree of freedom for ideal display configuration is scale. Increasing the display size increases the ideal viewing position distance to the screen proportionally.\nFigure 2-1. Top: Capture Configuration; Bottom: Ideal Display Configuration\nThe ideal viewing position is usually when the user is in front of the center of the screen, at a particular distance. This leads to the use of cameras that have a symmetric frustum, and where only field of view (FOV) is taken into account. However, it might be necessary to use an asymmetric frustum when the viewport does not cover the entire screen.\nFor the sake of simplicity, the following sections assume that the viewport covers the entire screen.\nHowever, viewing a picture from a position that does not match the ideal viewing position does not break the comprehension of the picture. In fact, this effect can be used artistically. For instance, cinema frequently uses different fields of view to convey different impressions to the audience. Some special effects can be performed by changing the field of view dynamically. (A common example is the dolly zoom.)\n",0]
,["Pages/Page_71309783.html#PageId_71309869","2.2. Ideal Stereoscopic Effect ",
"\n2.2. Ideal Stereoscopic Effect\n3D viewing is performed by displaying two images on the same screen. One image is seen by the left eye, and the other by the right eye.\nYou can achieve an ideal stereoscopic effect by displaying the images such that the ideal viewing positions in virtual space match the positions of the eyes of the user in the standard viewing position.\nThe standard viewing position of the user is in front of the center of the screen. In this configuration there is horizontal symmetry. The standard eye positions are then positioned on each side of the center line. (See Figure 2-2.)\nFigure 2-2. Standard Viewing Configuration\nA stereoscopic camera comprises two base cameras that match the standard eye positions in virtual space.\nAn ideal stereoscopic camera is created by horizontally displacing two base cameras so that they match the eye positions and FOV of the user in the standard viewing position.\nHowever, one degree of freedom remains. The virtual window can be placed at any distance from the center of the camera (see Figure 2-3). As the developer, you must determine the position of this window for stereoscopy. You must decide which plane in virtual space will be the plane of the screen, and which objects will appear on, behind, and in front of the screen. Changing the window depth (that is, its position) changes the apparent size of displayed objects.\nFigure 2-3. Left: Base Camera Configuration; Center: Stereoscopic Camera Configuration Using Window 1; Right: Stereoscopic Camera Configuration Using Window 2\nThe window can be set at any distance from the base camera. Consider window 1 and window 2 in Figure 2-3. In the stereoscopic camera configuration using window 1 (center), objects appear behind the screen. In the configuration using window 2 (right), objects appear in front of the screen.\nAs shown in Figure 2-3, the windows for both cameras must coincide. As a result, the camera viewing directions (the window plane normals) must be the same, and their frustums must be made asymmetrical to encompass the window.\nThe FOV on the screen is determined by the standard viewing distance, based on the size of the screen. For ideal stereoscopy, have the camera use this FOV.\n",0]
,["Pages/Page_71309783.html#PageId_71309926","2.3. Ideal Dynamic Base Camera ",
"\n2.3. Ideal Dynamic Base Camera\nIf you can detect the user position relative to the screen, you can compute an image whose ideal position matches the current viewing position. This method is called a dynamic perspective.\nFigure 2-4. Dynamic Perspective\nAssume that you know the exact position of the user in the viewport frame of reference. To position the camera, you must define the window frame of reference in the virtual world. You can derive the window from the base camera that corresponds to the standard viewing position in real space.\nYou can position the window on any plane orthogonal to the viewing direction, at any positive distance. The window is the positional center of rotation, so the appearance of rendered objects changes depending on the distance from the camera to the window. To achieve an ideal dynamic camera, you must rotate the base camera, such as to the position of the dynamic camera in Figure 2-4. The same is true for a stereoscopic camera.\n",0]
,["Pages/Page_71309783.html#PageId_71309944","2.4. Ideal Dynamic Stereoscopic Camera ",
"\n2.4. Ideal Dynamic Stereoscopic Camera\nTo develop a dynamic stereoscopic camera, define the position of the window in the virtual world, and use the positions of both user eyes to position cameras, as was the case for dynamic base cameras.\n",0]
,["Pages/Page_71309783.html#PageId_71309965","2.5. Using a Viewport Not Covering the Entire Screen ",
"\n2.5. Using a Viewport Not Covering the Entire Screen\nThe operations described to this point assume that a picture is displayed on the entire screen. However, displaying a 3D picture in a sub-area needs special care.\nThe system can only know the relative position of the user to the screen. If the developer plans to display a 3D picture in a sub-area of the screen, the viewport and the position of the user relative to the viewport must be expressed to compute the correct camera position. To do so, the developer must specify the exact position of the viewport in the screen to the ideal dynamic stereo camera.\nFigure 2-5. Dynamic Camera and Viewport\nCONFIDENTIAL\n",0]
,["Pages/Page_71309974.html","3. Integration in SNAKE ",
"3. Integration in SNAKE\n",0]
,["Pages/Page_71309974.html#PageId_71309979","3.1. Face Tracking System Limitations ",
"3.1. Face Tracking System Limitations\n",0]
,["Pages/Page_71309974.html#PageId_71309982","3.1.1. Estimating Depth ",
"3.1.1. Estimating Depth\nSo far the position of the user relative to the screen is assumed to be known. However, that is not the case with SNAKE. The system’s face tracking feature can determine the user’s angle to the screen with good precision, but it cannot compute the user’s distance from the screen.\nInstead, the system assumes that the user is located at the optimal distance, which is the distance at which the 3D effect is best perceived.\n",0]
,["Pages/Page_71309974.html#PageId_71309986","3.1.2. Using an Average For Distance Between Eyes ",
"\n3.1.2. Using an Average For Distance Between Eyes\nEven if the position of both eyes can be tracked by the system, the real distance between the eyes for a particular user is not known. The apparent distance between the eyes in the image retrieved from the camera varies depending on the user's distance from the screen, so it cannot be used for the actual distance between the eyes. A virtual user with an average distance between eyes is assumed. An average value is assumed for the distance between eyes for a user positioned at the optimal distance from the screen. The same assumption is used for pictures of the user retrieved from the camera. The eye positions of this virtual user are used for all computations instead of the raw ones.\n",0]
,["Pages/Page_71309974.html#PageId_71309996","3.1.3. Handling User Detection Failure ",
"\n3.1.3. Handling User Detection Failure\nSometimes, the user might move out of the field of view of the tracking system. Other times, poor conditions might make recognition impossible. In these situations, the system calculates an estimate for the user’s position, based on prior position information and input from the gyro sensors. The estimate begins with the last-detected position of the user. If there is no gyro sensor input for some time—implying that the SNAKE system is not moving—the estimated position is shifted over time toward the center of the screen. If the tracking system remains unable to detect the user, it assumes that the user has returned to a position facing the center of the screen. If the SNAKE system is moving, the calculations assume that the user is static, and estimate the user's position relative to the moving console.\nThis behavior has been designed to provide a user experience at least as good as with 3D screens that do not support Super-Stable 3D. In addition, it provides a good user experience when the user moves the device.\n",0]
,["Pages/Page_71309974.html#PageId_71310007","3.2. Comfort and Maximum Parallax ",
"\n3.2. Comfort and Maximum Parallax\nExperiments have shown the importance of maintaining parallax within limits. There is a limit distance for parallax, and this value must be taken into consideration when setting the distance for an object in the images presented to the left and right eyes. If this limit is exceeded, most users will feel uncomfortable.\n",0]
,["Pages/Page_71309974.html#PageId_71310013","3.2.1. Near and Far Planes ",
"Using the ideal stereoscopic camera described in the previous section, this can happen when the object is situated very far from the window plane. If an object located infinitely far from the user is rendered at a distance equal to the distance between eyes, the parallax is very large. The following methods can be used to ensure that this constraint is enforced.\n3.2.1. Near and Far Planes\nSetting near planes and far planes nearer to the window plane reduces the maximum parallax that a visible object can have.\n",0]
,["Pages/Page_71309974.html#PageId_71310016","3.2.2. Reducing Distance Between Eyes ",
"\n3.2.2. Reducing Distance Between Eyes\nAnother way to reduce maximum parallax is to render the scene for a reduced distance between eyes. This reduces the parallax on the farthest objects. However, it also reduces the 3D effect on the nearest objects.\n",0]
,["Pages/Page_71309974.html#PageId_71310023","3.2.3. Increasing FOV ",
"\n3.2.3. Increasing FOV\nAs shown in the following section, parallax in the far plane can also be reduced by increasing the FOV.\nCONFIDENTIAL\n",0]
,["Pages/Page_71310103.html","4. Dynamic Stereoscopic Camera Settings ",
"4. Dynamic Stereoscopic Camera Settings\nWith the previously defined theoretical model in mind, you can relax some constraints to enable some control over the camera for artistic, comfort, or technical reasons.\nTo implement the following features, you must consider the position of the user’s eyes in two different ways: in terms of the absolute position of the user, and in terms of the relative positions of the eyes. This enables you to understand stereoscopic effects and dynamic perspective effects independently.\nFor dynamic perspective effects, the center point between the eyes is used as the user’s position. For stereoscopic effects, the positions of the eyes relative to this center point are used.\nFigure 4-1. Decoupling Stereoscopic Effects From Dynamic Perspective Effects\n",0]
,["Pages/Page_71310103.html#PageId_71310247","4.1. Wide FOV ",
"\n4.1. Wide FOV\nWhen dealing with a base camera, using a specific FOV is natural for game developers. However, ideal stereoscopy needs a fixed FOV—a standard FOV—that matches the angular appearance of the screen as seen by the user.\nMost of the time, these specific FOVs are larger than the standard FOV. These larger FOVs are called wide FOVs.\nFigure 4-2. Wide-FOV Camera\nIt is possible to compute a realistic camera that matches the standard FOV and whose window matches the window of the base camera. However, using another FOV may be needed for artistic reasons.\nIf you want to use a wide FOV, you can reduce the deformation of objects displayed in the center of the screen by adeptly setting the distance between the left and right cameras. To minimize these deformations, match the direction at which both cameras are pointing toward the center of the window to the direction at which the user is looking toward the center of the screen.\nAs shown in Figure 4-3, deformation is minimized by setting the inter-camera distance for the ideal stereo cameras and the distance between the left and right wide FOV cameras proportional to their distances from the window.\nFigure 4-3. Ideal Inter-Camera Distance for Wide-FOV Cameras\nWhen using dynamic perspective, you need to consider methods for computing the camera movement from the user’s position. Consider the following methods.\nThe first method scales the movement as the inter-eye distance was scaled. The effect of this method is that an object at the center of the screen is viewed from the appropriate angle. However, far objects seem to move less than is normal.\nThe second method reproduces the movement of the user on the cameras without scaling. This method gives the impression that far objects move realistically, while near objects seem to rotate or deform.\nBoth methods can be justified, and intermediate behaviors can be useful depending on the scene. Any intermediate behavior can be described by a scalar factor between 0 and 1, the wide FOV motion factor.\nFigure 4-4. FOV Motion Factor Set to 0\nFigure 4-5. FOV Motion Factor Set to 1\n",0]
,["Pages/Page_71310103.html#PageId_71310366","4.2. Realistic FOV Factor ",
"\n4.2. Realistic FOV Factor\nIf you want to smoothly transform a wide FOV stereoscopic camera to an ideal stereoscopic camera, try interpolating between them for an interesting result. Use linear interpolation to move the virtual camera from one position to another. This linear interpolation factor is the realistic FOV factor.\nFigure 4-6. Realistic FOV Factor\n",0]
,["Pages/Page_71310103.html#PageId_71310381","4.3. Dynamic Perspective Amplitude ",
"\n4.3. Dynamic Perspective Amplitude\nYou can amplify or reduce a dynamic perspective effect by applying a multiplicative factor. This dynamic perspective amplitude factor corresponds to the user’s displacement from the standard position.\nYou can modify this amplitude independently on each axis depending on the desired effect or specific constraints.\n",0]
,["Pages/Page_71310103.html#PageId_71310400","4.4. Dynamic Perspective Clamping ",
"\n4.4. Dynamic Perspective Clamping\nThe area where the user can be detected by the system is limited by the field of view of the camera. When the user leaves this area, the system loses the user's position. To provide visual feedback to the user before leaving this area, you can clamp the maximum viewing angle to a narrower one.\nThis clamping can also be used to control the volume of space that can be seen by the camera.\n",0]
,["Pages/Page_71310103.html#PageId_71310543","4.5. Tilt and Improved Static Stereoscopic Camera ",
"\n4.5. Tilt and Improved Static Stereoscopic Camera\nIf camera displacement is deactivated by setting either the clamping factor or the amplitude factor to 0, the camera works like a static stereoscopic camera.\nEven so, the angle between the user and the console can be determined from the detected position of the eyes, and this tilt information can be applied to the virtual cameras. This can improve stereoscopic quality when the user tilts the screen as if turning a steering wheel.\nFigure 4-7. Improved Static Stereoscopic Camera\nHowever, in rare instances using tilt this way can cause problems. Also, moving the virtual camera can generate artifacts. If there are problems, disable the tilt so that the camera remains horizontal.\n",0]
,["Pages/Page_71310103.html#PageId_71310547","4.6. Parallax Distance Limit ",
"\n4.6. Parallax Distance Limit\nTo guarantee the comfort of the user, the developer can specify a maximum parallax distance. A maximum inter-camera distance is derived from this constraint, such that an object on the far plane would have at most this parallax distance. However, this constraint is not enforced for objects on the near plane.\n",0]
,["Pages/Page_71310103.html#PageId_71310556","4.7. 3D Slider and Stereo Factor ",
"\n4.7. 3D Slider and Stereo Factor\nFor users to tune the 3D effect to their liking, the inter-camera distance must be adjusted using the 3D slider. The developer can provide a stereo factor that takes values in the range from 0 through 1. The inter-camera distance can be shortened by multiplying the 3D slider value by this factor. You can use this technique to reduce the 3D effect if necessary, or to perform smooth transitions between 2D and 3D images.\nCONFIDENTIAL\n",0]
,["Pages/Page_71310767.html","6. DynamicStereoCamera Class ",
"6. DynamicStereoCamera Class\n",0]
,["Pages/Page_71310767.html#PageId_71310770","6.1. Specifying the Window ",
"6.1. Specifying the Window\n",0]
,["Pages/Page_71310767.html#PageId_71310785","6.1.1. Setting the Window From Its Size in Virtual Space ",
"When you are using a static camera, you specify the frustum. But when you are using a dynamic camera, you need to specify the window. You must also specify the near and far planes.\n6.1.1. Setting the Window From Its Size in Virtual Space\nYou can specify the window width and the depth at which it is placed from the center of the camera. The resulting window is centered on the z-axis of the camera, and its height is set to match the aspect ratio of the device's upper screen.\nNote that the ratio of the window width and the window depth determines the FOV.\nThis method assumes that the resulting image will be displayed on the entire upper screen.\nCode 6-1. SetWindowFromSize Function Definition\n void SetWindowFromSize(const f32 windowWidth,\n const f32 nearDepth,\n const f32 windowDepth,\n const f32 farDepth);\n",0]
,["Pages/Page_71310767.html#PageId_71310800","6.1.2. Deriving the Window From the Base Camera Frustum ",
"\n6.1.2. Deriving the Window From the Base Camera Frustum\nYou can give left, right, bottom, and top tangent angles of the base camera frustum. The window is then defined as the intersection of this frustum and the plane at window depth. You can use this method to describe viewports that do not cover the entire screen. Compute the viewport border tangents from a standard viewing position in front of the center of the screen. It is assumed that the resulting image will be displayed on the largest area of the screen that matches the viewport description.\nCode 6-2. SetWindowFromTangent Function Definition\n void SetWindowFromTangent(const f32 tangentLeft,\n const f32 tangentRight,\n const f32 tangentBottom,\n const f32 tangentTop,\n const f32 nearDepth,\n const f32 windowDepth,\n const f32 farDepth);\n",0]
,["Pages/Page_71310767.html#PageId_71310804","6.1.3. Setting the Window From the Viewport ",
"\n6.1.3. Setting the Window From the Viewport\nTo deal with a viewport that only covers a portion of the screen, you need to know the location of the viewport within the screen. The following member function is useful when you are using this kind of viewport and a realistic FOV.\nCode 6-3. SetWindowFromViewport Function Definition\n void SetWindowFromViewport(const f32 screenHorizontalHalfFOVTangent,\n const f32 viewportLeftInUnitSceen,\n const f32 viewportRightInUnitSceen,\n const f32 viewportBottomInUnitSceen,\n const f32 viewportTopInUnitSceen,\n const f32 nearDepth,\n const f32 windowDepth,\n const f32 farDepth);\n",0]
,["Pages/Page_71310767.html#PageId_71310810","6.1.4. Setting the Window Based on an Existing Projection Matrix ",
"\n6.1.4. Setting the Window Based on an Existing Projection Matrix\nYou can also set the window by specifying the window depth and the projection matrix of the base camera used to calculate the frustum.\nHowever, this method can only be used for a projection matrix that targets the entire upper screen and assumes a symmetric frustum.\nCode 6-4. SetWindowFromProjectionMatrix Function Definition\n void SetWindowFromProjectionMatrix(const nn::math::Matrix44 *proj,\n const f32 windowDepth);\n",0]
,["Pages/Page_71310767.html#PageId_71310881","6.2. Specifying the Base Camera’s View ",
"\n6.2. Specifying the Base Camera’s View\nCode 6-5. SetView Function Definition\n void SetView(const nn::math::MTX34* viewMatrix);\nvoid SetView(const nn::math::VEC3& position,\n const nn::math::VEC3& upDirection,\n const nn::math::VEC3& forwardDirection);\nEither specify the view matrix created by a function such as nn::math::MTX34LookAt, or directly specify the parameters for the base camera’s position. Make sure that you use the right-handed coordinate system adopted by the CTR graphics system when specifying the view matrix or the nn::math::Vector3 arguments.\n",0]
,["Pages/Page_71310767.html#PageId_71310951","6.3. Computing Left and Right Cameras ",
"\n6.3. Computing Left and Right Cameras\nCode 6-6. ComputeViewsAndProjections Function Definition\n f32 ComputeViewsAndProjections(\n nn::math::Matrix44* projL, nn::math::Matrix34* viewL,\n nn::math::Matrix44* projR, nn::math::Matrix34* viewR,\n const nn::math::VEC2* leftEyeTangent,\n const nn::math::VEC2* rightEyeTangent,\n const Setting& setting,\n const nn::math::PivotDirection pivot = nn::math::PIVOT_UPSIDE_TO_TOP\n) const;\nThis method is used to compute projection and view matrices to render left and right images for the left and right eyes.\nThis method assumes that the user is placed at the optimal distance from the surface of the LCD screen. It can provide a realistic stereoscopic view and other non-realistic (or artistic) views based on the setting parameter, explained below. This method can take into account the user's head position, and adjust the dynamic perspective accordingly.\nThe leftEyeTangent and rightEyeTangent parameters describe the position of the left and right eyes of the user relative to the screen. These values can be retrieved by a call to the nn::qtm::CTR::GetTrackingData function.\nWhen either leftEyeTangent or rightEyeTangent is NULL, this method produces the same behavior as a static stereoscopic camera. Set these parameters to NULL when developing for the CTR platform.\nThe upward direction of the camera in the generated projection matrix is the upward direction as determined by the pivot parameter. If pivot is set to nn::math::PIVOT_NONE, no rotation is applied. If an invalid value is specified for pivot, the function halts on an assertion. The default value is nn::math::PIVOT_UPSIDE_TO_TOP.\nThe function returns the factor for the distance between cameras, ƒI. This factor is described in the appendix.\n",0]
,["Pages/Page_71310767.html#PageId_71310943","6.4. Setting Class ",
"\n6.4. Setting Class\nCode 6-7. Setting Class Definition\n struct Setting\n{\n f32 StereoFactor;\n f32 ParallaxDistanceLimit;\n f32 DynamicPerspectiveAmplitudeX;\n f32 DynamicPerspectiveAmplitudeY;\n f32 DynamicPerspectiveClampingLeft;\n f32 DynamicPerspectiveClampingRight;\n f32 DynamicPerspectiveClampingBottom;\n f32 DynamicPerspectiveClampingTop;\n f32 RealisticFovFactor;\n f32 WideFovMotionFactor;\n bool TiltEnabled;\n}\nYou can use the Setting class passed as an argument to the ComputeViewsAndProjections function to adjust the behavior of the stereoscopic camera.\n\tStereoFactor adjusts the stereoscopic effect, as described in 4.7 3D Slider and Stereo Factor. \tParallaxDistanceLimit specifies the parallax distance limit, in millimeters. This limit is the maximum distance between the object in the image for the left eye and the object in the image for the right eye. For more information, see 4.6 Parallax Distance Limit. \tDynamicPerspectiveAmplitudeX and DynamicPerspectiveAmplitudeY are multiplication factors for the user position. They amplify or reduce the dynamic perspective effect for a head movement. \tDynamicPerspectiveClampingLeft, DynamicPerspectiveClampingRight, DynamicPerspectiveClampingBottom, and DynamicPerspectiveClampingTop are used to clamp the user’s position horizontally and vertically. They take values in the range from 0 through 1. When set to 1, the position is clamped to the limit value obtained using GetTrackingData. If set to a value in the middle of the range, the position is clamped to the corresponding fraction of the limit value. If set to 0, the camera stops moving in that direction. Note that the position is clamped before the amplitude set by DynamicPerspectiveAmplitude is applied. \tRealisticFovFactor is used to interpolate the camera position between the realistic camera and the base camera, as described in 4.2 Realistic FOV Factor. \tWideFovMotionFactor adjusts how the camera moves in response to movements by the user when a wide FOV is being used, as described in 4.1 Wide FOV. ",0]
,["Pages/Page_71310767.html#PageId_71311039","6.5. Occupancy Volume ",
" \tTiltEnabled indicates whether tilt is enabled or disabled. For more information, see 4.5. Tilt and Improved Static Stereoscopic Camera.\n6.5. Occupancy Volume\nCode 6-8. OccupancyVolume Structure Definition\n struct OccupancyVolume\n(\n math::VEC3 CameraCenter;\n math::VEC3 ForwardDirection;\n math::VEC3 OrthoUpDirection;\n math::VEC3 RightDirection;\n f32 NearPlaneDepth;\n f32 NearLeft;\n f32 NearRight;\n f32 NearBottom;\n f32 NearTop;\n f32 CameraLeft;\n f32 CameraRight;\n f32 CameraBottom;\n f32 CameraTop;\n};\nUse the OccupancyVolume structure to express the occupancy volume of the camera. To guarantee that artifacts do not appear unnatural due to the camera crossing paths with objects, make sure that no objects are located inside this volume.\n\t CameraCenter is the camera position when dynamic perspective is disabled. If RealisticFovFactor is set to 0, this position is the same as that specified by the SetView member function. For any value greater than 0, the camera position is adjusted along the axis in the forward direction, approaching the realistic FOV. The plane on which the camera moves for dynamic perspective is called the camera plane.\n \tForwardDirection is the unit vector in the forward direction. The value is the same as that set by the SetView member function. \tOrthoUpDirection is the unit vector in the upward direction. It bisects at right angles with ForwardDirection. \t RightDirection is the unit vector in the rightward direction. Because it bisects at right angles with ForwardDirection and OrthoUpDirection, you can calculate the rightward vector from their cross product.\nRightDirection = ForwardDirection × OrthoUpDirection\n\tNearPlaneDepth is the distance from the camera position to the near plane. \tNearLeft and NearRight define the range of the occupancy volume in the horizontal direction (in the direction of the rightward vector) in the near plane. \tNearBottom and NearTop define the range of the occupancy volume in the vertical direction (in the direction of the upward vector) in the near plane. \tCameraLeft and CameraRight define the range of the occupancy volume in the horizontal direction in the camera plane. \tCameraBottom and CameraTop define the range of the occupancy volume in the vertical direction in the camera plane.\n Figure 6-1. Occupancy Volume Parameters\nCONFIDENTIAL\n",0]
,["Pages/Page_71311936.html","7. Appendix: Formulas ",
"7. Appendix: Formulas\n",0]
,["Pages/Page_71311936.html#PageId_71311944","7.1. Constants ",
"7.1. Constants\nThe current system uses the following values.\n The standard distance between user eyes: 62 mm.\n The standard distance from the surface of the upper screen to the user: 316.5 mm (the average of SNAKE (301 mm) and CLOSER (332 mm)).\n The screen width: 95.4 mm (the average of SNAKE (84.6 mm) and CLOSER (106.2 mm)).\n The distance from the center of the screen to the center of the camera: 37.62 mm (the average of SNAKE (34.38 mm) and CLOSER (40.86 mm)).\n",0]
,["Pages/Page_71311936.html#PageId_71312021","7.2. Converting Between Virtual and Real Spaces ",
"\n7.2. Converting Between Virtual and Real Spaces\nFigure 7-1. Left: Real Space Frame of Reference; Right: Virtual Space Frame of Reference\nThe frames of reference in real space and virtual space were defined as shown in Figure 7-1. In both frames of reference, the origin is the center point of the screen.\nAssume that the orthogonal projection of the camera position on the window is the center of the window.\nThe following scale is used for converting the distance from real coordinates to virtual coordinates.\nWwindow is the width of the window in virtual space,\nWviewport is the width of the corresponding viewport in real space.\n",0]
,["Pages/Page_71311936.html#PageId_71312034","7.3. User Position ",
"\n7.3. User Position\nXlefteye and Xrighteye are defined as the positions of the user's eyes in real space. These positions are only known by their angular tangents.\nAssuming that the user is placed at optimal distance, where the 3D effect is best perceived, the user’s eye positions can be extrapolated.\nThese positions can be broken down into a user position Xuser and an inter-eye vector ⊿eyes (the vector from the left eye to the right eye).\nThe unit vector in the direction of ⊿eyes is called δeyes.\nThe position of the user is also represented as a displacement X′user from the standard position.\n",0]
,["Pages/Page_71311936.html#PageId_71312174","7.4. Computing the Distance Between Cameras ",
"\n7.4. Computing the Distance Between Cameras\nI is the distance between the cameras for the left and right images.\nThis distance is equal to the realistic inter-camera distance Irealistic when configured for realism. It can be computed from the distance between eyes, Ieye..\n",0]
,["Pages/Page_71311936.html#PageId_71312189","7.4.1. Computing Wide FOV ",
"\n7.4.1. Computing Wide FOV\nIf the camera distance is not at the ideal distance (wide FOV configuration), the distance between cameras must be scaled down with the factor.\ndwindow is the distance between the window and the camera, computed as an interpolation between the distance specified by the developer, dwindow base, and the standard distance dideal in virtual space.\nƒ realistic is the realistic FOV factor.\n",0]
,["Pages/Page_71311936.html#PageId_71312204","7.4.2. Computing the Parallax Distance Limit ",
"\n7.4.2. Computing the Parallax Distance Limit\nTo guarantee the comfort of the user, the developer can specify a parallax distance limit Plimit and ensure that this limit is not exceeded. A maximum inter-camera distance Imax is derived from the following constraints.\ndfar is the depth of the far plane, derived from the value of dfar base specified by the developer.\n",0]
,["Pages/Page_71311936.html#PageId_71312210","7.4.3. Inter-Camera Distance Factor ",
"\n7.4.3. Inter-Camera Distance Factor\nThe final inter-camera distance I is used for rendering.\nThe inter-camera distance factor ƒI, which represents how much the realistic distance between cameras has been scaled down, can be derived from I.\n",0]
,["Pages/Page_71311936.html#PageId_71312294","7.5. Computing the Central Camera Position ",
"\n7.5. Computing the Central Camera Position\nLet Xleftcam and Xrightcam be the positions of the cameras for the left and right eyes in the virtual space frame of reference. The center point between the two cameras is defined as Xcam.\nThe reference camera position, Xcamref, lies on the z-axis, at the standard distance in virtual space.\nXcam is obtained by calculating the user's displacement to this reference camera position.\nƒ amplitude is the dynamic perspective amplitude factor.\nkfov motion is the amplitude factor computed from the wide FOV motion factor.\nclamp(X'user) is the user displacement, clamped to the maximum detectable user displacement.\n",0]
,["Pages/Page_71311936.html#PageId_71312298","7.6. Computing Final Eye Camera Positions ",
"\n7.6. Computing Final Eye Camera Positions\n",0]
,["Pages/Page_71311936.html#PageId_71312301","7.6.1. When Tilt Is Enabled ",
"The final positions of the eye cameras are computed by displacing both cameras from the central camera position. This computation can include or exclude the tilt parameter.\n7.6.1. When Tilt Is Enabled\n",0]
,["Pages/Page_71311936.html#PageId_71312307","7.6.2. When Tilt Is Disabled ",
"\n7.6.2. When Tilt Is Disabled\nCONFIDENTIAL\n",0]
,["Pages/Page_71314143.html","5. Notes for Integration ",
"5. Notes for Integration\n",0]
,["Pages/Page_71314143.html#PageId_71310675","5.1. Determining Camera Occupancy ",
"5.1. Determining Camera Occupancy\nCamera placement is constrained to a set of acceptable positions in the game. It is desirable to position the camera so that it is out of the way of any geometry objects. When using a dynamic stereoscopic camera, you want to accurately reproduce the movements of the user. For this reason, the camera cannot be placed in locations where it would be possible to locate a static base camera but not the final eye cameras. This placement would lead to results that do match the movement of the player.\nTake these constraints into consideration before deriving the two eye cameras from the base camera. To avoid collisions with geometry objects, you must secure regions for the camera that are out of the way.\nSome particular camera positions are acceptable. In these positions, there are no geometry objects in the camera FOV between the camera and the near plane. Otherwise, rendering produces artifacts.\nFigure 5-1. Camera Occupancy for a Static Base Camera\nFor a dynamic stereoscopic camera, you must confirm that a particular camera is in an acceptable position for all possible user positions. For this purpose, the camera occupancy volume can be defined as the frustum from the camera plane to the near plane. The sides of this volume are the planes that stretch from the extreme reachable positions of the camera to the edges of the window. See Figure 5-2.\nFigure 5-2. Camera Occupancy for a Dynamic Stereoscopic Camera\nCONFIDENTIAL\n",0]
];
for (var x = 0; x < Search.additionalData.length; x++) Search.indexData.push(Search.additionalData[x]);
