Search.additionalData = [
["history.html","Revision History ",
"Revision History\n",0]
,["history.html","Version 1.0 (2014-09-04) ",
"Version 1.0 (2014-09-04)\n",0]
,["history.html","Additions and Changes ",
"Additions and Changes\n\tInitial version.\nCONFIDENTIAL\n",0]
,["title.html","3DS Programming Manual",
"3DS Programming Manual: GD Library\nVersion 1.0\n\tNintendo Confidential\n\tThe content of this document is highly confidential and should be handled accordingly.\nThe content of this document cannot be duplicated, copied, reprinted, transferred, distributed, or loaned in whole or in part without the prior approval of Nintendo.\nThis document contains confidential and proprietary information of Nintendo and is also protected under the copyright laws of the United States and foreign countries.No part of this document may be released, distributed, transmitted or reproduced in any form or by any electronic or mechanical means, including information storage and retrieval systems, without permission in writing from Nintendo.\n© 2014 Nintendo. All rights reserved.\nCTR-06-0174-002-A\nAll company and product names in this document are the trademarks or registered trademarks of their respective companies.\n",0]
,["Pages/Page_54693503.html","1. Introduction ",
"1. Introduction\nThis document describes programming using the CTR-SDK's GD library. It assumes that the reader is already familiar with basic graphics concepts and how to use some of the libraries (such as the GX library), and focuses specifically on the GD library.\n",0]
,["Pages/Page_54693503.html#PageId_54693504","1.1. The GD Library ",
"This document also assumes that the reader has an understanding of the content in the 3DS Programming Manual: Basic Graphics and the CTR Programming Manual: Advanced Graphics. Please read those documents first.\n1.1. The GD Library\nThe GD library is one of the CTR-SDK graphics libraries. It is mainly used to generate 3D commands.\nThe library generates only the 3D commands necessary to synchronize its internal state with the GPU state, allowing you to implement graphics processing without keeping track of the GPU state. The API is structured in a way that is tightly linked with the 3DS graphics hardware. It has been designed to achieve high performance while at the same time making it easy for applications to use.\n",0]
,["Pages/Page_54693503.html#PageId_54693505","1.1.1. Command Lists ",
"The GD library is incompatible with DMPGL (gl functions). You must use the GX library (nngx functions) for command list management and some other system-related features. In addition, only a C++ interface is provided.\n1.1.1. Command Lists\nAs already mentioned, the GD library does not manage command list operations; the features provided by the GX library are used for that. In other words, applications are solely responsible for implementing ways of preparing, using, and synchronizing command lists.\n",0]
,["Pages/Page_54693503.html#PageId_54693506","1.1.2. Immediate and Non-Immediate Functions ",
"\n1.1.2. Immediate and Non-Immediate Functions\nGD library functions are classified as either immediate or non-immediate functions, depending on when they write 3D commands to the 3D command buffer specified by the current command list object.\nImmediate functions write data directly to the 3D command buffer when they are called. This has the advantage of allowing you to directly edit recorded packets when you use the packet recording feature (see 6. Packet Recording).\nNon-immediate functions update the internal state of the GD library without writing to the 3D command buffer. These internal states are loaded when rendering functions (nn::gd::System::Draw or nn::gd::System::DrawIndexed) are called and data is written to the 3D command buffer as necessary.\nSome functions have the characteristics of both immediate and non-immediate functions and update the GD library's internal state while continuing to write to the 3D command buffer.\n",0]
,["Pages/Page_54693503.html#PageId_54693507","1.1.3. Error Handling ",
"\n1.1.3. Error Handling\nFunctions that may fail internally are implemented to return an nnResult object. In other words, when an error occurs a corresponding error value is returned.\nThere are two ways to handle errors with the GD library. The first is to always check the return values from GD functions. The second is to call the nn::gd::System::SetCallbackFunctionError function to register a callback function for handling errors. You can use both of these methods at the same time, getting errors from the return values of the GD functions after the callback is invoked.\nCode 1-1. Setting Callback Functions for Error Handling\n typedef void (*nn::gd::System::gdCallbackfunctionErrorPtr)(\n nnResult result, const char* functionName);\nstatic void nn::gd::System::SetCallbackFunctionError(\n nn::gd::System::gdCallbackfunctionErrorPtr callbackFunctionError);\n Warning:\n The build type (Debug/Development/Release) determines the granularity of error checking within GD functions.\nYou can also use the nn::gd::System::GetErrorStringFromResult function for debugging purposes to get a message that corresponds to the error for a given nnResult object.\nCode 1-2. Getting an Error Message (Only for Debugging)\n static char* nn::gd::System::GetErrorStringFromResult(nnResult result);\n",0]
,["Pages/Page_54693503.html#PageId_54693508","1.2. Module ",
"\n1.2. Module\nModules bundle related processing within the 3DS rendering pipeline.\nThe following modules are defined by the GD library.\nTable 1-1. List of Modules\n \tModule \tDefinition Name\n \tVertex Input Stage \tnn::gd::System::MODULE_VERTEX_INPUT\n \tShader Stage \tnn::gd::System::MODULE_SHADER\n \tRasterizer Stage \tnn::gd::System::MODULE_RASTERIZER\n \tTexture Stage \tnn::gd::System::MODULE_TEXTURE\n \tProcedural Texture Stage \tnn::gd::System::MODULE_TEXTURE_PROCEDURAL\n \tLighting Stage \tnn::gd::System::MODULE_LIGHTING\n \tCombiner Stage \tnn::gd::System::MODULE_TEXTURE_COMBINER\n \tFog Stage \tnn::gd::System::MODULE_GAS_FOG\n \tOutput Stage \tnn::gd::System::MODULE_OUTPUT\n Note:\n Pipeline processing is sometimes split into stages. Modules are also sometimes called stages.\n",0]
,["Pages/Page_54693503.html#PageId_54693509","1.2.1. Static Class Definitions ",
"\n1.2.1. Static Class Definitions\nMost GD library functions are accessed through static classes defined in each module. These stateless classes only have static functions and are never instantiated. In other words, these classes are simply used to group function definitions for each feature.\nEach module has one or more classes, as shown in the following table.\nTable 1-2. Modules and Their Classes\n \tModule \tClasses \tProcessing\n \tVertex Input Stage \tVertexInputStage \tVertex buffer input.\n \tShader Stage \tShaderStage \tAttaching shader programs.\n \tRasterizer Stage \tRasterizerStage \tCulling, clipping, viewport settings, scissoring, and early depth tests.\n \tTexture Stage \tTextureStage \tTexture unit settings.\n \tProcedural Texture Stage \tProceduralTextureStage \tProcedural texture settings.\n \tLighting Stage \tLightingStage \tFragment lighting settings.\n \tCombiner Stage \tCombinerStage \tTexture combiner settings.\n \tFog Stage \tFogStage \tFog and gas settings.\n \tOutput Stage \tOutputStage \tDepth tests, stencil tests, alpha tests, logical operations, blending, write masks, render target settings, and fragment operation settings.\n ",0]
,["Pages/Page_54693503.html#PageId_54693510","1.2.2. Managing Module States ",
"\n1.2.2. Managing Module States\nThe library manages its internal state in several modules. If the internal state does not change for two rendering passes, commands related to those modules are not resent. This prevents unnecessary command packets from being generated and can improve performance. If, however, some GD function call has modified the internal state, the corresponding module is considered to be \"dirty\" (in other words, the module contains changes). Commands that reference the internal state of any module considered to be dirty are sent the next time rendering occurs.\nThe library tracks whether modules are dirty and automatically processes them. Accordingly, applications that use the GD library normally do not need to track which modules are dirty. However, sometimes it is advantageous to ensure that all the commands related to a module are re-sent. To that end, the GD library provides the following function to explicitly mark modules as dirty.\nCode 1-3. Function for Explicitly Marking Modules as Dirty\n void nn::gd::System::ForceDirty(u32 moduleFlag);\nSpecify the modules to mark as dirty to moduleFlag using a bitwise OR of the definitions in Table 1-1.\nThis function is useful when the hardware state has been updated (after prepared command packets are executed, for example) or when another graphics library (such as NintendoWare, DMPGL, and GR) is used, and you want to ensure that the entire GD library state is applied.\n",0]
,["Pages/Page_54693503.html#PageId_54693511","1.3. State Objects and Resource Objects ",
"\n1.3. State Objects and Resource Objects\nState objects are immutable objects that are never modified after they are created. They are typically created during initialization and then used to set a state when necessary at run time. Although the act of creating an object involves memory allocation and data processing time, after a state is created it can be simply output to the 3D command buffer, to minimize later processing. There is no way to directly access the content of these state objects.\nWhen the GD library receives a request to create a state object, it first determines whether it has already created an object that has the exact same settings. If it finds an object with the same settings, it simply returns a pointer to that object rather than creating a new one.\nThe following are defined as state objects.\nTable 1-3. State Objects\n \tState Object \tStage Used \tBundled Settings\n \tCombinerState \tCombiner Stage \tCombiner and combiner buffer.\n \tBlendState \tOutput Stage \tLogical operations and blending.\n \tDepthStencilState \tOutput Stage \tDepth and stencil tests.\n \tInputLayout \tVertex Input Stage \tVertex buffers.\n \tSamplerState \tTexture Stage \tTexture parameters.\n Apart from state objects, there are also the following resource objects. These are used in the same way in the sense that the necessary processing is almost entirely finished when the objects are created.\nTable 1-4. Resource Objects\n \tResource Objects \tDescription\n \tShaderBinary \tSaves and loads shader binaries.\n \tShader \tLoads floating-point constants into the GPU and sets input and output registers.\n \tShaderPipeline \tAttaches shader programs to the shader pipeline.\n \tVertexBufferResource \tMaintains vertex buffer data.\n \tTexture2DResource \tMaintains resources that have a width and height, such as textures and color buffers.\n \tTexture2D \tHandles a Texture2DResource object as a 2D texture.\n \tTextureCube \tHandles six Texture2DResource objects as a cube-map texture.\n \tRenderTarget \tHandles a Texture2DResource object as a color buffer.\n \tDepthStencilTarget \tHandles a Texture2DResource object as a depth (stencil) buffer.\n Note:\n The memory required to create state and resource objects is allocated as a system buffer (NN_GX_MEM_SYSTEM) from device memory (NN_GX_MEM_FCRAM) through the allocator specified when the GX library was initialized. We recommend that you consider how much memory is used by this library and leave extra space in device memory.\n For more information about how much memory is allocated, see 9.2. Size of Memory Allocated Within Functions.\n",0]
,["Pages/Page_54693503.html#PageId_54693512","1.3.1. Descriptor Classes ",
"\n1.3.1. Descriptor Classes\nPass a descriptor class that defines the necessary settings to functions that create objects, such as state and resource objects, which are used to configure the pipeline.\nAll of a descriptor class's member variables are defined to be public. Descriptor classes also sometimes define member functions for easily setting multiple member variables.\n",0]
,["Pages/Page_54693503.html#PageId_54693513","1.4. Data Flow ",
"\n1.4. Data Flow\nThe following figure describes each stage of the GD library pipeline from a data perspective and shows how functions are split into classes.\nFigure 1-1. Data Flow in the GD Library\nCONFIDENTIAL\n",0]
,["Pages/Page_54693514.html","2. Basic Flow of Operations ",
"2. Basic Flow of Operations\nThe following list shows the minimum number and (with slight variations) the order of operations necessary to display graphics using the GD library.\n\tInitializing the GD Library \tAllocating the Framebuffer \tLoading a Shader Program \tPreparing Vertex Data \tViewport Settings \tRendering \tDisplaying the Rendered Results \tFinalization\nFor all other operations, see other chapters, such as 4. Module Descriptions.\n Note:\n This chapter does not provide detailed information on each function. For more information, see later chapters in this document or the CTR-SDK API Reference.\n",0]
,["Pages/Page_54693514.html#PageId_54693515","2.1. Initializing the GD Library ",
"\n2.1. Initializing the GD Library\nThe nn::gd::System::Initialize function initializes the GD library. The GD library must be initialized before any other GD functions are called.\nThe nn::gd::System::Initialize function uses the allocator passed to the nngxInitialize function to allocate internal working memory. You must initialize the GX library using nngxInitialize and allocate a command list for the 3D commands executed within functions before you initialize the GD library.\nThe following example shows the steps needed to initialize the GD library.\nCode 2-1. Initialization Sample\n nngxInitialize(...);\nnngxGenCmdlists(1, &cmdListId);\nnngxBindCmdlist(cmdListId);\nnngxCmdlistStorage(0x80000, 128);\nnnResult result = nn::gd::System::Initialize();\nThe GD library can be used in parallel with other frameworks (such as NintendoWare) even when they use the same command lists. However, if command lists are already allocated outside the GD library, do not allocate other command lists until you have initialized the GD library.\nThe GD library does not have any functions for allocating display buffers. Use nngx functions to allocate display buffers.\n",0]
,["Pages/Page_54693514.html#PageId_54693516","2.2. Allocating the Framebuffer ",
"\n2.2. Allocating the Framebuffer\nAllocate a color buffer and depth (stencil) buffer with the GD library.\nAlthough they have different descriptor class settings, both buffers require the creation of a Texture2DResource object. To make it more efficient to clear and render the buffers, we recommend that you allocate the color and depth (stencil) buffers separately in VRAM-A and VRAM-B, as shown in the following sample code.\nCode 2-2. Creating the Texture2DResource Objects Used by the Framebuffer\n static nn::gd::Texture2DResource* s_texture2DResource_ColorBuffer = 0;\nstatic nn::gd::Texture2DResource* s_texture2DResource_DepthStencilBuffer = 0;\n//Color buffer\nnn::gd::Texture2DResourceDescription Text2DResDesc_ColorBuffer =\n {nn::gx::DISPLAY0_WIDTH, nn::gx::DISPLAY0_HEIGHT, 1,\n nn::gd::Resource::NATIVE_FORMAT_RGBA_8888,\n nn::gd::Memory::LAYOUT_BLOCK_8, nn::gd::Memory::VRAMA};\nnnResult res = nn::gd::Resource::CreateTexture2DResource(\n &Text2DResDesc_ColorBuffer, 0, GD_FALSE,\n &s_texture2DResource_ColorBuffer);\nif ( GD_NNRESULT_IS_FAILURE(res) ) {NN_PANIC(\"\\n\");}\n//Depth stencil buffer\nnn::gd::Texture2DResourceDescription Text2DResDesc_DepthStencilBuffer =\n {nn::gx::DISPLAY0_WIDTH, nn::gx::DISPLAY0_HEIGHT, 1,\n nn::gd::Resource::NATIVE_FORMAT_DEPTH_24_STENCIL_8,\n nn::gd::Memory::LAYOUT_BLOCK_8, nn::gd::Memory::VRAMB};\nres = nn::gd::Resource::CreateTexture2DResource(\n &Text2DResDesc_DepthStencilBuffer, 0, GD_FALSE,\n &s_texture2DResource_DepthStencilBuffer);\nif ( GD_NNRESULT_IS_FAILURE(res) ) {NN_PANIC(\"\\n\");}\nUsing the Texture2DResource objects that were created, create a RenderTarget object for the color buffer and a DepthStencilTarget object for the depth (stencil) buffer.\nCode 2-3. Creating the RenderTarget and DepthStencilTarget Objects\n static nn::gd::RenderTarget* s_RenderTarget = 0;\nstatic nn::gd::DepthStencilTarget* s_DepthStencilTarget = 0;\n//Color buffer\nnn::gd::RenderTargetDescription descRenderTarget = {0};\nres = nn::gd::OutputStage::CreateRenderTarget(\n s_texture2DResource_ColorBuffer, &descRenderTarget,\n &s_RenderTarget);\nif ( GD_NNRESULT_IS_FAILURE(res) ) {NN_PANIC(\"\\n\");}\n//Depth stencil buffer\nnn::gd::DepthStencilTargetDescription descDepthStencilTarget = {0};\nres = nn::gd::OutputStage::CreateDepthStencilTarget(\n s_texture2DResource_DepthStencilBuffer, &descDepthStencilTarget,\n &s_DepthStencilTarget);\nif ( GD_NNRESULT_IS_FAILURE(res) ) {NN_PANIC(\"\\n\");}\nConfigure the pipeline in the output stage, calling the SetRenderTarget function for the RenderTarget object and the SetDepthStencilTarget function for the DepthStencilTarget object.\nCode 2-4. Configuring the Pipeline's Framebuffer\n //Set the color/depthstencil targets.\nnn::gd::OutputStage::SetRenderTarget(s_RenderTarget);\nnn::gd::OutputStage::SetDepthStencilTarget(s_DepthStencilTarget);\n",0]
,["Pages/Page_54693514.html#PageId_54693517","2.3. Loading a Shader Program ",
"\n2.3. Loading a Shader Program\nYou must perform the following procedures before applying a shader program to the pipeline so that it can be used for rendering.\n\tLoad a shader binary. \tCreate a ShaderBinary object. \tCreate a Shader object. \tCreate a ShaderPipeline object. \tApply settings to the pipeline.\nThe following sample code applies the vertex shader included in a shader binary (with an rSize-byte file already loaded in rBuf) to the pipeline.\nCode 2-5. Loading a Shader Program\n static nn::gd::ShaderBinary* shaderBinary = 0;\nstatic nn::gd::ShaderPipeline* shaderPipeline = 0;\nstatic nn::gd::Shader* vertexShader = 0;\nnnResult res;\nres = nn::gd::ShaderStage::CreateShaderBinary(rBuf, rSize, &shaderBinary);\nif ( GD_NNRESULT_IS_FAILURE(res) ) {NN_PANIC(\"\\n\");}\nres = nn::gd::ShaderStage::CreateShader(shaderBinary, 0, &vertexShader);\nif ( GD_NNRESULT_IS_FAILURE(res) ) {NN_PANIC(\"\\n\");}\nres = nn::gd::ShaderStage::CreateShaderPipeline(\n vertexShader, NULL, &shaderPipeline);\nif ( GD_NNRESULT_IS_FAILURE(res) ) {NN_PANIC(\"\\n\");}\nnn::gd::ShaderStage::SetShaderPipeline(shaderPipeline);\nThe register values used by the shader program are set through UniformLocation objects that are obtained by specifying a ShaderPipeline object and data name as arguments to the GetShaderUniformLocation function in the shader stage.\nCode 2-6. Setting Register Values Using Uniform Settings\n static nn::gd::UniformLocation s_shaderVariable_proj;\nstatic nn::gd::UniformLocation s_shaderVariable_view;\ns_shaderVariable_proj = nn::gd::ShaderStage::GetShaderUniformLocation(\n shaderPipeline, \"uProjection\");\ns_shaderVariable_view = nn::gd::ShaderStage::GetShaderUniformLocation(\n shaderPipeline, \"uModelView\");\nNN_ASSERT(s_shaderVariable_proj.IsValid());\nNN_ASSERT(s_shaderVariable_view.IsValid());\nnn::math::Matrix44 proj, mv;\nnn::gd::ShaderStage::SetShaderPipelineConstantFloat(\n shaderPipeline, s_shaderVariable_proj, static_cast<f32*>(proj));\nnn::gd::ShaderStage::SetShaderPipelineConstantFloat(\n shaderPipeline, s_shaderVariable_view, static_cast<f32*>(mv));\n",0]
,["Pages/Page_54693514.html#PageId_54693518","2.4. Preparing Vertex Data ",
"\n2.4. Preparing Vertex Data\nThe GD library requires the use of vertex buffers to render primitives.\nTo use a vertex buffer, you must do the following.\n\tCreate an InputLayout object. \tCreate a VertexBufferResource object. \tApply the settings (the input layout and vertex buffer) to the pipeline.\nThe following sample code uses vertex coordinates and vertex color as interleaved vertex data.\nCode 2-7. Preparing Vertex Data\n u16 idxs[] = {0, 1, 2};\nnnResult res;\nfloat coords_color[] = {\n 0.5f, 0.0f, 0.f, 1.f, 1.f, 0.0f, 0.0f,\n -0.5f, 0.5f, 0.f, 1.f, 0.f, 1.0f, 0.0f,\n -0.5f,-0.5f, 0.f, 1.f, 0.f, 0.0f, 1.0f,\n};\nnn::gd::InputElementDescription descs[] =\n{\n { 0, \"aPosition\",\n nn::gd::VertexInputStage::STREAM_TYPE_FLOAT, 4, 0},\n { 0, \"aColor\",\n nn::gd::VertexInputStage::STREAM_TYPE_FLOAT, 3, sizeof(float) * 4},\n};\nu32 strides[] = { sizeof(float) * 7 };\nres = nn::gd::VertexInputStage::CreateInputLayout(\n descs, 2, strides, vertexShader, &inputLayout);\nif ( GD_NNRESULT_IS_FAILURE(res) ) {NN_PANIC(\"\\n\");}\nnn::gd::VertexBufferResourceDescription desc;\ndesc.m_ByteSize = sizeof(coords_color);\ndesc.m_MemLocation = nn::gd::Memory::FCRAM;\nres = nn::gd::Resource::CreateVertexBufferResource(\n &desc, coords_color, &bufferCoord);\nif ( GD_NNRESULT_IS_FAILURE(res) ) {NN_PANIC(\"\\n\");}\ndesc.m_ByteSize = sizeof(idxs);\nres = nn::gd::Resource::CreateVertexBufferResource(&desc, idxs, &bufferIndex);\nif ( GD_NNRESULT_IS_FAILURE(res) ) {NN_PANIC(\"\\n\");}\nnn::gd::VertexBufferResource* buffers[] = { bufferCoord };\nu32 offsets[] = { 0 };\nnn::gd::VertexInputStage::SetVertexBuffers(0, 1, buffers, offsets);\nnn::gd::VertexInputStage::SetInputLayout(inputLayout);\nnn::gd::VertexInputStage::SetIndexBuffer(\n bufferIndex, nn::gd::VertexInputStage::INDEX_FORMAT_USHORT, 0);\n",0]
,["Pages/Page_54693514.html#PageId_54693519","2.5. Configuring the Viewport ",
"\n2.5. Configuring the Viewport\nUse the SetViewport function in the rasterizer stage to configure the viewport.\nThe following sample code configures the viewport using information about the render target obtained by calling the GetRenderTargetProperties function in the output stage.\nCode 2-8. Configuring the Viewport\n nn::gd::TargetProperties renderTargetProperty;\nnn::gd::OutputStage::GetRenderTargetProperties(\n s_RenderTarget, &renderTargetProperty);\nnn::gd::Viewport viewPort(0, 0,\n renderTargetProperty.m_Width, renderTargetProperty.m_Height);\nnn::gd::RasterizerStage::SetViewport(viewPort);\n",0]
,["Pages/Page_54693514.html#PageId_54693520","2.6. Rendering ",
"\n2.6. Rendering\nYou call different rendering functions depending on whether you use vertex indices. Call the nn::gd::System::DrawIndexed function when you use vertex indices and the nn::gd::System::Draw function when you do not. The library also provides nn::gd::System::DrawImmediate and nn::gd::System::DrawImmediateIndexed for rendering without using a vertex buffer resource. Although it is more flexible to handle vertex data if you render without using a vertex buffer resource, rendering is performed more slowly.\nUse the nn::gd::Memory::ClearTargets function to clear the framebuffer.\nThe following sample code clears the framebuffer and then uses vertex indices for rendering.\nCode 2-9. Rendering\n //Clear the render buffers.\nu8 clearColor[] = {25, 25, 122, 255};\nnn::gd::Memory::ClearTargets(s_RenderTarget, s_DepthStencilTarget,\n clearColor, 1.f, 0);\n//Draw.\nnn::gd::System::DrawIndexed(3, 0);\n",0]
,["Pages/Page_54693514.html#PageId_54693521","2.7. Displaying the Rendered Results ",
"\n2.7. Displaying the Rendered Results\nUse the nn::gd::Memory::CopyTexture2DResourceBlockToLinear function to copy the content of the color buffer to the display buffer and then display the rendered results on the LCDs. This function has nearly the same functionality as the nngxTransferRenderImage function, but the application must use a function such as nngxGetDisplaybufferParameteri to get the copy destination address.\nCode 2-10. Displaying the Rendered Results\n //Transfer the framebuffer to the display buffer.\nint dstAddr;\nnngxActiveDisplay(NN_GX_DISPLAY0);\nnngxGetDisplaybufferParameteri(NN_GX_DISPLAYBUFFER_ADDRESS, &dstAddr);\nnn::gd::Memory::CopyTexture2DResourceBlockToLinear(\n s_texture2DResource_ColorBuffer, //source\n 0, //sourceMipLevelIndex\n 0, //srcOffsetY\n (u8*)dstAddr, //dstAddr\n nn::gx::DISPLAY0_WIDTH, //dstWidth\n nn::gx::DISPLAY0_HEIGHT, //dstHeight\n nn::gd::Resource::NATIVE_FORMAT_RGB_888, //dstFormat\n nn::gd::Memory::DOWNSCALING_NONE, //DownScalingMode\n GD_FALSE //yFlip\n);\nnngxWaitCmdlistDone();\nnngxSwapBuffers(NN_GX_DISPLAY_BOTH);\n",0]
,["Pages/Page_54693514.html#PageId_54693522","2.8. Finalizing ",
"\n2.8. Finalizing\nThe nn::gd::System::Finalize function shuts down the GD library. You can no longer call GD functions after this function has completed.\nFinalization automatically deallocates memory, including resources managed by the library and those allocated by the application. However, when possible, have applications explicitly deallocate objects that they create (such as state and resource objects).\nCONFIDENTIAL\n",0]
,["Pages/Page_54693523.html","3. Resource Objects ",
"3. Resource Objects\nIn general, the nn::gd::Resource class defines functions that handle resource objects.\n",0]
,["Pages/Page_54693523.html#PageId_54693524","3.1. Texture 2D Resources ",
"The nn::gd::Resource class can create and release resource objects. It can also get detailed information and data pointers for overwriting data.\n3.1. Texture 2D Resources\n",0]
,["Pages/Page_54693523.html#PageId_54693525","3.1.1. Creating a Texture2DResource Object ",
"Texture 2D resources are resources that have a width and height, such as texture images and color buffers. These resources are handled by Texture2DResource objects.\n3.1.1. Creating a Texture2DResource Object\nYou can use the nn::gd::Resource::CreateTexture2DResource function to create Texture2DResource objects.\nCode 3-1. Creating a Texture2DResource Object\n static nnResult nn::gd::Resource::CreateTexture2DResource(\n const nn::gd::Texture2DResourceDescription* description,\n const void* initialData, gdBool autoGenerateMipMapsFromData,\n nn::gd::Texture2DResource** texture2DResource,\n gdBool copyInitialData = GD_TRUE);\nSet resource information in a descriptor object (Texture2DResourceDescription) specified by the description parameter. Further details on descriptor class settings are provided in 3.1.1.2. Descriptor Class for Texture2DResource Objects.\nIf you are creating a resource that has initial data, such as texture data loaded from a file, specify the starting address of that data in the initialData parameter. If you specify GD_TRUE for the copyInitialData parameter, the initial data is copied to the memory specified by the descriptor class when the resource is created. Because this function adds a DMA transfer command request to the command list when the resource data is allocated in VRAM, do not overwrite or deallocate the memory region that contains the initial data until the command list finishes executing. If you specify GD_FALSE for copyInitialData, the address specified in initialData is used as the resource. Note that if a resource contains mipmap data, you must allocate enough memory to store all of it. You also need to pay attention to memory alignment. Applications are responsible for deallocating resource memory.\nSpecify NULL for initialData when you create resources that do not have initial data. A memory region large enough to hold the resource data will be allocated from the memory specified in the descriptor class.\nSpecify GD_TRUE for the autoGenerateMipMapsFromData parameter if you want texture mipmaps to be automatically generated from the initial data. For more information about generating mipmaps automatically, see 3.1.7. Generating Mipmaps Automatically.\n",0]
,["Pages/Page_54693523.html#PageId_54693526","3.1.1.1. Helper Functions for Converting Textures Into the Native Format ",
"Use the pointer that the function sets in texture2DResource to access the Texture2DResource object that you have created.\n3.1.1.1. Helper Functions for Converting Textures Into the Native Format\nYou must specify initial data in the native 3DS format when you create Texture2DResource objects to be pasted as textures on 3D models. The nn::gd::Resource::Helper class provides helper functions for converting standard textures, such as those used by OpenGL, into the native 3DS format. However, these helper functions have a high processing load and are not well-suited for runtime processing.\nThere are different helper functions for converting compressed and uncompressed textures, but with the exception of the source pixel format they accept the same arguments.\nCode 3-2. Helper Functions for Converting Textures Into Native Format\n static nnResult nn::gd::Resource::Helper::ConvertTextureResourceToNativeFormat(\n nn::gd::Resource::Format format,\n u32 width, u32 height, const u8* dataSrc, u8* dataDst,\n nn::gd::Resource::NativeFormat* pnativeFormat = NULL);\nstatic nnResult nn::gd::Resource::Helper::ConvertCompressedTextureResourceToNativeFormat(\n nn::gd::Resource::CompressedFormat format,\n u32 width, u32 height, u8* dataSrc, u8* dataDst,\n nn::gd::Resource::NativeFormat* pnativeFormat = NULL);\nSpecify the original pixel format in the format parameter.\nYou can specify the following pixel formats to the ConvertTextureResourceToNativeFormat function.\n\tFORMAT_RGBA_8888 \tFORMAT_RGB_888 \tFORMAT_RGBA_5551 \tFORMAT_RGB_565 \tFORMAT_RGBA_4444 \tFORMAT_LUMINANCE_ALPHA_88 \tFORMAT_HILO_88 \tFORMAT_LUMINANCE_8 \tFORMAT_ALPHA_8 \tFORMAT_LUMINANCE_ALPHA_44 \tFORMAT_LUMINANCE_4 \tFORMAT_ALPHA_4\nYou can specify the following pixel format in the ConvertCompressedTextureResourceToNativeFormat function.\n\tFORMAT_ETC1_RGB8\nDepending on the pixel format, a buffer large enough to hold the data to convert is temporarily allocated from device memory. A temporary buffer is always allocated when the same address is specified for both dataSrc and dataDst.\nFor more information about the differences between the standard and native texture formats, see the 3DS Programming Manual: Basic Graphics.\n",0]
,["Pages/Page_54693523.html#PageId_54693527","3.1.1.2. Descriptor Class for Texture2DResource Objects ",
"\n3.1.1.2. Descriptor Class for Texture2DResource Objects\nThis section introduces the member variables of the descriptor class (Texture2DResourceDescription) for Texture2DResource objects, and describes setting-related cautions and information.\nCode 3-3. Texture2DResourceDescription Class Definition\n class nn::gd::Texture2DResourceDescription\n{\n u32 m_Width;\n u32 m_Height;\n u32 m_CountMipLevels;\n nn::gd::Resource::NativeFormat m_Format;\n nn::gd::Memory::MemoryLayout m_MemLayout;\n nn::gd::Memory::MemoryLocation m_MemLocation;\n};\n",0]
,["Pages/Page_54693523.html#Anchor_54693527_幅（m_Width）","Width (m_Width) ",
"Width (m_Width)\nSets the width of the resource data in pixels. Specify the width of the top-level texture data for any texture that has mipmaps.\n",0]
,["Pages/Page_54693523.html#Anchor_54693527_高さ（m_Height）","Height (m_Height) ",
"Height (m_Height)\nSets the height of the resource data in pixels. Specify the height of the top-level texture data for any texture that has mipmaps.\n",0]
,["Pages/Page_54693523.html#Anchor_54693527_ミップマップレベル（m_CountMipLevels）","Mipmap Levels (m_CountMipLevels) ",
"Mipmap Levels (m_CountMipLevels)\nSets the number of mipmap levels in resource data that has mipmaps. Specify a value of 1 for any texture that does not have mipmaps. Specifying a value of 0 is the same as specifying the maximum number of mipmap levels possible for the resource data's width and height.\n",0]
,["Pages/Page_54693523.html#Anchor_54693527_ピクセルフォーマット（m_Format）","Pixel Format (m_Format) ",
"Pixel Format (m_Format)\nThe pixel formats that you can specify for a resource depend upon how you plan to use it.\nThe following table shows which pixel formats can be specified for texture data (Texture2D or TextureCube objects), color buffers (RenderTarget objects), and depth stencil buffers (DepthStencilTarget objects).\nTable 3-1. Specifiable Pixel Formats\n \tPixel Formats \tTexture2D \tTextureCube \tRenderTarget \tDepthStencilTarget\n \tResource::NATIVE_FORMAT_RGBA_8888 1 \t○ \t○ \t○ \tError\n \tResource::NATIVE_FORMAT_RGB_888 1 \t○ \t○ \tError \tError\n \tResource::NATIVE_FORMAT_RGBA_5551 1 \t○ \t○ \t○ \tError\n \tResource::NATIVE_FORMAT_RGB_565 1 \t○ \t○ \t○ \tError\n \tResource::NATIVE_FORMAT_RGBA_4444 1 \t○ \t○ \t○ \tError\n \tResource::NATIVE_FORMAT_LUMINANCE_ALPHA_88 \t○ \t○ \tError \tError\n \tResource::NATIVE_FORMAT_HILO_88 \t○ \t○ \tError \tError\n \tResource::NATIVE_FORMAT_LUMINANCE_8 \t○ \t○ \tError \tError\n \tResource::NATIVE_FORMAT_ALPHA_8 \t○ \t○ \tError \tError\n \tResource::NATIVE_FORMAT_LUMINANCE_ALPHA_44 \t○ \t○ \tError \tError\n \tResource::NATIVE_FORMAT_LUMINANCE_4 \t○ \t○ \tError \tError\n \tResource::NATIVE_FORMAT_ALPHA_4 \t○ \t○ \tError \tError\n \tResource::NATIVE_FORMAT_ETC1_RGB8 \t○ \t○ \tError \tError\n \tResource::NATIVE_FORMAT_ETC1_A4 \t○ \t○ \tError \tError\n \tResource::NATIVE_FORMAT_GAS 2 \t○ \tError \t○ \tError\n \tResource::NATIVE_FORMAT_SHADOW 2 \t○ \t○ \t○ \tError\n \tResource::NATIVE_FORMAT_DEPTH_16 \t○ \t○ \tError \t○\n \tResource::NATIVE_FORMAT_DEPTH_24 \t○ \t○ \tError \t○\n \tResource::NATIVE_FORMAT_DEPTH_24_STENCIL_8 \t○ \t○ \tError \t○\n Note 1: Supports automatic mipmap generation.\n Note 2: Does not support mipmaps.\n",0]
,["Pages/Page_54693523.html#Anchor_54693527_メモリレイアウト（m_MemLayout）","Memory Layout (m_MemLayout) ",
"Memory Layout (m_MemLayout)\nYou must specify Memory::LAYOUT_BLOCK_8 as the memory layout for (1) resource objects that are used to create Texture2D and TextureCube objects, or (2) resource objects that you create with initial data specified.\nYou can specify Memory::LAYOUT_BLOCK_8 or Memory::LAYOUT_BLOCK_32 as the memory layout for resource objects that are used to create RenderTarget and DepthStencilTarget objects.\nYou can specify Memory::LAYOUT_LINEAR as the memory layout for any other resource objects.\n",0]
,["Pages/Page_54693523.html#Anchor_54693527_リソースデータを確保するメモリ（m_MemLocation）","Memory to Allocate for Resource Data (m_MemLocation) ",
"Memory to Allocate for Resource Data (m_MemLocation)\nSpecifies the memory that will hold the resource data. You cannot create a RenderTarget or DepthStencilTarget object using a resource object created with Memory::FCRAM (device memory) specified.\nTable 3-2. Memory That Can Be Specified for Allocating Resource Data\n \tMemory Used to Hold Resource Data \tTexture2D \tTextureCube \tRenderTarget \tDepthStencilTarget\n \tMemory::FCRAM (device memory) \t○ \t○ \tError \tError\n \tMemory::VRAMA (VRAM-A) \t○ \t○ \t○ \t○\n \tMemory::VRAMB (VRAM-B) \t○ \t○ \t○ \t○\n",0]
,["Pages/Page_54693523.html#PageId_54693528","3.1.2. Releasing a Texture2DResource Object ",
"\n3.1.2. Releasing a Texture2DResource Object\nCall the nn::gd::Resource::ReleaseTexture2DResource function to release a Texture2DResource object. The memory region allocated to create the object is deallocated when the object is released.\nCode 3-4. Releasing a Texture2DResource Object\n static nnResult nn::gd::Resource::ReleaseTexture2DResource(\n nn::gd::Texture2DResource* texture2DResource);\n",0]
,["Pages/Page_54693523.html#PageId_54693529","3.1.3. Getting Detailed Information About a Texture2DResource Object ",
"\n3.1.3. Getting Detailed Information About a Texture2DResource Object\nYou can use the nn::gd::Resource::GetTexture2DResourceProperties function to get detailed information (using the Texture2DResourceProperties class) stored in a Texture2DResource object.\nCode 3-5. Getting Detailed Information About a Texture2DResource Object\n static nnResult nn::gd::Resource::GetTexture2DResourceProperties(\n const nn::gd::Texture2DResource* texture2DResource,\n nn::gd::Texture2DResourceProperties* properties);\n",0]
,["Pages/Page_54693523.html#PageId_54693530","3.1.3.1. Detailed Information About a Texture2DResource Object ",
"When the function returns, the properties parameter contains a pointer to the detailed information stored in the Texture2DResource object that was specified in the texture2DResource parameter. Ensure that you do not modify the content of the Texture2DResource object through the pointer that is returned.\n3.1.3.1. Detailed Information About a Texture2DResource Object\nAn object's detailed information (the Texture2DResourceProperties class) has nearly the same settings as the descriptor class that was specified when the object was created.\nCode 3-6. Texture2DResourceProperties Class Definition\n class nn::gd::Texture2DResourceProperties\n{\n u32 m_Width;\n u32 m_Height;\n u32 m_CountMipLevels;\n u32 m_PixelSize;\n nn::gd::Resource::NativeFormat m_Format;\n nn::gd::Memory::MemoryLayout m_MemLayout;\n nn::gd::Memory::MemoryLocation m_MemLocation;\n u8* m_MemAddr;\n nn::gd::MipmapResourceInfo GetMipmapAddress(u32 mipmapLevel);\n};\nThe following member variables include only those that could differ from settings in the descriptor class or those that exist only in this class.\n",0]
,["Pages/Page_54693523.html#Anchor_54693530_ミップマップレベル（m_CountMipLevels）","Mipmap Levels (m_CountMipLevels) ",
"Mipmap Levels (m_CountMipLevels)\nIf a value of 0 is specified in the descriptor, this variable is configured with the maximum number of mipmap levels possible for the resource data's width and height.\n",0]
,["Pages/Page_54693523.html#Anchor_54693530_ピクセルサイズ（m_PixelSize）","Pixel Size (m_PixelSize) ",
"Pixel Size (m_PixelSize)\nThis variable is configured with the number of bits in the resource data's pixel format.\n",0]
,["Pages/Page_54693523.html#Anchor_54693530_リソースデータの先頭アドレス（m_MemAddr）","Starting Address of Resource Data (m_MemAddr) ",
"Starting Address of Resource Data (m_MemAddr)\nThis variable specifies the physical starting address of the resource data.\n",0]
,["Pages/Page_54693523.html#Anchor_54693530_ミップマップデータのアドレス情報の取得（GetMipmapAddress()）","Getting Address Information for Mipmap Data (GetMipMapAddress) ",
"Getting Address Information for Mipmap Data (GetMipMapAddress)\nThis variable defines a helper function that, given a mipmap level, gets address information (using the MipmapResourceInfo class) for the mipmap data.\nCode 3-7. Definition of Address Information for Mipmap Data\n class nn::gd::MipmapResourceInfo\n{\n u32 m_Width;\n u32 m_Height;\n u8* m_MemAddr;\n};\nThis variable defines the width (m_Width), height (m_Height), and the data’s starting address (m_MemAddr) of the mipmap data.\n",0]
,["Pages/Page_54693523.html#PageId_54693531","3.1.4. Getting a Data Pointer in a Texture2DResource Object ",
"\n3.1.4. Getting a Data Pointer in a Texture2DResource Object\nYou can use the nn::gd::Resource::MapTexture2DResource function to get a data pointer for accessing resource data.\nCode 3-8. Getting a Pointer to the Data in a Texture2DResource Object\n static nnResult nn::gd::Resource::MapTexture2DResource(\n nn::gd::Texture2DResource* texture2DResource,\n s32 mipLevelIndex, nn::gd::Resource::MapUsage usage, void** data);\nstatic nnResult nn::gd::Resource::UnmapTexture2DResource(\n nn::gd::Texture2DResource* texture2DResource);\nThis function returns a memory address in the data parameter that indicates the starting address of the texture data, at the mipmap level specified by the mipLevelIndex parameter, within the resource data of the Texture2DResource object specified by the texture2DResource parameter. Specify the following mapping methods (types of resource data access) in usage.\nTable 3-3. Data Pointer Mapping Methods\n \tDefinition \tMapping Method \tDevice Memory \tVRAM\n \tResource::MAP_READ_ONLY \tRead-only \t○ \t○\n \tResource::MAP_WRITE_ONLY \tWrite-only \t○ \tError\n \tResource::MAP_READ_WRITE \tRead/write \t○ \tError\n Note:\n The mapping method is only checked when you get a data pointer. Although writing to the resource data in device memory obtained by MAP_READ_ONLY does not result in an error, the cache is not flushed when the mapping is removed.\n You cannot directly access resource data in VRAM. Use a DMA transfer or some other means to copy the data to memory that can be directly accessed by the CPU.\nAfter you have finished accessing the resource data, remove the mapping using the nn::gd::Resource::UnmapTexture2DResource function. If you get a data pointer to resource data in device memory using a mapping method that includes writes, the cache is flushed when the mapping is removed.\n",0]
,["Pages/Page_54693523.html#PageId_54693532","3.1.5. Clearing Resource Data (Texture2DResource Objects) ",
"\n3.1.5. Clearing Resource Data (Texture2DResource Objects)\nYou can clear resource data using a specified value. You can also specify a range of mipmap levels to clear in resources that have mipmap data. However, you cannot clear resource data allocated in device memory.\nCode 3-9. Clearing Resource Data (Texture2DResource Objects)\n static nnResult nn::gd::Memory::ClearTexture2DResource(\n const nn::gd::Texture2DResource* tex2DResource,\n s32 mipLevelIndexStart, s32 mipLevelIndexEnd, const u8 Components[4]);\nstatic nnResult nn::gd::Memory::ClearTexture2DResource(\n const nn::gd::Texture2DResource* tex2DResource,\n s32 mipLevelIndexStart, s32 mipLevelIndexEnd, u32 value);\nThis function attempts to clear any resource when the value parameter is specified, but it is limited to the following pixel formats when the Components parameter is specified. The right side of the following figure shows how values specified to Components are applied to the data to be cleared.\nFigure 3-1. Clearing Resource Data: Structure and Valid Pixel Formats\n",0]
,["Pages/Page_54693523.html#PageId_54693533","3.1.6. Partially Copying Resource Data (Texture2DResource Objects) ",
"\n3.1.6. Partially Copying Resource Data (Texture2DResource Objects)\nYou can specify a mipmap level and rectangular region to create a partial copy of resource data. Its format is not converted because this function calls the nngxAddBlockImageCopyCommand function. Data is actually copied when the command list is executed.\nCode 3-10. Partially Copying Resource Data (Texture2DResource Objects)\n static nnResult nn::gd::Memory::CopyTextureSubResource(\n const nn::gd::Texture2DResource* source,\n s32 srcMipLevelIndex, const nn::gd::Memory::Rect& rect,\n const nn::gd::Texture2DResource* dest, s32 dstMipLevelIndex,\n s32 destPosX, s32 destPosY);\nThe resource data and the viewport (LCD) use different origins, which this function takes into account. Specify the rectangular region (rect) and the coordinates to copy to (destPosX and destPosY), based on the viewport's origin.\nAn error (the ResultInvalidTextureFormat function) is returned when the source and destination have different pixel sizes.\n",0]
,["Pages/Page_54693523.html#PageId_54693534","3.1.7. Generating Mipmaps Automatically ",
"\n3.1.7. Generating Mipmaps Automatically\nYou can automatically generate mipmap data for Texture2DResource objects.\nCode 3-11. Generating Mipmaps Automatically\n static nnResult nn::gd::Memory::GenerateMipMaps(\n const nn::gd::Texture2DResource* tex2DResource,\n u32 mipLevelSourceIndex, s32 mipLevelLastDestinationIndex);\n The Texture2DResource object specified for tex2DResource must be created so that it can hold the resource data generated for the mipmap levels. You can only generate mipmap data automatically for Texture2DResource objects created in the following formats.\n\tnn::gd::Resource::NATIVE_FORMAT_RGBA_8888 \tnn::gd::Resource::NATIVE_FORMAT_RGBA_5551 \tnn::gd::Resource::NATIVE_FORMAT_RGBA_4444 \tnn::gd::Resource::NATIVE_FORMAT_RGB_888 \tnn::gd::Resource::NATIVE_FORMAT_RGB_565\nYou can use the nn::gd::Resource::CreateTexture2DResourceCastFrom function to convert an object from a format that is not in this list to a format such as NATIVE_FORMAT_RGBA_8888, which generates mipmap data. This method is not guaranteed to produce proper mipmap data, however, because pixels comprise different bits in each format.\n",0]
,["Pages/Page_54693523.html#PageId_54693535","3.1.7.1. Using the CPU to Generate Mipmaps Automatically ",
"Mipmap data is automatically generated from the level specified by the mipLevelSourceIndex parameter to the level specified by the mipLevelLastDestinationIndex parameter. To specify the maximum number of mipmap levels for that resource, specify a value of -1 for mipLevelLastDestinationIndex.\n3.1.7.1. Using the CPU to Generate Mipmaps Automatically\nYou can also call the nn::gd::Resource::Helper::GenerateMipMapsCPU function to generate mipmap data automatically. This function uses the CPU to generate mipmaps and is slower than nn::gd::Memory::GenerateMipMaps, which uses the GPU, but it has the advantage of supporting more formats and allowing you to generate mipmap data with a minimum width and height of 8 pixels. (The minimum width and height is either 32 or 64 pixels, depending on the format, when the GPU is used.)\nCode 3-12. Using the CPU to Generate Mipmaps Automatically\n static nnResult nn::gd::Resource::Helper::GenerateMipMapsCPU(\n nn::gd::Resource::NativeFormat format,\n u32 width, u32 height, const u8* dataSrc, u8* dataDst,\n u32 countMipLevelToGenerate);\nYou can specify the following formats for the format parameter.\n\tnn::gd::Resource::NATIVE_FORMAT_RGBA_8888 \tnn::gd::Resource::NATIVE_FORMAT_RGBA_5551 \tnn::gd::Resource::NATIVE_FORMAT_RGBA_4444 \tnn::gd::Resource::NATIVE_FORMAT_RGB_888 \tnn::gd::Resource::NATIVE_FORMAT_RGB_565 \tnn::gd::Resource::NATIVE_FORMAT_LUMINANCE_ALPHA_88 \tnn::gd::Resource::NATIVE_FORMAT_LUMINANCE_ALPHA_44 \tnn::gd::Resource::NATIVE_FORMAT_LUMINANCE_8 \tnn::gd::Resource::NATIVE_FORMAT_ALPHA_8 \tnn::gd::Resource::NATIVE_FORMAT_HILO_88\nSpecify the width and height of the original resource data in width and height, respectively. Both the width and height must be at least 8 and a power of 2.\nSpecify the starting address of the resource data in the dataSrc parameter, and specify the address to store the mipmap data in the dataDst parameter. Allocate enough memory to store all of the generated mipmap data.\nFor the countMipLevelToGenerate parameter, specify the number of mipmap levels to generate.\n",0]
,["Pages/Page_54693523.html#PageId_54693536","3.1.8. Converting a Texture2DResource Object to Another Format ",
"\n3.1.8. Converting a Texture2DResource Object to Another Format\nYou can use the following function to convert an existing Texture2DResource object to another format. This allows the resource data for RenderTarget objects that were used for rendering (such as gas textures, shadow textures, and depth buffers) to be reused as texture data (resource data for Texture2D objects).\nCode 3-13. Converting a Texture2DResource Object to Another Format\n static nnResult nn::gd::Resource::CreateTexture2DResourceCastFrom(\n const nn::gd::Texture2DResource* initialTexture2DResource,\n nn::gd::Resource::NativeFormat format,\n nn::gd::Memory::MemoryLayout layout,\n nn::gd::Texture2DResource** texture2DResource);\nThe object returned in the texture2DResource parameter has the pixel format and memory layout of the Texture2DResource object specified for the initialTexture2DResource parameter changed to the values specified by the format and layout parameters, respectively. An error is returned if the two pixel formats are not compatible (if they have different pixel sizes).\nThis function creates a new Texture2DResource object that contains modified format information. However, the data is not actually converted from one format to another, nor is data actually copied. The original object remains unchanged. All information other than the pixel format and memory layout, such as the location at which data is stored, remains unchanged. If you overwrite resource data using the object created by this function, you also overwrite the resource data of the original object.\n",0]
,["Pages/Page_54693523.html#PageId_54693537","3.2. Vertex Buffer Resources ",
"\n3.2. Vertex Buffer Resources\n",0]
,["Pages/Page_54693523.html#PageId_54693538","3.2.1. Creating a VertexBufferResource Object ",
"Vertex buffer resources are vertex coordinates, vertex indices, and other resources that are used as vertex buffers. These resources are handled by VertexBufferResource objects.\n3.2.1. Creating a VertexBufferResource Object\nYou can use the nn::gd::Resource::CreateVertexBufferResource function to create VertexBufferResource objects.\nCode 3-14. Creating a VertexBufferResource Object\n static nnResult nn::gd::Resource::CreateVertexBufferResource(\n const nn::gd::VertexBufferResourceDescription* description,\n const void* initialData, nn::gd::VertexBufferResource** buffer,\n gdBool copyInitialData = GD_TRUE);\nSet resource information in a descriptor object (VertexBufferResourceDescription) specified by the description parameter. For more information about descriptor class settings, see 3.2.1.1. Descriptor Class for VertexBufferResource Objects.\nIf you are creating a resource that has initial data, such as vertex data loaded from a file, specify the starting address of that data in the initialData parameter. If you specify GD_TRUE for the copyInitialData parameter, the initial data is copied to the memory specified by the descriptor class when the resource is created. Because this function adds a DMA transfer command request to the command list when the resource data is allocated in VRAM, do not overwrite or deallocate the memory region that contains the initial data until the command list finishes executing. Pay attention to the memory alignment of the address specified in initialData, because it is used as the resource if you specify GD_FALSE for copyInitialData. Applications are responsible for deallocating resource memory.\nSpecify NULL for initialData when you create resources that do not have initial data. A memory region large enough to hold the resource data is allocated from the memory specified in the descriptor class.\n",0]
,["Pages/Page_54693523.html#PageId_54693539","3.2.1.1. Descriptor Class for VertexBufferResource Objects ",
"The function sets a pointer in the buffer parameter that you can use to access the VertexBufferResource that it just created.\n3.2.1.1. Descriptor Class for VertexBufferResource Objects\nThis section introduces the member variables of the descriptor class (VertexBufferResourceDescription) for VertexBufferResource objects and provides setting-related cautions and information.\nCode 3-15. VertexBufferResourceDescription Class Definition\n class nn::gd::VertexBufferResourceDescription\n{\n u32 m_ByteSize;\n nn::gd::Memory::MemoryLocation m_MemLocation;\n};\n",0]
,["Pages/Page_54693523.html#Anchor_54693539_頂点バッファのサイズ（m_ByteSize）","Vertex Buffer Size (m_ByteSize) ",
"Vertex Buffer Size (m_ByteSize)\nSets the number of bytes in the vertex buffer (resource data).\n",0]
,["Pages/Page_54693523.html#Anchor_54693539_リソースデータを確保するメモリ（m_MemLocation）","Memory to Allocate for Resource Data (m_MemLocation) ",
"Memory to Allocate for Resource Data (m_MemLocation)\nSpecifies the memory that will hold the resource data.\nThe memory to be specified can be selected from Memory::FCRAM (device memory), Memory::VRAMA (VRAM-A), or Memory::VRAMB (VRAM-B).\n",0]
,["Pages/Page_54693523.html#PageId_54693540","3.2.2. Releasing a VertexBufferResource Object ",
"\n3.2.2. Releasing a VertexBufferResource Object\nCall the nn::gd::Resource::ReleaseVertexBufferResource function to release a VertexBufferResource object. The memory region allocated to create the object is deallocated when the object is released.\nCode 3-16. Releasing a VertexBufferResource Object\n static nnResult nn::gd::Resource::ReleaseVertexBufferResource(\n nn::gd::VertexBufferResource *buffer);\n",0]
,["Pages/Page_54693523.html#PageId_54693541","3.2.3. Getting Detailed Information About a VertexBufferResource Object ",
"\n3.2.3. Getting Detailed Information About a VertexBufferResource Object\nYou can use the nn::gd::Resource::GetVertexBufferResourceProperties function to get detailed information (using the VertexBufferResourceProperties class) stored in a VertexBufferResource object.\nCode 3-17. Getting Detailed Information About a VertexBufferResource Object\n static nnResult nn::gd::Resource::GetVertexBufferResourceProperties(\n const nn::gd::VertexBufferResource* buffer,\n nn::gd::VertexBufferResourceProperties* properties);\n",0]
,["Pages/Page_54693523.html#PageId_54693542","3.2.3.1. Detailed Information About a VertexBufferResource Object ",
"This function sets a pointer in the properties parameter that points to the detailed information stored in the VertexBufferResource object specified in the buffer parameter. Ensure that you do not modify the content of the VertexBufferResource object using the pointer that you get.\n3.2.3.1. Detailed Information About a VertexBufferResource Object\nAn object's detailed information (the VertexBufferResourceProperties class) has nearly the same settings as the descriptor class that was specified when the object was created.\nCode 3-18. VertexBufferResourceProperties Class Definition\n class nn::gd::VertexBufferResourceProperties\n{\n u32 m_ByteSize;\n nn::gd::Memory::MemoryLocation m_MemLocation;\n u8* m_MemAddr;\n};\nThe rest of this section describes member variables that only exist in this class.\n",0]
,["Pages/Page_54693523.html#Anchor_54693542_リソースデータの先頭アドレス（m_MemAddr）","Starting Address of Resource Data (m_MemAddr) ",
"Starting Address of Resource Data (m_MemAddr)\nThis variable specifies the physical starting address of the resource data.\n",0]
,["Pages/Page_54693523.html#PageId_54693543","3.2.4. Getting a Pointer to the Data in a VertexBufferResource Object ",
"\n3.2.4. Getting a Pointer to the Data in a VertexBufferResource Object\nYou can use the nn::gd::Resource::MapVertexBufferResource function to get a data pointer for accessing resource data.\nCode 3-19. Getting a Pointer to the Data in a VertexBufferResource Object\n static nnResult nn::gd::Resource::MapVertexBufferResource(\n nn::gd::VertexBufferResource* buffer,\n nn::gd::Resource::MapUsage usage, void** data);\nstatic nnResult nn::gd::Resource::UnmapVertexBufferResource(\n nn::gd::VertexBufferResource* buffer);\nThe starting address of the resource data for the VertexBufferResource object specified by the buffer parameter is returned in the data parameter. Specify the following mapping methods (types of resource data access) in usage.\nTable 3-4. Data Pointer Mapping Methods\n \tDefinition \tMapping Method \tDevice Memory \tVRAM\n \tResource::MAP_READ_ONLY \tRead-only \t○ \t○\n \tResource::MAP_WRITE_ONLY \tWrite-only \t○ \tError\n \tResource::MAP_READ_WRITE \tRead/write \t○ \tError\n Note:\n The mapping method is only checked when you get a data pointer. Although writing to the resource data in device memory obtained by MAP_READ_ONLY does not result in an error, the cache is not flushed when the mapping is removed.\n You cannot directly access resource data in VRAM. Use a method such as DMA transfer to copy the data to memory that can be directly accessed by the CPU.\nAfter you have finished accessing the resource data, remove the mapping using the nn::gd::Resource::UnmapVertexBufferResource function. If you get a data pointer to resource data in device memory using a mapping method that includes writes, the cache is flushed when the mapping is removed.\n",0]
,["Pages/Page_54693523.html#PageId_54693544","3.2.5. Partially Copying Resource Data (VertexBufferResource Objects) ",
"\n3.2.5. Partially Copying Resource Data (VertexBufferResource Objects)\nYou can create a partial copy of resource data. Data is actually copied when the command list is executed because this function calls the nngxAddBlockImageCopyCommand function.\nCode 3-20. Partially Copying Resource Data (VertexBufferResource Objects)\n static nnResult nn::gd::Memory::CopyVertexBufferSubResource(\n const nn::gd::VertexBufferResource* source,\n u32 sourceOffset, u32 size,\n const nn::gd::VertexBufferResource* dest, u32 destOffset);\nData is not copied if the arguments passed to the sourceOffset, size, and destOffset parameters are not all multiples of 16.\nCONFIDENTIAL\n",0]
,["Pages/Page_54693545.html","4. Module Descriptions ",
"4. Module Descriptions\n",0]
,["Pages/Page_54693545.html#PageId_54693546","4.1. Shader Stage ",
"4.1. Shader Stage\nThe vertex shader and geometry shader are configured in the shader stage by functions defined in the nn::gd::ShaderStage class.\nThe following figure provides an overview of the settings made in the shader stage.\nFigure 4-1. Overview of the Shader Stage\nApplications load vertex shaders and geometry shaders as precompiled shader binary files. A shader binary file can contain one or more vertex shader and geometry shader programs.\nTo access a particular shader in a shader binary file, pass the loaded shader binary file to the nn::gd::ShaderStage::CreateShaderBinary function and then use the ShaderBinary object that is generated.\nCode 4-1. Creating a ShaderBinary Object\n static nnResult nn::gd::ShaderStage::CreateShaderBinary(\n const void* shaderBytecode, u32 bytecodeLength,\n nn::gd::ShaderBinary** shaderBinary);\nSpecify the starting address of the buffer storing the loaded shader binary code in the shaderBytecode parameter, and specify the size of the code (in bytes) in the bytecodeLength parameter.\nUse the pointer that you receive from shaderBinary to access the ShaderBinary object that you have created.\nUse the nn::gd::ShaderStage::CreateShader function to create a Shader object for accessing the shader program contained within the ShaderBinary object. A Shader object represents the code for a single shader (either a vertex shader or a geometry shader).\nCode 4-2. Creating a Shader Object\n static nnResult nn::gd::ShaderStage::CreateShader(\n nn::gd::ShaderBinary* shaderBinary, u32 shaderBinaryIndex,\n nn::gd::Shader** shader);\nIf the ShaderBinary object specified in the shaderBinary parameter has multiple shader programs, specify the shader program index in the shaderBinaryIndex parameter to create a Shader object.\nThe function sets a pointer in the shader parameter to the created Shader object.\nCall the nn::gd::ShaderStage::CreateShaderPipeline function with one or two Shader objects to create a ShaderPipeline object that you can use to apply the shader programs to the pipeline.\nCode 4-3. Creating a ShaderPipeline Object and Applying Shader Programs to the Pipeline\n static nnResult nn::gd::ShaderStage::CreateShaderPipeline(\n nn::gd::Shader* vertexShader, nn::gd::Shader* geometryShader,\n nn::gd::ShaderPipeline** shaderPipeline,\n nn::gd::ShaderPipelineUnmanagedRegisters* unmanagedRegister = NULL);\nstatic nnResult nn::gd::ShaderStage::SetShaderPipeline(\n nn::gd::ShaderPipeline* shaderPipeline);\nUse the vertexShader parameter to specify the Shader object to use as the vertex shader. Use the geometryShader parameter to specify the Shader object to use as the geometry shader, or NULL if a geometry shader is not required.\nUse the unmanagedRegister parameter to specify a range of registers for which the library will not automatically generate commands. For more information about this setting, see 4.1.2. Commands Automatically Generated by the Library to Set Registers.\n Warning:\n The GD library does not check whether the vertex shader outputs the proper vertex attributes to the geometry shader. To ensure that the vertex attributes output by the vertex shader match the vertex attributes used as input to the geometry shader, create a ShaderPipeline object with the correct combination of Shader objects.\nThe following sample code first creates a ShaderPipeline object that only uses a vertex shader, and then applies that object to the pipeline.\nCode 4-4. Configuring the Shader Stage\n nn::gd::ShaderBinary* shaderBinary = 0;\nnn::gd::ShaderPipeline* shaderPipeline = 0;\nnn::gd::Shader* vertexShader = 0;\nnn::gd::ShaderStage::CreateShaderBinary(\n shaderBinaryFileBuffer, bufferSize, &shaderBinary);\nnn::gd::ShaderStage::CreateShader(shaderBinary, 0, &vertexShader);\nnn::gd::ShaderStage::CreateShaderPipeline(vertexShader, NULL, &shaderPipeline);\nnn::gd::ShaderStage::SetShaderPipeline(shaderPipeline);\n",0]
,["Pages/Page_54693545.html#PageId_54693547","4.1.1. Configuring Registers for Shader Settings ",
"\n4.1.1. Configuring Registers for Shader Settings\nA ShaderPipeline object maintains values configured for floating-point constant registers and other registers that are used for shader settings. The GD library allows you to use shader stage functions to change register values for shader settings. These functions update variables that are temporarily saved as the internatl state of the ShaderPipeline object, and are not output to the 3D command buffer until a render command request.\nThe following table shows which functions to use to set values in different types of registers.\nTable 4-1. Register Types and Functions Called\n \tType of Register \tFunction\n \tFloating-point constant register \tnn::gd::ShaderStage::SetShaderPipelineConstantFloat\n \tBoolean register \tnn::gd::ShaderStage::SetShaderPipelineConstantBoolean\n \tInteger register \tnn::gd::ShaderStage::SetShaderPipelineConstantInteger\n Code 4-5. Functions Used to Configure Values to Registers for Shader Settings\n static nnResult nn::gd::ShaderStage::SetShaderPipelineConstantFloat(\n nn::gd::ShaderPipeline* shaderPipeline,\n nn::gd::UniformLocation uniformLocation, f32* v);\nstatic nnResult nn::gd::ShaderStage::SetShaderPipelineConstantBoolean(\n nn::gd::ShaderPipeline* shaderPipeline,\n nn::gd::UniformLocation uniformLocation, u16 v, u32 count);\nstatic nnResult nn::gd::ShaderStage::SetShaderPipelineConstantInteger(\n nn::gd::ShaderPipeline* shaderPipeline,\n nn::gd::UniformLocation uniformLocation, u8* v);\nstatic nn::gd::UniformLocation nn::gd::ShaderStage::GetShaderUniformLocation(\n nn::gd::ShaderPipeline* shaderPipeline, const char* name);\nGet a UniformLocation object using the nn::gd::ShaderStage::GetShaderUniformLocation function, and then use the object to specify which register value to change. Use the name parameter to specify the name of the uniform setting when getting the object. The IsValid function of the UniformLocation object'returns a value of 1 when the correct uniform setting has been obtained.\n",0]
,["Pages/Page_54693545.html#PageId_54693548","4.1.2. Commands Automatically Generated by the Library to Set Registers ",
"\n4.1.2. Commands Automatically Generated by the Library to Set Registers\nCommands to set registers are normally generated automatically when a different ShaderPipeline object (from the one currently being applied) is applied to the pipeline. However, performance may be improved in some cases by generating commands that write directly to the registers, rather than having commands generated automatically.\nThe nn::gd::ShaderStage::SetFloatConstantBuffer function takes this into account and allows you to change register settings independently.\nCode 4-6. Changing Register Settings Independently\n static nnResult nn::gd::ShaderStage::SetFloatConstantBuffer(\n nn::gd::ShaderStage::ShaderType shaderType,\n u32 firstRegisterIndex, u32 registerCount, f32* constantBufferSrc);\nThis function can only change the value of unmanaged floating-point constant registers, which are not set by automatically-generated commands. The range of unmanaged registers is specified by the ShaderPipelineUnmanagedRegisters object given as the unmanagedRegister argument when a ShaderPipeline object is created.\nThe ShaderPipelineUnmanagedRegisters class comprises an array of the UnmanagedRegistersInterval objects and the number of elements in that array. The UnmanagedRegistersInterval class comprises the shader type (m_ShaderType), the number of the first unmanaged register (m_FirstUnmanagedRegister), and the number of unmanaged registers (m_RegisterCount).\nCode 4-7. Classes Used to Specify Unmanaged Registers\n class nn::gd::UnmanagedRegistersInterval\n{\n nn::gd::ShaderStage::ShaderType m_ShaderType;\n u32 m_FirstUnmanagedRegister;\n u32 m_RegisterCount;\n};\nclass nn::gd::ShaderPipelineUnmanagedRegisters\n{\n nn::gd::UnmanagedRegistersInterval* m_ArrayUnmanagedRegistersInterval;\n u32 m_CountUnmanagedRegistersInterval;\n};\nThe following sample code shows how to create a ShaderPipeline object that includes unmanaged registers.\nCode 4-8. Creating a ShaderPipeline Object Including Unmanaged Registers\n nn::gd::ShaderPipelineUnmanagedRegisters unmanagedRegs;\nnn::gd::UnmanagedRegistersInterval arrayUnmanagedRegistersInterval[] =\n{\n {nn::gd::ShaderStage::SHADER_TYPE_VERTEX, 0, 4},\n {nn::gd::ShaderStage::SHADER_TYPE_VERTEX, 4, 3},\n {nn::gd::ShaderStage::SHADER_TYPE_VERTEX, 8, 3}\n};\nunmanagedRegs.m_ArrayUnmanagedRegistersInterval =\n arrayUnmanagedRegistersInterval;\nunmanagedRegs.m_CountUnmanagedRegistersInterval = 3;\nnn::gd::ShaderStage::CreateShaderPipeline(\n s_vertexShader, NULL, &s_shaderPipeline, &unmanagedRegs);\nThe following code is an example of writing directly to unmanaged registers.\nCode 4-9. Writing to Unmanaged Registers\n nn::gd::UniformLocation s_shaderVariable_proj =\n nn::gd::ShaderStage::GetShaderUniformLocation(\n s_shaderPipeline, \"uProjection\");\nnn::gd::ShaderStage::SetFloatConstantBuffer(\n nn::gd::ShaderStage::SHADER_TYPE_VERTEX,\n s_shaderVariable_proj.getRegister(), 4, data);\nIf a ShaderPipeline object has already been applied to the pipeline and you apply a new one that was generated from different ShaderBinary objects, all of the data in the ShaderBinary objects is output to the 3D command buffer during rendering. Because of the associated processing load, we recommend using unmanaged registers instead to minimize the number of 3D commands that are output.\n",0]
,["Pages/Page_54693545.html#PageId_54693549","4.2. Vertex Input Stage ",
"\n4.2. Vertex Input Stage\nThe vertex input stage handles settings related to vertex data (vertex attributes) input to the pipeline. The nn::gd::VertexInputStage class defines functions that configure these settings.\n",0]
,["Pages/Page_54693545.html#PageId_54693550","4.2.1. Registering Vertex Buffers ",
"The following diagram provides an overview of the settings made in the vertex input stage.\nFigure 4-2. Overview of the Vertex Input Stage\n4.2.1. Registering Vertex Buffers\nVertex buffers are registered in the vertex input stage's 12 slots. By specifying an array of VertexBufferResource objects and an array of starting offsets to vertex data, you can register vertex buffers in multiple consecutive slots by making a single call to the nn::gd::VertexInputStage::SetVertexBuffers function.\nCode 4-10. Registering Vertex Buffers\n static nnResult nn::gd::VertexInputStage::SetVertexBuffers(\n u32 startSlot, u32 numBuffers,\n nn::gd::VertexBufferResource** vertexBuffers, u32* offsets);\nUse the startSlot parameter to specify the first slot number (0–11) for registering a vertex buffer. Use the numBuffers parameter to specify the number of elements in the array of VertexBufferResource objects given to vertexBuffers. Use the offsets parameter to specify the starting offset for the vertex data array. You must register vertex buffers with the same number of vertex data elements defined by the InputLayout object (described in 4.2.4. Input Layout).\n",0]
,["Pages/Page_54693545.html#PageId_54693551","4.2.2. Using Vertex Indices ",
"\n4.2.2. Using Vertex Indices\nWhen you use vertex indices to render primitives, create a VertexBufferResource object by specifying an array of vertex indices as resource data, and then call the nn::gd::VertexInputStage::SetIndexBuffer function to register that object to the vertex input stage.\nCode 4-11. Registering Vertex Indices\n static nnResult nn::gd::VertexInputStage::SetIndexBuffer(\n nn::gd::VertexBufferResource* indexBuffer,\n nn::gd::VertexInputStage::IndexFormat format, u32 offset);\nThe VertexBufferResource object specified for the indexBuffer parameter is registered to the vertex input stage. When you register VertexBufferResource objects interleaved with vertex and other data, specify the starting offset (in bytes) of the vertex indices in the offset parameter.\nYou can specify one of the following vertex index formats for the format parameter.\nTable 4-2. Specifying the Vertex Index Format\n \tDefinition \tDescription\n \tINDEX_FORMAT_UBYTE \tVertex indices are treated as an unsigned byte array.\n \tINDEX_FORMAT_USHORT \tVertex indices are treated as an unsigned short array.\n ",0]
,["Pages/Page_54693545.html#PageId_54693552","4.2.3. Types of Primitives ",
"\n4.2.3. Types of Primitives\nUse the nn::gd::VertexInputStage::SetPrimitiveTopology function to set the type of primitives to render.\nCode 4-12. Setting the Type of Primitives to Render\n static nnResult nn::gd::VertexInputStage::SetPrimitiveTopology(\n nn::gd::VertexInputStage::PrimitiveTopology primitiveTopology);\nSpecify one of the following types of primitives for the primitiveTopology parameter.\nTable 4-3. Types of Primitives\n \tDefinition \tDescription\n \tPRIMITIVE_TOPOLOGY_TRIANGLELIST \tTriangles (GL_TRIANGLES)\n \tPRIMITIVE_TOPOLOGY_TRIANGLESTRIP \tTriangle strip (GL_TRIANGLE_STRIP)\n \tPRIMITIVE_TOPOLOGY_TRIANGLEFAN \tTriangle fan (GL_TRIANGLE_FAN)\n \tPRIMITIVE_TOPOLOGY_GEOMETRY \tGeometry shader (GL_GEOMETRY_PRIMITIVE_DMP)\n ",0]
,["Pages/Page_54693545.html#PageId_54693553","4.2.4. Input Layout ",
"\n4.2.4. Input Layout\nAn input layout (InputLayout object) defines how input variables are connected between a vertex shader and vertex buffer. A vertex buffer comprises either a simple scalar value or several interleaved scalar values. Call the nn::gd::VertexInputStage::CreateInputLayout function to create an InputLayout object. This function's parameters include an array of descriptor (InputElementDescription) objects for the input elements (vertex data) and a Shader object that was created in the shader stage.\nCode 4-13. Creating, Releasing, and Applying an Input Layout to the Pipeline\n static nnResult nn::gd::VertexInputStage::CreateInputLayout(\n nn::gd::InputElementDescription* inputElementDescs, u32 numElements,\n u32* strides, nn::gd::Shader* vertexShader,\n nn::gd::InputLayout** inputLayout);\nstatic nnResult nn::gd::VertexInputStage::ReleaseInputLayout(\n nn::gd::InputLayout* inputLayout);\nstatic nnResult nn::gd::VertexInputStage::SetInputLayout(\n nn::gd::InputLayout* inputLayout);\nSpecify an array of descriptor (InputElementDescription) objects with information on the input elements in inputElementDescs, and pass the number of descriptor objects in the array to numElements. For more information about descriptor class settings, see 4.2.4.1. Descriptor Class for InputLayout Objects.\nSpecify the amount of vertex data used by a single vertex as an array in strides for each slot number. If all of the input elements are interleaved in a single vertex buffer, this array only has a single element that represents the total size of the input elements. If the input elements are not interleaved, this array has multiple elements for the size of each input element.\nUse vertexShader to specify the Shader object for the vertex shader.\nYou can use the nn::gd::VertexInputStage::SetInputLayout function to apply a created InputLayout object to the pipeline.\nUse the nn::gd::VertexInputStage::ReleaseInputLayout function to release an InputLayout object that is no longer needed. This also deallocates memory regions that were allocated internally when the CombinerState object was created.\nIn the following sample code, an input layout is created from a descriptor class that describes a single vertex buffer with three interleaved input elements. The layout is then applied to the pipeline (vertex input stage).\nCode 4-14. Creating an Input Layout\n nn::gd::InputLayout* inputLayout = 0;\nnn::gd::InputElementDescription descs[] =\n{\n { 0, \"aPosition\",\n nn::gd::VertexInputStage::STREAM_TYPE_FLOAT, 3, 0 },\n { 0, \"aColor\",\n nn::gd::VertexInputStage::STREAM_TYPE_FLOAT, 4, sizeof(float) * 3 },\n { 0, \"aNormal\",\n nn::gd::VertexInputStage::STREAM_TYPE_FLOAT, 3, sizeof(float) * 7 },\n};\nu32 strides[] = { sizeof(float) * 10 };\nnn::gd::VertexInputStage::CreateInputLayout(\n descs, 3, strides, vertexShader, &inputLayout);\nnn::gd::VertexInputStage::SetInputLayout(inputLayout);\n",0]
,["Pages/Page_54693545.html#PageId_54693554","4.2.4.1. Descriptor Class for InputLayout Objects ",
"\n4.2.4.1. Descriptor Class for InputLayout Objects\nThis section introduces the members of the descriptor class (InputElementDescription) for InputLayout objects and provides setting-related cautions and information.\nCode 4-15. Definition of the InputElementDescription Class\n class nn::gd::InputElementDescription\n{\n u32 m_StreamSlot;\n const char* m_SemanticName;\n nn::gd::VertexInputStage::ShaderStreamFormatType m_Format;\n u32 m_Count;\n u32 m_AlignedByteOffset;\n static const u32 Append = 0x00080000;\n};\n",0]
,["Pages/Page_54693545.html#Anchor_54693554_スロット番号（m_StreamSlot）","Slot Number (m_StreamSlot) ",
"Slot Number (m_StreamSlot)\nThe slot number (0–11) used by an input element. To use the same slot number for more than one input element, the vertex buffers registered to that slot number must be interleaved. Slot numbers must be registered consecutively in ascending order, starting at 0.\n",0]
,["Pages/Page_54693545.html#Anchor_54693554_入力変数の名前（m_SemanticName）","Input Variable Name (m_SemanticName) ",
"Input Variable Name (m_SemanticName)\nThe name of an input variable defined by the vertex shader.\n",0]
,["Pages/Page_54693545.html#Anchor_54693554_入力要素のデータフォーマット（m_Format）","Input Element Data Format (m_Format) ",
"Input Element Data Format (m_Format)\nThe data format of an input element. Specify one of the values from the following table.\nTable 4-4. Input Element Data Formats\n \tDefinition \tDescription\n \tVertexInputStage::STREAM_TYPE_BYTE \tbyte (GL_BYTE)\n \tVertexInputStage::STREAM_TYPE_UNSIGNED_BYTE \tunsigned byte (GL_UNSIGNED_BYTE)\n \tVertexInputStage::STREAM_TYPE_SHORT \tshort (GL_SHORT)\n \tVertexInputStage::STREAM_TYPE_FLOAT \tfloat (GL_FLOAT)\n ",0]
,["Pages/Page_54693545.html#Anchor_54693554_入力要素のサイズ（m_Count）","Input Element Size (m_Count) ",
"Input Element Size (m_Count)\nThe size of (number of components in) an input element. It can be a value between 1 and 4. Note that this is not the size of the input element in bytes.\n",0]
,["Pages/Page_54693545.html#Anchor_54693554_入力要素のオフセット（m_AlignedByteOffset）","Input Element Offset (m_AlignedByteOffset) ",
"Input Element Offset (m_AlignedByteOffset)\nThe offset (in bytes) to the starting input element. By specifying InputElementDescription::Append, the offset can be calculated automatically, thus simplifying descriptor class settings.\n",0]
,["Pages/Page_54693545.html#PageId_54693555","4.3. Texture Stage ",
"\n4.3. Texture Stage\nThe texture stage creates texture objects and handles texture unit-related settings. The nn::gd::TextureStage class defines functions that configure these settings.\n",0]
,["Pages/Page_54693545.html#PageId_54693556","4.3.1. 2D Textures ",
"The following diagram provides an overview of the settings made in the texture stage.\nFigure 4-3. Overview of the Texture Stage\n4.3.1. 2D Textures\n2D textures are handled by Texture2D objects in the GD library. A Texture2D object is created from a single Texture2DResource object and a descriptor class (Texture2DDescription). If the Texture2DResource object has mipmap data, you can use descriptor class settings to specify which levels of mipmap data to use when you create a Texture2D object. For more information about the descriptor class settings, see 4.3.1.1. Descriptor Class for Texture2D Objects.\nYou can create and release a Texture2D object using the following functions.\nCode 4-16. Creating and Releasing a Texture2D Object\n static nnResult nn::gd::TextureStage::CreateTexture2D(\n const nn::gd::Texture2DResource* tex2DResource,\n nn::gd::Texture2DDescription* desc,\n nn::gd::Texture2D** texture2D);\nstatic nnResult nn::gd::TextureStage::ReleaseTexture2D(\n nn::gd::Texture2D* texture2D);\nOnly management information is released when a Texture2D object is released. Texture2DResource objects are not released.\nThe following sample code creates a Texture2DResource object and uses it to create a Texture2D object.\nCode 4-17. Creating a Texture2D Object\n nn::gd::Texture2DResource* texture2DResource = 0;\nnn::gd::Texture2DResourceDescription Text2DResDesc =\n {256, 256, 1, nn::gd::Resource::NATIVE_FORMAT_RGB_888,\n nn::gd::Memory::LAYOUT_BLOCK_8, nn::gd::Memory::FCRAM};\nnn::gd::Resource::CreateTexture2DResource(\n &Text2DResDesc, data, GD_FALSE, &texture2DResource);\nnn::gd::Texture2D* texture2D = 0;\nnn::gd::Texture2DDescription tex2DDesc = {0, 0};\nnn::gd::TextureStage::CreateTexture2D(\n texture2DResource, &tex2DDesc, &texture2D);\n",0]
,["Pages/Page_54693545.html#PageId_54693557","4.3.1.1. Descriptor Class for Texture2D Objects ",
"\n4.3.1.1. Descriptor Class for Texture2D Objects\nThis section introduces the member variables of the descriptor class (Texture2DDescription) for Texture2D objects and provides setting-related cautions and information.\nCode 4-18. Definition of the Texture2DDescription Class\n class nn::gd::Texture2DDescription\n{\n s32 m_MinMipLevelIndex;\n s32 m_MaxMipLevelIndex;\n};\n",0]
,["Pages/Page_54693545.html#Anchor_54693557_最小ミップマップレベルのインデックス（m_MinMipLevelIndex）","Index of the Smallest Mipmap Level (m_MinMipLevelIndex) ",
"Index of the Smallest Mipmap Level (m_MinMipLevelIndex)\nThis is the mipmap level of the Texture2DResource object used as the smallest mipmap level of a Texture2D object. If you set a value of -1, the largest mipmap level of the Texture2DResource object is used.\n",0]
,["Pages/Page_54693545.html#Anchor_54693557_最大ミップマップレベルのインデックス（m_MaxMipLevelIndex）","Index of the Largest Mipmap Level (m_MaxMipLevelIndex) ",
"Index of the Largest Mipmap Level (m_MaxMipLevelIndex)\nThis is the mipmap level of the Texture2DResource object used as the largest mipmap level of a Texture2D object. If you set a value of -1, the largest mipmap level of the Texture2DResource object is used.\n",0]
,["Pages/Page_54693545.html#PageId_54693558","4.3.1.2. Getting Detailed Information About a Texture2D Object ",
"\n4.3.1.2. Getting Detailed Information About a Texture2D Object\nYou can use the nn::gd::TextureStage::GetTexture2DProperties function to get detailed information (using the Texture2DProperties class).\nCode 4-19. Getting Detailed Information on a Texture2D Object\n static nnResult nn::gd::TextureStage::GetTexture2DProperties(\n const nn::gd::Texture2D* texture2D,\n nn::gd::Texture2DProperties* properties);\nAlthough the width, height, mipmap levels, and starting address of resource data may change with the mipmap levels specified when an object was created, they are generally the same as the detailed information for the Texture2DResource object.\nCode 4-20. Definition of the Texture2DProperties Class\n class nn::gd::Texture2DProperties\n{\n u32 m_Width;\n u32 m_Height;\n u32 m_CountMipLevels;\n u32 m_PixelSize;\n nn::gd::Resource::NativeFormat m_Format;\n nn::gd::Memory::MemoryLayout m_MemLayout;\n nn::gd::Memory::MemoryLocation m_MemLocation;\n u8* m_MemAddr;\n nn::gd::MipmapResourceInfo GetMipmapAddress(u32 mipmapLevel);\n};\n",0]
,["Pages/Page_54693545.html#PageId_54693559","4.3.2. Cube Map Textures ",
"\n4.3.2. Cube Map Textures\nCube map textures are handled by TextureCube objects in the GD library. A TextureCube object is created using six Texture2DResource objects. As with Texture2D objects, you can specify which mipmap levels to use. Each face (Texture2DResource object) of the cube map texture must have the same format, mipmap levels, resolution (which depends on the mipmap level), and memory placement.\nYou can create and release a TextureCube object using the following functions.\nCode 4-21. Creating and Releasing a TextureCube Object\n static nnResult nn::gd::TextureStage::CreateTextureCube(\n nn::gd::Texture2DResource** tex2DResources,\n nn::gd::TextureCubeDescription* desc,\n nn::gd::TextureCube** textureCube);\nstatic nnResult nn::gd::TextureStage::ReleaseTextureCube(\n nn::gd::TextureCube* textureCube);\nSpecify an array of six Texture2DResource objects, one for each face of the cube map texture, in the tex2DResources parameter. These textures are used for the +X, -X, +Y, -Y, +Z, and -Z faces, respectively, from the beginning of the array. Cube map texture faces are specified in the same order as the elements in descriptor class arrays.\n",0]
,["Pages/Page_54693545.html#PageId_54693560","4.3.2.1. Descriptor Class for TextureCube Objects ",
"Only management information is released when a TextureCube object is released. Texture2DResource objects are not released.\n4.3.2.1. Descriptor Class for TextureCube Objects\nThis section introduces the members of the descriptor class (TextureCubeDescription) for TextureCube objects and provides setting-related cautions and information.\nCode 4-22. Definition of the TextureCubeDescription Class\n class nn::gd::TextureCubeDescription\n{\n s32 m_MinMipLevelIndex[6];\n s32 m_MaxMipLevelIndex[6];\n TextureCubeDescription();\n TextureCubeDescription(int minMipLevelIndex, int maxMipLevelIndex);\n};\nThis class differs from Texture2DDescription in that it specifies mipmap levels for six faces and has a constructor to simplify the process of doing so.\n",0]
,["Pages/Page_54693545.html#Anchor_54693560_最小ミップマップレベルのインデックス（m_MinMipLevelIndex）","Index of the Smallest Mipmap Level (m_MinMipLevelIndex) ",
"Index of the Smallest Mipmap Level (m_MinMipLevelIndex)\nThis is the mipmap level of the Texture2DResource object used as the smallest mipmap level of a TextureCube object. If you set a value of -1, the largest mipmap level of the Texture2DResource object is used.\nThe values in the array correspond to the Texture2DResource objects for the +X, -X, +Y, -Y, +Z, and -Z faces, in that order.\n",0]
,["Pages/Page_54693545.html#Anchor_54693560_最大ミップマップレベルのインデックス（m_MaxMipLevelIndex）","Index of the Largest Mipmap Level (m_MaxMipLevelIndex) ",
"Index of the Largest Mipmap Level (m_MaxMipLevelIndex)\nThis is the mipmap level of the Texture2DResource object used as the largest mipmap level of a TextureCube object. If you set a value of -1, the largest mipmap level of the Texture2DResource object is used.\nThe values in the array correspond to the Texture2DResource objects for the +X, -X, +Y, -Y, +Z, and -Z faces, in that order.\n",0]
,["Pages/Page_54693545.html#PageId_54693561","4.3.2.2. Getting Detailed Information About a TextureCube Object ",
"\n4.3.2.2. Getting Detailed Information About a TextureCube Object\nYou can use the nn::gd::TextureStage::GetTextureCubeProperties function to get detailed information (using the TextureCubeProperties class) about a TextureCube object.\nCode 4-23. Getting Detailed Information About a TextureCube Object\n static nnResult nn::gd::TextureStage::GetTextureCubeProperties(\n const nn::gd::TextureCube* TextureCube,\n nn::gd::TextureCubeProperties* properties);\nThis class is the same as Texture2DProperties with the following exceptions: it maintains the starting address of six faces of resource data and accepts a cube map face in the faceIndex parameter, to get address information for mipmap data.\nCode 4-24. Definition of the TextureCubeProperties Class\n class nn::gd::TextureCubeProperties\n{\n u32 m_Width;\n u32 m_Height;\n u32 m_CountMipLevels;\n u32 m_PixelSize;\n nn::gd::Resource::NativeFormat m_Format;\n nn::gd::Memory::MemoryLayout m_MemLayout;\n nn::gd::Memory::MemoryLocation m_MemLocation;\n u8* m_MemAddr[6];\n nn::gd::MipmapResourceInfo GetMipmapAddress(u32 faceIndex, u32 mipmapLevel);\n};\n",0]
,["Pages/Page_54693545.html#PageId_54693562","4.3.3. Texture Units ",
"\n4.3.3. Texture Units\nThe GD library provides four logical texture units to match the 3DS hardware.\nTable 4-5. Types of Textures That Can Be Set for Each Texture Unit\n \tDefinition \tTypes of Textures That Can Be Set\n \t TextureStage::TEXTURE_UNIT_0\n (Texture unit 0)\n \t 2D textures (including gas and shadows)\n Cube map textures (including shadows)\n Projection textures\n \t TextureStage::TEXTURE_UNIT_1\n (Texture unit 1)\n \t2D textures\n \t TextureStage::TEXTURE_UNIT_2\n (Texture unit 2)\n \t2D textures\n \t TextureStage::TEXTURE_UNIT_3_PROCEDURAL\n (Texture unit 3)\n \tProcedural textures\n Note:\n The procedural texture stage handles all TextureStage::TEXTURE_UNIT_3_PROCEDURAL settings except for texture coordinates.\n",0]
,["Pages/Page_54693545.html#Anchor_54693562_2次元テクスチャ","2D textures ",
"2D textures\nUse the nn::gd::TextureStage::SetTexture function to set a 2D texture. You can set a 2D texture in three texture units: 0, 1, and 2.\nCode 4-25. Setting a 2D Texture in a Texture Unit\n static nnResult nn::gd::TextureStage::SetTexture(\n nn::gd::TextureStage::TextureUnitId textureUnitId,\n nn::gd::Texture2D& texture2D);\n",0]
,["Pages/Page_54693545.html#Anchor_54693562_そのほかのテクスチャ（プロシージャルテクスチャを除く）","Other (Non-Procedural) Textures ",
"Other (Non-Procedural) Textures\nAll other textures (except for procedural textures) can only be set in texture unit 0.\nCode 4-26. Setting Other Textures\n static nnResult nn::gd::TextureStage::SetTextureUnit0(\n nn::gd::TextureCube& textureCube);\nstatic void nn::gd::TextureStage::SetTexture2DProjectionForUnit0(gdBool value);\nstatic void nn::gd::TextureStage::SetPerspectiveShadow(gdBool v);\nstatic void nn::gd::TextureStage::SetShadowZBias(f32 zBias);\nUse the nn::gd::TextureStage::SetTextureUnit0 function to set a cube map texture.\nBy calling the nn::gd::TextureStage::SetTexture2DProjectionForUnit0 function and passing GD_TRUE as an argument, you can treat the 2D texture set in texture unit 0 as a projection texture.\n",0]
,["Pages/Page_54693545.html#PageId_54693566","4.3.3.1. Texture Coordinates ",
"If you apply perspective projection to create texture coordinates when you access a shadow texture or shadow cube map texture, pass GD_TRUE as an argument to nn::gd::TextureStage::SetPerspectiveShadow. You can also use nn::gd::TextureStage::SetShadowZBias to set a bias value that will be subtracted from the distance to the light source when a shadow texture is generated.\n4.3.3.1. Texture Coordinates\nThe following functions set the texture coordinates input to texture units 2 and 3.\nCode 4-27. Setting Texture Coordinates\n // For TextureUnit2\nstatic void nn::gd::TextureStage::SetTextureCoordinateSourceForUnit2(\n nn::gd::TextureStage::TextureCoordinateSourceUnit2 u);\n// For TextureUnit3\nstatic void nn::gd::TextureStage::SetTextureCoordinateSourceForUnit3Procedural(\n nn::gd::TextureStage::TextureCoordinateSourceUnit3Procedural u);\nYou can specify the following arguments to these functions.\nTable 4-6. Texture Coordinates That Can Be Set for Texture Unit 2\n \tDefinition \tInput Texture Coordinates\n \tUNIT2_TEXTURE_COORDINATE_FROM_UNIT_2 \tTexture coordinate 2 (default).\n \tUNIT2_TEXTURE_COORDINATE_FROM_UNIT_1 \tTexture coordinate 1\n Table 4-7. Texture Coordinates That Can Be Set for Texture Unit 3\n \tDefinition \tInput Texture Coordinates\n \tUNIT3_PROCEDURAL_TEXTURE_COORDINATE_FROM_UNIT_0 \tTexture coordinate 0 (default).\n \tUNIT3_PROCEDURAL_TEXTURE_COORDINATE_FROM_UNIT_1 \tTexture coordinate 1\n \tUNIT3_PROCEDURAL_TEXTURE_COORDINATE_FROM_UNIT_2 \tTexture coordinate 2\n",0]
,["Pages/Page_54693545.html#PageId_54693567","4.3.4. Sampler Settings ",
"\n4.3.4. Sampler Settings\nTo configure the texture sampling method (sampler settings), create a state (SamplerState) object and set it in a texture unit.\nCode 4-28. Creating, Releasing, and Specifying Sampler Settings in a Texture Unit\n static nnResult nn::gd::TextureStage::CreateSamplerState(\n const nn::gd::SamplerStateDescription* desc,\n nn::gd::SamplerState** sampler);\nstatic nnResult nn::gd::TextureStage::ReleaseSamplerState(\n nn::gd::SamplerState* sampler);\nstatic nnResult nn::gd::TextureStage::SetSamplerState(\n nn::gd::TextureStage::TextureUnitId textureUnitId,\n nn::gd::SamplerState* sampler);\nConfigure sampler settings in a descriptor class (SamplerStateDescription) in advance, specified in the desc parameter. For more information about descriptor class settings, see 4.3.4.1. Descriptor Class for SamplerState Objects.\nYou can use the nn::gd::TextureStage::SetSamplerState function to set a created SamplerState object in a texture unit. You can configure sampler settings in three texture units: 0, 1, and 2.\nUse the nn::gd::TextureStage::ReleaseSamplerState function to release a SamplerState object that is no longer needed. This also deallocates memory regions that were allocated internally when the CombinerState object was created.\nThe following sample code creates a SamplerState object with the default settings, changes the minification and magnification filters, and then sets that object in texture unit 0.\nCode 4-29. Sampler Settings\n nn::gd::SamplerState* sampler = 0;\nnn::gd::SamplerStateDescription samplerdesc;\nsamplerdesc.ToDefault();\nsamplerdesc.m_MinFilter = nn::gd::TextureStage::SAMPLER_MIN_FILTER_LINEAR;\nsamplerdesc.m_MagFilter = nn::gd::TextureStage::SAMPLER_MAG_FILTER_LINEAR;\nnn::gd::TextureStage::CreateSamplerState(&samplerdesc, &sampler);\nnn::gd::TextureStage::SetSamplerState(\n nn::gd::TextureStage::TEXTURE_UNIT_0, sampler);\n",0]
,["Pages/Page_54693545.html#PageId_54693568","4.3.4.1. Descriptor Class for SamplerState Objects ",
"\n4.3.4.1. Descriptor Class for SamplerState Objects\nThis section introduces the members of the descriptor (SamplerStateDescription) class for SamplerState objects and provides setting-related cautions and information.\nCode 4-30. Definition of the SamplerStateDescription Class\n class nn::gd::SamplerStateDescription\n{\n nn::gd::TextureStage::SamplerMinFilter m_MinFilter;\n nn::gd::TextureStage::SamplerMagFilter m_MagFilter;\n nn::gd::TextureStage::SamplerWrapMode m_WrapS;\n nn::gd::TextureStage::SamplerWrapMode m_WrapT;\n u8 m_BorderColor[4];\n u32 m_LodBias;\n u32 m_MinLod;\n u32 m_MaxLod;\n void SetMinFilter(nn::gd::TextureStage::SamplerMinFilter filter);\n void SetMagFilter(nn::gd::TextureStage::SamplerMagFilter filter);\n void SetWrapS(nn::gd::TextureStage::SamplerWrapMode wrap);\n void SetWrapT(nn::gd::TextureStage::SamplerWrapMode wrap);\n void SetBorderColor(u8 colorR, u8 colorG, u8 colorB, u8 colorA);\n void SetBorderColor(f32 colorR, f32 colorG, f32 colorB, f32 colorA);\n void SetLodBias(f32 biasValue);\n void SetMinLod(u32 lodValue);\n void SetMaxLod(u32 lodValue);\n SamplerStateDescription();\n void ToDefault();\n void SetShadow();\n void SetShadowCube();\n void SetGas();\n};\n",0]
,["Pages/Page_54693545.html#Anchor_54693568_縮小時のフィルタ（m_MinFilter）","Minification Filter (m_MinFilter) ",
"Minification Filter (m_MinFilter)\nThis filter is applied during texture minification. You can set this value using the SetMinFilter function.\n",0]
,["Pages/Page_54693545.html#Anchor_54693568_拡大時のフィルタ（m_MagFilter）","Magnification Filter (m_MagFilter) ",
"Magnification Filter (m_MagFilter)\nThis filter is applied during texture magnification. You can set this value using the SetMagFilter function.\n",0]
,["Pages/Page_54693545.html#Anchor_54693568_S方向のラッピングモード（m_WrapS）","Wrapping Mode in the S Direction (m_WrapS) ",
"Wrapping Mode in the S Direction (m_WrapS)\nThis wrapping mode is applied to repeating textures in the S direction. You can set this value using the SetWrapS function.\n",0]
,["Pages/Page_54693545.html#Anchor_54693568_T方向のラッピングモード（m_WrapT）","Wrapping Mode in the T Direction (m_WrapT) ",
"Wrapping Mode in the T Direction (m_WrapT)\nThis wrapping mode is applied to repeating textures in the T direction. You can set this value using the SetWrapT function.\n",0]
,["Pages/Page_54693545.html#Anchor_54693568_ボーダーカラー（m_BorderColor）","Border Color (m_BorderColor) ",
"Border Color (m_BorderColor)\nThis border color is used when TextureStage::SAMPLER_WRAP_CLAMP_TO_BORDER is specified as the wrapping mode. You can call the SetBorderColor function to set each RGBA component to a value between 0 and 255 or between 0.0 and 1.0.\n",0]
,["Pages/Page_54693545.html#Anchor_54693568_LODのバイアス値（m_LodBias）","LOD Bias (m_LodBias) ",
"LOD Bias (m_LodBias)\nThis bias value is used to determine the level of detail (LOD). Because you must set this value using the same format as when setting a GPU register, do so by passing a value between -16.0 and 16.0 to the SetLoadBias function.\nThe bias value is a 13-bit signed fixed-point decimal number with 8 fractional bits. To change member variables directly, use the Utils::Float32ToFix13Fraction8 utility function.\n",0]
,["Pages/Page_54693545.html#Anchor_54693568_LODの最小レベル（m_MinLod）","Minimum LOD (m_MinLod) ",
"Minimum LOD (m_MinLod)\nThis is the smallest mipmap level used for LOD. If you use LOD, set the smallest mipmap level to a value of 0 or greater. (Specify 0 if you do not use LOD.) You can set this value using the SetMinLod function.\n",0]
,["Pages/Page_54693545.html#Anchor_54693568_LODの最大レベル（m_MaxLod）","Maximum LOD (m_MaxLod) ",
"Maximum LOD (m_MaxLod)\nThis is the largest mipmap level used for LOD. Specify a value of 0 when you do not use LOD and a value that is one less than the maximum number of mipmap levels in the texture when you do use LOD. You can set this value using the SetMaxLod function.\nSpecify UINT_MAX or another large value when the same sampler settings are used by textures that have different numbers of mipmap levels. This value will be one less than the number of mipmap levels in the texture when it is rendered. It is determined from the default settings and source.\n",0]
,["Pages/Page_54693545.html#Anchor_54693568_デフォルトの設定（ToDefault()）","Default Settings (ToDefault) ",
"Default Settings (ToDefault)\nYou can call the ToDefault function to change all members to their default values. This is also done in the constructor.\nTable 4-8. Default Sampler Settings\n \tMember \tSetting Value\n \tm_MinFilter \tTextureStage::SAMPLER_MIN_FILTER_NEAREST\n \tm_MagFilter \tTextureStage::SAMPLER_MAG_FILTER_NEAREST\n \tm_WrapS \tTextureStage::SAMPLER_WRAP_REPEAT\n \tm_WrapT \tTextureStage::SAMPLER_WRAP_REPEAT\n \tm_BorderColor \t(0, 0, 0, 0)\n \tm_LodBias \t0.0\n \tm_MinLod \t0\n \tm_MaxLod \t0xFFFFFFFF\n ",0]
,["Pages/Page_54693545.html#Anchor_54693568_シャドウテクスチャ用の設定（SetShadow()）","Shadow Texture Settings (SetShadow) ",
"Shadow Texture Settings (SetShadow)\nYou can call the SetShadow function to change all members to their values for shadow textures.\nTable 4-9. Sampler Settings for Shadow Textures\n \tMember \tSetting Value\n \tm_MinFilter \tTextureStage::SAMPLER_MIN_FILTER_LINEAR\n \tm_MagFilter \tTextureStage::SAMPLER_MAG_FILTER_LINEAR\n \tm_WrapS \tTextureStage::SAMPLER_WRAP_CLAMP_TO_BORDER\n \tm_WrapT \tTextureStage::SAMPLER_WRAP_CLAMP_TO_BORDER\n \tm_BorderColor \t(0, 0, 0, 0)\n \tm_LodBias \t0.0\n \tm_MinLod \t0\n \tm_MaxLod \t0xFFFFFFFF\n Warning:\n Do not change the filter, wrapping mode, and LOD settings.\n",0]
,["Pages/Page_54693545.html#Anchor_54693568_シャドウキューブテクスチャ用の設定（SetShadowCube()）","Shadow Cube Texture Settings (SetShadowCube) ",
"Shadow Cube Texture Settings (SetShadowCube)\nYou can call the SetShadowCube function to change all members to their values for shadow cube map textures.\nTable 4-10. Sampler Settings for Shadow Cube Map Textures\n \tMember \tSetting Value\n \tm_MinFilter \tTextureStage::SAMPLER_MIN_FILTER_LINEAR\n \tm_MagFilter \tTextureStage::SAMPLER_MIN_FILTER_LINEAR\n \tm_WrapS \tTextureStage::SAMPLER_WRAP_CLAMP_TO_EDGE\n \tm_WrapT \tTextureStage::SAMPLER_WRAP_CLAMP_TO_EDGE\n \tm_BorderColor \t(0, 0, 0, 0)\n \tm_LodBias \t0.0\n \tm_MinLod \t0\n \tm_MaxLod \t0xFFFFFFFF\n Warning:\n Do not change the filter, wrapping mode, and LOD settings.\n",0]
,["Pages/Page_54693545.html#Anchor_54693568_ガステクスチャ用の設定（SetGas()）","Gas Texture Settings (SetGas) ",
"Gas Texture Settings (SetGas)\nYou can call the SetGas function to change all members to their values for gas textures.\nTable 4-11. Sampler Settings for Gas Textures\n \tMember \tSetting Value\n \tm_MinFilter \tTextureStage::SAMPLER_MIN_FILTER_NEAREST\n \tm_MagFilter \tTextureStage::SAMPLER_MAG_FILTER_NEAREST\n \tm_WrapS \tTextureStage::SAMPLER_WRAP_CLAMP_TO_EDGE\n \tm_WrapT \tTextureStage::SAMPLER_WRAP_CLAMP_TO_EDGE\n \tm_BorderColor \t(0, 0, 0, 0)\n \tm_LodBias \t0.0\n \tm_MinLod \t0\n \tm_MaxLod \t0xFFFFFFFF\n Warning:\n Do not change the filter, wrapping mode, and LOD settings.\n",0]
,["Pages/Page_54693545.html#PageId_54693569","4.4. Procedural Texture Stage ",
"\n4.4. Procedural Texture Stage\nThe procedural texture stage handles settings related to procedural textures. The nn::gd::ProceduralTextureStage class defines functions that configure these settings.\n",0]
,["Pages/Page_54693545.html#PageId_54693570","4.4.1. Setting Procedural Texture Parameters ",
"The following figure gives an overview of the settings made in the procedural texture stage.\nFigure 4-4. Overview of the Procedural Texture Stage\n4.4.1. Setting Procedural Texture Parameters\nFunctions have been provided for setting each of the procedural texture parameters.\nCode 4-31. Functions for Setting Procedural Texture Parameters\n class nn::gd::ProceduralTextureStage\n{\n static void SetRgbMap(UvMap rgbMap);\n static void SetAlphaSeparate(gdBool alphaSeparate);\n static void SetAlphaMap(UvMap alphaMap);\n static void SetClampUV(Clamp u, Clamp v);\n static void SetShiftUV(Shift u, Shift v);\n static void SetMinFilter(MinFilter minFilter);\n static void SetTexBias(f32 texBias);\n static void SetNoiseEnable(gdBool noiseEnable);\n static void SetNoiseUV(f32 noiseU[3], f32 noiseV[3]);\n static void SetTexWidth(u8 texWidth);\n static void SetTexOffset(u8 texOffset);\n};\nThe following table shows which parameters are set by each function. For more information about how to specify arguments and what effect these parameters have, see the CTR-SDK API Reference and other documentation.\nTable 4-12. Parameters Set by Functions in the Procedural Texture Stage\n \tFunction \tParameters\n \tSetRgbMap \tThe G function for RGB mapping. Default value: UV_MAP_U\n \tSetAlphaSeparate \tWhether to enable independent alpha mode. Default value: GD_FALSE\n \tSetAlphaMap \tThe G function for alpha mapping. Default value: UV_MAP_U\n \tSetClampUV \tThe clamp mode. Default value: CLAMP_TO_ZERO\n \tSetShiftUV \tThe shift mode. Default value: SHIFT_NONE\n \tSetMinFilter \tThe minification filter mode. Default value: MIN_FILTER_LINEAR\n \tSetTexBias \tThe LOD bias. Default value: 0.5\n \tSetNoiseEnable \tWhether to enable noise. Default value: GD_FALSE\n \tSetNoiseUV \tNoise parameters. Default values: F=0.0, P=0.0, and A=0.0 for both U and V\n \tSetTexWidth \tColor lookup table width. Default value: 0\n \tSetTexOffset \tColor lookup table offset. Default value: 0\n Note:\n The SetTexBias and SetNoiseUV functions convert the values passed as arguments into the format used when setting GPU registers.\n",0]
,["Pages/Page_54693545.html#PageId_54693571","4.4.2. Loading Lookup Tables Used by the Procedural Texture Stage ",
"\n4.4.2. Loading Lookup Tables Used by the Procedural Texture Stage\nThere are two ways to load F functions (for RGB mapping and alpha mapping), noise modulation data, color lookup tables, and other lookup tables used by procedural textures: The first is to use functions (with the Float suffix) that take arrays of floating-point numbers. The second is to use functions (with the Native suffix) that take arrays of data to be written unchanged to the GPU registers. Because these are immediate functions, data is written to the 3D command buffer when they are run.\nCode 4-32. Functions for Loading Lookup Tables Used by the Procedural Texture Stage\n class nn::gd::ProceduralTextureStage\n{\n static nnResult UploadLookUpTableRgbMapFloat(\n u32 index, f32* Map, f32* MapDelta, u32 lutSize);\n static nnResult UploadLookUpTableRgbMapNative(\n u32 index, u32* Map, u32 lutSize);\n static nnResult UploadLookUpTableAlphaMapFloat(\n u32 index, f32* Map, f32* MapDelta, u32 lutSize);\n static nnResult UploadLookUpTableAlphaMapNative(\n u32 index, u32* Map, u32 lutSize);\n static nnResult UploadLookUpTableNoiseMapFloat(\n u32 index, f32* Map, f32* MapDelta, u32 lutSize);\n static nnResult UploadLookUpTableNoiseMapNative(\n u32 index, u32* Map, u32 lutSize);\n static nnResult UploadLookUpTableColorMapFloat(\n u32 index, f32** Map, f32** MapDelta, u32 lutSize);\n static nnResult UploadLookUpTableColorMapNative(\n u32 index, u32* Map, u32* MapDelta, u32 lutSize);\n};\nYou can use index and lutSize values to partially overwrite a lookup table. However, an error occurs if the total exceeds the maximum number of elements in the following table.\nTo load a lookup table with arrays of floating-point numbers, you must provide one array for the data (the Map parameter) and one for the deltas (the MapDelta parameter). However, you need eight arrays to load a color lookup table: two for each of the color's four components.\nIf you load any lookup table (except for a color lookup table) with an array that has been converted into native format, you only need to provide one array because both the data and the delta have been combined into a single element.\nThe following table shows the format of and maximum number of elements in the arrays used by each function to load a lookup table.\nTable 4-13. Functions for Loading Lookup Tables Used by the Procedural Texture Stage\n \tFunction \tLookup Table \tFormat \tMaximum Number of Elements\n \tUploadLookUpTableRgbMapFloat \tRGB mapping \tFloating-point numbers \t128\n \tUploadLookUpTableRgbMapNative \tRGB mapping \tNative \t128\n \tUploadLookUpTableAlphaMapFloat \tAlpha mapping \tFloating-point numbers \t128\n \tUploadLookUpTableAlphaMapNative \tAlpha mapping \tNative \t128\n \tUploadLookUpTableNoiseMapFloat \tNoise modulation table \tFloating-point numbers \t128\n \tUploadLookUpTableNoiseMapNative \tNoise modulation table \tNative \t128\n \tUploadLookUpTableColorMapFloat \tColor lookup table \tFloating-point numbers \t256\n \tUploadLookUpTableColorMapNative \tColor lookup table \tNative \t256\n ",0]
,["Pages/Page_54693545.html#PageId_54693572","4.4.2.1. Helper Functions for Loading Lookup Tables ",
"4.4.2.1. Helper Functions for Loading Lookup Tables\nHelper functions have been provided for converting arrays of floating-point numbers into native format. Functions that load lookup tables from arrays of floating-point numbers internally convert the arrays into native format. This is expensive and not suited for runtime use. We recommend that you use some other strategy, such as providing data that has already been converted into native format or using the helper functions to convert the data when the lookup table is loaded for the first time.\nCode 4-33. Helper Functions for Loading Lookup Tables\n class nn::gd::ProceduralTextureStage::Helper\n{\n static nnResult ConvertLookUpTableDataFloatToNative(\n f32* valueData, f32* deltaData, u32 lutSize, u32* destination);\n static nnResult ConvertColorLookUpTableDataFloatToNative(\n f32** refArray, f32** deltaArray, u32 lutSize,\n u32* destRef, u32* destDelta);\n};\nFor all lookup tables except for color lookup tables, a single array of data (destination) is generated by combining the data and deltas. The original data is converted into 12-bit unsigned fixed-point numbers with 12 fractional bits in bits [11:0] of the generated data. The original deltas are converted into 12-bit signed fixed-point numbers with 11 fractional bits (and negative numbers expressed in two's complement) in bits [23:12] of the generated data.\nFor color lookup tables, a combination of data and deltas for each RGBA component (a total of eight arrays) is converted into a combination of a single color and delta value (the destRef and destDelta parameters). Although they are converted through different methods, every color and delta value that is generated has eight bits each for the alpha, blue, green, and red components, arranged respectively from the most- to the least-significant bit. Color values are converted from 8-bit signed fixed-point numbers with 7 fractional bits, between 0.0 and 1.0, into unsigned 8-bit integers between 0 and 255. Delta values between -1.0 and 1.0 are converted into 8-bit signed (two’s complement) fixed-point numbers with 7 fractional bits.\n Note:\n For more information about how these values are converted, see the 3DS Programming Manual: Advanced Graphics.\n The GD library provides utility functions for these conversions.\n",0]
,["Pages/Page_54693545.html#PageId_54693573","4.5. Lighting Stage ",
"\n4.5. Lighting Stage\nThe lighting stage configures the global lighting environment and individual lights. The nn::gd::LightingStage class defines functions that configure these settings. Applications do not need to enable or disable the lighting environment. The library manages lighting settings. The lighting environment is only enabled when lighting results are required by combiner settings.\n",0]
,["Pages/Page_54693545.html#PageId_54693574","4.5.1. Configuring the Global Lighting Environment ",
"The following diagram provides an overview of the settings made in the lighting stage. (This does not include individual lights).\nFigure 4-5. Overview of the Lighting Stage (Except for Individual Lights)\n4.5.1. Configuring the Global Lighting Environment\nFunctions have been provided for setting each of the global lighting environment parameters.\nCode 4-34. Functions for Setting Global Lighting Parameters\n class nn::gd::LightingStage\n{\n static void SetGlobalColorAmbient(u8 R, u8 G, u8 B);\n static void SetClampHightlight(gdBool value);\n static void SetFresnelSelector(FresnelSelectorType fresnelSelectorType);\n static void SetShadowSelectorTexture(\n nn::gd::TextureStage::TextureUnitId textureUnit);\n static void SetShadowPrimary(gdBool value);\n static void SetShadowSecondary(gdBool value);\n static void SetShadowAlpha(gdBool value);\n static void SetInvertShadow(gdBool value);\n static void SetBumpMode(BumpMode bumpMode, gdBool bumpRenorm);\n static void SetBumpSelectorTexture(\n nn::gd::TextureStage::TextureUnitId textureUnit);\n};\nThe following table shows which parameters are set by each function. For more information about how to specify arguments and what effect these parameters have, see the CTR-SDK API Reference and other documentation.\nTable 4-14. Parameters Configured by Functions in the Lighting Stage\n \tFunction \tParameters\n \tSetGlobalColorAmbient \tThe light source color of global ambient light. Default value: (10, 10, 10)\n \tSetClampHightlight \tSpecular light clamping. Default value: GD_TRUE\n \tSetFresnelSelector \tFresnel factor. Default value: FRESNEL_SELECTOR_TYPE_NO_FRESNEL\n \tSetShadowSelectorTexture \tThe texture unit used by shadows. Default value: TEXTURE_UNIT_0\n \tSetShadowPrimary \tEffect of shadows on the primary color. Default value: GD_FALSE\n \tSetShadowSecondary \tEffect of shadows on the secondary color. Default value: GD_FALSE\n \tSetShadowAlpha \tEffect of shadows on the alpha component. Default value: GD_FALSE\n \tSetInvertShadow \tShadow term inversion. Default value: GD_FALSE\n \tSetBumpMode \tBump-map perturbation mode and whether to regenerate the third component of normal vectors. Default values: BUMPMODE_NOT_USED, GD_FALSE\n \tSetBumpSelectorTexture \tThe texture unit used for bump mapping. Default value: TEXTURE_UNIT_0\n Note:\n SetGlobalColorAmbient is an immediate function.\n",0]
,["Pages/Page_54693545.html#PageId_54693575","4.5.2. Setting the Layer Configuration and Lookup Tables ",
"\n4.5.2. Setting the Layer Configuration and Lookup Tables\nFunctions have been provided to set the following: the layer configuration (this determines which lookup tables are used for lighting), lookup table input values, scaling of lookup table output values, distribution factors, and reflections.\nCode 4-35. Functions for Setting Lookup Table-Related Parameters\n class nn::gd::LightingStage\n{\n static void SetLayerConfiguration(LayerConfiguration layerConfiguration);\n static void SetLookUpTableInputValue(\n LookUpTableId lutId, LookUpTableInputValue lutInputValue);\n static void SetLookUpTableAbsInput(LookUpTableId lutId, gdBool value);\n static void SetLookUpTableOutputScaling(\n LookUpTableId lutId, LookUpTableOutputScaleValue outputScalingValue);\n static void EnableLookUpTableD0(gdBool value);\n static void EnableLookUpTableD1(gdBool value);\n static void EnableLookUpTableReflection(gdBool value);\n};\nThe following table shows which parameters are set by each function. For more information about how to specify arguments and what effect these parameters have, see the CTR-SDK API Reference and other documentation.\nTable 4-15. Lookup Table-Related Parameters Set by Functions in the Lighting Stage\n \tFunction \tParameters\n \tSetLayerConfiguration \tThe layer configuration. Default value: LAYER_CONFIGURATION_0\n \tSetLookUpTableInputValue \tLookup table input value. Default value: INPUT_VALUE_NH for all lookup tables\n \tSetLookUpTableAbsInput \tWhether to take the absolute value of lookup table input value. Default value: GD_FALSE for all lookup tables\n \tSetLookUpTableOutputScaling \tHow to scale lookup table output values. Default value: OUTPUT_SCALE_VALUE_1 for all lookup tables\n \tEnableLookUpTableD0 \tWhether to apply lookup tables to distribution factor 0. Default value: GD_FALSE\n \tEnableLookUpTableD1 \tWhether to apply lookup tables to distribution factor 1. Default value: GD_FALSE\n \tEnableLookUpTableReflection \tWhether to apply lookup tables to reflections. Default value: GD_FALSE\n ",0]
,["Pages/Page_54693545.html#PageId_54693576","4.5.3. Loading Lookup Tables Used by the Lighting Stage ",
"\n4.5.3. Loading Lookup Tables Used by the Lighting Stage\nThere are two functions for loading lookup tables used by lighting: UploadLookUpTableFloat, which takes arrays of floating-point numbers, and UploadLookUpTableNative, which takes arrays of data to be written unchanged to the GPU registers. Because these are immediate functions, data is written to the 3D command buffer when they are run.\nCode 4-36. Functions for Loading Lookup Tables Used by the Lighting Stage\n class nn::gd::LightingStage\n{\n static nnResult UploadLookUpTableFloat(\n LookUpTableUploadId lutID, u32 lutStartIndex,\n const f32* valueData, const f32* deltaData, u32 dataCount);\n static nnResult UploadLookUpTableNative(\n LookUpTableUploadId lutID, u32 lutStartIndex,\n const u32* data, u32 dataCount);\n class Helper\n {\n static nnResult ConvertLookUpTableDataFloatToNative(\n const f32* valueData, const f32* deltaData, u32 dataCount,\n u32* __restrict destination);\n };\n};\nLighting uses lookup tables with 256 elements. You can use lutStartIndex and dataCount values to partially overwrite a lookup table, but an error will occur if the total exceeds 256.\nA lookup table's structure differs depending upon whether its input values are converted into absolute values. Note that the input values corresponding to the 128th and 129th elements of a lookup table will be discontinuous (they will jump from 0.9922 to -1.0) if the lookup table input values are not converted into absolute values (if they are between -1.0 and 1.0).\nTo load a lookup table with arrays of floating-point numbers, you need to provide one array for the data (the valueData parameter) and one array for the deltas (the deltaData parameter).\nIf you load any lookup table with an array that has been converted into native format, you only need to provide one array because both the data and the delta have been combined into a single element.\nHelper functions have been provided for converting arrays of floating-point numbers into native format. These helper functions generate a single array of data (the destination parameter) by combining the data and deltas. The original data is converted into 12-bit unsigned fixed-point numbers with 12 fractional bits. The original deltas are converted into 12-bit signed fixed-point numbers with 11 fractional bits (which are absolute values). These are stored in bits [11:0] and [23:12] of the generated data, respectively.\n Note:\n For more information about how these values are converted, see the 3DS Programming Manual: Advanced Graphics.\n The GD library provides utility functions for these conversions.\n",0]
,["Pages/Page_54693545.html#PageId_54693577","4.5.4. Lights ",
"\n4.5.4. Lights\nThe 3DS hardware supports lighting with 8 lights. The GD library represents individual lights as separate instances of the nn::gd::Light class. The nn::gd::LightingStage class contains eight Light objects that are created when the library is initialized. As a result, applications do not need to create Light objects. Access them using the members defined by the lighting stage. For example, you could call nn::gd::LightingStage::light[0].EnableLight(GD_TRUE).\n",0]
,["Pages/Page_54693545.html#PageId_54693578","4.5.4.1. Light Settings ",
"The following diagram provides an overview of the settings handled by Light objects.\nFigure 4-6. Overview of Light Objects\n4.5.4.1. Light Settings\nFunctions have been provided for setting each of the light parameters.\nCode 4-37. Functions for Setting Light Parameters\n class nn::gd::Light\n{\n void EnableLight(gdBool value);\n void SetLightType(SourceType sourceType);\n void EnableTwoSideDiffuse(gdBool value);\n void EnableShadowed(gdBool value);\n void EnableGeomFactor0(gdBool value);\n void EnableGeomFactor1(gdBool value);\n void SetColorAmbient(u8 R, u8 G, u8 B);\n void SetColorDiffuse(u8 R, u8 G, u8 B);\n void SetColorSpecular0(u8 R, u8 G, u8 B);\n void SetColorSpecular1(u8 R, u8 G, u8 B);\n void SetPosition(f32 X, f32 Y, f32 Z);\n void SetDirection(f32 X, f32 Y, f32 Z);\n void EnableSpotLight(gdBool value);\n void SetSpotDirection(f32 X, f32 Y, f32 Z);\n void EnableDistanceAttenuation(gdBool value);\n void SetDistanceAttenuationScaleBias(\n f32 attenuationScale, f32 attenuationBias);\n nnResult UploadLookUpTableFloat(\n LightLookUpTableUploadId lightLutID, u32 lutStartIndex,\n const f32* valueData, const f32* deltaData, u32 dataCount);\n nnResult UploadLookUpTableNative(\n LightLookUpTableUploadId lightLutID, u32 lutStartIndex,\n const u32* data, u32 dataCount);\n};\nThe following table shows which parameters are set by each function. For more information about how to specify arguments and what effect these parameters have, see the CTR-SDK API Reference and other documentation.\nTable 4-16. Parameters Set by Light Object Functions\n \tFunction \tParameters\n \tEnableLight \tWhether to enable light. Default value: GD_FALSE\n \tSetLightType \tType of light source. Default value: SOURCE_TYPE_POINT\n \tEnableTwoSideDiffuse \tWhether to enable two-sided lighting. Default value: GD_FALSE\n \tEnableShadowed \tWhether to enable shadow attenuation. Default value: GD_FALSE\n \tEnableGeomFactor0 \tWhether to enable geometry factor 0. Default value: GD_FALSE\n \tEnableGeomFactor1 \tWhether to enable geometry factor 1. Default value: GD_FALSE\n \tSetColorAmbient \tAmbient light color. Default value: (0, 0, 0)\n \tSetColorDiffuse \tDiffuse light color. Default value: (255, 255, 255)\n \tSetColorSpecular0 \tColor of specular light 0. Default value: (255, 255, 255)\n \tSetColorSpecular1 \tColor of specular light 1. Default value: (255, 255, 255)\n \tSetPosition \tPoint light source position. Default value: (0.0, 0.0, 0.0)\n \tSetDirection \tDirectional light orientation. No default value.\n \tEnableSpotLight \tWhether to enable spotlights. Default value: GD_FALSE\n \tSetSpotDirection \tSpotlight direction. Default value: (0.0, 0.0, 0.0)\n \tEnableDistanceAttenuation \tWhether to enable distance attenuation. Default value: GD_FALSE\n \tSetDistanceAttenuationScaleBias \t Scale and bias for distance attenuation. Default values: 0.0, 0.0\n A utility function (Utils::ConvertStartEndToScaleBias) has been provided to find the appropriate setting values from both a starting and ending distance.\n Lookup tables are loaded in the same way as they are for the lighting stage, although they are specified differently. The lookup tables used by distance attenuation (DA) must be created to accept input values between 0.0 and 1.0.\n Note:\n Every function that configures lights is an immediate function, except for EnableLight, EnableShadowed, EnableSpotLight, and EnableDistanceAttenuation.\n",0]
,["Pages/Page_54693545.html#PageId_54693579","4.6. Rasterizer Stage ",
"\n4.6. Rasterizer Stage\nThe rasterizer stage configures the process of rasterizing pixels that are output to the fragment pipeline. The nn::gd::RasterizerStage class defines functions that configure these settings.\n",0]
,["Pages/Page_54693545.html#PageId_54693580","4.6.1. Culling Settings ",
"The following diagram provides an overview of the settings made in the rasterizer stage.\nFigure 4-7. Overview of the Rasterizer Stage\n4.6.1. Culling Settings\nUse the nn::gd::RasterizerStage::SetCulling function to configure culling.\nCode 4-38. Function for Configuring Culling\n static void nn::gd::RasterizerStage::SetCulling(\n nn::gd::RasterizerStage::Culling culling);\nSpecify the culling method in the culling parameter. Clockwise culling (CULLING_CLOCKWISE) is specified by default.\n",0]
,["Pages/Page_54693545.html#PageId_54693581","4.6.2. Configuring the Viewport ",
"\n4.6.2. Configuring the Viewport\nUse the nn::gd::RasterizerStage::SetViewport function to configure the viewport.\nCode 4-39. Functions for Configuring the Viewport\n class nn::gd::Viewport\n{\n u32 m_X;\n u32 m_Y;\n u32 m_Width;\n u32 m_Height;\n Viewport();\n Viewport(u32 x, u32 y, u32 width, u32 height);\n void Set(u32 x, u32 y, u32 width, u32 height);\n};\nstatic void nn::gd::RasterizerStage::SetViewport(\n nn::gd::Viewport& viewport);\nThe nn::gd::Viewport class defines the viewport region. Note that the viewport has a different orientation than the LCDs. Viewport(0, 0, 240, 320) is set by default.\n",0]
,["Pages/Page_54693545.html#PageId_54693582","4.6.3. Clipping Settings ",
"\n4.6.3. Clipping Settings\nUse the following functions to configure clipping.\nCode 4-40. Functions for Configuring Clipping\n static void nn::gd::RasterizerStage::EnableClippingPlane(gdBool value);\nstatic void nn::gd::RasterizerStage::SetClippingPlane(\n f32 param1, f32 param2, f32 param3, f32 param4);\nBy default, clipping is disabled (with GD_FALSE) and all parameters are set to 0.\n",0]
,["Pages/Page_54693545.html#PageId_54693583","4.6.4. Scissor Test Settings ",
"\n4.6.4. Scissor Test Settings\nUse the following functions to configure the scissor test.\nCode 4-41. Functions for Configuring the Scissor Test\n static void nn::gd::RasterizerStage::EnableScissor(gdBool value);\nstatic void nn::gd::RasterizerStage::SetScissor(\n nn::gd::Viewport& area);\nBy default, the scissor test is disabled (GD_FALSE) and Viewport(0, 0, 0, 0) is set as the scissor box.\n",0]
,["Pages/Page_54693545.html#PageId_54693584","4.6.5. Early Depth Test Settings ",
"\n4.6.5. Early Depth Test Settings\nUse the following functions to configure the early depth test.\nCode 4-42. Functions for Configuring the Early Depth Test\n static void nn::gd::RasterizerStage::SetEarlyDepthTest(\n gdBool enable, nn::gd::RasterizerStage::EarlyDepthFunction func);\nstatic void nn::gd::RasterizerStage::ClearEarlyDepth(f32 depthClearValue);\nBy default, the early depth test is disabled (GD_FALSE and EARLYDEPTH_FUNCTION_LESS) and 1.0 is set as the depth value to use for clearing.\nBlock-32 mode must be set as the block mode when you use the early depth test. For more information, see Block Mode Settings in the 3DS Programming Manual: Advanced Graphics.\n",0]
,["Pages/Page_54693545.html#PageId_54693585","4.7. Combiner Stage ",
"\n4.7. Combiner Stage\nThe combiner stage configures (texture) combiners and combiner buffers. The nn::gd::CombinerStage class defines functions that configure these settings.\n",0]
,["Pages/Page_54693545.html#PageId_54693586","4.7.1. Combiner Settings ",
"The following diagram provides an overview of the settings made in the combiner stage.\nFigure 4-8. Overview of the Combiner Stage\n4.7.1. Combiner Settings\nThe combiner stage creates a state (CombinerState) object that consolidates all combiner and combiner buffer settings. By switching this CombinerState object, you can change all combiner-related settings in one operation, except for the constant color.\nCombinerState objects are created from a descriptor class (CombinerDescription) that defines operands and other settings.\nCode 4-43. Creating, Releasing, and Applying Combiner Settings to the Pipeline\n static nnResult nn::gd::CombinerStage::CreateTextureCombinerState(\n const nn::gd::CombinerDescription* description,\n nn::gd::CombinerState** textureCombinerState);\nstatic nnResult nn::gd::CombinerStage::ReleaseTextureCombinerState(\n nn::gd::CombinerState* textureCombinerState);\nstatic nnResult nn::gd::CombinerStage::SetTextureCombinerState(\n nn::gd::CombinerState* textureCombinerState);\nCombiner settings are configured in the descriptor object (CombinerDescription) specified by the description parameter in advance. For more information about the descriptor class settings, see 4.9.5.1. Descriptor Class for BlendState Objects.\nYou can use the nn::gd::CombinerStage::SetTextureCombinerState function to set a created CombinerState object in the combiner stage.\nUse the nn::gd::CombinerStage::ReleaseTextureCombinerStage function to release a state object that is no longer needed. This also deallocates memory regions that were allocated internally when the CombinerState object was created.\nUse the following function to set the constant color. You must set a constant color if it is used by combiner settings, because the constant color is undefined by default.\nCode 4-44. Setting the Constant Color\n static nnResult nn::gd::CombinerStage::SetTextureCombinerUnitConstantColor(\n nn::gd::CombinerStage::UnitId unit,\n u8 colorR, u8 colorG, u8 colorB, u8 colorA);\n",0]
,["Pages/Page_54693545.html#PageId_54693587","4.7.1.1. Descriptor Class for CombinerState Objects ",
"Because this is an immediate function, 3D commands are accumulated in the command buffer when it is run.\n4.7.1.1. Descriptor Class for CombinerState Objects\nThis section introduces the members of the descriptor (CombinerDescription) class for CombinerState objects and provides setting-related cautions and information.\nThe CombinerDescription class contains settings for six combiner units. Because you will not necessarily configure every unit, this class is designed with member functions that set member variables for a specified unit.\nCode 4-45. Definition of the CombinerDescription Class\n class nn::gd::CombinerDescription\n{\n struct BufferInput\n {\n nn::gd::CombinerStage::BufferInput m_Rgb1;\n nn::gd::CombinerStage::BufferInput m_Rgb2;\n nn::gd::CombinerStage::BufferInput m_Rgb3;\n nn::gd::CombinerStage::BufferInput m_Rgb4;\n nn::gd::CombinerStage::BufferInput m_Alpha1;\n nn::gd::CombinerStage::BufferInput m_Alpha2;\n nn::gd::CombinerStage::BufferInput m_Alpha3;\n nn::gd::CombinerStage::BufferInput m_Alpha4;\n };\n struct BufferInput m_BufferInput;\n void SetSourceRGB(\n nn::gd::CombinerStage::UnitId unit,\n nn::gd::CombinerStage::Source sourceRGB1,\n nn::gd::CombinerStage::Source sourceRGB2,\n nn::gd::CombinerStage::Source sourceRGB3);\n void SetOperandRGB(\n nn::gd::CombinerStage::UnitId unit,\n nn::gd::CombinerStage::OperandRgb opRGB1,\n nn::gd::CombinerStage::OperandRgb opRGB2,\n nn::gd::CombinerStage::OperandRgb opRGB3);\n void SetSourceAlpha(\n nn::gd::CombinerStage::UnitId unit,\n nn::gd::CombinerStage::Source sourceA1,\n nn::gd::CombinerStage::Source sourceA2,\n nn::gd::CombinerStage::Source sourceA3);\n void SetOperandAlpha(\n nn::gd::CombinerStage::UnitId unit,\n nn::gd::CombinerStage::OperandAlpha opA1,\n nn::gd::CombinerStage::OperandAlpha opA2,\n nn::gd::CombinerStage::OperandAlpha opA3);\n void SetCombineRGB(\n nn::gd::CombinerStage::UnitId unit,\n nn::gd::CombinerStage::CombineRgb combineRgb);\n void SetCombineAlpha(\n nn::gd::CombinerStage::UnitId unit,\n nn::gd::CombinerStage::CombineAlpha combineAlpha);\n void SetScaleRGB(\n nn::gd::CombinerStage::UnitId unit,\n nn::gd::CombinerStage::Scale scaleRgb);\n void SetScaleAlpha(\n nn::gd::CombinerStage::UnitId unit,\n nn::gd::CombinerStage::Scale scaleAlpha);\n void SetBufferColor(u8 colorR, u8 colorG, u8 colorB, u8 colorA);\n void SetCombinerInUse(nn::gd::CombinerStage::UnitId unit, gdBool mode);\n CombinerDescription();\n void ToDefault();\n};\n",0]
,["Pages/Page_54693545.html#Anchor_54693587_入力ソースの設定（SetSourceRGB()、SetSourceAlpha()）","Input Source Settings (SetSourceRGB, SetSourceAlpha) ",
"Input Source Settings (SetSourceRGB, SetSourceAlpha)\nThese functions set input sources 0 to 2 for the specified combiner unit.\n",0]
,["Pages/Page_54693545.html#Anchor_54693587_オペランドの設定（SetOperandRGB()、SetOperandAlpha()）","Operand Settings (SetOperandRGB, SetOperandAlpha) ",
"Operand Settings (SetOperandRGB, SetOperandAlpha)\nThese functions set operands 0 to 2 for the specified combiner unit.\n",0]
,["Pages/Page_54693545.html#Anchor_54693587_コンバイナ関数の設定（SetCombineRGB()、SetCombineAlpha()）","Combiner Function Settings (SetCombineRGB, SetCombineAlpha) ",
"Combiner Function Settings (SetCombineRGB, SetCombineAlpha)\nThese functions set the combiner function for the specified combiner unit.\n",0]
,["Pages/Page_54693545.html#Anchor_54693587_スケーリング値の設定（SetScaleRGB()、SetScaleAlpha()）","Scaling Value Settings (SetScaleRGB, SetScaleAlpha) ",
"Scaling Value Settings (SetScaleRGB, SetScaleAlpha)\nThese functions set the scaling values for the specified combiner unit.\n",0]
,["Pages/Page_54693545.html#Anchor_54693587_コンバイナバッファ0の定数カラーの設定（SetBufferColor()）","Constant Color for Combiner Buffer 0 Setting (SetBufferColor) ",
"Constant Color for Combiner Buffer 0 Setting (SetBufferColor)\nThis function sets the constant color for combiner buffer 0.\n",0]
,["Pages/Page_54693545.html#Anchor_54693587_コンバイナバッファの入力ソース（m_BufferInput）","Combiner Buffer Input Source (m_BufferInput) ",
"Combiner Buffer Input Source (m_BufferInput)\nSets the input source for combiner buffers 1 to 4. No setter functions are defined for it.\n",0]
,["Pages/Page_54693545.html#Anchor_54693587_コンバイナユニットの使用設定（SetCombinerInUse()）","Configuring How Combiner Units Are Used (SetCombinerInUse) ",
"Configuring How Combiner Units Are Used (SetCombinerInUse)\nThis function either enables (GD_TRUE) or disables (GD_FALSE) the settings for the specified combiner unit. If you do not enable settings for a combiner unit, it uses its default settings.\n",0]
,["Pages/Page_54693545.html#Anchor_54693587_デフォルトの設定（ToDefault()）","Default Settings (ToDefault) ",
"Default Settings (ToDefault)\nYou can call the ToDefault function to change all members to their default values. This is also done in the constructor.\nTable 4-17. Default Combiner Settings\n \tMember \tDefault Settings\n \t SetSourceRGB\n SetSourceAlpha\n \t SOURCE_CONSTANT (all input sources for unit 0)\n SOURCE_PREVIOUS (all input sources for units 1 to 5)\n \tSetOperandRGB \tOPERAND_RGB_SRC_COLOR (all operands)\n \tSetOperandAlpha \tOPERAND_ALPHA_SRC_ALPHA (all operands)\n \tSetCombineRGB \tCOMBINE_RGB_REPLACE (all units)\n \tSetCombineAlpha \tCOMBINE_ALPHA_REPLACE (all units)\n \t SetScaleRGB\n SetScaleAlpha\n \tSCALE_1 (all units)\n \tSetBufferColor \t(0,0,0,0)\n \tm_BufferInput \tINPUT_PREVIOUS_BUFFER (all inputs)\n \tSetCombinerInUse \tGD_FALSE (all units)\n",0]
,["Pages/Page_54693545.html#PageId_54693588","4.8. Fog Stage ",
"\n4.8. Fog Stage\nThe fog stage handles all settings related to rendering fog and gas. The nn::gd::FogStage class defines functions that configure these settings.\n",0]
,["Pages/Page_54693545.html#PageId_54693589","4.8.1. Fog Mode ",
"The following diagram provides an overview of the settings made in the fog stage.\nFigure 4-9. Overview of the Fog Stage\n4.8.1. Fog Mode\nUse the nn::gd::FogStage::SetGasFogMode function to switch the mode for GPU fog features.\nCode 4-46. Switching the Fog Mode\n static void nn::gd::FogStage::SetGasFogMode(nn::gd::FogStage::Mode mode);\nYou can specify one of the following values in the mode parameter.\nTable 4-18. Specifying the Fog Mode\n \tDefinition \tDescription\n \tFogStage::MODE_NONE \tDo not use fog features. (Default.)\n \tFogStage::MODE_FOG \tUse fog features normally in fog mode.\n \tFogStage::MODE_GAS \tUse fog features in gas mode.\n ",0]
,["Pages/Page_54693545.html#PageId_54693590","4.8.2. Fog Settings ",
"\n4.8.2. Fog Settings\nFunctions have been provided for setting each of the fog parameters.\nCode 4-47. Functions for Setting Fog Parameters\n class nn::gd::FogStage\n{\n static void SetFogColor(u8 R, u8 G, u8 B);\n static void SetFogZFlip(gdBool zFlip);\n};\nThe following table shows which parameters are set by each function. For more information about how to specify arguments and what effect these parameters have, see the CTR-SDK API Reference and other documentation.\nTable 4-19. Parameters Configured by Functions in the Fog Stage\n \tFunction \tParameters\n \tSetFogColor \tFog color. Default value: (0, 0, 0)\n \tSetFogZFlip \tWhether to invert input depth value. Default value: GD_FALSE\n Note:\n For more information about how to load lookup tables, see 4.8.4. Loading Lookup Tables Used by the Fog Stage.\n",0]
,["Pages/Page_54693545.html#PageId_54693591","4.8.3. Gas Settings ",
"\n4.8.3. Gas Settings\nFunctions have been provided for setting each of the gas parameters.\nCode 4-48. Functions for Setting Gas Parameters\n class nn::gd::FogStage\n{\n static void SetGasShadingDensity(\n GasShadingDensitySource shadingDensitySrc);\n static void SetGasAttenuation(f32 attenuation);\n static void SetGasAccumulationMax(f32 maxAccumulation);\n static void SetGasAutoAccumulation(gdBool enableAutoAccumulation);\n static void SetGasLightXY(\n u8 lightMinimum, u8 lightMaximum, u8 lightAttenuation);\n static void SetGasLightZ(\n u8 scattMinimum, u8 scattMaximum, u8 scattAttenuation, u8 lz);\n static void SetGasDeltaZ(f32 deltaZ);\n static void SetGasLightColorLutInput(\n GasColorLookUpTableInput colorLutInput);\n};\nThe following table shows which parameters are set by each function. For more information about how to specify arguments and what effect these parameters have, see the CTR-SDK API Reference and other documentation.\nTable 4-20. Gas Parameters Configured by Functions in the Fog Stage\n \tFunction \tParameters\n \tSetGasShadingDensity \tDensity information used for shading. Default value: GAS_SHADING_DENSITY_SOURCE_PLAIN\n \tSetGasAttenuation \tDensity attenuation coefficient to use when calculating alpha value for shading. Default value: 1.0\n \tSetGasAccumulationMax \tThe reciprocal of the maximum density information value. Default value: 1.0\n \tSetGasAutoAccumulation \tWhether to automatically calculate the reciprocal of the maximum density information value. Default value: GD_FALSE\n \tSetGasLightXY \tFactors used to control planar shading: the minimum intensity, maximum intensity, and density attenuation. Default value: (0,0,0)\n \tSetGasLightZ \tFactors used to control view shading: the minimum intensity, maximum intensity, density attenuation, and effect in the view direction. Default value: (0,0,0,0)\n \tSetGasDeltaZ \tAttenuation coefficient in the depth direction. Default value: 10.0\n \tSetGasLightColorLutInput \tInput value to the shading lookup table. Default value: GAS_COLOR_INPUT_LIGHT_FACTOR\n Note:\n For more information about how to load lookup tables, see 4.8.4. Loading Lookup Tables Used by the Fog Stage.\n",0]
,["Pages/Page_54693545.html#PageId_54693592","4.8.4. Loading Lookup Tables Used by the Fog Stage ",
"\n4.8.4. Loading Lookup Tables Used by the Fog Stage\nThere are two ways to load the lookup tables used for coefficients and shading in the fog stage: The first is through functions (with the Float suffix) that accept arrays of floating-point numbers. The second is through functions (with the Native suffix) that accept arrays of data to be written unchanged to the GPU registers. Because these are immediate functions, data is written to the 3D command buffer when they are run.\nCode 4-49. Functions for Loading Lookup Tables Used by the Fog Stage\n class nn::gd::FogStage\n{\n static nnResult UploadFogLookUpTableNative(\n u32 lutStartIndex, u32* data, u32 countData);\n static nnResult UploadFogLookUpTableFloat(\n u32 lutStartIndex, const f32* dataValue, const f32* dataDelta,\n u32 countData);\n static nnResult UploadGasLookUpTableNative(\n u32 lutStartIndex, u32* data, u32 countData);\n static nnResult UploadGasLookUpTableFloat(\n u32 lutStartIndex, f32* dataValue, f32* dataDelta, u32 countData);\n};\nYou can use lutStartindex and countData values to partially overwrite a lookup table. However, an error occurs if the total exceeds the maximum number of elements in the following table.\nTo load a lookup table using arrays of floating-point numbers, you must provide one array for the data (the dataValue parameter) and one for the deltas (the dataDelta parameter). However, note that because each element in the shading lookup table is generated from three elements in these source arrays (which store the color components in RGB order), dataValue and dataDelta must have three times the number of elements specified in countData.\nIf you load any lookup table using an array that has been converted into native format, you only need to provide one array because both the data and the delta have been combined into a single element. However, the native and floating-point arrays used to load the shading lookup table have different maximum sizes.\nThe following table shows the format of and maximum number of elements in the arrays used by each function to load a lookup table.\nTable 4-21. Functions for Loading Lookup Tables Used by the Fog Stage\n \tFunction \tLookup Table \tFormat \tMaximum Number of Elements\n \tUploadFogLookUpTableFloat \tFog Coefficients \tFloating-point numbers \t128\n \tUploadFogLookUpTableNative \tFog Coefficients \tNative \t128\n \tUploadGasLookUpTableFloat \tShading \tFloating-point numbers \t8\n \tUploadGasLookUpTableNative \tShading \tNative \t16\n ",0]
,["Pages/Page_54693545.html#PageId_54693593","4.8.4.1. Helper Functions for Loading Lookup Tables ",
"4.8.4.1. Helper Functions for Loading Lookup Tables\nHelper functions have been provided for converting arrays of floating-point numbers into native format. Functions that load lookup tables from arrays of floating-point numbers internally convert the arrays into native format. This is expensive and not suited for runtime use. We recommend that you use some other strategy, such as providing data that has already been converted into native format or using the helper functions to convert the data when the lookup table is loaded for the first time.\nCode 4-50. Helper Functions for Loading Lookup Tables\n class nn::gd::FogStage::Helper\n{\n static nnResult ConvertFogLookUpTableDataFloatToNative(\n const f32* dataValue, const f32* dataDelta, u32 countData,\n u32 *__restrict dest);\n static nnResult ConvertGasLookUpTableDataFloatToNative(\n const f32* dataValue, const f32* dataDelta, u32 countData,\n u32 *__restrict dest);\n};\nBoth helper functions provided by the fog stage generate a single array (the dest parameter) from a data array and a delta array.\nEach data element is converted into an 11-bit unsigned fixed-point number with 11 fractional bits. Each delta is converted into a 13-bit signed (two's complement) fixed-point number with 11 fractional bits. These are stored in bits [23:13] and [12:0], respectively, of the data generated for the fog coefficient lookup table.\nData is generated for the shading lookup table in a special format. Specifically, note that the three elements of the source array (in R/G/B order) are converted into a single data value. Conversely, each element of the array dest does not contain both data and delta values. Instead, the first eight elements of the array store delta values, and the second eight elements store data values.\nThe first eight elements are generated from the deltas. The deltas for the R, G, and B components are converted into 8-bit signed integers that are packed into bits [7:0], [15:8], and [23:16] respectively.\nThe last eight elements are generated from the data. The data for the R, G, and B components are converted into 8-bit unsigned fixed-point numbers (with no fractional bits) that are packed into bits [7:0], [15:8], and [23:16] respectively.\n Note:\n For more information about how these values are converted, see the 3DS Programming Manual: Advanced Graphics.\n The GD library provides utility functions for these conversions.\n",0]
,["Pages/Page_54693545.html#PageId_54693594","4.9. Output Stage ",
"\n4.9. Output Stage\nThe output stage configures blending, the depth test, the stencil test, the target buffer, and other settings related to pixel output. The nn::gd::OutputStage class defines functions that configure these settings.\n",0]
,["Pages/Page_54693545.html#PageId_54693595","4.9.1. Fragment Operation Mode ",
"The following diagram provides an overview of the settings made in the output stage.\nFigure 4-10. Overview of the Output Stage\n4.9.1. Fragment Operation Mode\nYou can use the nn::gd::OutputStage::SetFragOpMode function to switch the fragment operation mode.\nCode 4-51. Function for Setting the Fragment Operation Mode\n static void nn::gd::OutputStage::SetFragOpMode(\n nn::gd::OutputStage::FragmentOperationMode fragOpMode);\nYou can specify one of the following values in the fragOpMode parameter.\nTable 4-22. Fragment Operation Modes\n \tDefinition \tDescription\n \tFRAGMENT_OPERATION_MODE_FRAGMENT_COLOR \tNormal fragment processing (this is the default value).\n \tFRAGMENT_OPERATION_MODE_GAS_ACCUMULATION \tFragment processing for the shadow accumulation path.\n \tFRAGMENT_OPERATION_MODE_SHADOW_MAP \tFragment processing for rendering gas density information.\n ",0]
,["Pages/Page_54693545.html#PageId_54693596","4.9.2. Framebuffers ",
"\n4.9.2. Framebuffers\nThe GD library uses RenderTarget and DepthStencilTarget objects to operate on the framebuffer (color and depth stencil buffer). Both objects are created from a single Texture2DResource object and a descriptor class (RenderTargetDescription or DepthStencilTargetDescription,). If the Texture2DResource object has mipmap data, you can use descriptor class settings to specify which levels of mipmap data to use when you create the object.\nYou can use the following functions to create, release, and apply a framebuffer object (a RenderTarget or DepthStencilTarget object) to the pipeline.\nCode 4-52. Creating, Releasing, and Applying a Framebuffer Object to the Pipeline\n static nnResult nn::gd::OutputStage::CreateRenderTarget(\n const nn::gd::Texture2DResource* texture2DResource,\n const nn::gd::RenderTargetDescription* desc,\n nn::gd::RenderTarget** renderTarget);\nstatic nnResult nn::gd::OutputStage::ReleaseRenderTarget(\n nn::gd::RenderTarget* renderTarget);\nstatic nnResult nn::gd::OutputStage::SetRenderTarget(\n const nn::gd::RenderTarget* renderTarget);\nstatic nnResult nn::gd::OutputStage::CreateDepthStencilTarget(\n const nn::gd::Texture2DResource* texture2DResource,\n const nn::gd::DepthStencilTargetDescription* desc,\n nn::gd::DepthStencilTarget** depthStencil);\nstatic nnResult nn::gd::OutputStage::ReleaseDepthStencilTarget(\n nn::gd::DepthStencilTarget* depthStencil);\nstatic nnResult nn::gd::OutputStage::SetDepthStencilTarget(\n const nn::gd::DepthStencilTarget* depthStencil);\nFunctions that have the RenderTarget suffix affect the color buffer. Functions that have the DepthStencilTarget suffix affect the depth (stencil) buffer.\nConfigure framebuffer settings in the corresponding descriptor object (RenderTargetDescription or DepthStencilTargetDescription) specified in the desc parameter. For more information about the descriptor class settings, see 4.9.2.1. Descriptor Class for Framebuffer Objects.\nOnly a limited number of pixel formats can be used by a Texture2DResource object in the framebuffer. For more information, see 3.1.1. Creating a Texture2DResource Object.\nAfter you have created a framebuffer object, you can set it as the framebuffer with the nn::gd::OutputStage::Set* functions.\n",0]
,["Pages/Page_54693545.html#PageId_54693597","4.9.2.1. Descriptor Class for Framebuffer Objects ",
"Use the nn::gd::OutputStage::Release* functions to release framebuffer objects that are no longer needed. Only management data is released when a framebuffer object is released. Texture2DResource objects are not released.\n4.9.2.1. Descriptor Class for Framebuffer Objects\nThis section introduces the members of the descriptor classes (RenderTargetDescription and DepthStencilTargetDescription) for framebuffer classes (RenderTarget and DepthStencilTarget) and provides setting-related cautions and information.\nCode 4-53. Descriptor Class Definitions for Framebuffer Objects\n class nn::gd::RenderTargetDescription\n{\n u32 m_MipLevelIndex;\n};\nclass nn::gd::DepthStencilTargetDescription\n{\n u32 m_MipLevelIndex;\n};\n",0]
,["Pages/Page_54693545.html#Anchor_54693597_ミップマップレベルのインデックス（m_MipLevelIndex）","Mipmap Level Index (m_MipLevelIndex) ",
"Mipmap Level Index (m_MipLevelIndex)\nThis is the mipmap level of the Texture2DResource object used to create a framebuffer object.\n",0]
,["Pages/Page_54693545.html#PageId_54693598","4.9.2.2. Getting Detailed Information About Framebuffer Objects ",
"\n4.9.2.2. Getting Detailed Information About Framebuffer Objects\nDetailed information (the TargetProperties class) includes framebuffer-related information that is based on the content of the Texture2DResource object specified when the framebuffer was created.\nThe RenderTarget and DepthStencilTarget objects use different functions to get detailed information, but they both do so using the TargetProperties class.\nCode 4-54. Getting Detailed Information About Framebuffer Objects\n static nnResult nn::gd::OutputStage::GetRenderTargetProperties(\n const nn::gd::RenderTarget* renderTarget,\n nn::gd::TargetProperties* properties);\nstatic nnResult nn::gd::OutputStage::GetDepthStencilTargetProperties(\n const nn::gd::DepthStencilTarget* depthStencil,\n nn::gd::TargetProperties* properties);\nAlthough the width, height, mipmap levels, and starting address of resource data may change with the mipmap levels specified when an object was created, they are generally the same as the detailed information for the Texture2DResource object.\nCode 4-55. Definition of the TargetProperties Class\n class nn::gd::TargetProperties\n{\n u32 m_Width;\n u32 m_Height;\n u32 m_PixelSize;\n nn::gd::Resource::NativeFormat m_Format;\n nn::gd::Memory::MemoryLayout m_MemLayout;\n nn::gd::Memory::MemoryLocation m_MemLocation;\n u8* m_MemAddr;\n};\n",0]
,["Pages/Page_54693545.html#PageId_54693599","4.9.2.3. Clearing the Framebuffer ",
"\n4.9.2.3. Clearing the Framebuffer\nYou can clear the color buffer and depth (stencil) buffer using a specified value.\nCode 4-56. Clearing the Framebuffer\n static nnResult nn::gd::Memory::ClearTargets(\n const nn::gd::RenderTarget* renderTarget,\n const nn::gd::DepthStencilTarget* depthStencilTarget,\n const u8 ColorRGBA[4], float depth, u8 stencil);\nThe framebuffer is actually cleared when the command list is executed because this function internally calls the nngxAddMemoryFillCommand function.\n",0]
,["Pages/Page_54693545.html#PageId_54693600","4.9.2.4. Color Mask ",
"\n4.9.2.4. Color Mask\nUsing the nn::gd::OutputStage::SetColorWriteMask function, you can change color mask settings and control what is written to the color buffer.\nCode 4-57. Function for Setting the Color Mask\n static void nn::gd::OutputStage::SetColorWriteMask(u32 mask);\nSpecify a bitwise OR of the following flags in the mask parameter.\nTable 4-23. Color Mask Flags\n \tDefinition \tDescription\n \tCOLOR_WRITE_MASK_ENABLE_RED \tEnables write operations for the red component.\n \tCOLOR_WRITE_MASK_ENABLE_GREEN \tEnables write operations for the green component.\n \tCOLOR_WRITE_MASK_ENABLE_BLUE \tEnables write operations for the blue component.\n \tCOLOR_WRITE_MASK_ENABLE_ALPHA \tEnables write operations for the alpha component.\n \tCOLOR_WRITE_MASK_ENABLE_ALL \tEnables write operations for all components. (This value is the default.)\n",0]
,["Pages/Page_54693545.html#PageId_54693601","4.9.3. Depth and Stencil Tests ",
"\n4.9.3. Depth and Stencil Tests\nBy creating a state (DepthStencilState) object and then switching the DepthStencilState object that is configured in the output stage, you can change all settings related to the depth and stencil test in one operation.\nYou can create, release, and apply a DepthStencilState object to the pipeline using the following functions.\nCode 4-58. Creating, Releasing, and Applying a DepthStencilState Object to the Pipeline\n static nnResult nn::gd::OutputStage::CreateDepthStencilState(\n const nn::gd::DepthStencilStateDescription* desc,\n nn::gd::DepthStencilState** DepthStencilState);\nstatic nnResult nn::gd::OutputStage::ReleaseDepthStencilState(\n nn::gd::DepthStencilState* DepthStencilState);\nstatic nnResult nn::gd::OutputStage::SetDepthStencilState(\n const nn::gd::DepthStencilState* state, u8 stencilRef);\nConfigure the depth and stencil test in a descriptor class (DepthStencilStateDescription) passed in the desc parameter. For more information about the descriptor class settings, see 4.9.3.1. Descriptor Class for DepthStencilState Objects.\nAfter you have created a state object, you can apply it to the pipeline using the nn::gd::OutputStage::SetDepthStencilState function. Specify the reference value to use in the stencil test in the stencilRef parameter.\n",0]
,["Pages/Page_54693545.html#PageId_54693602","4.9.3.1. Descriptor Class for DepthStencilState Objects ",
"Use the nn::gd::OutputStage::ReleaseDepthStencilState function to release a state object that is no longer needed. This also releases memory regions that were allocated internally when the BlendState object was created as a state object.\n4.9.3.1. Descriptor Class for DepthStencilState Objects\nThis section introduces the members of the descriptor (DepthStencilStateDescription) class for DepthStencilState objects and provides setting-related cautions and information.\nCode 4-59. Definition of the DepthStencilStateDescription Class\n class nn::gd::DepthStencilStateDescription\n{\n gdBool m_DepthEnable;\n gdBool m_StencilEnable;\n u8 m_StencilWriteMask;\n u8 m_StencilReadMask;\n nn::gd::OutputStage::DepthWriteMask m_DepthMask;\n nn::gd::OutputStage::DepthFunction m_DepthFunc;\n nn::gd::OutputStage::StencilFunction m_StencilFunc;\n nn::gd::OutputStage::StencilOperation m_StencilFail;\n nn::gd::OutputStage::StencilOperation m_StencilZFail;\n nn::gd::OutputStage::StencilOperation m_StencilZPass;\n DepthStencilStateDescription();\n void ToDefault();\n};\n",0]
,["Pages/Page_54693545.html#Anchor_54693602_デプステストの有効化（m_DepthEnable）","Enabling the Depth Test (m_DepthEnable) ",
"Enabling the Depth Test (m_DepthEnable)\nThe depth test is enabled if this is GD_TRUE.\n",0]
,["Pages/Page_54693545.html#Anchor_54693602_デプステストのライトマスク（m_DepthMask）","Write Mask for the Depth Test (m_DepthMask) ",
"Write Mask for the Depth Test (m_DepthMask)\nSets the write mask for the depth test.\n",0]
,["Pages/Page_54693545.html#Anchor_54693602_デプステストの比較関数（m_DepthFunc）","Comparison Function for the Depth Test (m_DepthFunc) ",
"Comparison Function for the Depth Test (m_DepthFunc)\nSets the comparison function for the depth test.\n",0]
,["Pages/Page_54693545.html#Anchor_54693602_ステンシルテストの有効化（m_StencilEnable）","Enabling the Stencil Test (m_StencilEnable) ",
"Enabling the Stencil Test (m_StencilEnable)\nThe stencil test is enabled if this is GD_TRUE.\n",0]
,["Pages/Page_54693545.html#Anchor_54693602_ステンシルテストのライトマスク（m_StencilWriteMask）","Write Mask for the Stencil Test (m_StencilWriteMask) ",
"Write Mask for the Stencil Test (m_StencilWriteMask)\nSets the write mask for the stencil test.\n",0]
,["Pages/Page_54693545.html#Anchor_54693602_ステンシルテストのリードマスク（m_StencilReadMask）","Read Mask for the Stencil Test (m_StencilReadMask) ",
"Read Mask for the Stencil Test (m_StencilReadMask)\nSets the read mask for the stencil test.\n",0]
,["Pages/Page_54693545.html#Anchor_54693602_ステンシルテストの比較関数（m_StencilFunc）","Comparison Function for the Stencil Test (m_StencilFunc) ",
"Comparison Function for the Stencil Test (m_StencilFunc)\nSets the comparison function for the stencil test.\n",0]
,["Pages/Page_54693545.html#Anchor_54693602_ステンシルテストの失敗時の処理（m_StencilFail）","Operation When the Stencil Test Fails (m_StencilFail) ",
"Operation When the Stencil Test Fails (m_StencilFail)\nSets the operation to run when the stencil test fails.\n",0]
,["Pages/Page_54693545.html#Anchor_54693602_ステンシルテストの成功時の処理（m_StencilZFail）","Operation When the Stencil Test Passes (m_StencilZFail) ",
"Operation When the Stencil Test Passes (m_StencilZFail)\nSets the operation to run when the stencil test passes, but the depth test fails.\n",0]
,["Pages/Page_54693545.html#Anchor_54693602_ステンシルテストとデプステストの成功時の処理（m_StencilZPass）","Operation When Both the Depth and Stencil Test Pass (m_StencilZPass) ",
"Operation When Both the Depth and Stencil Test Pass (m_StencilZPass)\nSets the operation to run when both the depth and stencil tests pass.\n",0]
,["Pages/Page_54693545.html#Anchor_54693602_デフォルトの設定（ToDefault()）","Default Settings (ToDefault) ",
"Default Settings (ToDefault)\nYou can call the ToDefault function to change all members to their default values. This is also done in the constructor.\nTable 4-24. Default Settings for the DepthStencilStateDescription Class\n \tMember \tDefault Settings\n \tm_DepthEnable \tGD_FALSE\n \tm_DepthMask \tDEPTH_WRITE_MASK_ALL\n \tm_DepthFunc \tDEPTH_FUNCTION_LESS\n \tm_StencilEnable \tGD_FALSE\n \t m_StencilWriteMask\n m_StencilReadMask\n \t0xFF\n \tm_StencilFunc \tSTENCIL_FUNCTION_NEVER\n \t m_StencilFail\n m_StencilZFail\n m_StencilZPass\n \tSTENCIL_OPERATION_KEEP\n",0]
,["Pages/Page_54693545.html#PageId_54693603","4.9.4. Alpha Test ",
"\n4.9.4. Alpha Test\nUse the nn::gd::OutputStage::SetAlphaTest function to configure the alpha test.\nCode 4-60. Configuring the Alpha Test\n static void nn::gd::OutputStage::SetAlphaTest(\n gdBool enable, nn::gd::OutputStage::AlphaFunction func, u8 alphaRef);\nTo enable the alpha test, specify GD_TRUE in the enable parameter, a comparison function in the func parameter, and a reference value in the alphaRef parameter. The default settings for these parameters are GD_FALSE, ALPHA_FUNCTION_NEVER, and 0, respectively.\n",0]
,["Pages/Page_54693545.html#PageId_54693604","4.9.5. Logical Operations and Blending ",
"\n4.9.5. Logical Operations and Blending\nBy creating a state (BlendState) object and then switching the BlendState object that is configured in the output stage, you can change all settings related to logical operations and blending in one operation.\nYou can create, release, and apply a BlendState object to the pipeline using the following functions.\nCode 4-61. Creating, Releasing, and Applying a BlendState Object to the Pipeline\n static nnResult nn::gd::OutputStage::CreateBlendState(\n const nn::gd::BlendStateDescription* desc,\n nn::gd::BlendState** blendState);\nstatic nnResult nn::gd::OutputStage::ReleaseBlendState(\n nn::gd::BlendState* blendState);\nstatic nnResult nn::gd::OutputStage::SetBlendState(\n const nn::gd::BlendState* blendState);\nConfigure logical operations and blending in a descriptor class (BlendStateDescription) that is specified in the desc parameter. For more information about the descriptor class settings, see 4.9.5.1. Descriptor Class for BlendState Objects.\nYou can use the nn::gd::OutputStage::SetBlendState function to apply a created BlendState object to the pipeline.\nUse the nn::gd::OutputStage::ReleaseBlendState function to release a state object that is no longer needed. This also releases memory regions that were allocated internally when the BlendState object was created as a state object.\nUse the following function to set the blend color. The default blend color is (0, 0, 0, 0).\nCode 4-62. Setting the Blend Color\n static void nn::gd::OutputStage::SetBlendColor(u8 R, u8 G, u8 B, u8 A);\n",0]
,["Pages/Page_54693545.html#PageId_54693605","4.9.5.1. Descriptor Class for BlendState Objects ",
"\n4.9.5.1. Descriptor Class for BlendState Objects\nThis section introduces the members of the descriptor (BlendStateDescription) class for BlendState objects and provides setting-related cautions and information.\nCode 4-63. Definition of the BlendStateDescription Class\n class nn::gd::BlendStateDescription\n{\n nn::gd::OutputStage::BlendType m_BlendType;\n nn::gd::OutputStage::LogicOperator m_LogicOp;\n nn::gd::OutputStage::BlendFunction m_SrcRgb;\n nn::gd::OutputStage::BlendFunction m_DstRgb;\n nn::gd::OutputStage::BlendFunction m_SrcAlpha;\n nn::gd::OutputStage::BlendFunction m_DstAlpha;\n nn::gd::OutputStage::BlendEquation m_EqRgb;\n nn::gd::OutputStage::BlendEquation m_EqAlpha;\n BlendStateDescription();\n void ToDefault();\n void SetBlendFunc(\n nn::gd::OutputStage::BlendFunction src,\n nn::gd::OutputStage::BlendFunction dst,\n nn::gd::OutputStage::BlendEquation eq);\n void SetBlendFunc(\n nn::gd::OutputStage::BlendFunction srcRgb,\n nn::gd::OutputStage::BlendFunction dstRgb,\n nn::gd::OutputStage::BlendFunction srcAlpha,\n nn::gd::OutputStage::BlendFunction dstAlpha,\n nn::gd::OutputStage::BlendEquation eqRgb,\n nn::gd::OutputStage::BlendEquation eqAlpha);\n void SetBlendMode_DefaultBlending();\n void SetBlendMode_NoBlend();\n void SetLogicOperatorMode(nn::gd::OutputStage::LogicOperator logicOp);\n void SetLogicOperatorMode_Default();\n};\n",0]
,["Pages/Page_54693545.html#Anchor_54693605_論理演算とブレンディングの選択（m_BlendType）","Choice Between Logical Operations and Blending (m_BlendType) ",
"Choice Between Logical Operations and Blending (m_BlendType)\nSet to BLEND_TYPE_LOGICOP when logical operations are used or BLEND_TYPE_BLENDING when blending is used.\n",0]
,["Pages/Page_54693545.html#Anchor_54693605_論理演算の演算方法（m_LogicOp）","Logical Operation (m_LogicOp) ",
"Logical Operation (m_LogicOp)\nIndicates which logical operation to use. You can set this value using the SetLogicOperatorMode function.\nSet to LOGIC_OPERATOR_COPY by the SetLogicOperatorMode_Default function.\n",0]
,["Pages/Page_54693545.html#Anchor_54693605_ブレンディングの重み係数（m_SrcRgb、m_DstRgb、m_SrcAlpha、m_DstAlpha）","Blend Weight Coefficients (m_SrcRgb, m_DstRgb, m_SrcAlpha, m_DstAlpha) ",
"Blend Weight Coefficients (m_SrcRgb, m_DstRgb, m_SrcAlpha, m_DstAlpha)\nThese are the blend weight coefficients. There are settings for the RGB and alpha components in both the source and destination.\n",0]
,["Pages/Page_54693545.html#Anchor_54693605_ブレンディングの計算式（m_EqRgb、m_EqAlpha）","Blend Equations (m_EqRgb, m_EqAlpha) ",
"Blend Equations (m_EqRgb, m_EqAlpha)\nThese are the blend equations. There is a setting for both the RGB and alpha components.\n",0]
,["Pages/Page_54693545.html#Anchor_54693605_ブレンディングの一括設定（SetBlendFunc()、SetBlendMode_NoBlend()）","Batch Blend Settings (SetBlendFunc, SetBlendMode_NoBlend) ",
"Batch Blend Settings (SetBlendFunc, SetBlendMode_NoBlend)\nThe SetBlendFunc function configures all blending operations at the same time. You can either configure the RGB and alpha components together or separately.\nThe SetBlendMode_NoBlend function changes settings so that there is no blending (the source is output unchanged).\nTable 4-25. Blend Settings Changed by SetBlendMode_NoBlend\n \tMember \tSetting\n \tm_SrcRgb, m_SrcAlpha \tBLEND_FUNCTION_ONE\n \tm_DstRgb, m_DstAlpha \tBLEND_FUNCTION_ZERO\n \tm_EqRgb, m_EqAlpha \tBLEND_EQUATION_ADD\n ",0]
,["Pages/Page_54693545.html#Anchor_54693605_デフォルトの設定（ToDefault()）","Default Settings (ToDefault) ",
"Default Settings (ToDefault)\nYou can call the ToDefault function to change all members to their default values. This is also done in the constructor.\nBlending is used by default, and the settings are the same as those configured by the SetBlendMode_NoBlend function.\n",0]
,["Pages/Page_54693545.html#PageId_54693606","4.9.6. W-Buffer and Polygon Offset ",
"\n4.9.6. W-Buffer and Polygon Offset\nUse the nn::gd::OutputStage::SetDepthRangeMode function to configure the w-buffer and polygon offset.\nCode 4-64. Setting the W-Buffer and Polygon Offset\n static void nn::gd::OutputStage::SetDepthRangeMode(\n nn::gd::OutputStage::DepthBufferType type,\n f32 depthRangeNear, f32 depthRangeFar, s32 offset);\nSpecify DEPTHBUFFER_LINEAR in the type parameter to use the w-buffer feature or specify DEPTHBUFFER_RECIPROCAL_FUNCTION to use the normal range of depth values instead.\nSpecify the near and far depth values in the depthRangeNear and depthRangeFar parameters when you do not use the w-buffer. Specify 0 in depthRangeNear and a scale value in depthRangeFar when you do use the w-buffer.\nWhen you use a polygon offset, specify the offset value in the offset parameter.\n",0]
,["Pages/Page_54693545.html#PageId_54693607","4.9.7. Soft Shadows ",
"\n4.9.7. Soft Shadows\nUse the nn::gd::OutputStage::SetPenumbraScaleBias function to set the scale and bias for soft shadows. Because this is an immediate function, 3D commands are accumulated in the command buffer when it is run.\nCode 4-65. Setting Soft Shadows\n static void nn::gd::OutputStage::SetPenumbraScaleBias(f32 Scale, f32 Bias);\nCONFIDENTIAL\n",0]
,["Pages/Page_54693608.html","5. Layout Conversion ",
"5. Layout Conversion\n",0]
,["Pages/Page_54693608.html#PageId_54693609","5.1. Converting From Block Format to Linear Format ",
"The nn::gd::Memory class defines functions that convert layouts from block format to linear format and vice versa.\n5.1. Converting From Block Format to Linear Format\nThe following functions are provided for converting from block format to linear format.\nCode 5-1. Functions for Converting From Block Format to Linear Format\n static nnResult nn::gd::Memory::CopyTexture2DResourceBlockToLinear(\n const nn::gd::Texture2DResource* source,\n s32 srcMipLevelIndex, u32 srcOffsetY, s32 srcCountRow,\n const nn::gd::Texture2DResource* dest,\n s32 dstMipLevelIndex, u32 dstOffsetY,\n nn::gd::Memory::DownScalingMode downScalingMode, gdBool yFlip);\nstatic nnResult nn::gd::Memory::CopyTexture2DResourceBlockToLinear(\n const nn::gd::Texture2DResource* source,\n s32 srcMipLevelIndex, u32 srcOffsetY,\n u8* dstAddr, u32 dstWidth, u32 dstHeight, u32 dstFormat,\n nn::gd::Memory::DownScalingMode downScalingMode, gdBool yFlip);\nThe resource data in the Texture2DResource object specified for the source parameter is converted from block format to linear format and then transferred. You can use the srcMipLevelIndex parameter to specify which mipmap level to convert. If you specify a value of -1, the largest mipmap level of the Texture2DResource object is used.\nUse the srcOffsetY parameter to specify the y-coordinate at which to start converting. Specify the raw resource data coordinates. These functions take care of addressing differences.\nWhen you call the function that takes a Texture2DResource object as the destination, use the srcCountRow parameter to specify the number of lines to convert. If you specify a value of -1, the height of the Texture2DResource object is used. Use the dstMipLevelIndex parameter and the dstOffsetY parameter to specify the mipmap level and starting y-coordinate, respectively, at the destination.\nWhen you call the function that takes an address as the destination, specify the starting address in the dstAddr parameter, the width in the dstWidth parameter, the height in the dstHeight parameter, and the pixel format in the dstFormat parameter.\nConfigure anti-aliasing using the downScalingMode parameter. To enable vertical flipping, specify GD_TRUE in the yFlip parameter.\nData is actually converted when the command list is executed, because these functions call the nngxAddB2LTransferCommand function.\nThese functions, which convert from block format to linear format, are mainly used when the color buffer is transferred to the display buffer. For sample code, see 2.7. Displaying the Rendered Results.\n",0]
,["Pages/Page_54693608.html#PageId_54693610","5.2. Converting From Linear Format to Block Format ",
"\n5.2. Converting From Linear Format to Block Format\nThe following functions have been provided for converting from linear format to block format.\nCode 5-2. Functions for Converting From Linear Format to Block Format\n static nnResult nn::gd::Memory::CopyTexture2DResourceLinearToBlock(\n const nn::gd::Texture2DResource* source,\n s32 srcMipLevelIndex, u32 srcOffsetY, s32 srcCountRow,\n const nn::gd::Texture2DResource* dest,\n s32 dstMipLevelIndex, u32 dstOffsetY);\nstatic nnResult nn::gd::Memory::CopyTexture2DResourceLinearToBlock(\n u8* srcAddr, u32 width, u32 height,\n const nn::gd::Texture2DResource* dest,\n s32 dstMipLevelIndex, u32 dstOffsetY);\nThe original data is converted from linear format to block format and then transferred to the Texture2DResource object specified for the dest parameter. You can use the dstMipLevelIndex parameter to specify the mipmap level in which to receive the converted data. If you specify a value of -1, the largest mipmap level of the Texture2DResource object is used. Use dstOffsetY to specify the y-coordinate at which to start receiving data. Specify the raw resource data coordinates. These functions take care of addressing differences.\nWhen you call the function that takes a Texture2DResource object as the source, use the srcCountRow parameter to specify the number of lines to convert. If you specify a value of -1, the height of the Texture2DResource object is used. Use the srcMipLevelIndex parameter and the srcOffsetY parameter to specify the mipmap level and starting y-coordinate, respectively, at the source.\nWhen you call the function that takes an address as the source, specify the starting address in the srcAddr parameter, the width in the width parameter, and the height in the height parameter.\nData is actually converted when the command list is executed, because these functions call the nngxAddL2BTransferCommand function.\nCONFIDENTIAL\n",0]
,["Pages/Page_54693611.html","6. Packet Recording ",
"6. Packet Recording\nUsing the packet recording feature, you can record a group of command packets and then replay them later. This feature is useful when the majority of an image requires repeated rendering (excluding camera, light, and other settings). This is the same as rendering a scene twice stereoscopically.\nThe following functions control the packet recording feature.\nCode 6-1. Functions Used to Control the Packet Recording Feature\n nn::gd::System::StartRecordingPackets(\n u32* forceDirtyModuleFlag,\n nn::gd::RecordingPacketUsage usage = RECORD_COMMAND_LIST_COPY);\nnn::gd::System::StopRecordingPackets(nn::gd::RecordedPacketId** packetId);\nnn::gd::System::ReplayPackets(nn::gd::RecordedPacketId* packetId,\n u32* forceDirtyModuleFlag);\nnn::gd::System::ReleasePackets(nn::gd::RecordedPacketId* packetId);\nYou can record both 3D commands output to the 3D command buffer and command requests (see 7.1. Internal Use of nngx Functions) output to the request buffer.\nYou can use the nn::gd::System::ReleasePackets function to release command packets that you no longer need.\n",0]
,["Pages/Page_54693611.html#Anchor_54693611_出力される3Dコマンドへの内部状態の影響","Effects of the Internal State on Outputting 3D Commands ",
"Effects of the Internal State on Outputting 3D Commands\nDepending on the internal state of the GD library, functions do not necessarily output 3D commands. This may cause behavior to deviate from what was expected when the command packets were recorded. Similarly, the state of the GPU may be inconsistent with the internal state of the GD library when the command packets are replayed. To work around this problem, the nn::gd::System::StartRecordingPackets and nn::gd::System::ReplayRecordingPackets functions each accept flags as arguments, allowing you to indicate which modules to mark as dirty before the function is executed.\nThe safest course of action is to specify the MODULE_ALL flag and mark all modules as dirty. This causes all 3D commands to be output and definitely prevents erroneous states. When you use this feature, if you know which module's state will be recorded and replayed and whether that particular state will cause a conflict, you can improve performance by specifying only the minimum number of necessary modules as dirty.\n",0]
,["Pages/Page_54693611.html#Anchor_54693611_コマンドパケットの編集","Editing Command Packets ",
"Editing Command Packets\nYou can edit recorded command packets before you replay them. To calculate the 3D command buffer position to update, combine two values: The first value is the offset for output into the 3D command buffer that is returned by the nn::gd::System::GetCommandBufferOffset function while packets are being recorded (between calls to the nn::gd::System::StartRecordingPackets and nn::gd::System::StopRecordingPackets functions). The second value is the base address of the 3D command buffer that is returned by the nn::gd::System::GetCommandBufferBaseAddress function after you have finished recording packets. However, to overwrite 3D commands, you can only use immediate functions, which output 3D commands to the 3D command buffer.\n",0]
,["Pages/Page_54693611.html#Anchor_54693611_引数usageによる3Dコマンドバッファのコピー指定","Using the usage Parameter to Issue an Instruction to Copy the 3D Command Buffer ",
"Using the usage Parameter to Issue an Instruction to Copy the 3D Command Buffer\nTo create a copy of the 3D command buffer, specify Record_Command_List_Copy in usage under the nn::gd::System::StartRecordingPackets function as was done before adding the argument. The 3D command in the command packet that was recorded can be overwritten for this purpose.\nOn the other hand, specifying Record_Command_List_No_Copy in usage means that a 3D command buffer copy will not be created. This shortens processing time, but the 3D command cannot be overwritten. In addition, clearing the 3D command buffer acting as the source of the copy renders the command packet unexecutable.\nWhen packet recording is started with RECORD_3D_COMMAND_BUFFER_FOR_JUMP specified in usage, the nn::gd::System::ReplyPackets function creates a copy of the 3D command buffer, and it is executed with a jump by inserting a jump command using the nngxAddSubroutineCommand function.\nCONFIDENTIAL\n",0]
,["Pages/Page_54693615.html","7. Relationship Between the GD and GX Libraries ",
"7. Relationship Between the GD and GX Libraries\n",0]
,["Pages/Page_54693615.html#PageId_54693616","7.1. Internal Use of nngx Functions ",
"This chapter explains which nngx functions are called by functions in this library, and when you need to pay particular attention to nngx functions.\n7.1. Internal Use of nngx Functions\nSome of the functions in the GD library call nngx functions. As long as no errors have occurred, the functions in the following table call nngx functions when the condition in the Condition column is met. However, the same function may call different nngx functions depending on the condition.\nIf \"Always\" is given as a condition, the corresponding nngx functions are always called unless an error has occurred.\nTable 7-1. GD Functions That Call nngx Functions\n \tGD Function \tConditions \tnngx Functions\n \tSystem::\nStartRecordingPackets \tAlways \tnngxStartCmdlistSave\n \tSystem::\nStopRecordingPackets \tAlways \t nngxSplitDrawCmdlist\n nngxStopCmdlistSave\n nngxExportCmdlist\n \tSystem::ReplayPackets \tAlways \tnngxImportCmdlist\n \tSystem::\nExecute3DCommandList \tAlways \tnngxSplitDrawCmdlist\n \tResource::\nCreateTexture2DResource \tThere is initial data, and data is copied to a region allocated in VRAM. \tnngxAddVramDmaCommand\n \tThere is initial data, and data is copied to a region allocated in device memory. \tnngxUpdateBuffer\n \tThere is initial data, and mipmaps are generated automatically. \tnngxFilterBlockImage\n \tMemory::GenerateMipMaps \tAlways \tnngxFilterBlockImage\n \tRendering dependency \tnngxSplitDrawCmdlist\n \tResource::\nCreateVertexBufferResource \tThere is initial data, and data is copied to a region allocated in VRAM. \tnngxAddVramDmaCommand\n \tThere is initial data, and data is copied to a region allocated in device memory. \tnngxUpdateBuffer\n \tMemory::\nCopyTextureSubResource \tAlways \tnngxAddBlockImageCopyCommand\n \tRendering dependency \tnngxSplitDrawCmdlist\n \tMemory::\nCopyVertexBufferSubResource \tAlways \tnngxAddBlockImageCopyCommand\n \tMemory::\nCopyTexture2DResource\nBlockToLinear \tAlways \tnngxAddB2LTransferCommand\n \tRendering dependency \tnngxSplitDrawCmdlist\n \tMemory::\nCopyTexture2DResource\nLinearToBlock \tAlways \tnngxAddL2BTransferCommand\n \tRendering dependency \tnngxSplitDrawCmdlist\n \tMemory::ClearTargets \tAlways \tnngxAddMemoryFillCommand\n \tRendering dependency \tnngxSplitDrawCmdlist\n \tMemory::\nClearTexture2DResource \tAlways \tnngxAddMemoryFillCommand\n \tRendering dependency \tnngxSplitDrawCmdlist\n \tResource::\nUnmapTexture2DResource \tA mapping that involves writing to resources in device memory \tnngxUpdateBuffer\n \tResource::\nUnmapVertexBufferResource \tA mapping that involves writing to resources in device memory \tnngxUpdateBuffer\n ",0]
,["Pages/Page_54693615.html#PageId_54693617","7.2. Interaction With nngx Functions ",
"\n7.2. Interaction With nngx Functions\n",0]
,["Pages/Page_54693615.html#PageId_54693618","7.2.1. Creating Textures With Initial Data in VRAM ",
"This section explains when you need to pay particular attention to nngx functions while using the GD library.\n7.2.1. Creating Textures With Initial Data in VRAM\nWhen you create a resource in VRAM using initial data, a DMA transfer command request is added to the command request queue. This command transfers the initial data from main memory to VRAM. Because the GD library does not manage command request execution, applications must confirm that the DMA transfer has completed before releasing the initial data.\nAs shown in the following sample code, you can call the nngxWaitCmdlistDone function to wait for all of the command requests to complete. However, if the command list is double-buffered, the DMA transfer command request is not executed until the next frame after it was added to the current command request queue.\nCode 7-1. Creating Textures With Initial Data in VRAM\n nn::gd::Texture2DResource* texture2DResource = 0;\nnn::gd::Texture2DResourceDescription Text2DResDesc =\n{\n width, height, 1, nn::gd::Resource::NATIVE_FORMAT_RGB_888,\n nn::gd::Memory::LAYOUT_BLOCK_8, nn::gd::Memory::VRAMA\n};\nnn::gd::Resource::CreateTexture2DResource(\n &Text2DResDesc, data, GD_TRUE, &texture2DResource);\n// We need to confirm that the DMA transfer has completed before releasing the resource, which is created in VRAM.\nnngxWaitCmdlistDone();\nfree(data);\n",0]
,["Pages/Page_54693615.html#PageId_54693619","7.2.2. Accessing Rendered Results ",
"\n7.2.2. Accessing Rendered Results\nThe CPU cannot directly access data in VRAM. If you want to access the contents of the color buffer, depth (stencil) buffer, and textures to directly overwrite something you have already rendered, you must take the following steps.\n\tAllocate a temporary resource in device memory. \tCopy the resource in VRAM to your temporary resource (transfer data from VRAM). \tOverwrite the temporary resource. \tWrite the content of the temporary resource back to the resource in VRAM (transfer data to VRAM).\nCommand requests transfer data to and from VRAM. You must wait for the command requests to complete before you overwrite the resource or deallocate the temporary resource.\nThis procedure can be expressed by the following code.\nCode 7-2. Accessing Rendered Results\n // Allocate a temporary resource in device memory (FCRAM).\nnn::gd::Texture2DResource* tmpResource= 0;\nnn::gd::Texture2DResourceDescription tmpResourceDesc =\n{\n s_RenderTextureWidth, s_RenderTextureHeight, 1,\n nn::gd::Resource::NATIVE_FORMAT_RGBA_8888,\n nn::gd::Memory::LAYOUT_BLOCK_8,\n nn::gd::Memory::FCRAM\n};\nres = nn::gd::Resource::CreateTexture2DResource(\n &tmpResourceDesc, 0, GD_FALSE, &tmpResource);\n// Copy the resource in VRAM to the temporary resource.\nnn::gd::Memory::Rect memRect(0, 0, -1, -1);\nres = nn::gd::Memory::CopyTextureSubResource(\n s_texture2DResource_ColorBuffer, 0, memRect, tmpResource, 0, 0, 0);\n// Map the resource to access it.\nu8* data;\nnn::gd::Resource::MapTexture2DResource(\n tmpResource, 0, nn::gd::Resource::MAP_READ_WRITE, (void**)&data);\n// You must wait for the command request to complete before accessing data.\nnngxWaitCmdlistDone();\n// Write to the mapped resource here.\nfor (int y=0; y<s_RenderTextureHeight*s_RenderTextureWidth>>1; y++)\n{\n data[k+] = 255; data[k] = 255; data[k] = 255; data[k+] = 255;\n}\n// Change to a temporary resource is finalized on unmapping.\nnn::gd::Resource::UnmapTexture2DResource(tmpResource);\n// Copy the temporary resource to the resource in VRAM.\nnn::gd::Memory::CopyTextureSubResource(\n tmpResource, 0, memRect, s_texture2DResource_ColorBuffer, 0, 0, 0);\nCONFIDENTIAL\n",0]
,["Pages/Page_54693620.html","8. Coordinating Operations With Other Libraries and Frameworks ",
"8. Coordinating Operations With Other Libraries and Frameworks\n",0]
,["Pages/Page_54693620.html#PageId_54693621","8.1. Cautions for Initialization ",
"This chapter provides cautions for using the GD library along with other graphics-related libraries and frameworks.\n8.1. Cautions for Initialization\nWhen you use the same command list with other frameworks (such as NintendoWare), make sure that you do not initialize it more than once.\nFor example, in Code 2-1, only the GD library's initialization function (nn::gd::System::Initialize) is called when NintendoWare's command list-related initialization has finished.\n",0]
,["Pages/Page_54693620.html#PageId_54693622","8.2. Using nn::gd::System::ForceDirty ",
"\n8.2. Using nn::gd::System::ForceDirty\nThe GD library's internal state may lose synchronization with the GPU when some other library or framework has written 3D commands to the 3D command buffer. If this happens, call the nn::gd::System::ForceDirty function to mark several modules as dirty and resend all 3D commands for the corresponding modules.\nModules that have been marked as dirty by the ForceDirty function generate 3D commands when a rendering command is executed. When 3D commands have already been generated, call the nn::gd::System::FlushDirtyModule function.\nThe GD library does not maintain the content of registers that are set by immediate functions. When you have used an immediate function to alter the content of a register, you must call the immediate function to reset the register, after the 3D commands have been generated.\nFor more information about the internal state, see 1.2.2. Managing Module States.\n",0]
,["Pages/Page_54693620.html#PageId_54693623","8.3. Duplicate Command Lists ",
"\n8.3. Duplicate Command Lists\nThe GD library has some functions that queue command requests. After command requests are queued in a command list, they are not executed until the command list is executed. You must avoid overwriting or deleting resources handled by those command requests before they are executed.\nThis caution also applies to frameworks that use two command lists, because command requests are queued in the command list that is used to create the next frame.\n",0]
,["Pages/Page_54693620.html#PageId_54693624","8.4. Inserting Dummy Commands ",
"\n8.4. Inserting Dummy Commands\nDummy commands that are not recognized by the GPU as commands can be added to the command buffer by calling the nn::gd::System::AddDummyCommands function. Dummy commands can be used when you need to wait for the next command to execute, or need to adjust the size of the command buffer.\nCONFIDENTIAL\n",0]
,["Pages/Page_54693625.html","9. Appendix ",
"9. Appendix\n",0]
,["Pages/Page_54693625.html#PageId_54693626","9.1. List of Immediate Functions ",
"9.1. List of Immediate Functions\nThe following table lists the immediate functions.\nTable 9.1. Immediate Functions\n \tClass \tFunction\n \tnn::gd::CombinerStage \tSetTextureCombinerUnitConstantColor\n \tnn::gd::FogStage \tUploadFogLookUpTableFloat\n \tUploadFogLookUpTableNative\n \tUploadGasLookUpTableFloat\n \tUploadGasLookUpTableNative\n \tnn::gd::LightingStage \tSetGlobalColorAmbient\n \tUploadGasLookUpTableFloat\n \tUploadLookUpTableNative\n \tnn::gd::Light \tSetColorAmbient\n \tSetColorDiffuse\n \tSetColorSpecular0\n \tSetColorSpecular1\n \tSetPosition\n \tSetDirection\n \tSetSpotDirection\n \tSetLightType\n \tSetDistanceAttenuationScaleBias\n \tEnableTwoSideDiffuse\n \tEnableGeomFactor0\n \tEnableGeomFactor1\n \tUploadLookUpTableFloat\n \tUploadLookUpTableNative\n \tnn::gd::ProceduralTextureStage \tUploadLookUpTableRgbMapFloat\n \tUploadLookUpTableRgbMapNative\n \tUploadLookUpTableAlphaMapFloat\n \tUploadLookUpTableAlphaMapNative\n \tUploadLookUpTableNoiseMapFloat\n \tUploadLookUpTableNoiseMapNative\n \tUploadLookUpTableColorMapFloat\n \tUploadLookUpTableColorMapNative\n \tnn::gd::ShaderStage \tSetFloatConstantBuffer\n \tnn::gd::OutputStage \tSetPenumbraScaleBias\n \tnn::gd::RasterizerStage \tSetCulling\n \tSetViewport\n \tEnableClippingPlane\n \tSetClippingPlane\n \tEnableScissor\n \tSetScissor\n ",0]
,["Pages/Page_54693625.html#PageId_54693627","9.2. Memory Allocation Within Functions ",
"\n9.2. Memory Allocation Within Functions\nThe sizes in the following table are passed to the allocator that is specified when the GX library is initialized.\nUnless specified otherwise in the Comments column, memory is allocated as a system buffer (NN_GX_MEM_SYSTEM) from device memory (NN_GX_MEM_FCRAM).\nTable 9-2. Memory Allocated Within Functions\n \tFunction \tSize \tComments\n \tSystem::Initialize \t24 bytes \tManagement information\n \t12 bytes \tManagement information\n \t12 bytes \tManagement information\n \t28 bytes \tManagement information\n \t12 bytes \tManagement information\n \t1536 bytes \tManagement information\n \t52 bytes \tManagement information\n \t28 bytes \tManagement information\n \t12 bytes \tManagement information\n \t1664 bytes \tManagement information\n \t56 bytes \tManagement information\n \t28 bytes \tManagement information\n \t12 bytes \tManagement information\n \t2688 bytes \tManagement information\n \t28 bytes \tManagement information\n \t12 bytes \tManagement information\n \t14,208 bytes \tManagement information\n \t616 bytes \tManagement information\n \t28 bytes \tManagement information\n \t12 bytes \tManagement information\n \t1664 bytes \tManagement information\n \t60 bytes \tManagement information\n \t28 bytes \tManagement information\n \t12 bytes \tManagement information\n \t640 bytes \tManagement information\n \t328 bytes \t3D command\n(Changes the shared processor settings to use the geometry shader.)\n \t328 bytes \t3D command\n(Changes the shared processor settings to not use the geometry shader.)\n \tResource::CreateTexture2DResource \t52 bytes \tManagement information\n \t16 bytes \tManagement information\n \tData-dependent \tAllocated with NN_GX_MEM_TEXTURE in the specified memory as long as there is initial data and copies are made.\n \tResource::\nCreateTexture2DResourceCastFrom \t64 bytes \tManagement information\n \tResource::Helper::\nConvertTextureResourceToNativeFormat \tData-dependent \tThis memory is allocated for a temporary buffer used by the conversion and is released when the function exits.\n \tResource::Helper::ConvertCompressed\nTextureResourceToNativeFormat \tData-dependent \tThis memory is allocated for a temporary buffer used by the conversion and is released when the function exits.\n \tResource::CreateRenderTarget \t48 bytes \tManagement information\n \tResource::CreateDepthStecilTarget \t52 bytes \tManagement information\n \tTextureStage::CreateTexture2D \t64 bytes \tManagement information\n \tTextureStage::CreateTextureCube \t112 bytes \tManagement information\n \tShaderStage::CreateShaderBinary \t28 bytes \tManagement information\n \tData-dependent \tManagement information\n \tData-dependent \tManagement information\n \tData-dependent \tManagement information\n \tData-dependent \tManagement information\n \tData-dependent \tManagement information\n \tData-dependent \tManagement information\n \t20 bytes \tManagement information\n \tData-dependent \t3D commands\n \tData-dependent \t3D commands\n \tShaderStage::CreateShader \t20 bytes \tManagement information\n \tData-dependent \t3D commands\n \tShaderStage::CreateShaderPipeline \t148 bytes \tManagement information\n \tData-dependent \t3D commands\n \tData-dependent \tUniform setting values\n(vertex shader)\n \tData-dependent \tUniform setting values (geometry shader)\n \tVertexInputStage::CreateInputLayout \t204 bytes \tManagement information\n \tData-dependent \tInputElementDescription classes\n \tResource::CreateVertexBufferResource \t24 bytes \tManagement information\n \tData-dependent \tAllocated with NN_GX_MEM_VERTEXBUFFER in the specified memory as long as there is initial data and copies are made.\n \tCombinerStage::\nCreateTextureCombinerState \t616 bytes \tManagement information\n \t92 bytes \tManagement information\n \tTextureStage::CreateSamplerState \t60 bytes \tManagement information\n \tOutputStage::CreateDepthStecilState \t52 bytes \tManagement information\n \tOutputStage::CreateBlendState \t56 bytes \tManagement information\n \tSystem::StartRecordingPackets \t36 bytes \tManagement information\n \tSystem::StopRecordingPackets \tData-dependent \tRecorded 3D commands\n ",0]
,["Pages/Page_54693625.html#PageId_54693628","9.3. Debugging Support Features ",
"\n9.3. Debugging Support Features\n",0]
,["Pages/Page_54693625.html#PageId_54693629","9.3.1. Filter Features ",
"The GD library provides various functions that are useful for debugging, such as filter features that modify the rendering behavior. Filter functions are only enabled when compiled with Debug or Development.\n9.3.1. Filter Features\nFilter features allow the rendering behavior to be modified by setting various flags. Filter features are only enabled when compiled with Debug or Development.\nCode 9-1. Function for Specifying Filter Features\n static nnResult nn::gd::System::Debug::SetFilter(\n u32 filterFlag, u32 drawStartIndex, s32 drawCount);\nThe filter features are enabled after the number of rendering operations specified by drawStartIndex, and are enabled for the number of render operations specified by drawCount.\nThe filters to be enabled are specified in filterFlag by logically ORing flag values from the following table.\nTable 9-3. Filter Feature Flags\n \tFlag \tDescription\n \tFILTER_NONE \tSpecifying this flag by itself disables all filter features.\n \tFILTER_TEXTURE_8X8 \tConvert all textures used in rendering to 8x8 textures.\n \tFILTER_TEXTURE_CUSTOM \tConvert all textures used in rendering to the custom textures specified using the SetCustomTexture function. A fixed 8×8 texture is used if no custom textures have been specified.\n \tFILTER_SAMPLER_STATE_FILTER_\nNEAREST_MIPMAP_NEAREST \tForces the filter setting for texture shrinking to GL_NEAREST_MIPMAP_NEAREST.\n \tFILTER_SAMPLER_STATE_FILTER_\nLINEAR_MIPMAP_NEAREST \tForces the filter setting for texture shrinking to GL_LINEAR_MIPMAP_NEAREST.\n \tFILTER_VIEWPORT_1X1 \tChanges the viewport setting to 1×1.\n \tFILTER_SIMPLE_COMBINER \tChanges the combiner settings so that all pixels are output as green.\n \tFILTER_DISABLE_BLENDING \tDisables blending.\n \tFILTER_DISABLE_DEPTH_STENCIL_TEST \tDisables depth stencil testing.\n \tFILTER_DISABLE_ALPHA_TEST \tDisables alpha testing.\n \tFILTER_LIGHTING_STAGE_1_LIGHT_MAX \tSets the maximum number of light sources to 1. If multiple lights have been configured, only the first light that is enabled will be enabled.\n \tFILTER_LIGHTING_STAGE_SIMPLE_\nLAYER_CONFIGURATION \tChanges to the layer configuration requiring the smallest number of cycles.\n \tFILTER_DISABLE_DRAW \tDisables rendering.\n \tFILTER_VISUALIZE_OVERDRAW \tAdds a blending configuration so that rendered pixels gradually go white, allowing pixels that are rendered repetitively to be visualized. Using this together with the filter disabling depth stencils allows pixels discarded (or not discarded) in testing to be checked.\n \tFILTER_DISABLE_CULLING \tDisables culling.\n \tFILTER_ENABLE_CULLING_CLOCKWISE \tChanges culling to clockwise.\n \tFILTER_ENABLE_CULLING_COUNTERCLOCKWISE \tChanges culling to counterclockwise.\n ",0]
,["Pages/Page_54693625.html#PageId_54693630","9.3.2. Mipmap Level Visualization ",
"\n9.3.2. Mipmap Level Visualization\nThe nn::gd::System::Debug::ColorizeMipmaps function supports the visualization of mipmap levels by filling in each mipmap level specified in the texture source with a specified color.\nCode 9-2. Function Supporting Visualization of Mipmap Levels\n static nnResult nn::gd::System::Debug::ColorizeMipmaps(\n nn::gd::Texture2DResource* texture2DResource,\n s32 minMipLevelIndex, s32 maxMipLevelIndex, u8 colors[][4]);\nThe minimum and maximum indices of the mipmap levels to be filled in are specified in minMipLevelIndex and maxMipLevelIndex respectively. When -1 is specified for either of these parameters, the maximum mipmap level of the texture resource specified by texture2DResource is specified.\nThe colors specified in colors are used for the mipmap levels, starting at minMipLevelIndex. In other words, the texture of the mipmap level specified by minMipLevelIndex is filled in with the color specified by colors[0], and the texture of the next mipmap level is filled in with the color specified by colors[1].\nIf NULL is specified for colors, a default color array is used to fill in the textures. For information about the default color array, see the API Reference.\nCONFIDENTIAL\n",0]
];
for (var x = 0; x < Search.additionalData.length; x++) Search.indexData.push(Search.additionalData[x]);
